<?php

namespace App\Http\Controllers;

use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;

class AuthController extends Controller
{
    public function validar(Request $request)
    {
        if ($request->user()){
            return response()->json(true);
        } else {
            return response()->json(false);
        }
    }

    public function login(Request $request)
    {

        if (!isset($request->idUser) || is_null($request->idUser)){
            $request->validate([
                'email' => 'required|string|email' ,
                'password' => 'required|string' ,
                'remember_me' => 'boolean' ,
            ] , [
                'email.required' => 'El email es requerido' ,
                'email.email' => 'Debe ser un correo válido test@gmail.com' ,
                'password.required' => 'El contraseña es requerida' ,
            ]);
        }
        $cre;

        if (isset($request->idUser) && !is_null($request->idUser)){
            //para autenticar solo con el id
            $cre = Auth::loginUsingId($request->idUser);
        } else {
            //para autenticar con el email y el password
            $credentials = request(['email' , 'password']);
            $cre         = Auth::attempt($credentials);
        }

        if (!$cre){
            if (count(User::where('email' , $request->email)->get()) > 0){
                return response()->json([
                    'message' => 'Clave errada' ,
                ] , 401);
            } else {

                return response()->json([
                    'message' => 'Usuario no existe' ,
                ] , 401);
            }
        }

        $user = is_null(@$request->user()) ? Auth::user() : $request->user();

        $tokenResult = $user->createToken('Personal Access Token');
        $token       = $tokenResult->token;

        if ($request->remember_me){
            $token->expires_at = Carbon::now()->addDay();
        }


        $token->save();


        return response()->json([
            'access_token' => $tokenResult->accessToken ,
            'token_type' => 'Bearer' ,
            'expires_at' => Carbon::parse($tokenResult->token->expires_at)->toDateTimeString() ,
            'user' => $request->user()
        ]);
    }

    /**
     * Logout user (Revoke the token)
     *
     * @return [string] message
     */
    public function logout(Request $request)
    {

//        /* HISTÓRICO DE VISITAS*/
//        MetricaVisitasUser::create([
//            'date' => Carbon::now()->format('Y-m-d H:i:s'),
//            'fk_idUser' => @$request->user()->id,
//            'detalle' => 'El usuario: ' . $request->user()->name . ', mail: ' . $request->user()->email . ', cerró sesión',
//            'status' => 0,
//        ]);

        @$request->user()->token()->revoke();

        return response()->json([
            'message' => 'Se ha desconectado correctamente' ,
        ]);
    }

    /**
     * Get the authenticated User
     *
     * @return [json] user object
     */
    public function user(Request $request)
    {
        try {
            $u = $request->user();
//            if (!is_null($u->fotoPerfil)) {
//                $u->fotoPerfilURL = asset('storage\\fotoPerfil\\' . $request->user()->fotoPerfil);
//            }
//            $u->metodoDePago; //plan de pago
//            $u->statusDePago;
//            //$u->reputacion;
//            $u->asistenciaPosibleEncuentro;
            //$u->statusUser2;
            return response()->json($u , 200);
        } catch (\Exception $e) {
            Log::error('Ha ocurrido un error en AuthController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de obtener los datos.' ,
            ] , 500);
        }
    }

    public function redirectToProvider()
    {
        return Socialite::driver('google')->redirect();
    }

    public function handleProviderCallback()
    {
        try {
            $user = Socialite::driver('google')->user();

        } catch (\Exception $e) {
            Log::error('Ha ocurrido un error en AuthController. Msj: ' . $e->getMessage() . ', Linea: ' . $e->getLine() . ', Archivo: ' . $e->getFile());

            return response()->json([
                'message' => 'Problemas al autenticar con el servicio de Google' ,
                'error' => $e->getMessage() ,
                'line' => $e->getLine() ,
            ] , 500);
        }
        try {
            $datos;
            // comprobar si existe el usuario
            $existingUser = User::where('email' , $user->email)->first();
            if ($existingUser){
                //loguar si existe
                $datos = $this->login(new Request(['idUser' => $existingUser->id]));
            } else {
                // crear un nuevo usuario y luego regresar el tocken
                $newUser                         = new User;
                $newUser->name                   = $user->name;
                $newUser->email                  = $user->email;
                $newUser->fk_statusUser          = 1; // activo
                $newUser->fk_rolId               = 1; //cliente por defecto
                $newUser->tockenGoogle           = $user->token;
                $newUser->fecha_vencimiento_pago = Carbon::now()->addMonth();
                $newUser->fk_statusPago          = 1; //por defecto metodo de pago gratis para el usuario
                $newUser->statusUser             = 1;

                $newUser->save();
                //Mail::to($user->email)->send(new NuevoRegistroMail($newUser->tockenActivarCuenta, $newUser));
                $datos = $this->login(new Request(['idUser' => $newUser->id]));
            }

            $tocken     = $datos->getData()->access_token;
            $ruta_front = env('URL_FRONT_RECIBE_TOCKEN');

            //$ruta_front = 'http://www.bioonix.com/libreando/#/auth/login/';

            return redirect()->to($ruta_front . $tocken);
        } catch (\Exception $e) {
            Log::error('Ha ocurrido un error en AuthController. Msj: ' . $e->getMessage() . ', Linea: ' . $e->getLine() . ', Archivo: ' . $e->getFile());

            return response()->json([
                'error' => $e->getMessage() ,
                'line' => $e->getLine() ,
            ] , 500);
        }

    }

    public function loginGoogleApp(Request $request)
    {

        $datos;
        // comprobar si existe el usuario
        $this->validate($request , [
            'email' => 'required' ,
        ] , [
            'email.required' => 'El Email es requerido' ,
        ]);

        $existingUser = User::where('email' , $request->email)->first();
        if ($existingUser){
            //loguar si existe
            $datos = $this->login(new Request(['idUser' => $existingUser->id]));
        } else {
            // crear un nuevo usuario y luego regresar el tocken
            $this->validate($request , [
                'name' => 'required' ,
                'email' => 'required' ,
                'token' => 'required' , //se debe enviar automaticamente de google desde la vista
            ] , [
                'name.required' => 'El Nombre es requerido' ,
                'email.required' => 'El Email es requerido' ,
                'token.required' => 'El token de google es requerido' ,
            ]);

            $newUser                         = new User;
            $newUser->name                   = $request->name;
            $newUser->email                  = $request->email;
            $newUser->fk_statusUser          = 1; // activo
            $newUser->fk_rolId               = 1; //cliente por defecto
            $newUser->tockenGoogle           = $request->token;
            $newUser->fk_statusPago          = 1; //por defecto metodo de pago gratis para el usuario
            $newUser->statusUser             = 1;
            $newUser->fecha_vencimiento_pago = Carbon::now()->addMonth();
            $newUser->save();
            //Mail::to($user->email)->send(new NuevoRegistroMail($newUser->tockenActivarCuenta, $newUser));
            $datos = $this->login(new Request(['idUser' => $newUser->id]));
        }

        return $datos;

        //$tocken     = $datos->getData()->access_token;
        //$ruta_front = env('URL_FRONT_RECIBE_TOCKEN');

        //$ruta_front = 'http://www.bioonix.com/libreando/#/auth/login/';

        //return redirect()->to($ruta_front.$tocken);


    }

    public function getUserGoogleFromToken($token)
    {
        try {
            $user = Socialite::driver('google')->userFromToken($token);

            return response()->json($user);
        } catch (\Exception $e) {
            Log::error('Ha ocurrido un error en AuthController. Msj: ' . $e->getMessage() . ', Linea: ' . $e->getLine() . ', Archivo: ' . $e->getFile());

            return 'Problemas al obtener los datos con Google';
        }

    }

    public function getRolUser(Request $request)
    {
        $response = [
            'msj' => 'Rol del user' ,
            'rol' => Rol::find($request->user()->fk_rolId) ,
        ];

        return response()->json($response , 202);

    }

    public function setTimeUser(Request $request)
    {
        return response()->json(["request" => $request->all()]);
    }
}
