<?php

namespace App\Http\Controllers;

use App\Exports\ComisionPorVentasExport;
use App\Exports\OperacionesExport;
use App\Mail\NotificacionStatusOrdenMail;
use App\Models\Caja;
use App\Models\Operacion;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\Sucursal;
use App\Models\TasaMoneda;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Maatwebsite\Excel\Facades\Excel;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class CajaController extends Controller
{

    public function periodos_de_cajas_cerradas(Request $request)
    {
        $cajas = Caja::where('status' , 2)
            ->get();
        $ar    = [];
        foreach ($cajas as $caja) {
            array_push($ar , [
                'idCaja' => $caja->idCaja ,
                'idSucursal' => $caja->sucursal->idSucursal ,
                'nombre' => $caja->sucursal->nombre . ' / ' . $caja['nombre'] . ' / ' . $caja['fecha_creacion'] . ' / ' . $caja['fecha_cierre'] ,
                'fecha_creacion' => $caja['fecha_creacion'] ,
                'fecha_cierre' => $caja['fecha_cierre'] ,
            ]);
        }


        return response()->json($ar);
    }

    public function export(Request $request)
    {
        $sucursal       = Sucursal::find($request->idSucursal);
        $nombre_archivo = null;
        $nombre_archivo = str_replace(' ' , '_' , $sucursal->nombre) . '_operaciones_caja_' . time() . '_' . strval(rand(100 , 999)) . '.xlsx';
        Excel::store(new OperacionesExport($request->idSucursal , $request->desde , $request->hasta) , '\\excel\\reportecajas\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG','').'\\storage\\app\\public\\excel\\reportecajas\\' . $nombre_archivo);


//        $nombre_archivo='Operaciones_caja.xlsx'.time() . '_' . strval(rand(100 , 999));
//        return Excel::download(new OperacionesExport($request->idSucursal , $request->desde , $request->hasta) , 'Operaciones_caja.xlsx');
//        return Excel::store(new OperacionesExport($request->idSucursal , $request->desde , $request->hasta) , '\\excel\\reportecajas\\'.$nombre_archivo,'public');
//        Storage::disk('public')->put('\\archivobodycotizacion\\' . $nombre_interno , File::get($file));
//        return Excel::download(new OperacionesExport(), 'Operaciones_caja.xlsx');
    }

    public function reportecomisionporventas(Request $request)
    {
        $r = OrderBody::with('orderHeader')
            ->where(function ($query) use ($request){
                $query->whereHas('orderHeader' , function ($query3) use ($request){
                    if (count($request->sucursales) > 0){
                        $query3->wherein('fk_sucursal' , $request->sucursales);
                    }
                    $query3->whereBetween('created_at' , [substr($request->desde , 0 , 10) . ' 00:00:00' , substr($request->hasta , 0 , 10) . ' 23:59:59']);
                });
                if (count($request->personales) > 0){
                    $query->wherein('idPersonal' , $request->personales);
                }
            })
            ->get();
//        return Excel::download(new ComisionPorVentasExport($r) , 'reportecomisionporventas.xlsx');
        $nombre_archivo = null;
        $nombre_archivo = 'reportecomisionporventas_' . time() . '_' . strval(rand(100 , 999)) . '.xlsx';
        Excel::store((new ComisionPorVentasExport($r)) , '\\excel\\reportecomisionporventas\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG','').'\\storage\\app\\public\\excel\\reportecomisionporventas\\' . $nombre_archivo);

    }

    public function reportecomisionporventas2()
    {
        $request = json_decode('{"sucursales":[4],"personales":[],"desde":"2022-05-20","hasta":"2022-06-30"}');
        $r       = OrderBody::with('orderHeader')
            ->where(function ($query) use ($request){
                $query->whereHas('orderHeader' , function ($query3) use ($request){
                    if (count($request->sucursales) > 0){
                        $query3->wherein('fk_sucursal' , $request->sucursales);
                    }
                    $query3->whereBetween('created_at' , [substr($request->desde , 0 , 10) . ' 00:00:00' , substr($request->hasta , 0 , 10) . ' 23:59:59']);

                });
                if (count($request->personales) > 0){
                    $query->wherein('idPersonal' , $request->personales);
                }
            })
            ->get();

        $nombre_archivo = 'Operaciones_caja_' . time() . '_' . strval(rand(100 , 999)) . '.xlsx';
        Excel::store(new ComisionPorVentasExport($r) , '\\excel\\reportecajas\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG','').'\\storage\\app\\public\\excel\\reportecajas\\' . $nombre_archivo);
//        return Excel::download(new ComisionPorVentasExport($r) , 'Operaciones_caja.xlsx');

    }

    public function cajascerradas(Request $request)
    {
        $this->validate($request , [
            'idSucursal' => 'required' ,
        ] , [
            'idSucursal.required' => ':attribute es requerido' ,
        ]);

        $cajas = Caja::where('status' , 2)
            ->where('idSucursal' , $request->idSucursal)
            ->get();
        return response()->json($cajas);
    }

    public function cerrarCaja(Request $request)
    {
        $idCaja = $request->idCaja;
        $caja   = Caja::find($idCaja);
//        return response()->json($caja->status , 404);
        DB::beginTransaction();
        try {
            if ($caja){
                if ($caja->status!=1){
                    return response()->json(['msj' => 'La caja ya esta cerrada'] , 404);
                } else {
                    //guardo una instancia de la caja antes de actualizar
                    $caja_old = $caja;

                    //actualizo la caja y la cierro
                    $caja->update(['status' => 2 , 'fecha_cierre' => now() , 'idUserClose' => $request->user()->id]);

                    //creo una nueva caja con los mismos atributos de la caja vieja, excepto la fecha de creacion
                    $caja_new                 = new Caja();
                    $caja_new->nombre         = $caja_old->nombre;
                    $caja_new->idSucursal     = $caja_old->idSucursal;
                    $caja_new->status         = 1; //abierta por defecto
                    $caja_new->fecha_creacion = now();
                    $caja_new->save();

                    DB::commit();
                    return response()->json(['msj' => 'Caja cerrada exitosamente']);
                }

            } else {
                return response()->json(['msj' => 'No se encontró la caja a cerrar'] , 404);
            }
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en CajaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }

    }

    public function cerrarOrder(Request $request)
    {
//        return response()->json($request->all());
        $idOrderHeader = $request->idOrderHeader;
        $idUser        = $request->idUser;
        $idSucursal    = $request->idSucursal;
        $order         = OrderHeader::find($idOrderHeader);
//        return response()->json(InventarioDeProductoController::crear_inventario_por_id_orden($idOrderHeader),404);
        InventarioDeProductoController::crear_inventario_por_id_orden($idOrderHeader);
        //buscando la caja activa  de la sucursal
        $caja = Caja::where('idSucursal' , $idSucursal)->active()->get();

        if (count($caja)==0){
            return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
        }
        $caja = $caja[0];
        DB::beginTransaction();
        $monto_pagado = '';
        try {
            if ($order){
                if (count($order->body) > 0){
                    foreach ($order->body as $key => $item) { //buscando en los productos de una orden
                        foreach ($item->servicios as $servicio) {  //buscando en los servicios de una orden
//                            return response()->json($servicio->servicio->producto->fk_tipoModalidad);
                            if (!is_null($servicio->fk_elemento) && $servicio->servicio){
                                if ($servicio->servicio->producto->fk_tipoModalidad==4){//buscando en los servicios de un producto
//                                    return response()->json($servicio->servicio->producto->fk_idTipopersonal);
                                    if ($servicio->servicio->producto->fk_idTipopersonal==1){
                                        if (!is_null($item->idPersonal) && $item->idPersonal!=0){
                                            $order->body[$key]->update(['fk_status' => 11]);
                                        } else {

                                            $order->body[$key]->update(['fk_status' => 12]);
                                        }
                                    } else {
                                        $cantidad = 3;
                                        if (count($order->body) >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                                            $cantidad = 4;
                                        }
                                        $fecha_de_entrega = OrderHeaderController::retornar_dia_disponible_para_fechaEntrega($cantidad);
                                        $order->body[$key]->update([
                                            'fk_status' => 2 ,
                                            'date_delivery' => $fecha_de_entrega
                                        ]); //el body se cambia a en proceso
                                    }
                                } else {
                                    $cantidad = 3;
                                    if (count($order->body) >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                                        $cantidad = 4;
                                    }
                                    $fecha_de_entrega = OrderHeaderController::retornar_dia_disponible_para_fechaEntrega($cantidad);
                                    $order->update([
                                        'fk_status' => 2 ,
                                        'enviar_a_taller' => true ,
                                        'date_delivery' => $fecha_de_entrega
                                    ]);
                                }
                            }

                        }
                        if ($item->producto->fk_tipoModalidad==4){ //si el producto del body es servicio
                            if ($item->producto->fk_idTipopersonal==1){
                                if (!is_null($item->idPersonal) && $item->idPersonal!=0){
                                    $order->body[$key]->update(['fk_status' => 11]);
                                } else {
                                    $order->body[$key]->update(['fk_status' => 12]);
                                }
                            } else {
                                $order->body[$key]->update(['fk_status' => 2]); //el body se cambia a en proceso
                            }
                        } else {
                            $cantidad = 3;
                            if (count($order->body) >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                                $cantidad = 4;
                            }
                            $fecha_de_entrega = OrderHeaderController::retornar_dia_disponible_para_fechaEntrega($cantidad);
                            $order->update([
                                'fk_status' => 2 ,
                                'enviar_a_taller' => true ,
                                'date_delivery' => $fecha_de_entrega
                            ]);
                        }
                    }
                }
                $total_pagado_dolares = 0;
                //actualizo la orden
                $order->update(['order_open' => false]);

                //buscando la caja abierta de la sucursal de la orden
                $caja = Caja::where('status' , 1)
                    ->where('idSucursal' , $request->idSucursal)
                    ->get();

                if (count($caja) > 0){
                    $caja  = $caja[0];
                    $array = collect();
                    $tasa  = TasaMoneda::active()->first();

                    //limpiando valores duplicados
                    $formas_pagos_depurada   = $request->collect('formas_pago')->unique();
                    $formas_pagos_depurada_1 = $formas_pagos_depurada->values()->all();
                    //fin depuracion

                    foreach ($formas_pagos_depurada_1 as $key => $forma_pago) {
                        $crear_operacion = true;
                        $operacion       = new Operacion();
                        if ($forma_pago['idTipoTransaccion']==2){
                            $operacion->descripcion = 'Cambio de la Orden Nº: ' . $order->n_order;
                        } else {
                            $operacion->descripcion = 'Cierre de Orden Nº: ' . $order->n_order;
                        }
                        if ($forma_pago['fk_moneda']==2){
                            $total_pagado_dolares += $forma_pago['monto'] / $tasa->tasa;
                        } else {
                            $total_pagado_dolares += $forma_pago['monto'];
                        }
                        $operacion->tasa_moneda = $tasa->tasa;
//                        }
                        $operacion->monto             = $forma_pago['monto'];
                        $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                        $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                        $operacion->idCaja            = $caja->idCaja;
                        $operacion->idOrderHeader     = $idOrderHeader;
                        $operacion->n_order           = $order->n_order;
                        $operacion->fk_moneda         = $forma_pago['fk_moneda'];
                        $operacion->idUser            = $idUser;
                        $operacion->save();
                    }
                } else {
                    return response()->json(['msj' => "No hay caja disponible para la sucursal de la orden"] , 404);
                }
                /*Envio mail de notificacion*/

                //actualizando la orden
                $order = OrderHeader::find($idOrderHeader);
//                $bodys = OrderBody::where('fk_orderHeader' , $idOrderHeader)->get();
//                foreach ($bodys as $body) {
//                    //si el producto en body es por unidad entonces creo el inventario
//                    if ($body->producto->fk_tipoModalidad==3){
//                        return response()->json(['arriba' => $body->producto]);
//                    } else {
//                        foreach ($body->elementos as $elemento) {
//                            if ($elemento->producto->fk_tipoModalidad==3 || $elemento->producto->fk_tipoModalidad==5){
//                                return response()->json(['debajo' => $body->producto]);
//                            }
//                        }
//                    }
//                }
                try {
                    if ($order->estatus->notificar_cliente){ //valido si el estado esta activo para notificar
//                        SmsController::sendSMS(new Request([
//                            "destination" => $order->customer->phone ,
//                            "message" => 'Tu orden en islapixel ' . $order->n_order . ' se encuentra en estatus ' . strtolower($order->estatus->nombre) ,
//                        ]));

//                    return response()->json($header);
//                        Mail::to($header->customer->email)->send(new NotificacionStatusOrdenMail($idOrderHeader , $header->n_order , $estadoOrden->nombre , $header));

                    }
                    PromocionesController::enviarMailOrdenNuevo($order->idOrderHeader);
                    SmsController::sendSMS(new Request([
                        "destination" => $order->customer->phone ,
//                        "message" => "Islapixel: Gracias tu pago de la orden " . $order->n_order . " fue verificado exitosamente" ,
                        "message" => "ISLAPIXEL-PAGO DE ORDEN " . $order->n_order . ", hemos recibido y verificado su pago por $" . round($total_pagado_dolares,2) . " con éxito. Pasará a estatus PRODUCCIÓN. Gracias por su confianza." ,

                    ]));
                } catch (\Exception $e) {
                    Log::error('OrderHeaderController no pudo enviar el sms ' . $e->getMessage() . ', Linea: ' . $e->getLine());
                }
                DB::commit();

                //notifico al socket
//                OrderHeaderController::enviar_notificacion_socket(new Request([
//                    'idSucursal' => $order->fk_sucursal ,
//                    'idOrderHeader' => $idOrderHeader
//                ]));
                return response()->json(['msj' => 'Orden cerrada exitosamente']);

            } else {
                return response()->json(['msj' => 'No se encontró la Orden a cerrar'] , 404);
            }
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en CajaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }

    }

    public function cerrarOrderCuentaPorCobrar(Request $request)
    {
        $idOrderHeader = $request->idOrderHeader;
        $idUser        = $request->idUser;
        $idSucursal    = $request->idSucursal;
        $order         = OrderHeader::find($idOrderHeader);

        //buscando la caja activa  de la sucursal
        $caja = Caja::where('idSucursal' , $idSucursal)->active()->get();

        if (count($caja)==0){
            return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
        }
        $caja = $caja[0];
        DB::beginTransaction();
        try {
            if ($order){
                if ($order->customer->permitir_credito==false){
                    return response()->json([
                        'msj' => 'Este cliente no tiene crédito permitido' ,
                        'bool' => false
                    ]);
                } else {
                    if (count($order->body) > 0){
                        foreach ($order->body as $key => $item) { //buscando en los productos de una orden
                            foreach ($item->servicios as $servicio) {  //buscando en los servicios de una orden
//                            return response()->json($servicio->servicio->producto->fk_tipoModalidad);
                                if (!is_null($servicio->fk_elemento) && $servicio->servicio){
                                    if ($servicio->servicio->producto->fk_tipoModalidad==4){//buscando en los servicios de un producto
//                                    return response()->json($servicio->servicio->producto->fk_idTipopersonal);
                                        if ($servicio->servicio->producto->fk_idTipopersonal==1){
                                            if (!is_null($item->idPersonal) && $item->idPersonal!=0){
                                                $order->body[$key]->update(['fk_status' => 11]);
                                            } else {

                                                $order->body[$key]->update(['fk_status' => 12]);
                                            }
                                        } else {
                                            $order->body[$key]->update(['fk_status' => 2]); //el body se cambia a en proceso
                                        }
                                    } else {
                                        $order->update(['fk_status' => 2 , 'enviar_a_taller' => true]);
                                    }
                                }

                            }
                            if ($item->producto->fk_tipoModalidad==4){ //si el producto del body es servicio
                                if ($item->producto->fk_idTipopersonal==1){
                                    if (!is_null($item->idPersonal) && $item->idPersonal!=0){
                                        $order->body[$key]->update(['fk_status' => 11]);
                                    } else {
                                        $order->body[$key]->update(['fk_status' => 12]);
                                    }
                                } else {
                                    $order->body[$key]->update(['fk_status' => 2]); //el body se cambia a en proceso
                                }
                            } else {
                                $order->update(['fk_status' => 2 , 'enviar_a_taller' => true]);
                            }
                        }
                    }
                    //actualizo la orden
                    $order->update(['order_open' => false , 'cuenta_por_cobrar' => true]);

                    //crear inventario
                    InventarioDeProductoController::crear_inventario_por_id_orden($idOrderHeader);

                    DB::commit();

//                    SmsController::sendSMS(new Request([
//                        "destination" => $order->customer->phone ,
//                        "message" => 'ISLAPIXEL: Gracias por tu abono. Tu pago fue verificado exitosamente. Para la orden: ' . $order->n_order
////                        "message" => 'ABONO A ORDEN N#ISP0201-8481, recibimos y verificado un abono por $100, adeuda $100. IMPORTANTE: Debe cancelar el 100% para retirar y/o se proceda a la instalación del trabajo.'
//                    ]));
                    //notifico al socket
//                OrderHeaderController::enviar_notificacion_socket(new Request([
//                    'idSucursal' => $order->fk_sucursal ,
//                    'idOrderHeader' => $idOrderHeader
//                ]));
                    return response()->json([
                        'msj' => 'Orden confirmada a crédito' ,
                        'bool' => true
                    ]);

                }

            } else {
                return response()->json(['msj' => 'No se encontró la Orden a cerrar'] , 404);
            }
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en CajaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }

    }

    public function filter($idFormaPago)
    {
        $caja = Caja::where('idFormaPago' , $idFormaPago)->get();
        return response()->json($caja);
    }

    public function index()
    {
        return Caja::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {

        $this->validate($request , [
            'nombre' => 'required' ,
            'fecha_creacion' => 'required' ,
            'idSucursal' => 'required' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
            'fecha_creacion.required' => ':attribute es requerido' ,
            'idSucursal.required' => ':attribute es requerido' ,
        ]);

        //busco las cajas activas de esa sucursal
        $e = Caja::where('idSucursal' , $request->idSucursal)->active()->get();
        if (count($e) > 0){
            $response = [
                'msj' => 'Ya existe una caja activa para esta sucursal, cierre la caja existente y automaticamente se creará una nueva' ,
                'caja_activa' => $e ,
            ];
            return response()->json($response , 401);
        } else {
            //busco las cajas activas de esa sucursal
            $e = Sucursal::where('idSucursal' , $request->idSucursal)->get();
            if (count($e)==0){
                $response = [
                    'msj' => 'La sucursal no existe' ,
                ];
                return response()->json($response , 401);
            } else {
                DB::beginTransaction();
                try {
                    $tipo                 = new Caja($request->all());
                    $tipo->status         = 1; //abierta por defecto
                    $tipo->fecha_creacion = now();
                    $tipo->save();
                    $response = [
                        'msj' => 'Creado exitosamente' ,
                        'tipo' => $tipo ,
                    ];
                    DB::commit();
                    return response()->json($response , 201);
                } catch (\Exception $e) {

                    DB::rollback();
                    Log::error('Ha ocurrido un error en CajaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

                    return response()->json([
                        'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                        'error' => $e->getMessage()
                    ] , 500);
                }
            }

        }

    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Caja::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'nombre' => 'required' ,
            'fecha_creacion' => 'required' ,
            'idSucursal' => 'required' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
            'fecha_creacion.required' => ':attribute es requerido' ,
            'idSucursal.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = Caja::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la caja solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Caja actualizada exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TipoCajaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Caja::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        //busco las cajas activas de esa sucursal
        $e = Caja::where('idSucursal' , $h->idSucursal)->active()->get();
        if (count($e) > 0){
            $response = [
                'msj' => 'No se puede eliminar la caja, pues esta activa para esta sucursal, cierre la caja existente y automaticamente se creará una nueva' ,
                'caja_activa' => $e ,
            ];
            return response()->json($response , 401);
        } else {
            $h->delete();
            $h->save();
            return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
        }

    }
}
