<?php

namespace App\Http\Controllers;

use App\Models\DatosAdicionalesOrdenes;
use App\Models\TipoPersonal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class DatosAdicionalesOrdenesController extends Controller
{
    public function index()
    {
        return DatosAdicionalesOrdenes::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'idUser' => 'required' ,
            'fecha_creacion' => 'required' ,
            'idOrderHeader' => 'required' ,
            'email' => 'required' ,
            'typeIdentification' => 'required' ,
            'numerIdentificacion' => 'required' ,
            'phone' => 'required' ,
        ] , [
            'idUser.required' => ':attribute es requerido' ,
            'fecha_creacion.required' => ':attribute es requerido' ,
            'idOrderHeader.required' => ':attribute es requerido' ,
            'email.required' => ':attribute es requerido' ,
            'typeIdentification.required' => ':attribute es requerido' ,
            'numerIdentificacion.required' => ':attribute es requerido' ,
            'phone.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new DatosAdicionalesOrdenes($request->all());
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TipoPersonalController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = DatosAdicionalesOrdenes::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    public function datos_adicionales_ordenes_por_idOrden($id = null)
    {
        if (is_null($id)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }
        $h = DatosAdicionalesOrdenes::where('idOrderHeader' , $id)->get();

        if (count($h)==0){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['resultado' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'idUser' => 'required' ,
            'fecha_creacion' => 'required' ,
            'idOrderHeader' => 'required' ,
            'email' => 'required' ,
            'typeIdentification' => 'required' ,
            'numerIdentificacion' => 'required' ,
            'phone' => 'required' ,
        ] , [
            'idUser.required' => ':attribute es requerido' ,
            'fecha_creacion.required' => ':attribute es requerido' ,
            'idOrderHeader.required' => ':attribute es requerido' ,
            'email.required' => ':attribute es requerido' ,
            'typeIdentification.required' => ':attribute es requerido' ,
            'numerIdentificacion.required' => ':attribute es requerido' ,
            'phone.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = DatosAdicionalesOrdenes::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el tipo de personal solicitado'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Datos actualizados exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TipoPersonalController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = DatosAdicionalesOrdenes::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
