<?php

namespace App\Http\Controllers;

use App\Models\FechaDeNoEntrega;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Models\User;

class FechaDeNoEntregaController extends Controller
{
    public function index()
    {
        return FechaDeNoEntrega::get();
    }

    public function parametro(Request $request)
    {
        $this->validate($request , [
            'fecha' => 'required' ,
        ] , [
            'fecha.required' => ':attribute es requerido' ,
        ]);

        $salida = true;
        try {
            $nueva_fecha = new Carbon($request->fecha);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'El formato de la fecha es erroneo, ejemplo de formato válido 2022-06-12' ,
            ] , 500);
        }
        do {
            $fecha     = '%' . $nueva_fecha->format('Y-m-d') . '%';
            $resultado = FechaDeNoEntrega::where('fecha_de_no_entrega' , 'like' , $fecha)->get();

            if (count($resultado) > 0){
                $nueva_fecha = $nueva_fecha->addDay();
            } else {
                $salida = false;
            }
        } while ($salida);
        return response()->json(['fecha_disponible' => $nueva_fecha->format('Y-m-d')]);
    }

    public static function parametro_($fecha) //funcion reusable
    {
        $salida = true;
        try {
            $nueva_fecha = new Carbon($fecha);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'El formato de la fecha es erroneo, ejemplo de formato válido 2022-06-12' ,
            ] , 500);
        }
        do {
            $fecha     = '%' . $nueva_fecha->format('Y-m-d') . '%';
            $resultado = FechaDeNoEntrega::where('fecha_de_no_entrega' , 'like' , $fecha)->get();

            if (count($resultado) > 0){
                $nueva_fecha = $nueva_fecha->addDay();
            } else {
                $salida = false;
            }
        } while ($salida);
        return $nueva_fecha->format('Y-m-d');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'id_user_creador' => 'required' ,
            'fecha_de_no_entrega' => 'required' ,
        ] , [
            'id_user_creador.required' => ':attribute es requerido' ,
            'fecha_de_no_entrega.required' => ':attribute es requerido' ,
        ]);
        $u = User::find($request->id_user_creador);
        if ($u){
            DB::beginTransaction();
            try {
                $tipo = new FechaDeNoEntrega($request->all());
                $tipo->save();
                $response = [
                    'msj' => 'Creado exitosamente' ,
                    'fecha' => $tipo ,
                ];
                DB::commit();
                return response()->json($response , 201);
            } catch (\Exception $e) {

                DB::rollback();
                Log::error('Ha ocurrido un error en FechaDeNoEntregaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

                return response()->json([
                    'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                    'error' => $e->getMessage()
                ] , 500);
            }
        } else {
            return response()->json([
                'message' => 'No se pudo encontrar el usuario' ,
            ] , 500);
        }

    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = FechaDeNoEntrega::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json($h , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'id_user_creador' => 'required' ,
            'fecha_de_no_entrega' => 'required' ,
        ] , [
            'id_user_creador.required' => ':attribute es requerido' ,
            'fecha_de_no_entrega.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = FechaDeNoEntrega::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la fecha de no entrega solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Fecha actualizada exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en FechaDeNoEntregaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = FechaDeNoEntrega::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
