<?php

namespace App\Http\Controllers;

use App\Models\FormaDePago;
use App\Models\FormasDePagoSucursal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class FormaDePagoController extends Controller
{
    public function index()
    {
        return FormaDePago::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
//            'porcentaje' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
//            'porcentaje.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new FormaDePago($request->all());
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'formaPago' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en FormaDePagoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = FormaDePago::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['formaPago' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
//            'porcentaje' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
//            'porcentaje.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = FormaDePago::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la forma de pago solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Forma de pago actualizada exitosamente' ,
                'formaPago' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en FormaDePagoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = FormaDePago::find($id);
        $r = FormasDePagoSucursal::where('fk_formaDePago' , $id)->get();

        if (count($r) > 0){
            return response()->json(['msj' => 'no se puede eliminar el registro pues existen formas de pago por sucursal asociadas' , 'data' => $r] , 404);
        } else {
            if (is_null($h)){
                return response()->json(['msj' => 'no se encontro resultado'] , 401);
            }
            $h->delete();
            $h->save();
            return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
        }
    }
}
