<?php

namespace App\Http\Controllers;

use App\Exports\InventarioExport;
use App\Models\InventarioDeProducto;
use App\Models\InventarioProductoHeader;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\Producto;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Maatwebsite\Excel\Facades\Excel;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class InventarioDeProductoController extends Controller
{
    public function export()
    {
        return Excel::download(new InventarioExport , 'Inventario.xlsx' , true);
    }

    public function p100()
    {
        $or = InventarioProductoHeader::get();
        foreach ($or as $key => $item) {
            $or[$key]->body()->update(['fk_idSucursal' => $item->fk_idSucursal]);
        }

        return response()->json($or);
    }

    public function inventario_cambiar_status(Request $request)
    {
        $inventario = InventarioProductoHeader::find($request->idInventarioProductoHeader);
        if ($inventario){
            $inventario->update([
                'status' => $request->status
            ]);
        } else {
            return response()->json([
                'msj' => 'No se ha encontrado el inventario'
            ] , 404);
        }
        return response()->json([
            'msj' => 'Se ha cambiado el estatus correctamente' ,
            'inv' => $inventario->refresh()
        ]);

    }

    public function index(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);
        return InventarioProductoHeader::orderBy('fechaUsuario' , 'desc')
            ->offset($request->offset)
            ->limit($request->limit)
            ->get();
    }

    public function listar_movimiento_por_producto($idProducto)
    {
        return InventarioDeProducto::where('fk_idProducto' , $idProducto)->get();
    }

    public function listar_productos_con_stock(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        return Producto::where('fk_status' , 1)
            ->wherein('fk_tipoModalidad' , [3 , 5])
            ->offset($request->offset)
            ->limit($request->limit)
            ->orderBy('name' , 'asc')
            ->get();
    }

    public function filtro(Request $request)
    {
        $this->validate($request , [
            'fk_idSucursal' => 'required' ,
        ] , [
            'fk_idSucursal.required' => 'La sucursal es obligatoria' ,
        ]);

        $busqueda = InventarioDeProducto::
        where(function ($sql) use ($request){
            if ($request->exists('fecha')){
                if ($request->input('fecha')!=''){
                    $date_separado = explode("-" , $request->fecha);
                    $año           = $date_separado[0];
                    $mes           = $date_separado[1];
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
                }
            }
            if ($request->exists('fk_idSucursal')){
                if ($request->input('fk_idSucursal')!=''){
                    $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                }
            }
            if ($request->exists('fk_idProducto')){
                if ($request->input('fk_idProducto')!=''){
                    $sql->where('fk_idProducto' , $request->fk_idProducto);
                }
            }

        })
            ->get();

        return response()->json($busqueda);
    }

    public function listar_productos_con_stock_por_fecha(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
            'fecha' => 'required' ,
            'fk_tipo_inventario' => 'required' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,
            'fecha.required' => 'El campo es requerido' ,
            'fk_tipo_inventario.required' => 'El campo es requerido' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);
        $date_separado = explode("-" , $request->fecha);

        //obtener el ultimo dia
        $fecha = new Carbon($request->fecha);
        $fecha->addMonth()->subDay();
        $dia = $fecha->format('d');

        $completa = $request->fecha . '-' . $dia;
        $año      = $date_separado[0];
        $mes      = $date_separado[1];

        $total_registros = 0;
        $total_registros = count(Producto::with('inventario')
            ->where('fk_status' , 1)
            ->whereHas('inventario' , function ($sql) use ($request , $año , $mes , $completa){
                $sql->whereHas('inventario_header' , function ($sql2) use ($request , $año , $mes , $completa){
                    if ($request->exists('fk_tipo_inventario') && $request->fk_tipo_inventario!=0){
                        $sql2->where('fk_tipo_inventario' , $request->fk_tipo_inventario);
                    }
                    if ($request->exists('fk_idSucursal')){
                        $sql2->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql2->whereDate('created_at' , '<=' , $completa);
                    $sql2->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"));
                });
            })
            ->whereHas('subTipo' , function ($sql){
                $sql->whereIn('fk_tipo' , [8 , 11]);
            })
            ->wherein('fk_tipoModalidad' , [3 , 5])
            ->orderBy('name' , 'asc')
            ->get()
        );

        $todo = Producto::with('inventario')
            ->where('fk_status' , 1)
            ->whereHas('inventario' , function ($sql) use ($request , $año , $mes , $completa){
                $sql->whereHas('inventario_header' , function ($sql2) use ($request , $año , $mes , $completa){
                    if ($request->exists('fk_tipo_inventario') && $request->fk_tipo_inventario!=0){
                        $sql2->where('fk_tipo_inventario' , $request->fk_tipo_inventario);
                    }
                    if ($request->exists('fk_idSucursal')){
                        $sql2->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql2->whereDate('created_at' , '<=' , $completa);
//                    $sql2->whereYear('created_at' , '<=' , $año);
//                    $sql2->whereMonth('created_at' , '<=' , $mes);
//                    $sql2->where('created_at' , '<=' , $mes);
//                    $sql2->where('created_at','like',"%".$request->fecha."%");
                    $sql2->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"));
                });
            })
            ->whereHas('subTipo' , function ($sql){
                $sql->whereIn('fk_tipo' , [8 , 11]);
            })
            ->wherein('fk_tipoModalidad' , [3 , 5])
            ->offset($request->offset)
            ->limit($request->limit)
            ->orderBy('name' , 'asc')
            ->get()/*->filter(function ($item){
                return $item->Stock > 0;
            })*/
        ;
//        return response()->json($todo);
        $todo->each(function ($todo) use ($request , $año , $mes , $completa){
            /*inventario inicial*/
            $inventarioentrada_pasado = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 1);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
//                $sql->whereYear('created_at_' , '<' , $año);
//                $sql->whereMonth('created_at' , '<' , $mes);
                    $sql->whereDate('created_at' , '<' , $completa);
                })->sum('cantidad');

            $inventariosalida_pasado      = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 2);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
//                $sql->whereYear('created_at' , '<' , $año);
//                $sql->whereMonth('created_at' , '<' , $mes);
                    $sql->whereDate('created_at' , '<' , $completa);
                })->sum('cantidad');
            $inventarioautoconsumo_pasado = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 3);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
//                $sql->whereYear('created_at' , '<' , $año);
//                $sql->whereMonth('created_at' , '<' , $mes);
                    $sql->whereDate('created_at' , '<' , $completa);
                })->sum('cantidad');
            /*inventario inicial*/


            /*Inventario del mes actual*/
            $inventarioentrada_actual = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 1);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
//                    $sql->whereDate('created_at' , 'like' , '%' . $completa . '%');
                })->sum('cantidad');

            $inventariosalida_actual      = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 2);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
//                    $sql->whereDate('created_at' , 'like' , '%' . $completa . '%');
                })->sum('cantidad');
            $inventarioautoconsumo_actual = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 3);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
//                    $sql->whereDate('created_at' , 'like' , '%' . $completa . '%');
                })->sum('cantidad');
            /*Inventario del mes actual*/

            $todo->stock_pasado                 = $inventarioentrada_pasado - ($inventariosalida_pasado + $inventarioautoconsumo_pasado);
            $todo->inventarioentrada_pasado     = $inventarioentrada_pasado;
            $todo->inventariosalida_pasado      = $inventariosalida_pasado;
            $todo->inventarioautoconsumo_pasado = $inventarioautoconsumo_pasado;

            $todo->stock_actual                 = $inventarioentrada_actual - ($inventariosalida_actual + $inventarioautoconsumo_actual);
            $todo->inventarioentrada_actual     = $inventarioentrada_actual;
            $todo->inventariosalida_actual      = $inventariosalida_actual;
            $todo->inventarioautoconsumo_actual = $inventarioautoconsumo_actual;

            $todo->inventarioentrada     = $todo->stock_pasado + $todo->inventarioentrada_actual;
            $todo->inventariosalida      = $todo->inventariosalida_actual;
            $todo->inventarioautoconsumo = $todo->inventarioautoconsumo_actual;

            $todo->stock = $todo->stock_pasado + $todo->inventarioentrada_actual - ($todo->inventariosalida_actual + $todo->inventarioautoconsumo_actual);
        });
        return response()->json(['data' => $todo , 'total_registros' => $total_registros]);

    }

    public function listar_productos_con_todos_los_pametros(Request $request)
    {
        $this->validate($request , [
            'fecha' => 'required' ,
            'fk_idSucursal' => 'required' ,
        ] , [
            'fecha.required' => 'El campo es requerido' ,
            'fk_idSucursal.required' => 'El campo es requerido' ,
        ]);
        $date_separado = explode("-" , $request->fecha);

        //obtener el ultimo dia
        $fecha = new Carbon($request->fecha);
        $fecha->addMonth()->subDay();
        $dia = $fecha->format('d');

        $completa = $request->fecha . '-' . $dia;
        $año      = $date_separado[0];
        $mes      = $date_separado[1];
//        return response()->json($completa);
//        exit();

        $todo = Producto::with('inventario')
            ->where('fk_status' , 1)
            ->whereHas('inventario' , function ($sql) use ($request , $año , $mes , $completa){
                $sql->whereHas('inventario_header' , function ($sql2) use ($request , $año , $mes , $completa){
                    if ($request->exists('fk_idSucursal')){
                        $sql2->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql2->whereDate('created_at' , '<=' , $completa);
//                    $sql2->whereYear('created_at' , '<=' , $año);
//                    $sql2->whereMonth('created_at' , '<=' , $mes);
//                    $sql2->where('created_at' , '<=' , $mes);
//                    $sql2->where('created_at','like',"%".$request->fecha."%");
                    $sql2->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"));
                });
            })
            ->whereHas('subTipo' , function ($sql){
                $sql->whereIn('fk_tipo' , [8 , 11]);
            })
            ->wherein('fk_tipoModalidad' , [3 , 5])
            ->orderBy('name' , 'asc')
            ->get()/*->filter(function ($item){
                return $item->Stock > 0;
            })*/
        ;
//        return response()->json($todo);
        $todo->each(function ($todo) use ($request , $año , $mes , $completa){
            /*inventario inicial*/
            $inventarioentrada_pasado = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 1);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
//                $sql->whereYear('created_at_' , '<' , $año);
//                $sql->whereMonth('created_at' , '<' , $mes);
                    $sql->whereDate('created_at' , '<' , $completa);
                })->sum('cantidad');

            $inventariosalida_pasado      = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 2);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
//                $sql->whereYear('created_at' , '<' , $año);
//                $sql->whereMonth('created_at' , '<' , $mes);
                    $sql->whereDate('created_at' , '<' , $completa);
                })->sum('cantidad');
            $inventarioautoconsumo_pasado = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 3);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
//                $sql->whereYear('created_at' , '<' , $año);
//                $sql->whereMonth('created_at' , '<' , $mes);
                    $sql->whereDate('created_at' , '<' , $completa);
                })->sum('cantidad');
            /*inventario inicial*/


            /*Inventario del mes actual*/
            $inventarioentrada_actual = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 1);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
//                    $sql->whereDate('created_at' , 'like' , '%' . $completa . '%');
                })->sum('cantidad');

            $inventariosalida_actual      = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 2);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
//                    $sql->whereDate('created_at' , 'like' , '%' . $completa . '%');
                })->sum('cantidad');
            $inventarioautoconsumo_actual = (int)$todo->inventario_header($request->fk_idSucursal)
                ->whereHas('inventario_header' , function ($sql) use ($request , $año , $mes , $completa){
                    $sql->where('fk_tipo_inventario' , 3);
                    if ($request->exists('fk_idSucursal')){
                        $sql->where('fk_idSucursal' , $request->fk_idSucursal);
                    }
                    $sql->whereYear('created_at' , '=' , $año);
                    $sql->whereMonth('created_at' , '=' , $mes);
//                    $sql->whereDate('created_at' , 'like' , '%' . $completa . '%');
                })->sum('cantidad');
            /*Inventario del mes actual*/

            $todo->stock_pasado                 = $inventarioentrada_pasado - ($inventariosalida_pasado + $inventarioautoconsumo_pasado);
            $todo->inventarioentrada_pasado     = $inventarioentrada_pasado;
            $todo->inventariosalida_pasado      = $inventariosalida_pasado;
            $todo->inventarioautoconsumo_pasado = $inventarioautoconsumo_pasado;

            $todo->stock_actual                 = $inventarioentrada_actual - ($inventariosalida_actual + $inventarioautoconsumo_actual);
            $todo->inventarioentrada_actual     = $inventarioentrada_actual;
            $todo->inventariosalida_actual      = $inventariosalida_actual;
            $todo->inventarioautoconsumo_actual = $inventarioautoconsumo_actual;

            $todo->inventarioentrada     = $todo->stock_pasado + $todo->inventarioentrada_actual;
            $todo->inventariosalida      = $todo->inventariosalida_actual;
            $todo->inventarioautoconsumo = $todo->inventarioautoconsumo_actual;

            $todo->stock = $todo->stock_pasado + $todo->inventarioentrada_actual - ($todo->inventariosalida_actual + $todo->inventarioautoconsumo_actual);
        });
        return $todo;
    }


    public function inventario_producto_agrupado($idProducto)
    {

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
//        $this->validate($request , [
//            'fk_idProducto' => 'required' ,
//            'cantidad' => 'required' ,
//            'costo' => 'required' ,
//            'date' => 'required' ,
//            'incremento' => 'required' ,
//        ] , [
//            'fk_idProducto.required' => ':attribute es requerido' ,
//            'cantidad.required' => ':attribute es requerido' ,
//            'costo.required' => ':attribute es requerido' ,
//            'date.required' => ':attribute es requerido' ,
//            'incremento.required' => ':attribute es requerido' ,
//        ]);
        DB::beginTransaction();
        try {
            //creando header
            $header                     = new InventarioProductoHeader();
            $header->status             = $request->status;
            $header->idUserCreador      = $request->idUserCreador;
            $header->fechaUsuario       = $request->fechaUsuario;
            $header->descripcion        = @$request->descripcion;
            $header->fk_tipo_inventario = @$request->fk_tipo_inventario;
            $header->fk_idSucursal      = @$request->fk_idSucursal;
            $header->save();
            //creo el body
            if (count($request->body) > 0){
                foreach ($request->body as $item) {
                    $body                = new InventarioDeProducto();
                    $body->fk_idProducto = $item['fk_idProducto'];
                    $body->cantidad      = $item['cantidad'];
                    $body->costo         = $item['costo'];
                    $body->date          = $item['date'];
                    $body->incremento    = $item['incremento'];
                    $body->fk_header     = $header->idInventarioProductoHeader;
                    $body->fk_idSucursal = $header->fk_idSucursal;
                    $body->save();
                }
            }

            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'inventario' => InventarioProductoHeader::find($header->idInventarioProductoHeader) ,
            ];
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en InventarioDeProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    public static function crear_inventario_por_id_orden($idOrderHeader)
    {
        $order = OrderHeader::find($idOrderHeader);
        if ($order){
            $bodys     = OrderBody::where('fk_orderHeader' , $idOrderHeader)->get();
            $header_ob = collect([
                'status' => 1 ,
                'idUserCreador' => @$order->idUserCreated ,
                'fechaUsuario' => now()->format('Y-m-d H:i:s') ,
                'descripcion' => 'Cierre de pedido Nº: ' . $order->n_order ,
                'fk_tipo_inventario' => 2 ,
                'fk_idSucursal' => @$order->fk_sucursal
            ]);
            $body_ob   = collect();
            $continuar = false;

            foreach ($bodys as $body) {
                if ($body->producto->fk_tipoModalidad==1 || $body->producto->fk_tipoModalidad==2){
                    //si el producto en body es por unidad entonces creo el inventario

                    foreach ($body->elementos as $elemento) {
//                        return response()->json($elemento->elemento->fk_producto);
                        if ($elemento->elemento->producto->fk_tipoModalidad==3 || $elemento->elemento->producto->fk_tipoModalidad==5){
                            $body_ob->push([
                                'fk_idProducto' => $elemento->elemento->fk_producto ,
                                'cantidad' => $body->cantidad ,
                                'costo' => $body->precio ,
                                'date' => now()->format('Y-m-d H:i:s') ,
                                'incremento' => false ,
                                'fk_idSucursal' => @$order->fk_sucursal
                            ]);
                        }
                    }

                } else {
                    //si el producto en body es por unidad entonces creo el inventario
                    if ($body->producto->fk_tipoModalidad==3){
                        $body_ob->push([
                            'fk_idProducto' => $body->idProduct ,
                            'cantidad' => $body->cantidad ,
                            'costo' => $body->precio ,
                            'date' => now()->format('Y-m-d H:i:s') ,
                            'incremento' => false ,
                            'fk_idSucursal' => @$order->fk_sucursal
                        ]);
                    } else {
                        foreach ($body->producto->elementos as $elemento) {
                            if ($elemento->producto->fk_tipoModalidad==3 || $elemento->producto->fk_tipoModalidad==5){
                                $body_ob->push([
                                    'fk_idProducto' => $body->idProduct ,
                                    'cantidad' => $body->cantidad ,
                                    'costo' => $body->precio ,
                                    'date' => now()->format('Y-m-d H:i:s') ,
                                    'incremento' => false ,
                                    'fk_idSucursal' => @$order->fk_sucursal
                                ]);
                            }
                        }
                    }
                }

            }
            if ($body_ob->count() > 0){
                $header_ob['body'] = $body_ob;
                $request           = new Request($header_ob->toArray());

                DB::beginTransaction();
                try {
                    //creando header
                    $header                     = new InventarioProductoHeader();
                    $header->status             = $request->status;
                    $header->idUserCreador      = @$request->idUserCreador;
                    $header->fechaUsuario       = $request->fechaUsuario;
                    $header->descripcion        = @$request->descripcion;
                    $header->fk_tipo_inventario = @$request->fk_tipo_inventario;
                    $header->fk_idSucursal      = @$request->fk_idSucursal;
                    $header->save();
                    //creo el body
                    if (count($request->body) > 0){
                        foreach ($request->body as $item) {
                            $body                = new InventarioDeProducto();
                            $body->fk_idProducto = $item['fk_idProducto'];
                            $body->cantidad      = $item['cantidad'];
                            $body->costo         = $item['costo'];
                            $body->date          = $item['date'];
                            $body->incremento    = $item['incremento'];
                            $body->fk_header     = $header->idInventarioProductoHeader;
                            $body->fk_idSucursal = $request->fk_idSucursal;
                            $body->save();
                        }
                    }

                    DB::commit();
                    $response = [
                        'msj' => 'Creado exitosamente' ,
                        'inventario' => InventarioProductoHeader::find($header->idInventarioProductoHeader) ,
                    ];
                    return response()->json($response , 201);
                } catch (\Exception $e) {

                    DB::rollback();
                    Log::error('Ha ocurrido un error en InventarioDeProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

                    return response()->json([
                        'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                        'error' => $e->getMessage() ,
                        'linea' => $e->getLine()
                    ] , 500);
                }
            }
        }
    }

    public static function crear_inventario_por_id_orden_punto_de_venta($idUserClose , $idOrderHeader)
    {

        $order = OrderHeader::find($idOrderHeader);
        if ($order){
            $bodys = OrderBody::where('fk_orderHeader' , $idOrderHeader)->get();

            $header_ob = collect([
                'status' => 1 ,
                'idUserCreador' => $idUserClose ,
                'fechaUsuario' => now()->format('Y-m-d H:i:s') ,
                'descripcion' => 'Cierre de pedido Nº: ' . $order->n_order ,
                'fk_tipo_inventario' => 2 ,
                'fk_idSucursal' => $order->fk_sucursal
            ]);
            $body_ob   = collect();
            $continuar = false;

            foreach ($bodys as $body) {
                //si el producto en body es por unidad entonces creo el inventario

//                     return $body->producto->elementos;
                if ($body->producto->fk_tipoModalidad==3 || $body->producto->fk_tipoModalidad==5){
                    $body_ob->push([
                        'fk_idProducto' => $body->idProduct ,
                        'cantidad' => $body->cantidad ,
                        'costo' => $body->precio ,
                        'date' => now()->format('Y-m-d H:i:s') ,
                        'incremento' => false ,
                        'fk_idSucursal' => $order->fk_sucursal
                    ]);
                } else {
                    foreach ($body->producto->elementos as $elemento) {
                        if ($elemento->producto->fk_tipoModalidad==3 || $elemento->producto->fk_tipoModalidad==5){
                            $body_ob->push([
                                'fk_idProducto' => $body->idProduct ,
                                'cantidad' => $body->cantidad ,
                                'costo' => $body->precio ,
                                'date' => now()->format('Y-m-d H:i:s') ,
                                'incremento' => false ,
                                'fk_idSucursal' => $order->fk_sucursal
                            ]);
                        }
                    }
                }
            }
            if ($body_ob->count() > 0){
                $header_ob['body'] = $body_ob;
                $request           = new Request($header_ob->toArray());

                DB::beginTransaction();
                try {
                    //creando header
                    $header                     = new InventarioProductoHeader();
                    $header->status             = $request->status;
                    $header->idUserCreador      = $request->idUserCreador;
                    $header->fechaUsuario       = $request->fechaUsuario;
                    $header->descripcion        = @$request->descripcion;
                    $header->fk_tipo_inventario = @$request->fk_tipo_inventario;
                    $header->fk_idSucursal      = @$request->fk_idSucursal;
                    $header->save();
                    //creo el body
                    if (count($request->body) > 0){
                        foreach ($request->body as $item) {
                            $body                = new InventarioDeProducto();
                            $body->fk_idProducto = $item['fk_idProducto'];
                            $body->cantidad      = $item['cantidad'];
                            $body->costo         = $item['costo'];
                            $body->date          = $item['date'];
                            $body->incremento    = $item['incremento'];
                            $body->fk_header     = $header->idInventarioProductoHeader;
                            $body->fk_idSucursal = $request->fk_idSucursal;
                            $body->save();
                        }
                    }

                    DB::commit();
                    $response = [
                        'msj' => 'Creado exitosamente' ,
                        'inventario' => InventarioProductoHeader::find($header->idInventarioProductoHeader) ,
                    ];
                    return response()->json($response , 201);
                } catch (\Exception $e) {

                    DB::rollback();
                    Log::error('Ha ocurrido un error en InventarioDeProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

                    return response()->json([
                        'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                        'error' => $e->getMessage() ,
                        'linea' => $e->getLine()
                    ] , 500);
                }
            }
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = InventarioProductoHeader::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['inventario' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
//        $this->validate($request , [
//            'fk_idProducto' => 'required' ,
//            'cantidad' => 'required' ,
//            'costo' => 'required' ,
//            'date' => 'required' ,
//            'incremento' => 'required' ,
//        ] , [
//            'fk_idProducto.required' => ':attribute es requerido' ,
//            'cantidad.required' => ':attribute es requerido' ,
//            'costo.required' => ':attribute es requerido' ,
//            'date.required' => ':attribute es requerido' ,
//            'incremento.required' => ':attribute es requerido' ,
//        ]);
        DB::beginTransaction();
        $h = InventarioProductoHeader::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el inventario solicitado'] , 401);
        }

        try {
            $h->status        = $request->status;
            $h->idUserCreador = $request->idUserCreador;
            $h->fechaUsuario  = $request->fechaUsuario;
            $h->descripcion   = @$request->descripcion;
            $h->save();

            if (count($request->body) > 0){
                foreach ($request->body as $item) {
                    $object = json_encode($item);
                    $item   = json_decode($object);
                    $body   = InventarioDeProducto::find(@$item->idInventario);
                    if ($body){
                        $body->fk_idProducto = @$item->fk_idProducto;
                        $body->cantidad      = @$item->cantidad;
                        $body->costo         = @$item->costo;
                        $body->date          = @$item->date;
                        $body->incremento    = @$item->incremento;
                        $body->save();
                    }
                }
            }
            $response = [
                'msj' => 'Inventario actualizado exitosamente' ,
                'inventario' => $h->refresh() ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en InventarioDeProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' . ', Linea: ' . $e->getLine() ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id) //borra todo
    {
        $h = InventarioProductoHeader::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }

    public function destroyBody($idBody) //para el body
    {
        $h = InventarioDeProducto::find($idBody);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }
        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
