<?php

namespace App\Http\Controllers;

use App\Models\Marca;
use App\Models\Operacion;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Image;

class MarcaController extends Controller
{
    public function index()
    {
        return Marca::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function prueba(Request $request)
    {
        Log::info(json_encode($request->all()));
    }



    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'titulo' => 'required' ,
            'descripcion' => 'required' ,
            'url' => 'required' ,
        ] , [
            'titulo' => 'required' ,
            'descripcion.required' => ':attribute es requerido' ,
            'url.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new Marca($request->all());
            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);

                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\marcas\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img = $nombre_interno;
                }
            }
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en NoticiasController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Marca::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
//            'img' => 'required' ,
            'url' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
//            'img.required' => ':attribute es requerido' ,
            'url.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = Marca::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el tipo de personal solicitado'] , 401);
        }

        try {
            $h->fill($request->all());
            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);

                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\marcas\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $h->img = $nombre_interno;
                }
            }
            $h->save();
            $response = [
                'msj' => 'Marca actualizada exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en MarcaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Marca::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
