<?php

namespace App\Http\Controllers;

use App\Mail\MetricasMail;
use App\Mail\PromocionesMail;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\ProductosMasBuscados;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;

class MetricasController extends Controller
{
    public function mail_reporte_semanal()
    {
        $fecha_actual = Carbon::now();
        $semana_pasada = Carbon::now()->subWeek();

        $lunes_semana_actual   = $fecha_actual->startOfWeek();
        $viernes_semana_actual = $fecha_actual->copy()->endOfWeek();

        $lunes_semana_pasada   = $semana_pasada->startOfWeek();
        $viernes_semana_pasada = $semana_pasada->copy()->endOfWeek();

        $prod_sem_act = null;
        $prod_sem_pas = null;

        $body_sem_act = OrderBody::withOnly('producto')
            ->whereBetween('created_at' , [$lunes_semana_actual , $viernes_semana_actual])
            ->get();
//        return response()->json($body_sem_act);

        $recuento = array();
        foreach ($body_sem_act as $body) {
            if (isset($recuento[$body['producto']['name']])){
                if ($body['producto']['used_size']==1){
                    $recuento[$body['producto']['name']] += 1;
                } else {
                    $recuento[$body['producto']['name']] += $body['cantidad'];
                }
            } else {
                $recuento[$body['producto']['name']] = $body['cantidad'];
            }
        }

        //se ordena ascendentemente
        arsort($recuento);

        //extraer los ultimos 10 productos
        $top_10_productos_semana_actual = array_slice($recuento , 0 , 10);

        $total_ordenes_semana_actual = DB::table('order_headers')
            ->select('idOrderHeader')
            ->whereBetween('created_at' , [$lunes_semana_actual , $viernes_semana_actual])
            ->count();


        $body_sem_pas = OrderBody::withOnly('producto')
            ->whereBetween('created_at' , [$lunes_semana_pasada , $viernes_semana_pasada])
            ->get();

        $recuento = array();
        foreach ($body_sem_pas as $body) {
            if (isset($recuento[$body['producto']['name']])){
                if ($body['producto']['used_size']==1){
                    $recuento[$body['producto']['name']] += 1;
                } else {
                    $recuento[$body['producto']['name']] += $body['cantidad'];
                }
            } else {
                $recuento[$body['producto']['name']] = $body['cantidad'];
            }
        }
        //se ordena ascendentemente
        arsort($recuento);
        //extraer los ultimos 10 productos
        $top_10_productos_semana_pasada = array_slice($recuento , 0 , 10);

        $total_ordenes_semana_pasada = DB::table('order_headers')
            ->select('idOrderHeader')
            ->whereBetween('created_at' , [$lunes_semana_pasada , $viernes_semana_pasada])
            ->count();


        $palabras_buscadas_semana_actual = ProductosMasBuscados::whereBetween('created_at' , [$lunes_semana_actual , $viernes_semana_actual])->get();
        $recuento                        = array();
        foreach ($palabras_buscadas_semana_actual as $body) {
            if (isset($recuento[$body['frase_buscada']])){
                $recuento[$body['frase_buscada']] += 1;
            } else {
                $recuento[$body['frase_buscada']] = 1;
            }
        }
        //se ordena ascendentemente
        asort($recuento);
        //extraer los ultimos 10 productos menos buscados
        $top_10_productos_menos_buscados_semana_actual = array_slice($recuento , 0 , 10);

        $palabras_buscadas_semana_pasada = ProductosMasBuscados::whereBetween('created_at' , [$lunes_semana_pasada , $viernes_semana_pasada])->get();
        $recuento                        = array();
        foreach ($palabras_buscadas_semana_pasada as $body) {
            if (isset($recuento[$body['frase_buscada']])){
                $recuento[$body['frase_buscada']] += 1;
            } else {
                $recuento[$body['frase_buscada']] = 1;
            }
        }
        //se ordena ascendentemente
        asort($recuento);
        //extraer los ultimos 10 productos menos buscados
        $top_10_productos_menos_buscados_semana_pasada = array_slice($recuento , 0 , 10);


        $todo = [
            'date_semana_actual' => ['lunes' => $lunes_semana_actual , 'viernes' => $viernes_semana_actual] ,
            'productos_semana_actual' => preg_replace('/(^[\"\']|[\"\']$)/' , '' , $top_10_productos_semana_actual) ,
            'total_ordenes_semana_actual' => $total_ordenes_semana_actual ,
            'frases_menos_buscadas_actual' => $top_10_productos_menos_buscados_semana_actual ,

            'date_semana_pasada' => ['lunes' => $lunes_semana_pasada , 'viernes' => $viernes_semana_pasada] ,

            'productos_semana_pasada' => preg_replace('/(^[\"\']|[\"\']$)/' , '' , $top_10_productos_semana_pasada) ,
            'frases_menos_buscadas_pasada' => $top_10_productos_menos_buscados_semana_pasada ,
            'total_ordenes_semana_pasada' => $total_ordenes_semana_pasada ,
        ];
        $todo = json_decode(json_encode($todo));
//        return response()->json($todo);

        //buscando los administradores
        $admins = DB::table('users')->select('email')->where('profile_fk' , )->get()->toArray();
        return view('correos.metricas')->with('request',$todo);
//        Mail::to(['alecortez240192@gmail.com'])->send(new MetricasMail($todo));
    }
}
