<?php

namespace App\Http\Controllers;

use App\Models\Moneda;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\Rule;

class MonedaController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    public function establecerMonedaPorDefecto(Request $request)
    {
        $idMoneda      = $request->idMoneda;
        $moneda        = Moneda::find($idMoneda);
        $monedaDefault = Moneda::where('porDefecto' , true)->get();
        if ($moneda){
            if (count($monedaDefault) > 0){
                $monedaDefault[0]->update(['porDefecto' => false]); //si existe una moneda por default entonces se desactiva
            }
            $moneda        = Moneda::find($idMoneda);
            $moneda->update(['porDefecto' => true]);

//            return response()->json($moneda);
            return response()->json(['msj' => 'La moneda fue asignada correctamente' , 'monedas' => Moneda::get()]);
        } else {
            return response()->json(['msj' => 'La moneda no existe'] , 404);
        }

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    public function cambiarStatus($idMoneda)
    {
        $moneda = Moneda::find($idMoneda);
        if ($moneda){
            if ($moneda->status){
                $moneda->update(['status' => false]);
                return response()->json(['msj' => 'La moneda fue desactivada correctamente' , 'monedas' => Moneda::all()]);
            } else {
                $moneda->update(['status' => true]);
                return response()->json(['msj' => 'La moneda fue activada correctamente' , 'monedas' => Moneda::all()]);
            }
        } else {
            return response()->json('La moneda no existe' , 404);
        }

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'descripcion' => 'required|unique:monedas,descripcion,NULL,idMoneda,deleted_at,NULL' ,
            'porDefecto' => Rule::unique('monedas')->where(function ($query){
                return $query->where('porDefecto' , 1);
            }) , 'monedas,porDefecto,NULL,idMoneda,deleted_at,NULL' ,
            'simbolo' => 'required' ,

        ] , [
            'descripcion.unique' => 'La moneda :input ya esta registrado(a)' ,
            'porDefecto.unique' => 'Ya hay una moneda por defecto' ,
            'descripcion.required' => ':attribute es requerido' ,
            'simbolo.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $re         = new Moneda($request->all());
            $re->status = true; //por defecto true
            $re->save();
            $response = [
                'msj' => 'Creada exitosamente' ,
                'moneda' => $re ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en MonedaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    public function desactivar($idMoneda)
    {

    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\Moneda $moneda
     * @return \Illuminate\Http\Response
     */
    public function show(Moneda $moneda)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\Moneda $moneda
     * @return \Illuminate\Http\Response
     */
    public function edit(Moneda $moneda)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Moneda $moneda
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , Moneda $moneda)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Moneda $moneda
     * @return \Illuminate\Http\Response
     */
    public function destroy(Moneda $moneda)
    {
        //
    }
}
