<?php

namespace App\Http\Controllers;

use App\Models\NotificacionCobro;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class NotificacionCobroController extends Controller
{

    public function index()
    {
        return NotificacionCobro::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
            'fk_idAsesor' => 'required' ,
            'fk_idCliente' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
            'fk_idAsesor.required' => ':attribute es requerido' ,
            'fk_idCliente.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new NotificacionCobro($request->all());
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'notificacion_cobro' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en NotificacionCobroController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($fk_idCliente)
    {
        $h = NotificacionCobro::with('cliente')
            ->where('fk_idCliente' , $fk_idCliente)
            ->get();

        if (count($h) < 0){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['notificacion_cobro' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {


    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = NotificacionCobro::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
