<?php

namespace App\Http\Controllers;

use App\Models\Caja;
use App\Models\FormaDePago;
use App\Models\Moneda;
use App\Models\Operacion;
use App\Models\OrderHeader;
use App\Models\Sucursal;
use App\Models\TasaMoneda;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use function Symfony\Component\String\b;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class OperacionController extends Controller
{

    public function actualizar_tasa(Request $request)
    {
        $operaciones_sin_tasa = Operacion::without('formaPago' , 'tipoOperacion' , 'moneda' , 'usuario_creador')
            ->wherenull('tasa_moneda')
            ->get();

        foreach ($operaciones_sin_tasa as $key => $item) {
            //busco la tasa con la fecha de creacion
            $tasa = TasaMoneda::where('created_at' , 'like' , "%" . $item->created_at->format('Y-m-d') . "%")
                ->orderby('idTasaMoneda' , 'desc')
                ->first();
            if ($tasa){
                $operaciones_sin_tasa[$key]->update(["tasa_moneda" => $tasa['tasa']]);
            } else {
                //si no esta la tasa en la fecha buscada, entonces busco una de dias anteriores
                $tasa = TasaMoneda::whereDate('created_at' , '<' , $item->created_at->format('Y-m-d'))
                    ->orderby('idTasaMoneda' , 'desc')
                    ->first();
                $operaciones_sin_tasa[$key]->update(["tasa_moneda" => $tasa['tasa']]);
//                dd(['tasa de fecha anterior', $tasa,$item->created_at->format('Y-m-d')]);
            }

        }
        return response()->json($operaciones_sin_tasa);
    }

    public function index()
    {
        return Operacion::get();
    }

    public function operaciones_por_mes(Request $request)
    {
        $todo       = [];
        $fecha      = $request->fecha;
        $idSucursal = $request->idSucursal;


        $tasa            = 0;
        $sum_consolidado = 0;

        $ordenes = OrderHeader::whereHas('sucursal_orden' , function ($sql) use ($idSucursal){
            $sql->where('idSucursal' , $idSucursal);
        })
            ->whereHas('transaccion' , function ($sql2) use ($idSucursal){
                $sql2->where('fk_moneda' , '=' , 1);
            })
            ->where('fecha_facturacion' , 'like' , '%' . $fecha . '%')
            ->wherenotnull('n_factura_seniat')
            ->orderBy('fecha_facturacion' , 'asc')
            ->get();


        foreach ($ordenes as $orden) {
            //busco los dias disponibles en las operaciones
            $total_operacion_por_dia_2 = Operacion::whereHas('caja' , function ($sql) use ($idSucursal){
                $sql->whereHas('sucursal' , function ($sql2) use ($idSucursal){
                    $sql2->where('idSucursal' , $idSucursal);
                });
            })
                ->where('idOrderHeader' , $orden->idOrderHeader)
                ->where('fk_moneda' , '=' , 1)
                ->sum('monto');


            $s = TasaMoneda::where('created_at' , 'like' , "%" . Carbon::create($orden->fecha_facturacion)->format('Y-m-d') . "%")->orderby('idTasaMoneda' , 'desc')->get();

            if (count($s) > 0){
                $tasa = $s[0]->tasa;
            } else {
                $tasa = TasaMoneda::whereDate('created_at' , '<' , $orden->fecha_facturacion->format('Y-m-d'))
                    ->orderby('idTasaMoneda' , 'desc')
                    ->first();
                $tasa = $tasa['tasa'];
            }


            $igtf_3          = round($total_operacion_por_dia_2 * 0.03 , 2);
            $bs_equi         = round($igtf_3 * $tasa , 2);
            $sum_consolidado += $bs_equi;
            array_push($todo , [
                'fecha_emision' => $orden->fecha_facturacion->format('Y-m-d') ,
                'n_factura_seniat' => $orden->n_factura_seniat ? $orden->n_factura_seniat : null ,
                'monto_total_bs' => round($total_operacion_por_dia_2 * $tasa , 2) ,
                'monto_pagado_divisas' => round($total_operacion_por_dia_2 , 2) ,
                'igtf_3' => $igtf_3 ,
                'porcentaje_de_percepcion' => '3%' ,
                'base_imponible' => round($total_operacion_por_dia_2 * $tasa , 2) ,
                'monto_igtf' => round($total_operacion_por_dia_2 * $tasa * 0.03 , 2) ,
                'tasa' => round($tasa , 2) ,
                'idOrderHeader' => $orden->idOrderHeader ,
                'monto_total_orden_bs' => round($orden->precio_total * $tasa , 2)

            ]);

        }

        return $todo;
    }

    public function operaciones_por_rango2(Request $request)
    {
        $this->validate($request , [
            'offset' => 'required|integer|min:0' ,
            'limit' => 'required|integer|min:0' ,
            'desde' => 'required|date_format:Y-m-d' ,
            'hasta' => 'required|date_format:Y-m-d' ,
            'idSucursal' => 'required' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,
            'offset.required' => ':attribute es requerido' ,
            'limit.required' => ':attribute es requerido' ,
            'desde.required' => ':attribute es requerido' ,
            'hasta.required' => ':attribute es requerido' ,
            'idSucursal.required' => ':attribute es requerido' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);
        $desde = $request->desde;
        $hasta = $request->hasta;

        $todo            = [];
        $idSucursal      = $request->idSucursal;
        $total_registros = 0;

        $total_registros = count(Operacion::whereHas('caja' , function ($sql) use ($idSucursal){
            $sql->whereHas('sucursal' , function ($sql2) use ($idSucursal){
                $sql2->where('idSucursal' , $idSucursal);
            });
        })
            ->whereHas('orderHeader' , function ($sql) use ($desde , $hasta){
                $sql->wherenotnull('n_factura_seniat');
                $sql->whereBetween('fecha_facturacion' , [$desde , $hasta]);
                $sql->groupBy(DB::raw("DATE_FORMAT(fecha_facturacion, '%Y-%m-%d')"));
            })
            ->where('fk_moneda' , '=' , 1)
//            ->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"))
            ->get());


        //busco los dias disponibles en las operaciones
        $operaciones_dia = Operacion::whereHas('caja' , function ($sql) use ($idSucursal){
            $sql->whereHas('sucursal' , function ($sql2) use ($idSucursal){
                $sql2->where('idSucursal' , $idSucursal);
            });
        })
            ->whereHas('orderHeader' , function ($sql) use ($desde , $hasta){
                $sql->wherenotnull('n_factura_seniat');

                $sql->whereBetween('fecha_facturacion' , [$desde , $hasta]);
            })
            ->where('fk_moneda' , '=' , 1)
//            ->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"))
            ->offset($request->offset)
            ->limit($request->limit)
            ->get();

        $tasa            = 0;
        $sum_consolidado = 0;

        //recorro los dias y las agrupo por operacion para sacar el n de factura seniat
        foreach ($operaciones_dia as $operacion_dia) {
//            $total_operacion_por_dia = Operacion::where('fk_moneda' , 1)
//                ->whereHas('caja' , function ($sql) use ($idSucursal){
//                    $sql->whereHas('sucursal' , function ($sql2) use ($idSucursal){
//                        $sql2->where('idSucursal' , $idSucursal);
//                    });
//                })
//                ->whereHas('orderHeader' , function ($sql) use ($operacion_dia){
//                    $sql->wherenotnull('n_factura_seniat');
//                    $sql->where('fecha_facturacion' , 'like' , '%' . $operacion_dia->created_at->format('Y-m-d') . '%');
//                })
////                ->where('created_at' , 'like' , '%' . $operacion_dia->created_at->format('Y-m-d') . '%')
//                ->groupBy('idOrderHeader')
//                ->get();

            //recorro los resultados para sumar el monto de cada operacion por dia, por n de factura de seniat
            $operacion_dia_por_orden   = $operacion_dia;
            $total_operacion_por_dia_2 = Operacion::where('fk_moneda' , 1)
                ->whereHas('caja' , function ($sql) use ($idSucursal){
                    $sql->whereHas('sucursal' , function ($sql2) use ($idSucursal){
                        $sql2->where('idSucursal' , $idSucursal);
                    });
                })
                ->whereHas('orderHeader' , function ($sql){
                    $sql->wherenotnull('n_factura_seniat');
                })
                ->where('idOrderHeader' , $operacion_dia_por_orden->idOrderHeader)
                ->sum('monto');
            $s                         = TasaMoneda::where('created_at' , 'like' , "%" . Carbon::create($operacion_dia_por_orden->created_at)->format('Y-m-d') . "%")->orderby('idTasaMoneda' , 'desc')->get();

            if (count($s) > 0){
                $tasa = $s[0]->tasa;
            } else {
                $tasa = TasaMoneda::whereDate('created_at' , '<' , $operacion_dia_por_orden->created_at->format('Y-m-d'))
                    ->orderby('idTasaMoneda' , 'desc')
                    ->first();
                $tasa = $tasa['tasa'];
            }

            $igtf_3          = round($total_operacion_por_dia_2 * 0.03 , 2);
            $bs_equi         = round($igtf_3 * $tasa , 2);
            $sum_consolidado += $bs_equi;
            array_push($todo , [
                'fecha_emision' => $operacion_dia_por_orden->created_at->format('Y-m-d') ,
                'n_factura_seniat' => $operacion_dia_por_orden->orderHeader->n_factura_seniat ? $operacion_dia_por_orden->orderHeader->n_factura_seniat : null ,
                'monto_total_bs' => round($total_operacion_por_dia_2 * $tasa , 2) ,
                'monto_pagado_divisas' => round($total_operacion_por_dia_2 , 2) ,
                'igtf_3' => $igtf_3 ,
                'porcentaje_de_percepcion' => '3%' ,
                'base_imponible' => round($total_operacion_por_dia_2 * $tasa , 2) ,
                'monto_igtf' => round($total_operacion_por_dia_2 * $tasa * 0.03 , 2) ,
                'tasa' => round($tasa , 2) ,
                'idOrderHeader' => $operacion_dia_por_orden->idOrderHeader ,
                'monto_total_orden_bs' => round($operacion_dia_por_orden->orderHeader->precio_total * $tasa , 2)

            ]);

        }
        return ['data' => $todo , 'total_regsitros' => $total_registros];
    }

    public function operaciones_por_rango3(Request $request)
    {
        $this->validate($request , [
            'offset' => 'required|integer|min:0' ,
            'limit' => 'required|integer|min:0' ,
            'desde' => 'required|date_format:Y-m-d' ,
            'hasta' => 'required|date_format:Y-m-d' ,
            'idSucursal' => 'required' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,
            'offset.required' => ':attribute es requerido' ,
            'limit.required' => ':attribute es requerido' ,
            'desde.required' => ':attribute es requerido' ,
            'hasta.required' => ':attribute es requerido' ,
            'idSucursal.required' => ':attribute es requerido' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);
        $desde = $request->desde;
        $hasta = $request->hasta;

        $todo            = [];
        $idSucursal      = $request->idSucursal;
        $total_registros = 0;


        $ordenes = OrderHeader::whereHas('sucursal_orden' , function ($sql) use ($idSucursal){
            $sql->where('idSucursal' , $idSucursal);
        })
            ->whereHas('transaccion' , function ($sql2) use ($idSucursal){
                $sql2->where('fk_moneda' , '=' , 1);
            })
            ->whereBetween('fecha_facturacion' , [$desde , $hasta])
            ->wherenotnull('n_factura_seniat')
            ->offset($request->offset)
            ->limit($request->limit)
            ->orderBy('fecha_facturacion' , 'asc')
            ->get();

        $total_registros = count(OrderHeader::whereHas('sucursal_orden' , function ($sql) use ($idSucursal){
            $sql->where('idSucursal' , $idSucursal);
        })
            ->whereHas('transaccion' , function ($sql2) use ($idSucursal){
                $sql2->where('fk_moneda' , '=' , 1);
            })
            ->whereBetween('fecha_facturacion' , [$desde , $hasta])
            ->wherenotnull('n_factura_seniat')
            ->get()
        );

        foreach ($ordenes as $orden) {
            //busco los dias disponibles en las operaciones
            $total_operacion_por_dia_2 = Operacion::whereHas('caja' , function ($sql) use ($idSucursal){
                $sql->whereHas('sucursal' , function ($sql2) use ($idSucursal){
                    $sql2->where('idSucursal' , $idSucursal);
                });
            })
                ->where('idOrderHeader' , $orden->idOrderHeader)
                ->where('fk_moneda' , '=' , 1)
                ->sum('monto');


            $tasa            = 0;
            $sum_consolidado = 0;

            $s = TasaMoneda::where('created_at' , 'like' , "%" . Carbon::create($orden->fecha_facturacion)->format('Y-m-d') . "%")->orderby('idTasaMoneda' , 'desc')->get();

            if (count($s) > 0){
                $tasa = $s[0]->tasa;
            } else {
                $tasa = TasaMoneda::whereDate('created_at' , '<' , $orden->fecha_facturacion->format('Y-m-d'))
                    ->orderby('idTasaMoneda' , 'desc')
                    ->first();
                $tasa = $tasa['tasa'];
            }

            $igtf_3          = round($total_operacion_por_dia_2 * 0.03 , 2);
            $bs_equi         = round($igtf_3 * $tasa , 2);
            $sum_consolidado += $bs_equi;
            array_push($todo , [
                'fecha_emision' => $orden->fecha_facturacion->format('Y-m-d') ,
                'n_factura_seniat' => $orden->n_factura_seniat ? $orden->n_factura_seniat : null ,
                'monto_total_bs' => round($total_operacion_por_dia_2 * $tasa , 2) ,
                'monto_pagado_divisas' => round($total_operacion_por_dia_2 , 2) ,
                'igtf_3' => $igtf_3 ,
                'porcentaje_de_percepcion' => '3%' ,
                'base_imponible' => round($total_operacion_por_dia_2 * $tasa , 2) ,
                'monto_igtf' => round($total_operacion_por_dia_2 * $tasa * 0.03 , 2) ,
                'tasa' => round($tasa , 2) ,
                'idOrderHeader' => $orden->idOrderHeader ,
                'monto_total_orden_bs' => round($orden->precio_total * $tasa , 2)

            ]);
        }

        return ['data' => $todo , 'total_regsitros' => $total_registros];
    }

    public
    function totalesPorSucursal($idSucursal)
    {
        //Busco la existencia de la sucursal
        $e = Sucursal::where('idSucursal' , $idSucursal)->get();

        if (count($e)==0){
            $response = [
                'msj' => 'La sucursal no existe' ,
            ];
            return response()->json($response , 404);

        } else {
            $caja_activa = Caja::where('idSucursal' , $idSucursal)->active()->get();
//            return response()->json($caja_activa , 404);
            $aux             = new \stdClass();
            $todo            = [];
            $todo2           = [];
            $formas_de_pago  = FormaDePago::get();
            $monedas_activas = null;
            if (count($caja_activa) > 0 && count($formas_de_pago) > 0){
                foreach ($formas_de_pago as $formas_de_pago) {
                    $monedas_activas = null;
                    $monedas_activas = Operacion::where('idCaja' , $caja_activa[0]->idCaja)
                        ->where('idFormaPago' , $formas_de_pago->idFormaPago)
                        ->groupby('fk_moneda')
                        ->get();
                    foreach ($monedas_activas as $monedas_activa) {
//                        return response()->json($monedas_activa->fk_moneda);
                        $operaciones = Operacion::where('idCaja' , $caja_activa[0]->idCaja)
                            ->where('idFormaPago' , $formas_de_pago->idFormaPago)
                            ->where('fk_moneda' , $monedas_activa->fk_moneda)
                            ->get();
                        $ingresos    = $operaciones->where('idTipoTransaccion' , 1)->sum('monto');
                        $egresos     = $operaciones->where('idTipoTransaccion' , 2)->sum('monto');

                        array_push($todo2 , [
                            'idMoneda' => $monedas_activa->fk_moneda ,
                            'nombre' => $monedas_activa->moneda->descripcion ,
                            'simbolo' => $monedas_activa->moneda->simbolo ,
                            'ingresos' => $ingresos ,
                            'egresos' => $egresos ,
                            'operaciones' => $operaciones
                        ]);
                    }
                    array_push($todo , [
                        'forma_pago' => $formas_de_pago ,
                        'monedas' => $todo2
                    ]);
                    $todo2 = [];

                }
//                return response()->json($todo);
            } else {
                $response = [
                    'msj' => 'la caja o las formas de pagos no existen ' ,
                ];
                return response()->json($response , 404);
            }
            $caja_activa[0]->operaciones_agrupadas_por_forma_pago = $todo;
            return response()->json($caja_activa);
        }
    }

    public
    function totalesPorCajaCerrada(Request $request)
    {
        $idCaja        = $request->idCaja;
        $seniat        = $request->seniat;
        $tasa          = TasaMoneda::active()->first();
        $caja_cerradas = Caja::where('idCaja' , $idCaja)->cerrada()->get();
        foreach ($caja_cerradas as $key => $caja_cerrada) {
            $s = TasaMoneda::where('created_at' , 'like' , "%" . Carbon::create($caja_cerrada->fecha_cierre)->format('Y-m-d') . "%")->orderby('idTasaMoneda' , 'desc')->get();
            if (count($s) > 0){
                $caja_cerradas[$key]->tasa_cierre = $s[0]['tasa'];
            } else {
                $caja_cerradas[$key]->tasa_cierre = 0;
            }
        }
        if (count($caja_cerradas)==0){
            $response = [
                'msj' => 'La Caja no esta cerrada' ,
            ];
            return response()->json($response , 404);
        }

        $aux             = new \stdClass();
        $todo            = [];
        $todo2           = [];
        $formas_de_pago  = FormaDePago::get();
        $monedas_activas = null;
        if ($seniat){
            if (count($caja_cerradas) > 0 && count($formas_de_pago) > 0){
                foreach ($formas_de_pago as $formas_de_pago) {
                    $monedas_activas = null;
                    $monedas_activas = Operacion::where('idCaja' , $caja_cerradas[0]->idCaja)
                        ->where('idFormaPago' , $formas_de_pago->idFormaPago)
                        ->groupby('fk_moneda')
                        ->get();
                    foreach ($monedas_activas as $monedas_activa) {
//                        return response()->json($monedas_activa->fk_moneda);
                        $operaciones = Operacion::where('idCaja' , $caja_cerradas[0]->idCaja)
                            ->whereHas('orderHeader' , function ($sql){
                                $sql->WhereNotNull('n_factura_seniat');
                            })
                            ->where('idFormaPago' , $formas_de_pago->idFormaPago)
                            ->where('fk_moneda' , $monedas_activa->fk_moneda)
                            ->get();

                        if ($monedas_activa->fk_moneda==1){
                            foreach ($operaciones as $key => $operacion) {
                                if (!is_null($operacion->tasa_moneda)){
                                    $operaciones[$key]->monto_bs = ($operacion->monto * $operacion->tasa_moneda);
                                } else {
                                    $operaciones[$key]->monto_bs = ($operacion->monto * $tasa->tasa);
                                }
                            }
                        } else {
                            foreach ($operaciones as $key => $operacion) {
                                $operaciones[$key]->monto_bs = $operacion->monto;
                            }
                        }
                        $ingresos = $operaciones->where('idTipoTransaccion' , 1)->sum('monto');
                        $egresos  = $operaciones->where('idTipoTransaccion' , 2)->sum('monto');

                        $ingresos_bs = $operaciones->where('idTipoTransaccion' , 1)->sum('monto_bs');
                        $egresos_bs  = $operaciones->where('idTipoTransaccion' , 2)->sum('monto_bs');

                        array_push($todo2 , [
                            'idMoneda' => $monedas_activa->fk_moneda ,
                            'nombre' => $monedas_activa->moneda->descripcion ,
                            'simbolo' => $monedas_activa->moneda->simbolo ,
                            'ingresos' => round($ingresos , 2) ,
                            'egresos' => round($egresos , 2) ,
                            'ingresos_bs' => round($ingresos_bs , 2) ,
                            'egresos_bs' => round($egresos_bs , 2) ,
                            'operaciones' => $operaciones
                        ]);
                    }
                    array_push($todo , [
                        'forma_pago' => $formas_de_pago ,
                        'monedas' => $todo2
                    ]);
                    $todo2 = [];

                }
//                return response()->json($todo);
            } else {
                $response = ['msj' => 'la caja o las formas de pagos no existen ' ,];
                return response()->json($response , 404);
            }
        } else {
            if (count($caja_cerradas) > 0 && count($formas_de_pago) > 0){
                foreach ($formas_de_pago as $formas_de_pago) {
                    $monedas_activas = null;
                    $monedas_activas = Operacion::where('idCaja' , $caja_cerradas[0]->idCaja)
                        ->where('idFormaPago' , $formas_de_pago->idFormaPago)
                        ->groupby('fk_moneda')
                        ->get();
                    foreach ($monedas_activas as $monedas_activa) {
//                        return response()->json($monedas_activa->fk_moneda);
                        $operaciones = Operacion::where('idCaja' , $caja_cerradas[0]->idCaja)
                            ->where('idFormaPago' , $formas_de_pago->idFormaPago)
                            ->where('fk_moneda' , $monedas_activa->fk_moneda)
                            ->get();
                        $ingresos    = $operaciones->where('idTipoTransaccion' , 1)->sum('monto');
                        $egresos     = $operaciones->where('idTipoTransaccion' , 2)->sum('monto');

                        array_push($todo2 , [
                            'idMoneda' => $monedas_activa->fk_moneda ,
                            'nombre' => $monedas_activa->moneda->descripcion ,
                            'simbolo' => $monedas_activa->moneda->simbolo ,
                            'ingresos' => round($ingresos , 2) ,
                            'egresos' => round($egresos , 2) ,
                            'operaciones' => $operaciones
                        ]);
                    }
                    array_push($todo , [
                        'forma_pago' => $formas_de_pago ,
                        'monedas' => $todo2
                    ]);
                    $todo2 = [];

                }
//                return response()->json($todo);
            } else {
                $response = [
                    'msj' => 'la caja o las formas de pagos no existen ' ,
                ];
                return response()->json($response , 404);
            }
        }


        $caja_cerradas[0]->operaciones_agrupadas_por_forma_pago = $todo;
        return response()->json($caja_cerradas);

    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public
    function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public
    function store(Request $request)
    {
        DB::beginTransaction();
        try {
            $caja = Caja::where('idSucursal' , $request->idSucursal)->active()->get();

            if (count($caja) > 0){
                $caja = $caja[0];
                foreach ($request->formas_pago as $formas_pago) {
                    $tipo         = new Operacion($formas_pago);
                    $tipo->idUser = $request->idUser;
                    $tipo->idCaja = $caja->idCaja;
                    $tipo->save();
                }
            } else {
                return response()->json(['msj' => "La caja no existe, cree una nueva caja para iniciar la transacción"] , 404);
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);


        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OperacionController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() . ' Linea: ' . $e->getLine()] , 500);
        }
    }

    public
    function storesincaja(Request $request)
    {
        $total_pagado_dolares = 0;
        $deuda                = 0;
        DB::beginTransaction();
        try {
            $caja   = Caja::where('idSucursal' , $request->idSucursal)->active()->get();
            $idUser = $request->idUser;
            $tasa   = TasaMoneda::active()->first();
//            $orden  = OrderHeader::withOnly('estatus')->find($request->formas_pago[0]['idOrderHeader']);
            if (count($request->formas_pago) > 0){
                $orden = OrderHeader::withOnly('customer')->find($request->formas_pago[0]['idOrderHeader']);
                if ($orden->customer->permitir_credito==false){
                    return response()->json([
                        'msj' => 'Este cliente no tiene crédito permitido' ,
                        'bool' => false
                    ]);
                }
            }
            if (count($caja) > 0){
                foreach ($request->formas_pago as $formas_pago) {
                    $tipo         = new Operacion($formas_pago);
                    $tipo->idCaja = $caja[0]->idCaja;
                    if ($formas_pago['fk_moneda']==2){
                        $total_pagado_dolares += $formas_pago['monto'] / $tasa->tasa;
                    } else {
                        $total_pagado_dolares += $formas_pago['monto'];
                    }
                    $tipo->tasa_moneda = $tasa->tasa;
                    $tipo->idUser      = $idUser;
                    $tipo->save();
                }
                // envio de sms
                DB::commit();
                if ($request->exists('enviar_sms') && $request->exists('idH')){
                    $orden = OrderHeader::withOnly('estatus')->find($request->idH);
                    //lo que ha pagado
                    $AbonoTotalDolares   = 0;
                    $AbonoTotalBolivares = 0;

                    $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                        ->where('fk_moneda' , 1)
                        ->where('idOrderHeader' , $request->idH)
                        ->sum('monto');

                    $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                        ->where('fk_moneda' , 2)
                        ->where('idOrderHeader' , $request->idH)
                        ->sum(DB::raw('monto/tasa_moneda'));

                    $totalAbono        = $AbonoTotalBolivares + $AbonoTotalDolares;
                    $orden->AbonoTotal = round($totalAbono , 2);
                    $orden->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);


                    if ($request->enviar_sms==true){
//                        $order = OrderHeader::withOnly('estatus')->find($request->idH);
//                        return response()->json($order , 201);
//                        $deuda = $order->precio_total - $total_pagado_dolares;
                        if ($orden){
                            SmsController::sendSMS(new Request([
                                "destination" => $orden->customer->phone ,
//                                "message" => 'ABONO A ORDEN ' . $order->n_order . ', recibimos y verificado un abono por $' . $total_pagado_dolares . ', adeuda $' . number_format($deuda , 2 , ',' , '') . '. IMPORTANTE: Debe cancelar el 100% para retirar y/o se proceda a la instalación del trabajo.'
                                "message" => 'ISLAPIXEL-ABONO ORDEN ' . $orden->n_order . ', recibimos un abono por $' . $total_pagado_dolares . ', adeuda $' . $orden->DeudaTotal . '. Recuerde cancelar el 100% antes de retirar o instalar el trabajo.'
                            ]));
                        }
                    } else {
//                        $order = OrderHeader::withOnly('estatus')->find($request->idH);
//                        return response()->json($order , 201);
//                        $deuda = $order->precio_total - $total_pagado_dolares;
                        SmsController::sendSMS(new Request([
                            "destination" => $orden->customer->phone ,
//                                "message" => 'ABONO A ORDEN ' . $order->n_order . ', recibimos y verificado un abono por $' . $total_pagado_dolares . ', adeuda $' . number_format($deuda , 2 , ',' , '') . '. IMPORTANTE: Debe cancelar el 100% para retirar y/o se proceda a la instalación del trabajo.'
                            "message" => 'ISLAPIXEL-PAGO DE ORDEN ' . $orden->n_order . ', hemos recibido y verificado su pago por $' . round($total_pagado_dolares , 2) . ' con éxito. Gracias por su confianza.'
                        ]));
                    }
                }

                $response = [
                    'msj' => 'Creado exitosamente' ,
                    'tipo' => $tipo ,
                ];

                return response()->json($response , 201);
            } else {
                return response()->json(['msj' => "La caja no existe, cree una nueva caja para iniciar la transacción"] , 404);
            }

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OperacionController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() . ' Linea: ' . $e->getLine()] , 500);
        }
    }

    public function operaciones_impuestos(Request $request)
    {
        $total_pagado_dolares = 0;
        $deuda                = 0;
        DB::beginTransaction();
        try {
            $caja   = Caja::where('idSucursal' , $request->idSucursal)->active()->get();
            $idUser = $request->idUser;

            $tasa   = @$request->tasa;
            if ($tasa==null){
                $tasa_activa = TasaMoneda::active()->first();
                $tasa = null;
                $orden = OrderHeader::withOnly('estatus')->find(@$request->idOrderHTax);
                if ($orden){
                    $tasa = $orden->tasa_moneda;
                } else {
                    $tasa = $tasa_activa->tasa;
                }
            }


            if (count($caja) > 0){
                foreach ($request->formas_pago as $formas_pago) {

                    $tipo = new Operacion($formas_pago);
                    $tipo->idCaja = $caja[0]->idCaja;

                    $tipo->impuesto    = @$request->impuesto;
                    $tipo->idOrderHTax = @$request->idOrderHTax;

                    $tipo->idCaja = $caja[0]->idCaja;

                    if ($formas_pago['fk_moneda']==2){
                        $total_pagado_dolares += $formas_pago['monto'] / $tasa;
                    } else {
                        $total_pagado_dolares += $formas_pago['monto'];
                    }
                    $tipo->tasa_moneda = $tasa;
                    $tipo->idUser      = $idUser;
                    $tipo->save();
                }
                // envio de sms
                DB::commit();

                $response = [
                    'msj' => 'Creado exitosamente' ,
                    'tipo' => $tipo ,
                ];

                return response()->json($response , 201);
            } else {
                return response()->json(['msj' => "La caja no existe, cree una nueva caja para iniciar la transacción"] , 404);
            }

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OperacionController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() . ' Linea: ' . $e->getLine()] , 500);
        }
    }

    public
    function cerrar_orden_cuenta_por_cobrar(Request $request)
    {
        DB::beginTransaction();
        try {
            $caja          = Caja::where('idSucursal' , $request->idSucursal)->active()->get();
            $idUser        = $request->idUser;
            $tasa          = TasaMoneda::active()->first();
            $idOrderHeader = null;
            if (count($caja) > 0){
                foreach ($request->formas_pago as $formas_pago) {
                    $idOrderHeader = $formas_pago['idOrderHeader'];
                    $tipo          = new Operacion($formas_pago);
                    $tipo->idCaja  = $caja[0]->idCaja;
                    if ($formas_pago['fk_moneda']==2){
                        $tipo->tasa_moneda = $tasa->tasa;
                    }
                    $tipo->idUser = $idUser;
                    $tipo->save();
                }
                $orden = OrderHeader::find($idOrderHeader);
                if ($orden){
                    $orden->update([
                        'cuenta_por_cobrar' => false
                    ]);
                    $orden->save();
                } else {
                    DB::rollback();
                    return response()->json(['msj' => "La Orden no existe"] , 404);
                }
                DB::commit();
                $response = [
                    'msj' => 'Creado exitosamente' ,
                    'tipo' => $tipo ,
                ];

                return response()->json($response , 201);
            } else {
                DB::rollback();
                return response()->json(['msj' => "La caja no existe, cree una nueva caja para iniciar la transacción"] , 404);
            }

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OperacionController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() . ' Linea: ' . $e->getLine()] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public
    function show($id)
    {
        $h = Operacion::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public
    function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function editar_operacion(Request $request , $id)
    {
        $this->validate($request , [
            'descripcion' => 'required' ,
            'tasa_moneda' => 'required' ,
            'monto' => 'required' ,
            'fk_moneda' => 'required' ,
            'idFormaPago' => 'required' ,
        ] , [
            'descripcion.required' => 'Es requerido el motivo de edicion de la transaccion' ,
            'tasa_moneda.required' => ':attribute es requerido' ,
            'monto.required' => ':attribute es requerido' ,
            'fk_moneda.required' => ':attribute es requerido' ,
            'idFormaPago.required' => ':attribute es requerido' ,
        ]);
        $operacion = Operacion::find($id);
        if ($operacion){
            $operacion->update([
                'historia_borrado' =>
                    'Operación editada por: ' . $request->user()->fullName
                    . ', monto original:' . $operacion->monto
                    . ', monto modificado:' . $request->monto
                    . ', moneda original:' . $operacion->fk_moneda
                    . ', moneda modificado:' . $request->fk_moneda
                    . ', tasa original:' . $operacion->tasa_moneda
                    . ', tasa modificado:' . $request->tasa_moneda
                    . ', idFormaPago original:' . $operacion->idFormaPago
                    . ', idFormaPago modificado:' . $request->idFormaPago
                    . ', id usuario: ' . $request->user()->id
                    . ', por el siguiente motivo: ' . $request->input('descripcion')
                    . ', en la fecha: ' . now()->format('Y-m-d H:i:s') . (!is_null($operacion->historia_borrado) ? ' - ' . $operacion->historia_borrado : '') ,
                'tasa_moneda' => $request->tasa_moneda ,
                'monto' => $request->monto ,
                'fk_moneda' => $request->fk_moneda ,
            ]);
            return response()->json(['msj' => "Edición Correcta"] , 200);
        } else {
            return response()->json(['msj' => "La transaccion no existe"] , 404);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy(Request $request , $id)
    {
//        dd($request->all());
//        $this->validate($request , [
//            'descripcion' => 'required' ,
//        ] , [
//            'descripcion.required' => 'Es requerido el motivo de edicion de la transaccion' ,
//        ]);

        if (!$request->exists('descripcion')){
            $request->descripcion = 'Eliminada por usuario';
        }

        $h = Operacion::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }
        $h->update([
            'historia_borrado' =>
                (!is_null($h->historia_borrado) ? $h->historia_borrado . ' - ' : '') . 'Operación borrada por: ' . $request->user()->fullName
                . ', por el siguiente motivo: ' . $request->input('descripcion')
                . ', en la fecha: ' . now()->format('Y-m-d H:i:s')
        ]);
        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}

