<?php

namespace App\Http\Controllers;

use App\Mail\AvisoIslaCreacionOrdenClienteMail;
use App\Mail\ClaveTemporalMail;
use App\Mail\NotificacionStatusOrdenMail;
use App\Models\ElementosOrden;
use App\Models\EstatusOrden;
use App\Models\HistoriaStatusOB;
use App\Models\HistoriaStatusOH;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\ServiciosOrden;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class OrderBodyController extends Controller
{

    public function cambiar_status_body_masivamente(Request $request)
    {
        $errors = [];
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'fk_status' => 'required' ,
            'idsOrderBody' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'fk_status.required' => ':attribute es requerido' ,
            'idsOrderBody.required' => ':attribute es requerido' ,
        ]);
        foreach ($request->idsOrderBody as $idOrderBody) {
            $body = OrderBody::where('fk_orderHeader' , $request->idOrderHeader)
                ->where('idOrderBody' , $idOrderBody)
                ->get();
            if (count($body) > 0){
                $body[0]->update(['fk_status' => $request->fk_status]);
            } else {
                array_push($errors , ['idOrderBody' => $idOrderBody]);
            }
        }

        if (count($errors) > 0){
            return response()->json(['msj' => 'No se pudieron cambiar los estados de los siguientes body' , 'data' => $errors] , 404);
        } else {
            return response()->json(['msj' => 'Se actualizó correctamente']);
        }

    }


    public function actualizar_precio_body(Request $request)
    {
        $idOrderBody = $request->idOrderBody;
        $precio      = $request->precio;
        $h           = OrderBody::find($idOrderBody);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el Order Body solicitada'] , 401);
        }
        try {
            $h->fill(['precio' => $precio]);
            $h->save();
            $response = [
                'msj' => 'Órden body actualizada exitosamente' ,
                'orden' => $h->refresh() ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    public function actualizarPersonal(Request $request)
    {
        $idOrderBody = $request->idOrderBody;
        $idPersonal  = $request->idPersonal;
        $h           = OrderBody::find($idOrderBody);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el Order Header solicitada'] , 401);
        }
        try {
            $h->fill(['fk_status' => 11 , 'idPersonal' => $idPersonal]);
            $h->save();
            $response = [
                'msj' => 'Órden actualizada exitosamente' ,
                'orden' => $h->refresh() ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    public function getOrdenPorIdBody(Request $request)
    {
        $idOrderBody = $request->idOrderBody;
        $body        = OrderBody::with('orderHeader' , 'estatus')->find($idOrderBody);
        if ($body){

            $body->fecha_entrega = Carbon::create($body->orderHeader->date_delivery)->format('Y-m-d');
        }

        if ($body)
            return response()->json(['producto' => $body , 'header' => $body->orderHeader]);
        return response()->json('sin Resultados' , 404);
    }

    public function cambiarStatusBody(Request $request)
    {
//        dd($request->all());
        $idOrderBody   = $request->idOrderBody;
        $idStatusOrden = $request->idStatusOrden;
        $idUser        = $request->idUser;
        $seguir        = [];
        $estadoOrden;
        if ($estadoOrden = EstatusOrden::find($idStatusOrden)){
            $seguir['status'] = true;
        } else {
            $seguir['status']  = false;
            $seguir['error'][] = 'No existe el estatus';
        }
        if (User::find($idUser)){
            $seguir['status'] = true;
        } else {
            $seguir['status']  = false;
            $seguir['error'][] = 'No existe el usuario';
        }

        $body = OrderBody::with('orderHeader')->find($idOrderBody);

        if ($body){
            $seguir['status'] = true;
        } else {
            $seguir['status']  = false;
            $seguir['error'][] = 'No existe el Body';
        }
        if ($seguir['status']){
            if ($body->fk_status==$idStatusOrden){
                return response()->json(['msj' => "La orden ya se encuentra en este estatus"]);
            }
            $body->update(['fk_status' => $idStatusOrden]);

            /*creo la historia del status*/
            $historia = new HistoriaStatusOB($request->all());
//            dd($historia);
            $historia->save();

            /*Envio mail de notificacion*/
            try {
                if ($estadoOrden->notificar_cliente){ //valido si el estado esta activo para notificar
                    SmsController::sendSMS(new Request([
                        "destination" => $body->orderHeader->customer->phone ,
                        "message" => 'Tu orden en islapixel ' . $body->orderHeader->n_order . ' se encuentra en estatus ' . strtolower($body->orderHeader->estatus->nombre) ,
                    ]));
                    $string     = $body->orderHeader->customer->email;
                    $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                    $resultado  = stristr($string , 'generico');
                    $resultado2 = stristr($string , 'noposee');
                    if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                    } else {
                        Mail::to($body->orderHeader->customer->email)->send(new NotificacionStatusOrdenMail($idOrderBody , $body->orderHeader->n_order , $body->orderHeader->estatus->nombre , $body->orderHeader));
                    }

                }
            } catch (\Exception $e) {
                Log::error('OrderBodyController funcion:cambiarStatusBody,  No se pudo enviar el correo al cambiar el estatus: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            return response()->json('Se cambió correctamente el estado del Body');
        } else {
            return response(['msj' => $seguir['error']] , 404);
        }
    }


    public function index()
    {
        return OrderBody::get();
    }

    public function listarPorId($idZonaEnvio)
    {
        return OrderBody::find($idZonaEnvio);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public static function store(Request $request , Request $mensajes)
    {
        try {
            $body = json_decode($request->body);
            foreach ($body->items as $item) {

                $orderBody = new OrderBody();
//                return response()->json($item);
//                $item                      = new Request($item);
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->precio                           = $item->precio;
                $orderBody->fk_moneda                        = $item->fk_moneda;
                $orderBody->fk_status                        = $item->fk_status;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->idPersonal                       = @$item->idPersonal;
                $orderBody->fk_idZonaEnvio                   = @$item->fk_idZonaEnvio;
                $orderBody->archivo                          = @$item->archivo;
                $orderBody->especificaciones_del_producto    = @$item->especificaciones_del_producto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;
                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }

                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicioOrden                = new ServiciosOrden();
                        $servicioOrden->fk_elemento   = $servicio->idElemento;
                        $servicioOrden->fk_producto   = $item->idProduct;
                        $servicioOrden->fk_orden_body = $orderBody->idOrderBody;
                        if (!is_null($servicio->fk_idPersonal)){
                            $orderBody->update([
                                'idPersonal' => $servicio->fk_idPersonal
                            ]);
                        }
                        $servicioOrden->fk_idPersonal       = $servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }

            }
//            $orderBody->save();
//envio de correo
            try {
                if (!is_null($mensajes->user)){
                    $string     = $mensajes->user->email;
                    $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                    $resultado  = stristr($string , 'generico');
                    $resultado2 = stristr($string , 'noposee');
                    if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                    } else {
                        @Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                    }

                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

//            dd([$mensajes->orden->idOrderHeader , $mensajes->orden->estadoOrden->nombre , $mensajes->orden->customer->email]);
            $dd = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'historiaStatus' ,
                'transaccion'
            )->find($mensajes->orden->idOrderHeader);

            if ($mensajes->orden->orden_creada_cliente==1){
                $string     = $mensajes->orden->customer->email;
                $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                $resultado  = stristr($string , 'generico');
                $resultado2 = stristr($string , 'noposee');
                if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                } else {
                    @Mail::to(env('MAIL_ISLA_NOTIFICACION'))->send(new AvisoIslaCreacionOrdenClienteMail($mensajes->orden->n_order , $mensajes->orden->customer->email , $mensajes->orden->customer->fullName));

                }
            }
            try {
                SmsController::sendSMS(new Request([
                    "destination" => $mensajes->orden->customer->phone ,
//                    "message" => 'Tu orden en islapixel ' . $mensajes->orden->n_order . ' se encuentra en estatus ' . strtolower($mensajes->orden->estatus->nombre) ,
                    "message" => 'ISLAPIXEL-ORDEN CREADA ' . $mensajes->orden->n_order . ' por un total de $' . $mensajes->orden->precio_total . '. Debemos verificar el pago para pasar a producción' ,
                ]));

                $string     = $mensajes->orden->customer->email;
                $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                $resultado  = stristr($string , 'generico');
                $resultado2 = stristr($string , 'noposee');
                if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                } else {
//                    PromocionesController::enviarMailOrdenNuevo($mensajes->orden->idOrderHeader);
//                    @Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre , $dd));
                }

            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //enviar el pdf con qr del pedido

//            $d = new OrderHeaderController();
//            @$d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);

            /*funcion para corregir id con numero de codigo cuando se repite*/
            $code = explode('-' , $mensajes->orden->n_order);
            if (count($code) > 0){
                if ($code[1]!=$mensajes->orden->idOrderHeader){
                    $orden   = OrderHeader::without(
                        'interacciones' ,
                        'estatus' ,
                        'descuento' ,
                        'usuario_creador' ,
                        'usuario_lector' ,
                        'customer' ,
                        'zonaEnvio' ,
                        'moneda_orderheader' ,
                        'estadoOrden' ,
                        'sucursal_orden' ,
                        'body' ,
                        'historiaStatus' ,
                        'transaccion'
                    )->find($mensajes->orden->idOrderHeader);
                    $n_order = $code . '-' . $mensajes->orden->idOrderHeader;
                    $orden->update(['n_order' => $n_order]);
                    $response = [
                        'msj' => 'Creado exitosamente' ,
                        'idOrden' => $mensajes->orden->idOrderHeader ,
                        'n_order' => $n_order ,
                    ];
                }
            }
            /*funcion para corregir id con numero de codigo cuadno se repite*/
            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public static function add_por_body_adicional(Request $request)
    {

        try {
            $body = json_decode(json_encode($request->body));
//            return response()->json($body->items);
//            $body = $request->body;
            foreach ($body->items as $item) {

                $orderBody                                   = new OrderBody();
                $orderBody->fk_orderHeader                   = $request->idOrderHeader;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->precio                           = $item->precio;
                $orderBody->fk_moneda                        = $item->fk_moneda;
                $orderBody->fk_status                        = $item->fk_status;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->idPersonal                       = @$item->idPersonal;
                $orderBody->fk_idZonaEnvio                   = @$item->fk_idZonaEnvio;
                $orderBody->archivo                          = @$item->archivo;
                $orderBody->fk_idTiempoEntrega               = @$item->fk_idTiempoEntrega;
                $orderBody->is_express_delivery              = (boolean)@$item->is_express_delivery;
                $orderBody->especificaciones_del_producto    = @$item->especificaciones_del_producto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }

                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicioOrden                = new ServiciosOrden();
                        $servicioOrden->fk_elemento   = $servicio->idElemento;
                        $servicioOrden->fk_producto   = $item->idProduct;
                        $servicioOrden->fk_orden_body = $orderBody->idOrderBody;
                        if (!is_null($servicio->fk_idPersonal)){
                            $orderBody->update([
                                'idPersonal' => $servicio->fk_idPersonal
                            ]);
                        }
                        $servicioOrden->fk_idPersonal       = $servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }

            }

            DB::commit();

            $response = [
                'msj' => 'Body agregado exitosamente' ,
            ];

            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public static function storecotizacion(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            $body = json_decode($request->body);
            foreach ($body->items as $item) {
                $orderBody                                   = new OrderBody();
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->archivo                          = @$item->archivo;
                $orderBody->especificaciones_del_producto    = @$item->especificaciones_del_producto;
                $orderBody->precio                           = is_null(@$item->precio) ? 0 : $item->precio; //falta subir
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                $orderBody->save();
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }

                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicioOrden                      = new ServiciosOrden();
                        $servicioOrden->fk_elemento         = $servicio->idElemento;
                        $servicioOrden->fk_producto         = $item->idProduct;
                        $servicioOrden->fk_orden_body       = $orderBody->idOrderBody;
                        $servicioOrden->fk_idPersonal       = $servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }


            }
            try {
                if (!is_null($mensajes->user)){
                    $string     = $mensajes->user->email;
                    $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                    $resultado  = stristr($string , 'generico');
                    $resultado2 = stristr($string , 'noposee');
                    if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                    } else {
                        @Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                    }
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $dd = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'historiaStatus' ,
                'transaccion'
            )->find($mensajes->orden->idOrderHeader);
            try {

                $string     = $mensajes->orden->customer->email;
                $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                $resultado  = stristr($string , 'generico');
                $resultado2 = stristr($string , 'noposee');
                if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                } else {
                    @Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre , $dd));
                }

            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

            try {
                if ($request->orden_creada_cliente==1){
                    @Mail::to(env('MAIL_ISLA_NOTIFICACION'))->send(new AvisoIslaCreacionOrdenClienteMail($mensajes->orden->n_order , $mensajes->orden->customer->email , $mensajes->orden->customer->fullName));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo a islapixel: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //enviar el pdf con qr del pedido
            $d = new OrderHeaderController();
            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function storecotizacionpostman(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            $body = $request->body;
            foreach ($body['items'] as $item) {

                $orderBody                                   = new OrderBody();
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->cantidad                         = $item['cantidad'];
                $orderBody->ancho                            = $item['ancho'];
                $orderBody->alto                             = $item['alto'];
                $orderBody->color                            = isset($item['color']) ? $item['color'] : null;
                $orderBody->tiempo_de_entrega                = isset($item['tiempo_de_entrega']) ? $item['tiempo_de_entrega'] : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item['monto_extra_de_tiempo_de_entrega']) ? $item['monto_extra_de_tiempo_de_entrega'] : null;

                $orderBody->idProduct                     = $item['idProduct'];
                $orderBody->archivo                       = @$item['archivo'];
                $orderBody->especificaciones_del_producto = @$item['especificaciones_del_producto'];
//                if ($request->exists('archivo')){
//                    if (!is_null($request->archivo)){
//                        //obtenemos el campo file definido en el formulario
//                        $file = $request->file('archivo');
//                        //obtenemos el nombre del archivo
//                        $nombre = $file->getClientOriginalName();
//                        //indicamos que queremos guardar un nuevo archivo en el disco local
////                        \Storage::disk('local')->put($nombre , \File::get($file));
//                        $nombre_interno = $nombre . '-' . time();
//                        Storage::disk('public')->put('\\archivobodycotizacion\\' . $nombre_interno , File::get($file));
//                        /*para la foto*/
//                        $orderBody->archivo = $nombre_interno;
//                    }
//                }
                $orderBody->save();
            }

            try {
                if (!is_null($mensajes->user)){
                    $string     = $mensajes->user->email;
                    $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                    $resultado  = stristr($string , 'generico');
                    $resultado2 = stristr($string , 'noposee');
                    if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                    } else {
                        Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                    }

                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $dd = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'historiaStatus' ,
                'transaccion'
            )->find($mensajes->orden->idOrderHeader);
            try {
                $string     = $mensajes->orden->customer->email;
                $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                $resultado  = stristr($string , 'generico');
                $resultado2 = stristr($string , 'noposee');
                if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                } else {
//                    Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre , $dd));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //enviar el pdf con qr del pedido
//            $d = new OrderHeaderController();
//            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public static function body_orden_por_valla(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            $body = $request->body;
            foreach ($body['items'] as $item) {

                $orderBody                                   = new OrderBody();
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->cantidad                         = @$item['cantidad'];
                $orderBody->ancho                            = @$item['ancho'];
                $orderBody->alto                             = @$item['alto'];
                $orderBody->color                            = isset($item['color']) ? $item['color'] : null;
                $orderBody->tiempo_de_entrega                = isset($item['tiempo_de_entrega']) ? $item['tiempo_de_entrega'] : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item['monto_extra_de_tiempo_de_entrega']) ? $item['monto_extra_de_tiempo_de_entrega'] : null;

                $orderBody->idProduct                     = @$item['idProduct'];
                $orderBody->fk_idValla                     = @$item['fk_idValla'];
                $orderBody->especificaciones_del_producto = @$item['especificaciones_del_producto'];

                $orderBody->save();
            }

            try {
                if (!is_null($mensajes->user)){
                    $string     = $mensajes->user->email;
                    $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                    $resultado  = stristr($string , 'generico');
                    $resultado2 = stristr($string , 'noposee');
                    if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                    } else {
//                        Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                    }

                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $dd = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'historiaStatus' ,
                'transaccion'
            )->find($mensajes->orden->idOrderHeader);
            try {
                $string     = $mensajes->orden->customer->email;
                $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                $resultado  = stristr($string , 'generico');
                $resultado2 = stristr($string , 'noposee');
                if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                } else {
//                    Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre , $dd));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //enviar el pdf con qr del pedido
//            $d = new OrderHeaderController();
//            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function prueba(Request $request)
    {
        if ($request->exists('archivo')){
            if (!is_null($request->archivo)){
                //obtenemos el campo file definido en el formulario
                $file = $request->file('archivo');

                //obtenemos el nombre del archivo
                $nombre = $file->getClientOriginalName();
                //indicamos que queremos guardar un nuevo archivo en el disco local
//                        \Storage::disk('local')->put($nombre , \File::get($file));
                $nombre_interno = $nombre . '-' . time();

                Storage::disk('public')->put('\\archivobodycotizacion\\' . $nombre_interno , File::get($file));
                /*para la foto*/
                $orderBody->archivo = $nombre_interno;
            }
        }
    }

    public function store2(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
//            $body = json_decode($request->body);
            foreach ($request->body['items'] as $item) {

                $orderBody = new OrderBody();
//                return response()->json($item);
                $item                                        = new Request($item);
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->precio                           = $item->precio;
                $orderBody->fk_moneda                        = $item->fk_moneda;
                $orderBody->fk_status                        = $item->fk_status;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                $orderBody->idPersonal     = @$item->idPersonal;
                $orderBody->fk_idZonaEnvio = @$item->fk_idZonaEnvio;
                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                foreach ($item->elementos as $elemento) {
                    $elementosOrden                = new ElementosOrden();
                    $elementosOrden->fk_elemento   = $elemento;
                    $elementosOrden->fk_producto   = $item->idProduct;
                    $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                    $elementosOrden->save();
                }
                //creando los servicios de productos de la orden
                foreach ($item->servicios as $servicio) {

                    $servicio = new Request($servicio);

                    $servicioOrden                      = new ServiciosOrden();
                    $servicioOrden->fk_elemento         = $servicio->idElemento;
                    $servicioOrden->fk_producto         = $item->idProduct;
                    $servicioOrden->fk_orden_body       = $orderBody->idOrderBody;
                    $servicioOrden->fk_idPersonal       = @$servicio->fk_idPersonal;
                    $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                    $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                    $servicioOrden->save();
                }

            }
//            $orderBody->save();
//envio de correo
            try {
                if (!is_null($mensajes->user)){
                    Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

//            dd([$mensajes->orden->idOrderHeader , $mensajes->orden->estadoOrden->nombre , $mensajes->orden->customer->email]);
            $dd = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'historiaStatus' ,
                'transaccion'
            )->find($mensajes->orden->idOrderHeader);

            try {
                Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre , $dd));
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //enviar el pdf con qr del pedido
            $d = new OrderHeaderController();
            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function store3(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            foreach ($request->body['items'] as $item) {
                $orderBody                                   = new OrderBody();
                $item                                        = new Request($item);
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->precio                           = $item->precio;
                $orderBody->fk_moneda                        = $item->fk_moneda;
                $orderBody->fk_status                        = 3;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                $orderBody->idPersonal = @$item->idPersonal;
                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }
                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicio                           = new Request($servicio);
                        $servicioOrden                      = new ServiciosOrden();
                        $servicioOrden->fk_elemento         = $servicio->idElemento;
                        $servicioOrden->fk_producto         = $item->idProduct;
                        $servicioOrden->fk_orden_body       = $orderBody->idOrderBody;
                        $servicioOrden->fk_idPersonal       = @$servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }
            }
//            $orderBody->save();
//envio de correo
            try {
                if (!is_null($mensajes->user)){
//                    Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

//            dd([$mensajes->orden->idOrderHeader , $mensajes->orden->estadoOrden->nombre , $mensajes->orden->customer->email]);

            try {
//                Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre));
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //creando el inventario
            InventarioDeProductoController::crear_inventario_por_id_orden($mensajes->orden->idOrderHeader);
            //enviar el pdf con qr del pedido
//            $d = new OrderHeaderController();
//            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public static function puntoventa(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            foreach ($request->body['items'] as $item) {
                $orderBody                                   = new OrderBody();
                $item                                        = new Request($item);
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->precio                           = $item->precio;
                $orderBody->fk_moneda                        = $item->fk_moneda;
                $orderBody->fk_status                        = 1;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                $orderBody->idPersonal                    = @$item->idPersonal;
                $orderBody->especificaciones_del_producto = @$item->especificaciones_del_producto;
                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }
                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicio                           = new Request($servicio);
                        $servicioOrden                      = new ServiciosOrden();
                        $servicioOrden->fk_elemento         = $servicio->idElemento;
                        $servicioOrden->fk_producto         = $item->idProduct;
                        $servicioOrden->fk_orden_body       = $orderBody->idOrderBody;
                        $servicioOrden->fk_idPersonal       = @$servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }
            }
//            $orderBody->save();
//envio de correo
            try {
                if (!is_null($mensajes->user)){
//                    Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

//            dd([$mensajes->orden->idOrderHeader , $mensajes->orden->estadoOrden->nombre , $mensajes->orden->customer->email]);

            try {
//                Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre));
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            //enviar el pdf con qr del pedido
//            $d = new OrderHeaderController();
//            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public static function puntoventa2(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            foreach ($request->body['items'] as $item) {
                $orderBody                                   = new OrderBody();
                $item                                        = new Request($item);
                $orderBody->fk_orderHeader                   = $mensajes->orden->idOrderHeader;
                $orderBody->idProduct                        = $item->idProduct;
                $orderBody->cantidad                         = $item->cantidad;
                $orderBody->precio                           = $item->precio;
                $orderBody->fk_moneda                        = $item->fk_moneda;
                $orderBody->fk_status                        = 3;
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                $orderBody->idPersonal                    = @$item->idPersonal;
                $orderBody->especificaciones_del_producto = @$item->especificaciones_del_producto;
                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }
                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicio                           = new Request($servicio);
                        $servicioOrden                      = new ServiciosOrden();
                        $servicioOrden->fk_elemento         = $servicio->idElemento;
                        $servicioOrden->fk_producto         = $item->idProduct;
                        $servicioOrden->fk_orden_body       = $orderBody->idOrderBody;
                        $servicioOrden->fk_idPersonal       = @$servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }
            }
//            $orderBody->save();

            //envio de correo
            try {
                if (!is_null($mensajes->user)){
//                    Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

//            dd([$mensajes->orden->idOrderHeader , $mensajes->orden->estadoOrden->nombre , $mensajes->orden->customer->email]);

            try {
//                Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre));
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [
                'msj' => 'Creado exitosamente' ,
                'idOrden' => $mensajes->orden->idOrderHeader ,
                'n_order' => $mensajes->orden->n_order ,
            ];
            DB::commit();
            InventarioDeProductoController::crear_inventario_por_id_orden_punto_de_venta($request->cerrar_orden['idUser'] , $mensajes->orden->idOrderHeader);
            //enviar el pdf con qr del pedido
//            $d = new OrderHeaderController();
//            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function puntoventacredito(Request $request , Request $mensajes)
    {

        DB::beginTransaction();
        try {
            foreach ($request->body['items'] as $item) {
                $orderBody                 = new OrderBody();
                $item                      = new Request($item);
                $orderBody->fk_orderHeader = $mensajes->orden->idOrderHeader;
                $orderBody->idProduct      = $item->idProduct;
                $orderBody->cantidad       = $item->cantidad;
                $orderBody->precio         = $item->precio;
                $orderBody->fk_moneda      = $item->fk_moneda;
                if ($mensajes->orden->customer->permitir_credito==false){
                    $orderBody->fk_status = 1;
                } else {
                    $orderBody->fk_status = 2;
                }
                $orderBody->ancho                            = $item->ancho;
                $orderBody->alto                             = $item->alto;
                $orderBody->color                            = isset($item->color) ? $item->color : null;
                $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                $orderBody->idPersonal                    = @$item->idPersonal;
                $orderBody->especificaciones_del_producto = @$item->especificaciones_del_producto;
                $orderBody->save();
                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elementosOrden                = new ElementosOrden();
                        $elementosOrden->fk_elemento   = $elemento;
                        $elementosOrden->fk_producto   = $item->idProduct;
                        $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                        $elementosOrden->save();
                    }
                }
                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicio                           = new Request($servicio);
                        $servicioOrden                      = new ServiciosOrden();
                        $servicioOrden->fk_elemento         = $servicio->idElemento;
                        $servicioOrden->fk_producto         = $item->idProduct;
                        $servicioOrden->fk_orden_body       = $orderBody->idOrderBody;
                        $servicioOrden->fk_idPersonal       = @$servicio->fk_idPersonal;
                        $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                        $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                        $servicioOrden->save();
                    }
                }
            }
//            $orderBody->save();
//envio de correo
            try {
                if (!is_null($mensajes->user)){
//                    Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
                }
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }

//            dd([$mensajes->orden->idOrderHeader , $mensajes->orden->estadoOrden->nombre , $mensajes->orden->customer->email]);

            try {
//                Mail::to($mensajes->orden->customer->email)->send(new NotificacionStatusOrdenMail($mensajes->orden->idOrderHeader , $mensajes->orden->n_order , $mensajes->orden->estadoOrden->nombre));
            } catch (\Exception $e) {
                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            $response = [];
            if ($mensajes->orden->customer->permitir_credito==false){
                $response = [
                    'msj' => 'Este cliente no tiene crédito permitido, pero se creó la orden' ,
                    'bool' => false ,
                    'idOrden' => $mensajes->orden->idOrderHeader ,
                    'n_order' => $mensajes->orden->n_order
                ];
            } else {
                $response = [
                    'msj' => 'Creado exitosamente' ,
                    'idOrden' => $mensajes->orden->idOrderHeader ,
                    'n_order' => $mensajes->orden->n_order ,
                ];
            }

            DB::commit();
//            return response()->json(InventarioDeProductoController::crear_inventario_por_id_orden_punto_de_venta($request->cerrar_orden['idUser'],$mensajes->orden->idOrderHeader),404);
            if ($mensajes->orden->customer->permitir_credito==true){
                InventarioDeProductoController::crear_inventario_por_id_orden_punto_de_venta($request->cerrar_orden['idUser'] , $mensajes->orden->idOrderHeader);
            }
            //enviar el pdf con qr del pedido
//            $d = new OrderHeaderController();
//            $d->gereararPdf($mensajes->orden->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            return response()->json($response , 201);

        } catch
        (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = OrderBody::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['zona_envio' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit(Request $request , $id)
    {
        $this->validate($request , [
            'cantidad' => 'required' ,
            'precio' => 'required' ,
        ] , [
            'cantidad.required' => ':attribute es requerido' ,
            'precio.required' => ':attribute es requerido' ,

        ]);
        DB::beginTransaction();
        $h = OrderBody::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el Order Header solicitada'] , 401);
        }

        try {
            $h->cantidad = $request->input('cantidad');
            $h->precio   = $request->input('precio');
            $h->save();
            $response = [
                'msj' => 'Order actualizada exitosamente'
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }
    }

    public
    function edit_all(Request $request)
    {
        DB::beginTransaction();
        try {
            $orderBody = null;
//            $body = json_decode($request->body);
            foreach ($request->items as $item) {
                $item = new Request($item);
                //si no envia el id del body, entonces creo un
                if ($item->idOrderBody==''){
                    $orderBody                                   = new OrderBody();
                    $orderBody->fk_orderHeader                   = $request->idOrderHeader;
                    $orderBody->idProduct                        = @$item->idProduct;
                    $orderBody->cantidad                         = @$item->cantidad;
                    $orderBody->precio                           = @$item->precio;
                    $orderBody->fk_moneda                        = @$item->fk_moneda;
                    $orderBody->fk_status                        = @$item->fk_status;
                    $orderBody->ancho                            = @$item->ancho;
                    $orderBody->alto                             = @$item->alto;
                    $orderBody->color                            = isset($item->color) ? $item->color : null;
                    $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                    $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                    $orderBody->idPersonal                    = @$item->idPersonal;
                    $orderBody->fk_idZonaEnvio                = @$item->fk_idZonaEnvio;
                    $orderBody->archivo                       = @$item->archivo;
                    $orderBody->especificaciones_del_producto = @$item->especificaciones_del_producto;
                    $orderBody->save();
//                    dd($orderBody->idOrderBody);
                } else {
                    $orderBody = OrderBody::find($item->idOrderBody);
                    if ($orderBody){
                        $orderBody->idProduct                        = $item->idProduct ? $item->idProduct : $orderBody->idProduct;
                        $orderBody->cantidad                         = $item->cantidad ? $item->cantidad : $orderBody->cantidad;
                        $orderBody->precio                           = $item->precio ? $item->precio : $orderBody->precio;
                        $orderBody->fk_moneda                        = $item->fk_moneda ? $item->fk_moneda : $orderBody->fk_moneda;
                        $orderBody->fk_status                        = $item->fk_status ? $item->fk_status : $orderBody->fk_status;
                        $orderBody->ancho                            = $item->ancho ? $item->ancho : $orderBody->ancho;
                        $orderBody->alto                             = $item->alto ? $item->alto : $orderBody->alto;
                        $orderBody->color                            = isset($item->color) ? $item->color : null;
                        $orderBody->tiempo_de_entrega                = isset($item->tiempo_de_entrega) ? $item->tiempo_de_entrega : null;
                        $orderBody->monto_extra_de_tiempo_de_entrega = isset($item->monto_extra_de_tiempo_de_entrega) ? $item->monto_extra_de_tiempo_de_entrega : null;

                        $orderBody->idPersonal                    = $item->idPersonal ? $item->idPersonal : $orderBody->idPersonal;
                        $orderBody->fk_idZonaEnvio                = @$item->fk_idZonaEnvio ? $item->fk_idZonaEnvio : $orderBody->fk_idZonaEnvio;
                        $orderBody->archivo                       = @$item->archivo ? $item->archivo : $orderBody->archivo;
                        $orderBody->especificaciones_del_producto = @$item->especificaciones_del_producto ? $item->especificaciones_del_producto : $orderBody->especificaciones_del_producto;
                        $orderBody->save();
                    }

                }


                //creando los elementos de productos de la orden
//                return response()->json($item->elementos);
                if (count($item->elementos) > 0){
                    foreach ($item->elementos as $elemento) {
                        $elemento = new Request($elemento);
                        if ($elemento->idElementoOrden==''){
                            $elementosOrden                = new ElementosOrden();
                            $elementosOrden->fk_elemento   = $elemento->elemento;
                            $elementosOrden->fk_producto   = $item->idProduct;
                            $elementosOrden->fk_orden_body = $orderBody->idOrderBody;
                            $elementosOrden->save();
                        } else {
                            $elementosOrden                = ElementosOrden::find($elemento->idElementoOrden);
                            $elementosOrden->fk_elemento   = $elemento->elemento ? $elemento->elemento : $elementosOrden->fk_elemento;
                            $elementosOrden->fk_producto   = $item->idProduct ? $item->idProduct : $elementosOrden->fk_producto;
                            $elementosOrden->fk_orden_body = $orderBody->idOrderBody ? $orderBody->idOrderBody : $elementosOrden->fk_orden_body;
                            $elementosOrden->save();
                        }

                    }
                }

                //creando los servicios de productos de la orden
                if (count($item->servicios) > 0){
                    foreach ($item->servicios as $servicio) {
                        $servicio = new Request($servicio);
                        if ($servicio->idElemento!=''){
                            $servicioOrden                = new ServiciosOrden();
                            $servicioOrden->fk_elemento   = $servicio->idElemento;
                            $servicioOrden->fk_producto   = $item->idProduct;
                            $servicioOrden->fk_orden_body = $orderBody->idOrderBody;
                            if (!is_null($servicio->fk_idPersonal)){
                                $orderBody->update([
                                    'idPersonal' => $servicio->fk_idPersonal
                                ]);
                            }
                            $servicioOrden->fk_idPersonal       = $servicio->fk_idPersonal;
                            $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                            $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                            $servicioOrden->save();
                        } else {
                            $servicioOrden                = ServiciosOrden::find($servicio->idElemento);
                            $servicioOrden->fk_elemento   = $servicio->idElemento;
                            $servicioOrden->fk_producto   = $item->idProduct;
                            $servicioOrden->fk_orden_body = $orderBody->idOrderBody;
                            if (!is_null($servicio->fk_idPersonal)){
                                $orderBody->update([
                                    'idPersonal' => $servicio->fk_idPersonal
                                ]);
                            }
                            $servicioOrden->fk_idPersonal       = $servicio->fk_idPersonal;
                            $servicioOrden->comision_porcentaje = $servicio->comision_porcentaje;
                            $servicioOrden->comision_moneda     = $servicio->comision_moneda;
                            $servicioOrden->save();
                        }

                    }
                }

            }
//            $orderBody->save();
//envio de correo
//            try {
//                if (!is_null($mensajes->user)){
//                    @Mail::to($mensajes->user->email)->send(new ClaveTemporalMail($mensajes->user , $mensajes->pass));
//                }
//            } catch (\Exception $e) {
//            DB::rollback();
//                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
//            }

            $dd = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'historiaStatus' ,
                'transaccion'
            )->find($request->idOrderHeader);
//            try {
//                SmsController::sendSMS(new Request([
//                    "destination" => $dd->customer->phone ,
//                    "message" => 'Tu orden en islapixel ' . $dd->n_order . ' se encuentra en estatus ' . strtolower($dd->estatus->nombre) ,
//                ]));
//                @Mail::to($dd->customer->email)->send(new NotificacionStatusOrdenMail($dd->idOrderHeader , $dd->n_order , $dd->estadoOrden->nombre , $dd));
//            } catch (\Exception $e) {
//                DB::rollback();
//                Log::error('no se pudo enviar el correo: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
//            }
            $response = [
                'msj' => 'Cuerpo de la orden modificada exitosamente' ,
                'idOrden' => $dd->idOrderHeader ,
                'n_order' => $dd->n_order ,
            ];
            //enviar el pdf con qr del pedido

            $d = new OrderHeaderController();
            @$d->gereararPdf($dd->idOrderHeader);
//            OrderHeaderController::gereararPdf($mensajes->orden->idOrderHeader);
            DB::commit();
            return response()->json($response , 201);

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'fk_orderHeader' => 'required' ,
            'fk_product' => 'required' ,
            'width' => 'required' ,
            'height' => 'required' ,
            'm2' => 'required' ,
            'precio_unitario' => 'required' ,
            'precio_total' => 'required' ,
        ] , [
            'fk_orderHeader.required' => ':attribute es requerido' ,
            'fk_product.required' => ':attribute es requerido' ,
            'width.required' => ':attribute es requerido' ,
            'height.required' => ':attribute es requerido' ,
            'm2.required' => ':attribute es requerido' ,
            'precio_unitario.required' => ':attribute es requerido' ,
            'precio_total.required' => ':attribute es requerido' ,

        ]);
        DB::beginTransaction();
        $h = OrderBody::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el Order Header solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Order actualizado exitosamente' ,
                'zona_envio' => $h->refresh() ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderBodyController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = OrderBody::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }

    public function destroy2(Request $request)
    {
        $ids = $request->ids_eliminar;
        $h   = OrderBody::whereIn('idOrderBody' , $ids);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            $h->delete();
            return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
        }

    }
}
