<?php

namespace App\Http\Controllers;

use App\Jobs\GenerarPdfJob;
use App\Mail\NotificacionStatusOrdenMail;
use App\Models\Caja;
use App\Models\Descuento;
use App\Models\EstatusOrden;
use App\Models\FormaDePago;
use App\Models\HistorialNotificacionCobroSms;
use App\Models\HistoriaRegistroSeniatOrden;
use App\Models\HistoriaStatusOH;
use App\Models\Operacion;
use App\Models\OrdenesEnviadasImpresas;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\Producto;
use App\Models\SubTipo;
use App\Models\Sucursal;
use App\Models\TasaMoneda;
use App\Models\TiemposDeEntrega;
use App\Models\User;
use App\Models\ZonaEnvio;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Image;
use phpDocumentor\Reflection\DocBlock\Tags\Uses;
use QrCode;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class OrderHeaderController extends Controller
{

    public function chek_enviado(Request $request)
    {
        $ordenes = OrderHeader::withOnly('estatus')
            ->find($request->idOrderHeader)
            ->update(['pedido_enviado' => 1]);

        return response()->json(['msj' => 'Orden actualizada correctamente'] , 200);

    }

    public function listar_imprimir(Request $request)
    {
        $ordenes = OrderHeader::
        where('pedido_enviado' , 1)
            ->has('fueImpresa' , '=' , 0) //para comprobar si tiene o no registros
            ->get();

        foreach ($ordenes as $key => $orden) {
            //se crea el registro para la impresion
            OrdenesEnviadasImpresas::create([
                "fk_idOrdenHeader" => $orden->idOrderHeader ,
                "fk_idUser" => $request->user()->id ,
                "esta_impreso" => 1
            ]);
        }


        return response()->json(['msj' => 'Órdenes para imprimir' , 'data' => $ordenes] , 200);

    }

    public function qw(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);
        $sin_tasa;
        $año     = $date_separado[0];
        $mes     = $date_separado[1];
        $ordenes = OrderHeader::withOnly('estadoOrden')
//            ->where('fk_status' , '!=' , 10)
            ->whereNotNull('fecha_facturacion')
            ->whereYear('fecha_facturacion' , '=' , $año)
            ->whereMonth('fecha_facturacion' , '=' , $mes)
            ->where(function ($sql) use ($request){
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->where('sucursal_facturada' , $request->idSucursal);
                    }
                }
            })
            ->get();
        foreach ($ordenes as $key => $orden) {

            //busco la tasa con la fecha de facturacion
            $tasa = TasaMoneda::where('created_at' , 'like' , "%" . $orden->fecha_facturacion->format('Y-m-d') . "%")
                ->orderby('idTasaMoneda' , 'desc')
                ->first();
            if ($tasa){
                $ordenes[$key]->mon_bs = ($orden->precio_total * $tasa->tasa);
            } else {

                //si no esta la tasa en la fecha buscada, entonces busco una de dias anteriores
                $tasa                  = TasaMoneda::where('created_at' , '<' , $orden->fecha_facturacion->format('Y-m-d'))
                    ->orderby('idTasaMoneda' , 'desc')
                    ->first();
                $ordenes[$key]->mon_bs = ($orden->precio_total * $tasa->tasa);
//                $sin_tasa[] = [
//                    'fecha' => $orden->fecha_facturacion->format('Y-m-d') ,
//                    'monto' => $ordenes[$key]->precio_total ,
//                    'tasa_aux' => $tasa->tasa,
//                    'conversion'=>$orden->precio_total * $tasa->tasa
//                ];
            }

        }
        return response()->json($ordenes->sum('mon_bs'));

//        return response()->json($sin_tasa);

//        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
//        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json([$sum_bs,$sum_dolares]);
//        $r->sum_bs      = $sum_bs;
//        $r->sum_dolares = $sum_dolares;
//        $nombre_archivo = null;
//        $nombre_archivo = 'reporte_ingresos_mensual' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
//        Excel::store((new ReporteIngresosMensual($r)) , '\\excel\\reporte_ingresos_mensual\\' . $nombre_archivo , 'public');
//        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ingresos_mensual\\' . $nombre_archivo);

    }

    public function qw2(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);
        $sin_tasa;
        $año     = $date_separado[0];
        $mes     = $date_separado[1];
        $ordenes = OrderHeader::withOnly('estadoOrden')
//            ->where('fk_status' , '!=' , 10)
            ->whereNotNull('fecha_facturacion')
            ->whereYear('fecha_facturacion' , '=' , $año)
            ->whereMonth('fecha_facturacion' , '=' , $mes)
            ->get();
        foreach ($ordenes as $key => $orden) {

            //busco la tasa con la fecha de facturacion
            $tasa = TasaMoneda::where('created_at' , 'like' , "%" . $orden->fecha_facturacion->format('Y-m-d') . "%")
                ->orderby('idTasaMoneda' , 'desc')
                ->first();
            if ($tasa){
                $ordenes[$key]->mon_bs = ($orden->precio_total * $tasa->tasa);
            } else {

                //si no esta la tasa en la fecha buscada, entonces busco una de dias anteriores
                $tasa                  = TasaMoneda::where('created_at' , '<' , $orden->fecha_facturacion->format('Y-m-d'))
                    ->orderby('idTasaMoneda' , 'desc')
                    ->first();
                $ordenes[$key]->mon_bs = ($orden->precio_total * $tasa->tasa);
                $sin_tasa[]            = [
                    'fecha' => $orden->fecha_facturacion->format('Y-m-d') ,
                    'monto' => $ordenes[$key]->precio_total ,
                    'tasa_aux' => $tasa->tasa ,
                    'conversion' => $orden->precio_total * $tasa->tasa
                ];
            }

        }
//        return response()->json($sin_tasa);
        return response()->json($ordenes->sum('mon_bs'));
//        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
//        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json([$sum_bs,$sum_dolares]);
//        $r->sum_bs      = $sum_bs;
//        $r->sum_dolares = $sum_dolares;
//        $nombre_archivo = null;
//        $nombre_archivo = 'reporte_ingresos_mensual' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
//        Excel::store((new ReporteIngresosMensual($r)) , '\\excel\\reporte_ingresos_mensual\\' . $nombre_archivo , 'public');
//        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ingresos_mensual\\' . $nombre_archivo);

    }

    public function updated_sucursal_facturada()
    {
        /*
         * nuevo campo para order header, sucursal_facturada integer
            luego buscar en las ordenes,

            si la fk_sucursal = 1 y (n_factura_seniat like '%*%' y es diferente de null) entonces sucursal_facturada=1
            si la fk_sucursal = 1 y (n_factura_seniat no like '%*%' y es diferente de null) entonces sucursal_facturada=6

            si la fk_sucursal = 6 y (n_factura_seniat no like '%*%' y es diferente de null) entonces sucursal_facturada=6
            si la fk_sucursal = 6 y (n_factura_seniat like '%*%' y es diferente de null) entonces sucursal_facturada=1
        */

        //busco todas las ordenes
        $ordenes_1 = OrderHeader::withOnly('estatus')->where('fk_sucursal' , 1)
            ->where('n_factura_seniat' , 'like' , '%*%')
            ->whereNotNull('n_factura_seniat')
            ->update(['sucursal_facturada' => 1]);

        $ordenes_2 = OrderHeader::withOnly('estatus')->where('fk_sucursal' , 1)
            ->where('n_factura_seniat' , 'not like' , '%*%')
            ->whereNotNull('n_factura_seniat')
//            ->get();
            ->update(['sucursal_facturada' => 6]);

        $ordenes_3 = OrderHeader::withOnly('estatus')->where('fk_sucursal' , 6)
            ->where('n_factura_seniat' , 'not like' , '%*%')
            ->whereNotNull('n_factura_seniat')
            ->update(['sucursal_facturada' => 6]);

        $ordenes_4 = OrderHeader::withOnly('estatus')->where('fk_sucursal' , 6)
            ->where('n_factura_seniat' , 'like' , '%*%')
            ->whereNotNull('n_factura_seniat')
            ->update(['sucursal_facturada' => 1]);

        return response()->json($ordenes_2);

    }

    public function actualizar_tiempo_entrega()
    {
        $ordenes_1 = OrderHeader::withOnly('estatus')
            ->where('fk_idTiempoEntrega' , 0)
            ->update(['fk_idTiempoEntrega' => 2]);

        $ordenes_2 = OrderHeader::withOnly('estatus')
            ->where('is_express_delivery' , 1)
            ->update(['fk_idTiempoEntrega' => 1]);

        return response()->json('listo');

    }

    public function mostrar_en_punto_de_venta()
    {
        $productos = Producto::withOnly('status')
            ->where('fk_tipoModalidad' , 5) //materia prima
            ->where('mostrar_en_punto_de_venta' , 1)
            ->update(['mostrar_en_punto_de_venta' => 0]);

        return response()->json('listo');

    }

    public
    function add_img_subtipo(Request $request)
    {
        $subtipo = SubTipo::find($request->input('idSubTipo'));
        if ($subtipo){
            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\subtipo\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $subtipo->img = $nombre_interno;
                    $subtipo->save();

                    // fin envio sms
                    return response()->json(['msj' => "Se actualizo exitosamente la imágen del sutipo" , 'data' => $subtipo]);
                }
            }
        } else {
            return response()->json(['msj' => "No existe el subtipo"] , 404);
        }

    }

    public function resumen_orden(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
        ] , [
            'idOrderHeader.required' => 'El campo idOrderHeader es obligatorio' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $body         = OrderBody::where('fk_orderHeader' , $request->idOrderHeader)->get();
            $precio_total = 0;
            $nuevo_precio = 0;
            foreach ($body as $key => $item) {
                if ($item->fk_moneda==2){ //si es bolivares
                    //busco la tasa con la fecha de creacion
                    $tasa = TasaMoneda::where('created_at' , 'like' , "%" . $item->created_at->format('Y-m-d') . "%")
                        ->orderby('idTasaMoneda' , 'desc')
                        ->first();
                    if ($tasa){
                        $precio       = $item->precio / $tasa['tasa'];
                        $precio_total += $precio;
                    } else {
                        //si no esta la tasa en la fecha buscada, entonces busco una de dias anteriores
                        $tasa = TasaMoneda::whereDate('created_at' , '<' , $item->created_at->format('Y-m-d'))
                            ->orderby('idTasaMoneda' , 'desc')
                            ->first();
                        if ($tasa){
                            $precio       = $item->precio / $tasa['tasa'];
                            $precio_total += $precio;
                        } else {
                            return response()->json(['msj' => 'La tasa de cambio para la fecha ' . $item->created_at->format('Y-m-d') . ' no existe']);

                        }
                    }


                } else {
                    $precio_total += $item->precio;
                }

            }

            $descuento = 0;
            if ($orden->fk_descuento){
                $descuento = $orden->descuento->valor;
            }
            $precio_con_descuento = $precio_total - ($precio_total * ($descuento / 100));
            $descuento_total      = ($precio_total * ($descuento / 100));

            /*Buscando los tiempos de entrega*/
            $tiempo                   = TiemposDeEntrega::where('idTiempoEntrega' , $orden->fk_idTiempoEntrega)->get();
            $total_por_tiempo_entrega = 0;

            if (count($tiempo) > 0){
                if ($orden->fk_idTiempoEntrega==1){
                    $nuevo_precio = ($precio_con_descuento * (($tiempo[0]->percentaje / 100) + 1)); //se incrementa el porcentaje asignado
                } elseif ($orden->fk_idTiempoEntrega==3) {
                    $nuevo_precio = ($precio_con_descuento - (($tiempo[0]->percentaje / 100) * $precio_con_descuento)); //se incrementa el porcentaje asignado
                } elseif ($orden->fk_idTiempoEntrega==2) {
                    $nuevo_precio = $precio_con_descuento;
                }


            }


            return response()->json(
                [
                    'totalOrden' => round($precio_total , 2) ,
                    'subTotalOrden' => round($precio_con_descuento , 2) ,
                    'totalDescuento' => round($descuento_total , 2) ,
                    'tipo' => ($orden->fk_idTiempoEntrega==1 ? 'Incremento' : ($orden->fk_idTiempoEntrega==2 ? 'Igual' : 'Descuento')) ,
                    'monto_x_tiempo_entrega' => round(($tiempo[0]->sum==0 ? -1 : 1) * ($precio_total * ($tiempo[0]->percentaje / 100)) , 2) ,
                    'total' => round($nuevo_precio , 2)
                ]);
        } else {
            return response()->json(['msj' => 'No se pudo encontrar la orden'] , 404);
        }

    }

    public static function resumen_orden_reusable($idOrderHeader)
    {
        $orden = OrderHeader::find($idOrderHeader);
        if ($orden){
            $body         = OrderBody::where('fk_orderHeader' , $idOrderHeader)->get();
            $precio_total = 0;
            $nuevo_precio = 0;
            foreach ($body as $key => $item) {
                if ($item->fk_moneda==2){ //si es bolivares
                    //busco la tasa con la fecha de creacion
                    $tasa = TasaMoneda::where('created_at' , 'like' , "%" . $item->created_at->format('Y-m-d') . "%")
                        ->orderby('idTasaMoneda' , 'desc')
                        ->first();
                    if ($tasa){
                        $precio       = $item->precio / $tasa['tasa'];
                        $precio_total += $precio;
                    } else {
                        //si no esta la tasa en la fecha buscada, entonces busco una de dias anteriores
                        $tasa = TasaMoneda::whereDate('created_at' , '<' , $item->created_at->format('Y-m-d'))
                            ->orderby('idTasaMoneda' , 'desc')
                            ->first();
                        if ($tasa){
                            $precio       = $item->precio / $tasa['tasa'];
                            $precio_total += $precio;
                        } else {
                            return response()->json(['msj' => 'La tasa de cambio para la fecha ' . $item->created_at->format('Y-m-d') . ' no existe']);

                        }
                    }


                } else {
                    $precio_total += $item->precio;
                }

            }

            $descuento = 0;
            if ($orden->fk_descuento){
                $descuento = $orden->descuento->valor;
            }
            $precio_con_descuento = $precio_total - ($precio_total * ($descuento / 100));
            $descuento_total      = ($precio_total * ($descuento / 100));

            /*Buscando los tiempos de entrega*/
            $tiempo                   = TiemposDeEntrega::where('idTiempoEntrega' , $orden->fk_idTiempoEntrega)->get();
            $total_por_tiempo_entrega = 0;

            if (count($tiempo) > 0){
                if ($orden->fk_idTiempoEntrega==1){
                    $nuevo_precio = ($precio_con_descuento * (($tiempo[0]->percentaje / 100) + 1)); //se incrementa el porcentaje asignado
                } elseif ($orden->fk_idTiempoEntrega==3) {
                    $nuevo_precio = ($precio_con_descuento - (($tiempo[0]->percentaje / 100) * $precio_con_descuento)); //se incrementa el porcentaje asignado
                } elseif ($orden->fk_idTiempoEntrega==2) {
                    $nuevo_precio = $precio_con_descuento;
                }
            }


            return response()->json(
                [
                    'totalOrden' => round($precio_total , 2) ,
                    'subTotalOrden' => round($precio_con_descuento , 2) ,
                    'totalDescuento' => round($descuento_total , 2) ,
                    'tipo' => ($orden->fk_idTiempoEntrega==1 ? 'Incremento' : ($orden->fk_idTiempoEntrega==2 ? 'Igual' : 'Descuento')) ,
                    'monto_x_tiempo_entrega' => round(($tiempo[0]->sum==0 ? -1 : 1) * ($precio_total * ($tiempo[0]->percentaje / 100)) , 2) ,
                    'total' => round($nuevo_precio , 2)
                ]);
        } else {
            return response()->json(['msj' => 'No se pudo encontrar la orden'] , 404);
        }

    }

    public
    function cambiar_orden_a_credito(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
        ] , [
            'idOrderHeader.required' => 'El cliente es obligatorio' ,
        ]);
        $idOrderHeader = $request->input('idOrderHeader');

        $order = OrderHeader::find($idOrderHeader);
        if ($order){
            $order->update(['cuenta_por_cobrar' => true]);
            return response()->json(['msj' => "Órden actualizada con éxito"]);
        } else {
            return response()->json(['msj' => "No se pudo encontrar la órden"] , 404);
        }

    }


    public function cambiar_orden_a_cotizacion(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
        ] , [
            'idOrderHeader.required' => 'El cliente es obligatorio' ,
        ]);
        $idOrderHeader = $request->input('idOrderHeader');

        $order = OrderHeader::find($idOrderHeader);
        if ($order){
            $order->update(['cotizacion' => true]);
            PromocionesController::enviarMailCotizacionNuevo($order->idOrderHeader);
            return response()->json(['msj' => "Órden actualizada con éxito"]);
        } else {
            return response()->json(['msj' => "No se pudo encontrar la órden"] , 404);
        }

    }

    public
    function asignar_sucursal(Request $request)
    {
        $this->validate($request , [
            'fk_sucursal' => 'required' ,
            'idOrderHeader' => 'required' ,
        ] , [
            'fk_sucursal.required' => 'La sucursal es obligatoria' ,
            'idOrderHeader.required' => 'El cliente es obligatorio' ,
        ]);
        $fk_sucursal   = $request->input('fk_sucursal');
        $idOrderHeader = $request->input('idOrderHeader');

        $order = OrderHeader::find($idOrderHeader);
        if ($order){
            $order->update(['fk_sucursal' => $fk_sucursal]);
            return response()->json(['msj' => "Órden actualizada con éxito"]);
        } else {
            return response()->json(['msj' => "No se pudo encontrar la órden"] , 404);
        }

    }

    public
    function cambiar_cliente(Request $request)
    {
        $this->validate($request , [
            'idUser' => 'required' ,
            'idOrderHeader' => 'required' ,
        ] , [
            'idUser.required' => 'El cliente es obligatorio' ,
            'idOrderHeader.required' => 'El cliente es obligatorio' ,
        ]);
        $idUser        = $request->input('idUser');
        $idOrderHeader = $request->input('idOrderHeader');

        $order = OrderHeader::find($idOrderHeader);
        if ($order){
            $order->update(['idUser' => $idUser]);
            return response()->json(['msj' => "Órden actualizada con éxito"]);
        } else {
            return response()->json(['msj' => "No se pudo encontrar la órden asociada al cliente"] , 404);
        }

    }

    public function enviar_notificacion_cobro(Request $request)
    {
        $debug = false;
        $tel   = '04120822226';

        $this->validate($request , [
            'idOrderHeader' => 'required' ,
        ] , [
            'idOrderHeader.required' => 'El campo idOrderHeader es obligatorio' ,
        ]);

        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $n = null;
            if ($debug==true){
                $n = $tel;
            } else {
                $n = $orden->customer->phone;
            }

            $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $request->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $request->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $orden->AbonoTotal = round($totalAbono , 2);
            $orden->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            // envio de sms
            SmsController::sendSMS(new Request([
                "destination" => $n ,
                "message" => "ISLAPIXEL-AVISO DE COBRO ORDEN N#" . $orden->idOrderHeader . " de fecha " . $orden->created_at->format('d-m-Y') . ", tiene una deuda por $" . $orden->DeudaTotal . ". Agradecemos su pago lo antes posible. Para mayor info. 04248257272" ,
//                "message" => "ISLAPIXEL le recuerda que tiene un pago pendiente de la orden: " . $orden->n_order . " por $" . $orden->DeudaTotal . ". Agradecemos su pronto pago" ,
            ]));

            //creando el historial
            $h = new HistorialNotificacionCobroSms([
                'sms' => "ISLAPIXEL - AVISO DE COBRO ORDEN N#" . $orden->idOrderHeader . " de fecha " . $orden->created_at->format('d-m-Y') . ", tiene una deuda por $" . $orden->DeudaTotal . ". Agradecemos su pago lo antes posible. Para mayor info. 04248257272" ,
                'idOrderHeader' => $request->idOrderHeader ,
                'idUserCreador' => $request->user()->id ,
                'idUserNotificado' => $orden->idUser ,
            ]);
            $h->save();
            return response()->json(['msj' => 'Envio de sms correcto']);
        } else {
            return response()->json(['msj' => 'No se pudo encontrar la orden'] , 404);
        }

    }

    public function enviar_notificacion_cobro_por_cliente(Request $request)
    {
        $debug = false;
        $tel   = '04120822226';

        $this->validate($request , [
            'idUser' => 'required' ,
        ] , [
            'idUser.required' => 'El campo idUser es obligatorio' ,
        ]);


        $ordenes = OrderHeader::orderBy('created_at' , 'desc')
            ->where('cuenta_por_cobrar' , true)
            ->where(function ($sql) use ($request){
                $sql->where('idUser' , $request->idUser);
            })
            ->where('fk_status' , '!=' , 10)
            ->get();

        if (count($ordenes) > 0){
            $n = null;
            if ($debug==true){
                $n = $tel;
            } else {
                $n = $ordenes[0]->customer->phone;
            }

            $AbonoTotalDolares   = null;
            $AbonoTotalBolivares = null;
            $deudaTotal          = 0;
            $lista_ordenes       = '';
            //convirtiendo de bs a dolares
            foreach ($ordenes as $key => $orden) {
                $lista_ordenes     .= $orden->idOrderHeader . ', ';
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
                $deudaTotal                += $ordenes[$key]->DeudaTotal;
            }
            // envio de sms
            SmsController::sendSMS(new Request([
                "destination" => $n ,
                "message" => "Islapixel - AVISO DE COBRO PENDIENTE POR $" . abs($deudaTotal) . " correspondientes a las órdenes " . $lista_ordenes . ". Agradecemos su pago lo antes posible." ,
            ]));

            //creando el historial
            $h = new HistorialNotificacionCobroSms([
                'sms' => "Islapixel - AVISO DE COBRO PENDIENTE POR $" . abs($deudaTotal) . " correspondientes a las órdenes " . $lista_ordenes . ". Agradecemos su pago lo antes posible." ,
                'idOrderHeader' => $ordenes[0]->idOrderHeader ,
                'idUserCreador' => $request->user()->id ,
                'idUserNotificado' => $request->idUser ,
            ]);
            $h->save();
            return response()->json(['msj' => 'Envio de sms correcto']);
        } else {
            return response()->json(['msj' => 'No se pudo encontrar la orden'] , 404);
        }

    }

    public
    function ordenes_canceladas(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        $ordenes = OrderHeader::without(
            'interacciones' ,
            'descuento' ,
            'usuario_creador' ,
            'usuario_lector' ,
            'customer' ,
            'zonaEnvio' ,
            'moneda_orderheader' ,
            'estadoOrden' ,
            'sucursal_orden' ,
            'historiaStatus' ,
            'transaccion' ,
            'body'
        )->where('fk_status' , 10)
            ->offset($request->offset)
            ->limit($request->limit)
            ->get();
        return response()->json($ordenes);
    }

    public
    function enviar_notificacion_socket(Request $request)
    {

//        return response()->json($request->all(),404);
        try {
//            $idSucursal=$request->idSucursal;
            $idSucursal = 1;
//            $endpoint   = env('URL_SOCKET_ISLA');
//            $response   = Http::post($endpoint , [
//                'idSucursal' => 'R' . $idSucursal ,
//                'idOrderHeader' => $request->idOrderHeader
//            ]);
            $v1 = 'R' . $idSucursal;
            $v2 = $request->idOrderHeader;

            $ch = curl_init();
            curl_setopt($ch , CURLOPT_URL , env('URL_SOCKET_ISLA'));
            curl_setopt($ch , CURLOPT_POST , 1);
            curl_setopt($ch , CURLOPT_POSTFIELDS , "idSucursal=" . $v1 . "&idOrderHeader=" . $v2);
            curl_setopt($ch , CURLOPT_RETURNTRANSFER , true);
            $server_output = curl_exec($ch);
            curl_close($ch);

            return response()->json(['msj' => 'Envio a socket correcto']);
        } catch (Exception $e) {
            Log::error('Ha ocurrido un error al enviar notificacion a socket: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            return response()->json(['msj' => 'Fallo al emitir el socket' , 404]);
        }
    }

    public
    function noprintbyorder($idSucursal , $idEnvironment , $idOrder)
    {
        $orden             = OrderHeader::find($idOrder);
        $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
            ->where('fk_moneda' , 1)
            ->where('idOrderHeader' , $idOrder)
            ->sum('monto');

        $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
            ->where('fk_moneda' , 2)
            ->where('idOrderHeader' , $idOrder)
            ->sum(DB::raw('monto/tasa_moneda'));

        //convirtiendo de bs a dolares
        $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
//        $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
        $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

        $orden = $this->diasDeEntregaSoloUno($orden);

        $abonoTotal = round($totalAbono , 2);
        $deudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);

        $header = [
            "asesor" => $orden->idUsuario ? $orden->usuario_lector->fullName : null ,
            "abonoTotal" => $abonoTotal ,
            "deudaTotal" => $deudaTotal ,
            "idOrderH" => $orden->idOrderHeader ,
            "idTableFk" => 0 ,
            "idOldTableFk" => null ,
            "numberOrden" => $orden->n_order ,
            "idUserOpenFk" => $orden->idUser ,
            "dateTranfer" => null ,
            "idClientFk" => null ,
            "idBranchFk" => 1 ,
            "idStatusOrderFk" => $orden->fk_status ,
            "discount" => null ,
            "comments" => $orden->notes ,
            "mpCash" => null ,
            "mpCreditCard" => null ,
            "mpDebitCard" => null ,
            "mpTranferBack" => null ,
            "mpMpago" => null ,
            "mpOtro" => 0 ,
            "titleOtro" => "" ,
            "idCurrencyFk" => $orden->fk_moneda ,
            "address" => @$orden->customer->direccion ,
            "addressLat" => null ,
            "addressLong" => null ,
            "phoneClient" => null ,
            "nameClient" => null ,
            "mailClient" => null ,
            "dateCancel" => @$orden->date_cancel ,
            "fechaEntrega" => @$orden->date_delivery ,
            "id_tipo_comprobanteFk" => null ,
            "id_cod_docFk" => null ,
            "id_tipo_monedaFk" => $orden->fk_moneda ,
            "n_comprobante" => "" ,
            "clouseCurrentAcount" => null ,
            "isacountCourrient" => 0 ,
            "deliveryEnTienda" => "0" ,
            "deliveryEnLocal" => "0" ,
            "deliveryExterno" => "0" ,
            "consumoEnLocal" => "0" ,
            "deliveryName" => null ,
            "isPriority" => null ,
            "mpOk" => null ,
            "fullNameClient" => $orden->customer->fullName ,
            "responseAfip" => null ,
            "cuitIngresosBrutos" => null ,
            "nameRestaurant" => "" ,
            "dateInitAfip" => null ,
            "facturaAfip" => null ,
            "totalBot" => $orden->precio_total ,
            "isPrint" => $orden->isPrint ,
            "isCocina" => null ,
            "moso" => "" ,
            "mpRappi" => null ,
            "mpGlovo" => null ,
            "mpUber" => null ,
            "mpPedidosya" => null ,
            "mpJust" => null ,
            "mpWabi" => null ,
            "mpOtro2" => null ,
            "created_at" => $orden->created_at ,
            "updated_at" => $orden->updated_at ,
            "isPrintBillin" => $orden->isPrint ,
            "idDeliveryExt" => null ,
            "imageBank" => null ,
            "isPayOk" => null ,
            "isDeliveryOk" => null ,
            "puntoVtaAfip" => null ,
            "idMp" => null ,
            "comprobanteAfit" => null ,
            "idEnvironmentFk" => 1 ,
            "sucursal" => @$orden->sucursal_orden->nombre ,
            "mpPersonal" => null ,
            "mpPedidosyacash" => null ,
            "mpPresent" => null ,
            "mpRapicash" => null ,
            "waste" => null ,
            "envio" => null ,
            "datePrepared" => null ,
            "dateDriver" => null ,
            "dateDeleted" => null ,
            "dateReady" => null ,
            "mpYumi" => null ,
            "mpPaypal" => null ,
            "mpZelle" => null ,
            "mpBofa" => null ,
            "isFromWeb" => null ,
            "commissionPercentaje" => null ,
            "discountAmount" => @$orden->descuento->valor ,
            "idStatusOrder" => $orden->fk_status ,
            "statusOrder" => $orden->estatus->nombre
        ];
        $body   = [];
        foreach ($orden->body as $item) {
            array_push($body , [
                "elementos" => $item->elementos ,
                "servicios" => $item->servicios ,
                "idOrderB" => $item->idOrderBody ,
                "idOrderHFk" => $orden->idOrderHeader ,
                "idProductFk" => $item->idProduct ,
                "quantityProduct" => $item->cantidad ,
                "idStatusFk" => $item->fk_status ,
                "idUserAddFk" => 0 ,
                "priceProductOrder" => $item->precio ,
                "observation" => $item->especificaciones_del_producto ,
                "idProduct2" => 0 ,
                "idProduct1" => 0 ,
                "created_at" => $item->created_at ,
                "updated_at" => $item->updated_at ,
                "weight" => 0 ,
                "isPrintProduct" => 0 ,
                "idProduct" => $item->idProduct ,
                "idProductFamilyFk" => 0 ,
                "idProductSubFamilyFk" => 0 ,
                "nameProduct" => $item->producto->name ,
                "alto" => round($item->alto * 100 , 2) . " cm" ,
                "ancho" => round($item->ancho * 100 , 2) . " cm" ,
                "apply_inventory" => null ,
                "minStock" => 0 ,
                "maxProducVenta" => 0 ,
                "is5050" => "0" ,
                "isPromo" => "0" ,
                "efectivo" => "0" ,
                "linkPago" => "1" ,
                "pricePurchase" => 0 ,
                "priceSale" => $item->producto->precio ,
                "idUnitMeasurePurchaseFk" => 0 ,
                "idUnitMeasureSaleFk" => 0 ,
                "urlImagenProduct" => "" ,
                "idSucursalFk" => 0 ,
                "idTypeProductFk" => 1 ,
                "maxAditionals" => 0 ,
                "minAditionals" => 0 ,
                "marketPrice" => null ,
                "percentageOfProfit" => null ,
                "isheavy" => 1 ,
                "nameKitchen" => null ,
                "idAdicionalCategoryFk" => null ,
                "barCode" => null ,
                "tax" => null ,
                "nameProductP1" => null ,
                "nameProductP2" => null ,
                "printer" => ""
            ]);

        }
        $header['body'] = $body;

        $response = [
            'status' => "ok" ,
            "data" => $header
        ];

        return response()->json($response);
    }

    public
    function isprint($idOrder)
    {
        $orden = OrderHeader::find($idOrder);
        $orden->update(['isPrint' => true]);
        $response = [
            'status' => "ok" ,
            "data" => $orden->refresh()
        ];

        return response()->json($response);
    }

    public
    function obtenercomprobantepago($idOrdenHeader)
    {
        //desencriptando
        $order = OrderHeader::find($idOrdenHeader);
        if ($order){
            $ruta_publica = Storage::disk('public')->get('\\comprobantepago\\' . $order->img_pago);
            return response($ruta_publica)->header('Content-type' , 'image/png');
        } else {
            return response()->json(['msj' => 'El comprobante no existe']);
        }
    }

    public
    function actualizar_descuento_en_orden(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'fk_descuento' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'fk_descuento.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update([
                'fk_descuento' => $request->fk_descuento ,
                'precio_total' => $request->precio_total
            ]);
            return response()->json(['msj' => "Se actualizó el descuento de la órden correctamente"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public
    function cambiar_zona_envio_orden(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'fk_zonaEnvio' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'fk_zonaEnvio.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update(['fk_zonaEnvio' => $request->fk_zonaEnvio]);
            $orden->save();
            return response()->json(['msj' => "Se actualizó la zona de envio de la orden correctamente"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public
    function add_img_orden(Request $request)
    {
        $orden = OrderHeader::find($request->input('idOrderHeader'));
        if ($orden){
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $orden->img_pago = $nombre_interno;
                    $orden->save();

                    // envio de sms
                    SmsController::sendSMS(new Request([
                        "destination" => env('NUMERO_ENVIO_SMS_ISLA_ANAMARIA') ,
                        "message" => 'Comprobante ' . $orden->n_order . ', del cliente ' . strtoupper(User::find($orden->idUser)->fullName) . ', ' . strtolower('https://islapixel.shop/islapixel-api/public/api/obtenercomprobantepago/' . $orden->idOrderHeader) ,
                    ]));
                    // fin envio sms
                    return response()->json(['msj' => "Se actualizo exitosamente la orden" , 'data' => $orden]);
                }
            }
        } else {
            return response()->json(['msj' => "No existe la orden"] , 404);
        }

    }

    public function quitar_n_seniat_orden(Request $request)
    {
        $orden = OrderHeader::find($request->input('idOrderHeader'));
        if ($orden && !is_null($orden->n_factura_seniat)){
            HistoriaRegistroSeniatOrden::create([
                'idOrderHeader' => @$request->input('idOrderHeader') ,
                'n_factura_seniat' => $orden->n_factura_seniat ,
                'idUserCreated' => @$request->input('idUserCreated') ,
                'motivo' => @$request->input('motivo') ,
            ]);
            $orden->update([
                'n_factura_seniat' => null
            ]);
            return response()->json(['msj' => 'Orden actualizada correctamente']);
        } else {
            return response()->json(['msj' => 'No se encontró la orden'] , 404);
        }
    }

    public
    function listar_sucursales_y_usuarios()
    {
//        $sucursales = Sucursal::get();
//        $user       = User::get();
    }

    public
    function cambiar_cotizacion_a_orden(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'precio_total' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'precio_total.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        Log::info('La orden idOrderHeader: ' . $request->idOrderHeader . ', con monto: ' . $orden->precio_total);
        Log::info('Ahora la orden tiene este nuevo precio: ' . $request->precio_total);
        if ($orden){
            $orden->update([
                'cotizacion' => false ,
                'precio_total' => $request->precio_total ,
            ]);
            $orden->save();
            return response()->json(['msj' => "Se correctamente"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public
    function actualizar_nota_de_credito(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'nota_de_credito' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'nota_de_credito.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update(['nota_de_credito' => $request->nota_de_credito]);
            $orden->save();
            return response()->json(['msj' => "Se actualizó nota_de_credito correctamente"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public
    function actualizar_comprobante_factura(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'comprobante_factura' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'comprobante_factura.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update(['comprobante_factura' => $request->comprobante_factura]);
            $orden->save();
            return response()->json(['msj' => "Se actualizó nota_de_credito correctamente"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public
    function actualizar_campo_entrega_orden(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'a_quien_entregan_orden' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'a_quien_entregan_orden.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update(['a_quien_entregan_orden' => $request->a_quien_entregan_orden]);
            $orden->save();
            return response()->json(['msj' => "Se actualizó a_quien_entregan_orden correctamente"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public
    function cambiar_cuenta_por_cobrar(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'cuenta_por_cobrar' => 'required|boolean' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'cuenta_por_cobrar.required' => ':attribute es requerido' ,
            'cuenta_por_cobrar.boolean' => ':attribute debe ser 1 o 0 boolean' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update(['cuenta_por_cobrar' => (boolean)$request->cuenta_por_cobrar]);
            $orden->save();
            return response()->json(['msj' => "Se cambio correctamente a cuenta por cobra"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }


    public
    function updateWHD(Request $request)
    {
        $this->validate($request , [
            'idOrderBody' => 'required' ,
            'ancho' => 'required' ,
            'alto' => 'required' ,
            'descripcion' => 'required'
        ] , [
            'idOrderBody.required' => ':attribute es requerido' ,
            'ancho.required' => ':attribute es requerido' ,
            'alto.required' => ':attribute es requerido' ,
            'descripcion.required' => ':attribute es requerido'
        ]);
        $orden = OrderBody::find($request->idOrderBody);
        if ($orden){
            $orden->update(['ancho' => $request->ancho , 'alto' => $request->alto , 'especificaciones_del_producto' => $request->descripcion]);
            $orden->save();
            return response()->json(['msj' => "Se actualizo exitosamente el producto"]);
        } else {
            return response()->json(['msj' => "No existe la orden"]);
        }
    }

    public function actualizarfacturaseniat(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'n_factura_seniat' => 'required' ,
            'sucursal_facturada' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'n_factura_seniat.required' => ':attribute es requerido' ,
            'sucursal_facturada.required' => ':attribute es requerido' ,
        ]);
        $orden = OrderHeader::find($request->idOrderHeader);
        if ($orden){
            $orden->update([
                'n_factura_seniat' => $request->n_factura_seniat ,
                'sucursal_facturada' => $request->sucursal_facturada ,
                'fecha_facturacion' => Carbon::now()->format('Y-m-d H:i:s')
            ]);
            $orden->save();
            return response()->json(['msj' => 'Se ha actualizado el Nº de factura seniat correctamente']);
        } else {
            return response()->json(['msj' => 'No se encontro la orden'] , 404);
        }
    }

    public function convertir_cotizacion_en_orden(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'idPersonal' => 'required' ,
            'enviar_a_taller' => 'required' ,
            'fk_status_header' => 'required' ,
            'fk_status_body' => 'required' ,
            'precio_total' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'idPersonal.required' => ':attribute es requerido' ,
            'enviar_a_taller.required' => ':attribute es requerido' ,
            'fk_status_header.required' => ':attribute es requerido' ,
            'fk_status_body.required' => ':attribute es requerido' ,
            'precio_total.required' => ':attribute es requerido' ,
        ]);
        $status_body = $request->fk_status_body;

        $orden = OrderHeader::find($request->idOrderHeader);
        if ($status_body==4){
            $orden->body()->update([
                'fk_status' => 1 ,
                'idPersonal' => $request->idPersonal ,
            ]);
            $status_body = 1;
        } else {
            $orden->body()->update([
                'fk_status' => $status_body ,
                'idPersonal' => $request->idPersonal ,
            ]);
        }
        $orden->save();

        $this->cambiarStatusHeader(new Request([
            "idOrderHeader" => $request->idOrderHeader ,
            "idStatusOrden" => $status_body ,
            "idUser" => $request->idUser
        ]));
//        return response()->json($orden->body);
//        exit();
        $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
        foreach ($orden->body as $body) {
            $ee = new OrderBodyController();
            $ee->cambiarStatusBody(new Request([
                "idOrderBody" => $body->idOrderBody ,
                "idStatusOrden" => $status_body ,
                "idUser" => $request->idUser ,
            ]));
        }


        if ($orden){
            //calcular los dias de entrega nuevamente
            $body                  = $orden->body;
            $cantidad_de_productos = count($body);
            $cantidad              = 3;
            if ($cantidad_de_productos >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                $cantidad = 4;
            }

            $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($cantidad);

            $orden->update([
                'cotizacion' => false ,
                'enviar_a_taller' => (boolean)$request->enviar_a_taller ,
                'fk_status' => 1 ,
                'precio_total' => $request->precio_total ,
                'date_delivery' => $fecha_de_entrega ,
                'created_at' => now() ,
            ]);

            //si la tasa de la orden es menor que la tasa actual, se actualiza a la tasa actual
            if (@$orden->tasa_moneda < $tasa['tasa']){
                $orden->update(['tasa_moneda' => $tasa['tasa']]);
            }

            //buscando la caja activa  de la sucursal
            $caja = Caja::where('idSucursal' , $orden->fk_sucursal)->active()->get();
//            dd($caja);
            if (count($caja)==0){
                DB::rollback();
                return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
            }
            $caja = $caja[0];


            //creo la transaccion
            if (count($request->formas_pago) > 0){

                foreach ($request->formas_pago as $forma_pago) {
//              return response()->json($forma_pago);
                    $operacion = new Operacion();
                    if ($forma_pago['idTipoTransaccion']==2){
                        $operacion->descripcion = 'Cambio de la Orden Nº: ' . $orden->n_order;
                    } else {
                        $operacion->descripcion = 'Cierre de Orden Nº: ' . $orden->n_order;
                    }
                    $operacion->monto             = $forma_pago['monto'];
                    $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                    $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                    $operacion->idCaja            = $caja->idCaja;
                    $operacion->idOrderHeader     = $request->idOrderHeader;
                    $operacion->n_order           = $orden->n_order;
                    $operacion->fk_moneda         = $forma_pago['fk_moneda'];
                    $operacion->tasa_moneda       = $tasa['tasa'];
                    $operacion->save();

                }
            }
//            else {
//                $orden->update([
//                    'cuenta_por_cobrar' => true ,
//                ]);
//            }
//            SmsController::sendSMS(new Request([
//                "destination" => $orden->customer->phone ,
//                "message" => 'Tu orden ' . $orden->n_order . ' se encuentra en estatus ' . strtolower($orden->estatus->nombre) ,
//            ]));

            $orden->save();
            return response()->json(['msj' => 'Se ha convertido la cotizacion en orden correctamente']);
        } else {
            return response()->json(['msj' => 'No se encontro la orden'] , 404);
        }
    }


    public
    function close_order_point_sale(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
            'idPersonal' => 'required' ,
            'enviar_a_taller' => 'required' ,
            'fk_status_header' => 'required' ,
            'fk_status_body' => 'required' ,
            'precio_total' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
            'idPersonal.required' => ':attribute es requerido' ,
            'enviar_a_taller.required' => ':attribute es requerido' ,
            'fk_status_header.required' => ':attribute es requerido' ,
            'fk_status_body.required' => ':attribute es requerido' ,
            'precio_total.required' => ':attribute es requerido' ,
        ]);
        $status_body = $request->fk_status_body;

        $orden = OrderHeader::find($request->idOrderHeader);
        if ($status_body==4){
            $orden->body()->update([
                'fk_status' => 2 ,
                'idPersonal' => $request->idPersonal ,
            ]);
            $status_body = 1;
        } else {
            $orden->body()->update([
                'fk_status' => $status_body ,
                'idPersonal' => $request->idPersonal ,
            ]);
        }
        $orden->save();

        $this->cambiarStatusHeader(new Request([
            "idOrderHeader" => $request->idOrderHeader ,
            "idStatusOrden" => $status_body ,
            "idUser" => $request->idUser
        ]));
//        return response()->json($orden->body);
//        exit();
        $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
        foreach ($orden->body as $body) {
            $ee = new OrderBodyController();
            $ee->cambiarStatusBody(new Request([
                "idOrderBody" => $body->idOrderBody ,
                "idStatusOrden" => $status_body ,
                "idUser" => $request->idUser ,
            ]));
        }


        if ($orden){
            //calcular los dias de entrega nuevamente
            $body                  = $orden->body;
            $cantidad_de_productos = count($body);
            $cantidad              = 3;
            if ($cantidad_de_productos >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                $cantidad = 4;
            }

            $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($cantidad);

            $orden->update([
                'cotizacion' => false ,
                'enviar_a_taller' => (boolean)$request->enviar_a_taller ,
                'fk_status' => 1 ,
                'precio_total' => $request->precio_total ,
                'date_delivery' => $fecha_de_entrega ,
                'created_at' => now() ,
            ]);

            //si la tasa de la orden es menor que la tasa actual, se actualiza a la tasa actual
            if (@$orden->tasa_moneda < $tasa['tasa']){
                $orden->update(['tasa_moneda' => $tasa['tasa']]);
            }

            //buscando la caja activa  de la sucursal
            $caja = Caja::where('idSucursal' , $orden->fk_sucursal)->active()->get();
//            dd($caja);
            if (count($caja)==0){
                DB::rollback();
                return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
            }
            $caja = $caja[0];


            //creo la transaccion
            if (count($request->formas_pago) > 0){

                foreach ($request->formas_pago as $forma_pago) {
//              return response()->json($forma_pago);
                    $operacion = new Operacion();
                    if ($forma_pago['idTipoTransaccion']==2){
                        $operacion->descripcion = 'Cambio de la Orden Nº: ' . $orden->n_order;
                    } else {
                        $operacion->descripcion = 'Cierre de Orden Nº: ' . $orden->n_order;
                    }
                    $operacion->monto             = $forma_pago['monto'];
                    $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                    $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                    $operacion->idCaja            = $caja->idCaja;
                    $operacion->idOrderHeader     = $request->idOrderHeader;
                    $operacion->n_order           = $orden->n_order;
                    $operacion->fk_moneda         = $forma_pago['fk_moneda'];
                    $operacion->tasa_moneda       = $tasa['tasa'];
                    $operacion->save();

                }
            }
//            else {
//                $orden->update([
//                    'cuenta_por_cobrar' => true ,
//                ]);
//            }
//            SmsController::sendSMS(new Request([
//                "destination" => $orden->customer->phone ,
//                "message" => 'Tu orden ' . $orden->n_order . ' se encuentra en estatus ' . strtolower($orden->estatus->nombre) ,
//            ]));

            $orden->save();
            return response()->json(['msj' => 'Se ha convertido la cotizacion en orden correctamente']);
        } else {
            return response()->json(['msj' => 'No se encontro la orden'] , 404);
        }
    }

    public
    function cambiar_cuenta_por_cobrar_false(Request $request)
    {
        $this->validate($request , [
            'idOrderHeader' => 'required' ,
        ] , [
            'idOrderHeader.required' => ':attribute es requerido' ,
        ]);

        $orden = OrderHeader::find($request->idOrderHeader);

        if ($orden){
            $orden->update([
                'cuenta_por_cobrar' => false
            ]);
//            SmsController::sendSMS(new Request([
//                "destination" => $orden->customer->phone ,
//                "message" => "Islapixel: Gracias tu pago de la orden " . $orden->n_order . " fue verificado exitosamente" ,
//            ]));
            $orden->save();
            return response()->json(['msj' => 'Se ha actualizado cuenta_por_cobrar de la orden correctamente']);
        } else {
            return response()->json(['msj' => 'No se encontro la orden'] , 404);
        }
    }

    public
    function OrdenesNoleidas()
    {
        $oh = count(OrderHeader::noLeido()->get());

        return response()->json(['cant_ordenes_no_leidas' => $oh]);
    }

    public
    function orden_cambio_a_taller(Request $request)
    {

        /*
         * Api que reciba el idOrdenHeader, id status, que va actualizar el campo enviar_a_taller a true y cambia el status de los productos a en  proceso
         */

        $idOrderHeader = $request->idOrderHeader;
//        $idOrderBody   = $request->idOrderBody;
        $header   = OrderHeader::find($idOrderHeader);
        $cantidad = 3;
        if (count($header->body) >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
            $cantidad = 4;
        }
        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($cantidad);
        $header->update([
            'fk_status' => 2 ,
            'enviar_a_taller' => true ,
            'date_delivery' => $fecha_de_entrega
        ]);

        $body = OrderBody::where('fk_orderHeader' , $idOrderHeader)->get();
        $body->update(['fk_status' => 2]);


        return response()->json('Cambiado exitosamente');
    }


    public
    function leer(Request $request)
    {
        $idUsuario     = $request->idUsuario;
        $esta_leido    = (boolean)$request->esta_leido;
        $fecha_leido   = now();
        $idOrderHeader = $request->idOrderHeader;

        $oh = OrderHeader::find($idOrderHeader);
//        return response()->json($oh);
        if ($oh){
            $oh->update([
                'idUsuario' => $idUsuario ,
                'esta_leido' => $esta_leido ,
                'fecha_leido' => $fecha_leido ,
            ]);
            return response()->json(['msj' => 'Se leyó correctamente el pedido']);
        } else {
            return response()->json(['msj' => 'No se pudo leer correctamente el pedido'] , 404);
        }

    }

    public
    function cambiarStatusHeader(Request $request)
    {
        $idOrderHeader = $request->idOrderHeader;
        $idStatusOrden = $request->idStatusOrden;
        $idUser        = $request->idUser;
        $seguir        = [];
        $estadoOrden;
        if ($estadoOrden = EstatusOrden::find($idStatusOrden)){
            $seguir['status'] = true;
        } else {
            $seguir['status']  = false;
            $seguir['error'][] = 'No existe el estatus';
        }
        if (User::find($idUser)){
            $seguir['status'] = true;
        } else {
            $seguir['status']  = false;
            $seguir['error'][] = 'No existe el usuario';
        }

        $header = OrderHeader::find($idOrderHeader);
        if ($header){
            $seguir['status'] = true;
        } else {
            $seguir['status']  = false;
            $seguir['error'][] = 'No existe la Orden';
        }
        if ($header->fk_status==10){
            return response()->json(['msj' => "No puede cambiar el estado de esta orden, la misma esta cancelada. "] , 404);
        }
        if ($seguir['status']){

            if ($idStatusOrden==2){

                $cantidad = 3;
                if (count($header->body) >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                    $cantidad = 4;
                }
                $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($cantidad);

                $header->update([
                    'fk_status' => $idStatusOrden ,
                    'date_delivery' => $fecha_de_entrega
                ]);
            } else {
                if ($idStatusOrden==4){
                    if ($header->cuenta_por_cobrar==1 || count($header->transaccion) > 0){
                        $header->update([
                            'fk_status' => $idStatusOrden
                        ]);
                    } else {
                        return response(['msj' => 'No se puede cambiar a estatus entregado ya que no ha sido confirmada la orden'] , 404);
                    }
                } else {
                    $header->update([
                        'fk_status' => $idStatusOrden
                    ]);
                }
            }
            /*creo la historia del status*/
            $historia = new HistoriaStatusOH($request->all());
            $historia->save();

            /*Envio mail de notificacion*/
            try {
                if ($estadoOrden->notificar_cliente){ //valido si el estado esta activo para notificar
                    if ($idStatusOrden==3){
                        SmsController::sendSMS(new Request([
                            "destination" => $header->customer->phone ,
//                            "message" => 'Tu orden en islapixel ' . $header->n_order . ' se encuentra en estatus ' . strtolower($estadoOrden->nombre) ,
                            "message" => 'ISLAPIXEL-ORDEN LISTA PARA RETIRAR,' . $header->n_order . ' está en la sucursal ' . strtoupper(substr($header->sucursal_orden->nombre , 10)) . '.Retirar de Lun.a Vie.de 8:30am a 4:30pm. SÁB.Y DOM.NO REALIZAMOS ENTREGAS.'
                        ]));
                    } else {
                        SmsController::sendSMS(new Request([
                            "destination" => $header->customer->phone ,
                            "message" => 'Tu orden en islapixel ' . $header->n_order . ' se encuentra en estatus ' . strtolower($estadoOrden->nombre) ,
                        ]));
                    } //entregada


//                    return response()->json($header);
                    $string     = $header->customer->email;
                    $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
                    $resultado  = stristr($string , 'generico');
                    $resultado2 = stristr($string , 'noposee');
                    if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
                    } else {
                        Mail::to($header->customer->email)->send(new NotificacionStatusOrdenMail($idOrderHeader , $header->n_order , $estadoOrden->nombre , $header));
                    }
                }
            } catch (\Exception $e) {
                Log::error('OrderHeaderController funcion:cambiarStatusHeader, No se pudo enviar el correo al cambiar el estatus: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            }
            return response()->json('Se cambió correctamente el estado del pedido');
        } else {
            return response(['msj' => $seguir['error']] , 404);
        }
    }

    public
    function findParametros(Request $request)
    {
        $parametro = $request->parametro;
        $todas     = OrderHeader::where(function ($query) use ($parametro){
            $query->whereHas('body' , function ($query2) use ($parametro){
                $query2->whereHas('producto' , function ($query3) use ($parametro){
                    $query3->where('name' , 'like' , '%' . $parametro . '%');
                    $query3->orWhere('descripcion' , 'like' , '%' . $parametro . '%');
                });
            });
            $query->orWhere('notes' , 'like' , '%' . $parametro . '%');
//            $query->where('note' , 'like' , '%' . $parametro . '%');

        })->orderBy('created_at' , 'desc')->get();

        return response()->json($this->diasDeEntrega($todas));
    }

    public
    function index(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $todas = OrderHeader::orderBy('created_at' , 'desc')
                ->where('cotizacion' , "!=" , true)
                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();
//
            return response()->json($this->diasDeEntrega($todas));
        } else {
            $todas = OrderHeader::without('body' , 'moneda_orderheader' , 'transaccion')
                ->orderBy('created_at' , 'DESC')
                ->where('cotizacion' , "!=" , true)
                ->where('fk_status' , 2)
                ->get();

            return response()->json($this->diasDeEntrega($todas));
        }
    }

    public
    function refrescar_fechas_de_entrega()
    {
        $todas = OrderHeader::without('moneda_orderheader' , 'transaccion')
            ->orderBy('created_at' , 'asc')
            ->where('cotizacion' , "!=" , true)
            ->where('fk_status' , 2)
            ->get();

        return response()->json($this->diasDeEntregaActualizar($todas));
    }


//aplica solo para actualizar los date_delivery
    public
    function diasDeEntregaActualizar($ordenes)
    {
        $todas             = $ordenes;
        $fechas_reservadas = collect();
        $probables         = [];
        foreach ($todas as $key => $toda) {
            $cantidad = 3;
            //calculo la cantidad de dias de entrega para la orden

            if (isset($toda['body'])){

                $cantidad_de_productos = count($toda['body']);
                if ($cantidad_de_productos >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
                    $cantidad = 4;
                }

                $todas[$key]->dias_entre_adicionales = $cantidad;

                $fecha_probable = null;
                $probable       = null;

                $fecha_probable = Carbon::create($todas[$key]->created_at)->addDay($cantidad)->format('Y-m-d'); //establezco la fecha inicial, que es la de creacion de la orden mas los 3 dias
//                dd($todas[$key]->created_at,$fecha_probable);
                //valido si la fecha probable es un dia disponible, si no me retorna el proximo
                $fecha_probable = FechaDeNoEntregaController::parametro_($fecha_probable);
//                dd($todas[$key]->created_at,$fecha_probable);
                if (count($fechas_reservadas)==0){ //si es la primera fecha reservada pues asigno la fecha
                    $fechas_reservadas->push(['idOrder' => $toda->idOrderHeader , 'fecha' => $fecha_probable]);
                    $todas[$key]->fecha_entrega = $fecha_probable;
                    OrderHeader::find($todas[$key]->idOrderHeader)->update(['date_delivery' => $fecha_probable]);
                } else {
                    $salida = true;
                    do {
                        $cantidad_reservaciones_para_el_dia = $fechas_reservadas->where('fecha' , '=' , $fecha_probable);
                        if (count($cantidad_reservaciones_para_el_dia) >= 30){
                            //aqui tengo que agregar otro dia y validarlo contra los dias no disponibles
                            $fecha_probable1 = Carbon::create($fecha_probable)->addDay(1)->format('Y-m-d');
                            $fecha_probable  = Carbon::create(FechaDeNoEntregaController::parametro_($fecha_probable1))->format('Y-m-d');
//                            array_push($probables , ['fecha' => $fecha_probable1 , 'fecha_disponible' => $fecha_probable]);

                            $salida = true; //para que entre de nuevo al ciclo
                        } else {
                            //agrego la fecha a la coleccion
                            $fechas_reservadas->push(['idOrder' => $toda->idOrderHeader , 'fecha' => $fecha_probable]);
                            $todas[$key]->fecha_entrega = $fecha_probable;
                            OrderHeader::find($todas[$key]->idOrderHeader)->update(['date_delivery' => $fecha_probable]);
                            $salida = false; //para ya salir
                        }
                    } while ($salida);
                }


            } else {
                $todas[$key]->dias_entre_adicionales = 0;
                $todas[$key]->fecha_entrega          = FechaDeNoEntregaController::parametro_($todas[$key]->created_at->format('Y-m-d'));
                OrderHeader::find($todas[$key]->idOrderHeader)->update(['date_delivery' => FechaDeNoEntregaController::parametro_($todas[$key]->created_at->format('Y-m-d'))]);
            }

        }
//        return response()->json($probables , 404);
        return $todas;
    }

    public
    function diasDeEntrega($ordenes)
    {
        $todas             = $ordenes;
        $fechas_reservadas = collect();
        $probables         = [];
        foreach ($todas as $key => $toda) {
            $todas[$key]->fecha_entrega = Carbon::create($todas[$key]->date_delivery)->format('Y-m-d');
        }
        return $todas;
    }

    public
    function diasDeEntregaSoloUno($orden)
    {
        $toda                = $orden;
        $toda->fecha_entrega = @Carbon::create($toda->date_delivery)->format('Y-m-d');
        return $toda;
    }

    public
    function ordenHeader_listar_con_buscador2(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit')){
            $limit  = 10;
            $offset = 0;
            if (is_null($request->limit)){
                $limit = 15;
            } else {
                $limit = $request->limit;
            }
            if (is_null($request->offset)){
                $offset = 15;
            } else {
                $offset = $request->offset;
            }
            $ordenes             = OrderHeader::orderBy('created_at' , 'desc')
                ->where('fk_status' , '!=' , '16')
                ->where('idUsuario' , $request->idUsuario)
                ->offset($offset)
                ->limit($limit)
                ->get();
            $AbonoTotalDolares   = null;
            $AbonoTotalBolivares = null;
            //convirtiendo de bs a dolares
            $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            foreach ($ordenes as $key => $orden) {
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            }
            $total = OrderHeader::orderBy('created_at' , 'desc')
                ->where('fk_status' , '!=' , '16')
                ->where('idUsuario' , $request->idUsuario)
                ->count();

            return response()->json(["data" => $this->diasDeEntrega($ordenes) , 'total' => $total]);
        }
    }


    public
    function orders_drafts_lists(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit')){
            $limit  = 10;
            $offset = 0;
            if (is_null($request->limit)){
                $limit = 15;
            } else {
                $limit = $request->limit;
            }
            if (is_null($request->offset)){
                $offset = 15;
            } else {
                $offset = $request->offset;
            }
            $ordenes             = OrderHeader::orderBy('created_at' , 'desc')
                ->where('fk_status' , '16')
                ->where('idUsuario' , $request->idUsuario)
                ->offset($offset)
                ->limit($limit)
                ->get();
            $AbonoTotalDolares   = null;
            $AbonoTotalBolivares = null;
            //convirtiendo de bs a dolares
            $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            foreach ($ordenes as $key => $orden) {
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            }
            $total = OrderHeader::orderBy('created_at' , 'desc')
                ->where('fk_status' , '16')
                ->where('idUsuario' , $request->idUsuario)
                ->count();

            return response()->json(["data" => $this->diasDeEntrega($ordenes) , 'total' => $total]);
        }
    }


    public
    function ordenHeader_listar_con_buscador(Request $request)
    {
//        return response()->json('asd');
        // debe buscar por nombre del cliente customer
        // n de telefono
        //correo electronico
        // n_pedido
        // n_documento

        if ($request->exists('offset') && $request->exists('limit')){
//            $this->validate($request , [
//                'offset' => 'integer|min:0' ,
//                'limit' => 'integer|min:0' ,
//            ] , [
//                'offset.integer' => 'Debe ser numérico' ,
//                'limit.integer' => 'Debe ser numérico' ,
//
//                'offset.min' => 'Debe tener al menos un número' ,
//                'limit.min' => 'Debe tener al menos un número' ,
//            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $todas = OrderHeader::orderBy('created_at' , 'desc')
                ->where('cotizacion' , "!=" , true)
                ->where('punto_de_venta' , false)
                ->where(function ($query) use ($request , $desde , $hasta){
                    if ($request->buscar!="null"){
                        $query->whereHas('customer' , function ($query2) use ($request){
                            $query2->where('fullName' , 'like' , "%" . $request->buscar . "%");
                            $query2->orwhere('phone' , 'like' , "%" . $request->buscar . "%");
                            $query2->orwhere('email' , 'like' , "%" . $request->buscar . "%");
                            $query2->orwhere('numerIdentificacion' , 'like' , "%" . $request->buscar . "%");
                        });
                        $query->orwhere('n_order' , 'like' , "%" . $request->buscar . "%");
                    }
                    if ($request->desde!=0){
                        $query->whereBetween('created_at' , [$desde , $hasta]);
                    }
                    if ($request->idUsuario!=0){
                        $query->where('idUsuario' , $request->idUsuario);
                    }
                })
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();


            return response()->json($this->diasDeEntrega($todas));
        }
    }

    public
    function buscador_general(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit')){
//            $this->validate($request , [
//                'offset' => 'integer|min:0' ,
//                'limit' => 'integer|min:0' ,
//            ] , [
//                'offset.integer' => 'Debe ser numérico' ,
//                'limit.integer' => 'Debe ser numérico' ,
//
//                'offset.min' => 'Debe tener al menos un número' ,
//                'limit.min' => 'Debe tener al menos un número' ,
//            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $todas = OrderHeader::orderBy('created_at' , 'desc')
                ->where(function ($query) use ($request , $desde , $hasta){
                    if ($request->buscar!="null"){
                        $query->whereHas('customer' , function ($query2) use ($request){
                            $query2->where('fullName' , 'like' , "%" . $request->buscar . "%");
                            $query2->orwhere('phone' , 'like' , "%" . $request->buscar . "%");
                            $query2->orwhere('email' , 'like' , "%" . $request->buscar . "%");
                            $query2->orwhere('numerIdentificacion' , 'like' , "%" . $request->buscar . "%");
                        });
                        $query->orwhere('n_order' , 'like' , "%" . $request->buscar . "%");
                    }
                    if ($request->desde!=0){
                        $query->whereBetween('created_at' , [$desde , $hasta]);
                    }
                    if ($request->idUsuario!=0){
                        $query->where('idUsuario' , $request->idUsuario);
                    }
                })
//                ->offset($request->offset)
//                ->limit($request->limit)
                ->get();


            return response()->json($this->diasDeEntrega($todas));
        }
    }

    public
    function buscador_general_2(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,
                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $ordenes = OrderHeader::orderBy('created_at' , 'desc')
                ->where(function ($query) use ($request){
                    if ($request->buscar!="null"){
                        $query->whereHas('customer' , function ($query2) use ($request){
                            $query2->where('fullName' , 'like' , "%" . $request->buscar . "%");
//                            $query2->orwhere('phone' , 'like' , "%" . $request->buscar . "%");
//                            $query2->orwhere('email' , 'like' , "%" . $request->buscar . "%");
//                            $query2->orwhere('numerIdentificacion' , 'like' , "%" . $request->buscar . "%");
                        });
                        $query->orwhere('n_order' , 'like' , "%" . $request->buscar . "%");
                    }

                })
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();

            $AbonoTotalDolares   = null;
            $AbonoTotalBolivares = null;
            //convirtiendo de bs a dolares
            $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            foreach ($ordenes as $key => $orden) {
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            }
            return response()->json($this->diasDeEntrega($ordenes));
        }
    }

    public
    function indexListarParametros(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';
            $todas = null;

            if ($request->order_open==true || $request->enviar_a_taller==true){
                if ($request->order_open==true){
                    $todas = OrderHeader::orderBy('created_at' , 'desc')
                        ->where(function ($query) use ($request , $desde , $hasta){
                            $query->whereHas('body' , function ($query3) use ($request , $desde , $hasta){
                                $query3->whereBetween('created_at' , [$desde , $hasta]);
                            });
                        })
                        ->where('order_open' , true)
                        ->offset($request->offset)
                        ->limit($request->limit)
                        ->get();
                } else {
                    if ($request->enviar_a_taller==true){
                        $todas = OrderHeader::orderBy('created_at' , 'desc')
                            ->where(function ($query) use ($request , $desde , $hasta){
                                $query->whereHas('body' , function ($query3) use ($request , $desde , $hasta){
                                    $query3->whereBetween('created_at' , [$desde , $hasta]);
                                });
                            })
                            ->where('enviar_a_taller' , true)
                            ->offset($request->offset)
                            ->limit($request->limit)
                            ->get();
                    }
                }

            } else {
                $todas = OrderHeader::orderBy('created_at' , 'desc')
                    ->where(function ($query) use ($request , $desde , $hasta){
                        $query->whereHas('body' , function ($query3) use ($request , $desde , $hasta){
                            $query3->whereBetween('created_at' , [$desde , $hasta]);
                            $query3->where('fk_status' , true);
                        });
                    })
                    ->offset($request->offset)
                    ->limit($request->limit)
                    ->get();
            }

            $databody;
            foreach ($this->diasDeEntrega($todas) as $key => $toda) {
                $databody[] = @$toda->body;
            }
            return response()->json(["ordenes" => $databody]);
        }


    }

    public
    function indexListarParametros2(Request $request)  //revisar agregar
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';
            $todas = null;

            if ($request->order_open==true || $request->enviar_a_taller==true){
                if ($request->order_open==true){
                    $todas = OrderBody::with('orderHeader')->orderBy('created_at' , 'desc')
                        ->where(function ($query){
                            $query->whereHas('orderHeader' , function ($query3){
                                $query3->where('order_open' , true);
                                $query3->where('cotizacion' , "!=" , true);
                            });
                        })
                        ->whereBetween('created_at' , [$desde , $hasta])
                        ->offset($request->offset)
                        ->limit($request->limit)
                        ->get();
                } else {
                    if ($request->enviar_a_taller==true){
                        $todas = OrderBody::with('orderHeader')->orderBy('created_at' , 'desc')
                            ->where(function ($query){
                                $query->whereHas('orderHeader' , function ($query3){
                                    $query3->where('enviar_a_taller' , true);
                                    $query3->where('cotizacion' , "!=" , true);
                                });
                            })
                            ->whereBetween('created_at' , [$desde , $hasta])
                            ->offset($request->offset)
                            ->limit($request->limit)
                            ->get();
                    }
                }

            } else {
                $todas = OrderBody::with('orderHeader')->orderBy('created_at' , 'desc')
                    ->where('fk_status' , $request->fk_status)
                    ->where(function ($query){
                        $query->whereHas('orderHeader' , function ($query3){
                            $query3->where('cotizacion' , "!=" , true);
                        });
                    })
//                    ->where(function ($query) use ($request){
//                        if ($request->idUsuario!=0){
//                            $query->where('idUsuario' , $request->idUsuario);
//                        }
//                        if ($request->fk_sucursal!=0){
//                            $query->where('fk_sucursal' , $request->fk_sucursal);
//                        }
//                    })
                    ->whereBetween('created_at' , [$desde , $hasta])
                    ->offset($request->offset)
                    ->limit($request->limit)
                    ->get();
            }
            return response()->json($todas);

        }


    }

    public
    function cod($cod) //para redireccionar la ruta de backend a frontend de manera trnasparente al usuario
    {
        $orden = OrderHeader::where('tocken_url' , $cod)->get();

        if (count($orden) > 0){
            $orden = $orden[0];
            return redirect()->to(env('APP_URL_FRONT') . '/#/detailOrder/' . $orden->idOrderHeader);
        } else {
            return response()->json('No pudimos encontrar la ruta');
        }

    }

    public
    function get_qr($palabra)
    {


//        $pdf = PDF::loadHTML('<h1>Styde.net</h1>');
//
//        return $pdf->download('mi-archivo.pdf');

//        $var = QrCode::generate('Make me into a QrCode!', '\\qrcodes\\qrcode.svg');
//        return $var;
//        $this->qr = asset('storage\\app\\public\\qrcodes\\qrcode.svg'); //server
//        $qr    = asset('storage\\qrcodes\\qrcode.svg'); //local
//        $qr_64 = base64_encode(QrCode::format('png')->size(1000)->generate($palabra));
        $qr_64 = base64_encode(QrCode::format('png')->size(1000)->generate($palabra));
//        dd($this->qr);

//        return view('correos.showStatusPedido2')->with('qr' , $qr);

        return $qr_64;
    }

    public function gereararPdf($idOrden)
    {
        GenerarPdfJob::dispatch($idOrden);
        return ['msj' => 'Se ha enviado correctamente el mail con el qr del producto'];
    }

    public
    function gereararPdf2($idOrden)
    {
        $qr_64 = $this->get_qr(env('APP_URL_FRONT') . "/detailOrder/" . $idOrden);
        return response()->json(['qr' => $qr_64] , 200);
    }

    public
    function gereararPdf3($idOrdenBody)
    {
        $qr_64 = $this->get_qr(env('APP_URL_FRONT') . "/detailOrder/" . $idOrdenBody);
        return response()->json(['qr' => $qr_64] , 200);
    }

    public
    function gereararQRUniversal(Request $request)
    {
        $stri  = $request->palabra;
        $qr_64 = $this->get_qr($stri);
        return response()->json(['qr' => $qr_64] , 200);
    }

    public
    function topPedidos(Request $request)
    {
        $respuesta = null;
        if ($request->todos){
            $respuesta = OrderHeader::where('idUser' , $request->idUser)->get();
        } else {
            $respuesta = OrderHeader::where('idUser' , $request->idUser)->orderBy('created_at' , 'desc')->limit(5)->get();
        }
        return response()->json($respuesta);
    }

    public
    function buscador_pedidos(Request $request)
    {
        $this->validate($request , [
            'offset' => 'required|integer|min:0' ,
            'limit' => 'required|integer|min:0' ,
            'desde' => 'required|date_format:Y-m-d' ,
            'hasta' => 'required|date_format:Y-m-d' ,
//            'palabra' => 'required' ,
            'idUser' => 'required' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,
            'offset.required' => ':attribute es requerido' ,
            'limit.required' => ':attribute es requerido' ,
            'desde.required' => ':attribute es requerido' ,
            'hasta.required' => ':attribute es requerido' ,
//            'palabra.required' => ':attribute es requerido' ,
            'idUser.required' => ':attribute es requerido' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
        $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

        $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
        $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';
        $todas = null;

        if ($request->exists('palabra') && $request->palabra!="null" && $request->palabra!=""){

            $palabra = "%" . $request->palabra . "%";
            $todas   = OrderHeader::orderBy('created_at' , 'desc')
                ->where('idUser' , $request->idUser)
                ->where(function ($query1) use ($palabra){
                    $query1->where('date_created' , 'like' , $palabra);
                    $query1->orwhere('n_order' , 'like' , $palabra);
                    $query1->orwhere(function ($query) use ($palabra){
                        $query->whereHas('body' , function ($query3) use ($palabra){
                            $query3->whereHas('producto' , function ($query4) use ($palabra){
                                $query4->where('name' , 'like' , $palabra);
                            });
                        });
                    });
                })
                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();

        } else {
            $todas = OrderHeader::orderBy('created_at' , 'desc')
                ->where('idUser' , $request->idUser)
                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();

        }

        return response()->json($todas);
    }

    public
    function initcarrito()
    {
        $zona_envio    = ZonaEnvio::active()->get();
        $sucursales    = Sucursal::get();
        $formaDePago   = FormaDePago::get();
        $estatus_orden = EstatusOrden::get();
//        $clientes      = User::where('profile_fk' , 3)->get();
        $descuentos = Descuento::active()->get();
        return response()->json([
            'zonas_envio' => $zona_envio ,
            'sucursales' => $sucursales ,
            'formas_pago' => $formaDePago ,
            'estatus_orden' => $estatus_orden ,
//            'clientes' => $clientes ,
            'descuentos' => $descuentos ,
        ]);
    }

    public
    function porid($idO)
    {
        //cargando elementos, servicios y productos eliminados por carga ansionsa
        $orden = OrderHeader::with([
            'body.producto' => function ($query){
                $query->withTrashed();
            } ,
//            'body.producto.elementos' => function ($query){
//                $query->withTrashed();
//            } ,
//            'body.producto.servicios' => function ($query){
//                $query->withTrashed();
//            } ,
            'body.elementos' => function ($query){
                $query->withTrashed();
            } ,
            'body.elementos.elemento' => function ($query){
                $query->withTrashed();
            } ,
            'body.servicios.servicio' => function ($query){
                $query->withTrashed();
            } ,
            'body.servicios' => function ($query){
                $query->withTrashed();
            }
        ])->find($idO);
        if ($orden){

            $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $idO)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $idO)
                ->sum(DB::raw('monto/tasa_moneda'));

            //convirtiendo de bs a dolares
            $tasa       = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;
            //$totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
            $orden = @$this->diasDeEntregaSoloUno($orden);

            $totalBsConvetidos = null;
            if ($orden->cuenta_por_cobrar==false && $orden->cotizacion==false){
                //Se conviertes las operaciones en dolares equivalenes
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto'));

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

                //total en dolares
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                if ($totalAbono >= $orden->precio_total){
                    //Se conviertes las operaciones en bs equivalenes
                    $AbonoTotalDolaresBs = Operacion::where('idTipoTransaccion' , 1)
                        ->where('fk_moneda' , 1)
                        ->where('idOrderHeader' , $orden->idOrderHeader)
                        ->sum(DB::raw('monto*tasa_moneda'));

                    $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                        ->where('fk_moneda' , 2)
                        ->where('idOrderHeader' , $orden->idOrderHeader)
                        ->sum(DB::raw('monto'));

                    $totalBsConvetidos = round($AbonoTotalBolivares + $AbonoTotalDolaresBs , 2);
                }

            }

            $orden->AbonoTotal        = round($totalAbono , 2);
            $orden->DeudaTotal        = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            $orden->totalBsConvetidos = $totalBsConvetidos;

            return $orden;
        } else {
            return response()->json([
                'msj' => 'No se encuentra la orden' ,
            ] , 404);
        }
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public
    function create()
    {
        //
    }

    /**
     * @param $cantidad
     * @return \Illuminate\Http\JsonResponse|string|null
     */
    public static function retornar_dia_disponible_para_fechaEntrega($cantidad)
    {
        $continuar      = false;
        $fecha_probable = null;
        $probable       = null;

        $fecha_probable = Carbon::now();
        /*funcion para contar x dias minimos de desarrollo y luego pasar a validar los ddemas dias disponibles*/
        for ($i = 0; $i < $cantidad; $i++) {
            $fecha_probable = $fecha_probable->addDay(1);
            $seguir         = false;
            do {
                if (!$fecha_probable->isWeekday()){ //es un sabado o domingo?
                    $seguir         = true;
                    $fecha_probable = $fecha_probable->addDay(); //entonces agrego un dia y valido
                } else {
                    $seguir = false;
                }
            } while ($seguir);
        }
        $fecha_probable = $fecha_probable->format('Y-m-d');
//        $fecha_probable = Carbon::now()->addDay($cantidad)->format('Y-m-d');
        //valido si la fecha probable es un dia disponible, si no me retorna el proximo
        $fecha_probable = FechaDeNoEntregaController::parametro_($fecha_probable);
        do {
            $ordenes   = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->where('fk_status' , 2)
                ->where('date_delivery' , 'like' , '%' . $fecha_probable . '%')
                ->get();
            $cantidad_ = count($ordenes);
            if ($cantidad_ >= 45){
                //se bloquea el dia
//                DiasDeEntregaBloquados::create([
//                    'dia' => $fecha_probable
//                ]);

                //agrego un dia y valido si esta disponible
                $fecha_probable = FechaDeNoEntregaController::parametro_(Carbon::create($fecha_probable)->addDay(1)->format('Y-m-d'));
                $continuar      = true;
            } else {
                $continuar = false;
            }
        } while ($continuar);
        return $fecha_probable;

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */

    public function add_body_extra(Request $request , $idOrderHeader = null)
    {
//        dd($request);
        //obteniendo la orden
        $orden = OrderHeader::find($idOrderHeader);
        if ($orden){
            return OrderBodyController::add_por_body_adicional($request);
        } else {
            return response()->json(['msj' => "No existe la Orden"] , 404);
        }
    }

    public function cambiar_tiempo_entrega(Request $request , $id)
    {
        $request->validate([
            'idOrderHeader' => 'required' ,
            'fk_idTiempoEntrega' => 'required' ,
            'is_express_delivery' => 'required' ,
        ] , [
            'idOrderHeader.required' => 'El campo :attribute es requerido' ,
            'fk_idTiempoEntrega.required' => 'El campo :attribute es requerido' ,
            'is_express_delivery.required' => 'El campo :attribute es requerido' ,
        ]);

        $orden            = OrderHeader::find($request->idOrderHeader);
        $precio           = 0;
        $isUrgente        = false;
        $fecha_de_entrega = null;
        if ($orden){
            if ($request->exists('fk_idTiempoEntrega')){
                /*Buscando los tiempos de entrega*/
                $tiempo = TiemposDeEntrega::where('idTiempoEntrega' , $request->fk_idTiempoEntrega)->get();
                if (count($tiempo) > 0){
                    if ($request->fk_idTiempoEntrega==1){
                        $isUrgente = true;
                        //establezco la fecha inicial
                        $fecha_probable = Carbon::now()->addDay($tiempo[0]->days);
                        //valido si la fecha es un dia de semana
                        $seguir = false;
                        do {
                            if (!$fecha_probable->isWeekday()){ //es un sabado o domingo?
                                $seguir         = true;
                                $fecha_probable = $fecha_probable->addDay(); //entonces agrego un dia y valido
                            } else {
                                $seguir = false;
                            }
                        } while ($seguir);
                        $fecha_de_entrega = $fecha_probable;


                        $precio = ($orden->precio_total * (($tiempo[0]->percentaje / 100) + 1)); //se incrementa el porcentaje asignado
                    } elseif ($request->fk_idTiempoEntrega==3) {
                        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                        $precio           = ($orden->precio_total - (($tiempo[0]->percentaje / 100) * $orden->precio_total)); //se incrementa el porcentaje asignado
                    } elseif ($request->fk_idTiempoEntrega==2) {
                        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);

                        $precio = $orden->precio_total;
                    }
                    /*formateando*/
                    $fecha_de_entrega = Carbon::create($fecha_de_entrega)->format('Y-m-d');

                    $orden->update([
                        'date_delivery' => $fecha_de_entrega ,
                        'fk_idTiempoEntrega' => $request->fk_idTiempoEntrega ,
                        'is_express_delivery' => (boolean)$request->is_express_delivery
                    ]);

                }
            }

            return response()->json(['msj' => "Éxito" , 'orden' => $orden]);
        } else {
            return response()->json(['msj' => "No existe la Orden"] , 404);
        }
    }


    public function store(Request $request)
    {
        //calcular los
        $body                  = json_decode($request->body);
        $cantidad_de_productos = count($body->items);
        $cantidad              = 3;
        if ($cantidad_de_productos >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
            $cantidad = 4;
        }
        $enviar_sms = false;

        $isUrgente        = false;
        $fecha_de_entrega = null;
        if ($request->exists('fk_idTiempoEntrega')){
            //busco si existe ese id
            $tiempo = TiemposDeEntrega::where('idTiempoEntrega' , $request->fk_idTiempoEntrega)->get();
            if (count($tiempo) > 0){
                if ($tiempo[0]->type=='URGENTE'){
                    $isUrgente = true;
                    //establezco la fecha inicial
                    $fecha_probable = Carbon::now()->addDay($tiempo[0]->days);
                    //valido si la fecha es un dia de semana
                    $seguir = false;
                    do {
                        if (!$fecha_probable->isWeekday()){ //es un sabado o domingo?
                            $seguir         = true;
                            $fecha_probable = $fecha_probable->addDay(); //entonces agrego un dia y valido
                        } else {
                            $seguir = false;
                        }
                    } while ($seguir);
                    $fecha_de_entrega = $fecha_probable;

                } elseif ($tiempo[0]->type=='NORMAL') {
                    $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                } elseif ($tiempo[0]->type=='RELAX') {
                    $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                }
            } else {
                Log::info('No se encontró el tiempo de espera ' . $request->fk_idTiempoEntrega);
                return response()->json(['msj' => 'No se encontró el tiempo de espera'] , 404);
            }


        }

        /*formateando*/
        $fecha_de_entrega = Carbon::create($fecha_de_entrega)->format('Y-m-d');

//        return response()->json($fecha_de_entrega , 404);

        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                      = new OrderHeader();
            $tipo->fk_payform          = $request->fk_payform;
            $tipo->fk_idTiempoEntrega  = @$request->fk_idTiempoEntrega;
            $tipo->recibo_pago         = $request->recibo_pago;
            $tipo->notes               = $request->notes;
            $tipo->total_diseno        = $request->total_diseno;
            $tipo->fk_moneda           = $request->fk_moneda;
            $tipo->is_express_delivery = $isUrgente;

            $tipo->fk_descuento = $request->fk_descuento;

            $tipo->fk_zonaEnvio  = (is_null($request->fk_zonaEnvio) || $request->fk_zonaEnvio=='') ? 0 : $request->fk_zonaEnvio;
            $tipo->precio_total  = $request->precio_total;
            $tipo->fk_status     = $request->fk_status;
            $tipo->idUserCreated = @$request->idUserCreated;
            $tipo->idUsuario     = @$request->idUserCreated;

            $tipo->date_recived       = $request->date_recived;
            $tipo->date_delivery      = $fecha_de_entrega;
            $tipo->date_cancel        = $request->date_cancel;
            $tipo->fk_sucursal        = $request->fk_sucursal;
            $tipo->response_mercantil = $request->response_mercantil=='null' ? null : $request->response_mercantil;


            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');
//            $tipo->order_open   = $request->order_open;
            $tipo->order_open                              = true;
            $tipo->tasa_moneda                             = $request->tasa_moneda;
            $tipo->date_created                            = now();
            $tipo->orden_creada_cliente                    = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;

            if (is_null($request->idUser) || $request->idUser=='null'){
                $datas = json_decode($request->data_user);

                $rr = User::where('numerIdentificacion' , $datas->numerIdentificacion)->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
//                    return response()->json(json_decode($request->data_user),404);
//                    $request->validate([
//                        'numerIdentificacion' => 'required|unique|users,numerIdentificacion,NULL,id,deleted_at,NULL' ,
//                        'email' => 'required|unique:users,email,NULL,id,deleted_at,NULL' ,
//                    ] , [
//                        'numerIdentificacion.required' => 'El número de identificación es requerida' ,
//                        'numerIdentificacion.unique' => 'El número de identificación es debe ser único' ,
//                        'email.unique' => 'Este email ya se encuentra en uso' ,
//                    ]);
                    $user                      = new User();
                    $user->email               = $datas->email;
                    $user->fullName            = $datas->fullName;
                    $user->numerIdentificacion = $datas->numerIdentificacion;
                    $user->phone               = $datas->phone;
                    $user->typeIdentification  = $datas->typeIdentification;
                    $user->user                = $datas->user;
                    $user->direccion           = @$datas->direccion;
//                    return response()->json($user , 404);
                    $user->profile_fk = 3; //como cliente
                    $user->name       = $datas->fullName;
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
//                        if ($rr[0]->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
//                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario1' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser) || $request->idUser!="null"){
                    $er = User::find($request->idUser);
                    if ($er){
                        $tipo->idUser = $request->idUser;
//                        if ($er->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
//                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario2' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);

                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/
                    $enviar_sms = true;

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;

            //buscando si el cliente tiene un descuento automatico activo
            $cliente = User::find($tipo->idUser);
            if ($cliente){
                if ($cliente->aplica_descuento_automatico==1){
                    $descuento = Descuento::where('idUsuario' , $tipo->idUser)
                        ->active()
                        ->get();
                    if (count($descuento) > 0){
                        $tipo->fk_descuento = $descuento[0]->idDescuento;
                    }

                }
            }

            $tipo->save();

            if ($enviar_sms){
                // envio de sms
                SmsController::sendSMS(new Request([
                    "destination" => env('NUMERO_ENVIO_SMS_ISLA_ANAMARIA') ,
                    "message" => 'Comprobante ' . $tipo->n_order . ', del cliente ' . strtoupper(User::find($tipo->idUser)->fullName) . ', ' . strtolower(env('BASE_URL_IMG') . '/public/api/obtenercomprobantepago/' . $tipo->idOrderHeader) ,
                ]));
                // fin envio sms
            }
//            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
//                'pass_rand' => $pass ,
//                'credenciales' => $credenciales->original ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];
//        $response['asa']="dsa";
//            return $response;
//            exit();
//            $response->append(['as'=>asdasd]);


            return OrderBodyController::store($request , new Request($response));
//            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function cotizacionpostman(Request $request)
    {
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                                          = new OrderHeader($request->all());
            $tipo->resumen_pago                            = @$request->input('resumen_pago');
            $tipo->comprobante_factura                     = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller                         = @$request->input('enviar_a_taller');
            $tipo->order_open                              = true;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;;

            $tipo->cotizacion   = true;
            $tipo->tasa_moneda  = $request->tasa_moneda;
            $tipo->date_created = now();

            $isUrgente        = false;
            $fecha_de_entrega = null;
            if ($request->exists('fk_idTiempoEntrega')){
                //busco si existe ese id
                $tiempo = TiemposDeEntrega::where('idTiempoEntrega' , $request->fk_idTiempoEntrega)->get();
                if (count($tiempo) > 0){
                    if ($tiempo[0]->type=='URGENTE'){
                        $isUrgente = true;
                        //establezco la fecha inicial
                        $fecha_probable = Carbon::now()->addDay($tiempo[0]->days);
                        //valido si la fecha es un dia de semana
                        $seguir = false;
                        do {
                            if (!$fecha_probable->isWeekday()){ //es un sabado o domingo?
                                $seguir         = true;
                                $fecha_probable = $fecha_probable->addDay(); //entonces agrego un dia y valido
                            } else {
                                $seguir = false;
                            }
                        } while ($seguir);
                        $fecha_de_entrega = $fecha_probable;
                    } elseif ($tiempo[0]->type=='NORMAL') {
                        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                    } elseif ($tiempo[0]->type=='RELAX') {
                        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                    }
                } else {
                    Log::info('No se encontró el tiempo de espera ' . $request->fk_idTiempoEntrega);
                    return response()->json(['msj' => 'No se encontró el tiempo de espera'] , 404);
                }
            }
            $fecha_de_entrega = Carbon::create($fecha_de_entrega)->format('Y-m-d');

            $tipo->date_delivery                           = $fecha_de_entrega;
            $tipo->is_express_delivery                     = $isUrgente;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;

            if (is_null($request->idUser) || $request->idUser=='null'){
                $datas = $request->data_user;

                $rr = User::where('numerIdentificacion' , $datas['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass                      = Str::random(10);
                    $user                      = new User();
                    $user->email               = $datas['email'];
                    $user->fullName            = $datas['fullName'];
                    $user->numerIdentificacion = $datas['numerIdentificacion'];
                    $user->phone               = $datas['phone'];
                    $user->typeIdentification  = $datas['typeIdentification'];
                    $user->user                = $datas['user'];
                    $user->profile_fk          = 3; //como cliente
                    $user->name                = $datas['fullName'];
                    $user->direccion           = @$datas['direccion'];
                    $user->password            = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
                    } else {
                        return response()->json([
                            'msj' => 'No se encuentra el usuario1' ,
                        ] , 404);
                    }
                }
            } else {
                if (!is_null($request->idUser) || $request->idUser!="null"){
                    if (User::find($request->idUser)){
                        $tipo->idUser = $request->idUser;
                    } else {
                        return response()->json([
                            'msj' => 'No se encuentra el usuario2' ,
                        ] , 404);
                    }
                }
            }

            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage  = $request->img_pago;
                    $thumbnailImage = Image::make($originalImage);
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/
                    $tipo->img_pago = $nombre_interno;
                }
            }

            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;

            $tipo->save();

            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::storecotizacionpostman($request , new Request($response));

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function crer_orden_desde_valla(Request $request)
    {
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo               = new OrderHeader($request->all());
            $tipo->resumen_pago = @$request->input('resumen_pago');

            $tipo->fecha_vencimiento = @$request->input('fecha_vencimiento');
            $tipo->estatus_contrato  = @$request->input('estatus_contrato');

            $tipo->comprobante_factura                     = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller                         = @$request->input('enviar_a_taller');
            $tipo->order_open                              = true;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;;

            $tipo->cotizacion   = false;
            $tipo->tasa_moneda  = $request->tasa_moneda;
            $tipo->date_created = now();

            $isUrgente        = false;
            $fecha_de_entrega = null;
            if ($request->exists('fk_idTiempoEntrega')){
                //busco si existe ese id
                $tiempo = TiemposDeEntrega::where('idTiempoEntrega' , $request->fk_idTiempoEntrega)->get();
                if (count($tiempo) > 0){
                    if ($tiempo[0]->type=='URGENTE'){
                        $isUrgente = true;
                        //establezco la fecha inicial
                        $fecha_probable = Carbon::now()->addDay($tiempo[0]->days);
                        //valido si la fecha es un dia de semana
                        $seguir = false;
                        do {
                            if (!$fecha_probable->isWeekday()){ //es un sabado o domingo?
                                $seguir         = true;
                                $fecha_probable = $fecha_probable->addDay(); //entonces agrego un dia y valido
                            } else {
                                $seguir = false;
                            }
                        } while ($seguir);
                        $fecha_de_entrega = $fecha_probable;
                    } elseif ($tiempo[0]->type=='NORMAL') {
                        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                    } elseif ($tiempo[0]->type=='RELAX') {
                        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($tiempo[0]->days);
                    }
                } else {
                    Log::info('No se encontró el tiempo de espera ' . $request->fk_idTiempoEntrega);
                    return response()->json(['msj' => 'No se encontró el tiempo de espera'] , 404);
                }
            }
            $fecha_de_entrega = Carbon::create($fecha_de_entrega)->format('Y-m-d');

            $tipo->date_delivery                           = $fecha_de_entrega;
            $tipo->is_express_delivery                     = $isUrgente;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;

            if (is_null($request->idUser) || $request->idUser=='null'){
                $datas = $request->data_user;

                $rr = User::where('numerIdentificacion' , $datas['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass                      = Str::random(10);
                    $user                      = new User();
                    $user->email               = $datas['email'];
                    $user->fullName            = $datas['fullName'];
                    $user->numerIdentificacion = $datas['numerIdentificacion'];
                    $user->phone               = $datas['phone'];
                    $user->typeIdentification  = $datas['typeIdentification'];
                    $user->user                = $datas['user'];
                    $user->profile_fk          = 3; //como cliente
                    $user->name                = $datas['fullName'];
                    $user->direccion           = @$datas['direccion'];
                    $user->password            = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
                    } else {
                        return response()->json([
                            'msj' => 'No se encuentra el usuario1' ,
                        ] , 404);
                    }
                }
            } else {
                if (!is_null($request->idUser) || $request->idUser!="null"){
                    if (User::find($request->idUser)){
                        $tipo->idUser = $request->idUser;
                    } else {
                        return response()->json([
                            'msj' => 'No se encuentra el usuario2' ,
                        ] , 404);
                    }
                }
            }

            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage  = $request->img_pago;
                    $thumbnailImage = Image::make($originalImage);
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/
                    $tipo->img_pago = $nombre_interno;
                }
            }

            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;

            $tipo->save();

            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::body_orden_por_valla($request , new Request($response));

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function cotizacion_listar(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $todas = OrderHeader::orderBy('created_at' , 'desc')
                ->where('cotizacion' , true)
                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();
//

            return response()->json($this->diasDeEntrega($todas));
        } else {
            $todas = OrderHeader::orderBy('created_at' , 'DESC')
                ->where('cotizacion' , true)
                ->get();
//

            return response()->json($this->diasDeEntrega($todas));
        }

    }

    public function ordenes_listar(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $primero_ordenes_urgentes = $ordenes = OrderHeader::orderBy('created_at' , 'desc')
                ->where('is_express_delivery' , true)
                ->whereBetween('created_at' , [$desde , $hasta])
                ->get(); //colocar de primero en el listado


            $primero = $ordenes = OrderHeader::orderBy('created_at' , 'desc')
//                ->where('fk_status' , '1')
                ->where('orden_creada_cliente' , true)
//                ->whereNull('idUsuario')
                ->whereBetween('created_at' , [$desde , $hasta])
                ->get(); //colocar de primero en el listado
//            dd($primero);

            $ordenes = OrderHeader::orderBy('created_at' , 'desc')
                ->where('fk_status' , '!=' , '16')
                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();

            //convirtiendo de bs a dolares
            $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            foreach ($ordenes as $key => $orden) {
                $AbonoTotalDolares   = null;
                $AbonoTotalBolivares = null;
                $totalBsConvetidos   = 0;
                if ($orden->cuenta_por_cobrar==false && $orden->cotizacion==false){
                    //Se conviertes las operaciones en dolares equivalenes
                    $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                        ->where('fk_moneda' , 1)
                        ->where('idOrderHeader' , $orden->idOrderHeader)
                        ->sum(DB::raw('monto'));

                    $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                        ->where('fk_moneda' , 2)
                        ->where('idOrderHeader' , $orden->idOrderHeader)
                        ->sum(DB::raw('monto/tasa_moneda'));

                    //total en dolares
                    $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                    if (round($totalAbono , 2) >= $orden->precio_total){
                        //Se conviertes las operaciones en bs equivalenes

                        $AbonoTotalDolaresBs = Operacion::where('idTipoTransaccion' , 1)
                            ->where('fk_moneda' , 1)
                            ->where('idOrderHeader' , $orden->idOrderHeader)
                            ->sum(DB::raw('monto*tasa_moneda'));

                        $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                            ->where('fk_moneda' , 2)
                            ->where('idOrderHeader' , $orden->idOrderHeader)
                            ->sum(DB::raw('monto'));

                        $totalBsConvetidos = round($AbonoTotalBolivares + $AbonoTotalDolaresBs , 2);
                    }

                } else {
                    $totalBsConvetidos = 0;
                }
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto'));

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal        = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal        = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
                $ordenes[$key]->totalBsConvetidos = $totalBsConvetidos;
            }

            $total   = OrderHeader::orderBy('created_at' , 'desc')
                ->whereBetween('created_at' , [$desde , $hasta])
                ->count();
            $ordenes = $this->diasDeEntrega($ordenes);

            foreach ($primero as $item) {
                $primero_ordenes_urgentes->push($item);
            }
            foreach ($ordenes as $item) {
                $primero_ordenes_urgentes->push($item);
            }
            $ordenes = $primero_ordenes_urgentes->unique(); //quitando duplicados

            return response()->json(["data" => $ordenes , 'total' => $total]);
        }

    }

    public
    function cotizacion_listar_punto_venta(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $todas = OrderHeader::orderBy('created_at' , 'desc')
//                ->where('cotizacion' , true)
                ->where('punto_de_venta' , true)
                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();
//

            return response()->json($this->diasDeEntrega($todas));
        } else {
            $todas = OrderHeader::orderBy('created_at' , 'DESC')
//                ->where('cotizacion' , true)
                ->where('punto_de_venta' , true)
                ->get();
//

            return response()->json($this->diasDeEntrega($todas));
        }

    }

    public
    function listado_ordenes_cuenta_por_cobrar(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit') && $request->exists('desde') && $request->exists('hasta')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

//            $desde = substr($request->desde , 0 , 10) . ' 00:00:00';
//            $hasta = substr($request->hasta , 0 , 10) . ' 23:59:59';

            $ordenes = OrderHeader::orderBy('created_at' , 'desc')
                ->where('cuenta_por_cobrar' , true)
                ->where(function ($sql) use ($request){
                    if ($request->exists('idOrderHeader')){
                        if ($request->idOrderHeader!='' && $request->idOrderHeader!=null){
                            $sql->where('idOrderHeader' , $request->idOrderHeader);
                        } else {
                            $sql->where('idUser' , $request->idUser);
                        }
                    } else {
                        $sql->where('idUser' , $request->idUser);
                    }
                })
                ->where('fk_status' , '!=' , 10)
//                ->whereBetween('created_at' , [$desde , $hasta])
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();

            $AbonoTotalDolares   = null;
            $AbonoTotalBolivares = null;
            //convirtiendo de bs a dolares
            $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            foreach ($ordenes as $key => $orden) {
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            }


            return response()->json($this->diasDeEntrega($ordenes));
        }
    }

    public
    function listado_ordenes_en_deuda_por_asesor(Request $request)
    {
        if ($request->exists('offset') && $request->exists('limit')){
            $this->validate($request , [
                'offset' => 'integer|min:0' ,
                'limit' => 'integer|min:0' ,
            ] , [
                'offset.integer' => 'Debe ser numérico' ,
                'limit.integer' => 'Debe ser numérico' ,

                'offset.min' => 'Debe tener al menos un número' ,
                'limit.min' => 'Debe tener al menos un número' ,
            ]);

            $ordenes = OrderHeader::orderBy('created_at' , 'desc')
                ->where('cuenta_por_cobrar' , true)
                ->where('idUsuario' , $request->idUsuario)
                ->where('fk_status' , '!=' , 10)
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();

            $AbonoTotalDolares   = null;
            $AbonoTotalBolivares = null;
            //convirtiendo de bs a dolares
            $tasa = TasaMoneda::active()->where('fk_moneda' , 2)->first();
            foreach ($ordenes as $key => $orden) {
                $AbonoTotalDolares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
                $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

                $ordenes[$key]->AbonoTotal = round($totalAbono , 2);
                $ordenes[$key]->DeudaTotal = ($orden->precio_total < $totalAbono) ? 0 : round(($orden->precio_total - $totalAbono) , 2);
            }


            return response()->json($this->diasDeEntrega($ordenes));
        }
    }

    public
    function cotizacion(Request $request)
    {
//        return response()->json($request->all(),404);
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                                          = new OrderHeader($request->all());
            $tipo->resumen_pago                            = @$request->input('resumen_pago');
            $tipo->comprobante_factura                     = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller                         = @$request->input('enviar_a_taller');
            $tipo->order_open                              = true;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;;

            $tipo->cotizacion           = true;
            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');

            if (is_null($request->idUser) || $request->idUser=='null'){
                $datas = json_decode($request->data_user);

                $rr = User::where('numerIdentificacion' , $datas->numerIdentificacion)->get();
                if (count($rr)==0){
                    $pass                      = Str::random(10);
                    $user                      = new User();
                    $user->email               = $datas->email;
                    $user->fullName            = $datas->fullName;
                    $user->numerIdentificacion = $datas->numerIdentificacion;
                    $user->phone               = $datas->phone;
                    $user->typeIdentification  = $datas->typeIdentification;
                    $user->user                = $datas->user;
                    $user->profile_fk          = 3; //como cliente
                    $user->name                = $datas->fullName;
                    $user->direccion           = @$datas->direccion;
                    $user->password            = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
//                        if ($rr[0]->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        return response()->json([
                            'msj' => 'No se encuentra el usuario1' ,
                        ] , 404);
                    }
                }
            } else {
                if (!is_null($request->idUser) || $request->idUser!="null"){
                    $re = User::find($request->idUser);
                    if ($re){
                        $tipo->idUser = $request->idUser;
//                        if ($re->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        return response()->json([
                            'msj' => 'No se encuentra el usuario2' ,
                        ] , 404);
                    }
                }
            }

//            if ($request->exists('img_pago')){
//                if (!is_null($request->img_pago)){
//                    /*para la foto*/
//                    $originalImage  = $request->img_pago;
//                    $thumbnailImage = Image::make($originalImage);
//                    $nombre_publico = $originalImage->getClientOriginalName();
//                    $extension      = 'png';
//                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
//                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
//                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
//                    /*para la foto*/
//                    $tipo->img_pago = $nombre_interno;
//                }
//            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;


            $tipo->save();

            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::storecotizacion($request , new Request($response));

        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public
    function store2(Request $request)
    {

        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo               = new OrderHeader();
            $tipo->fk_payform   = $request->fk_payform;
            $tipo->recibo_pago  = $request->recibo_pago;
            $tipo->notes        = $request->notes;
            $tipo->total_diseno = $request->total_diseno;
            $tipo->fk_moneda    = $request->fk_moneda;
            $tipo->fk_descuento = $request->fk_descuento;
            $tipo->fk_zonaEnvio = $request->fk_zonaEnvio;
            $tipo->precio_total = $request->precio_total;
            $tipo->fk_status    = $request->fk_status;

            $tipo->date_recived       = $request->date_recived;
            $tipo->date_delivery      = $request->date_delivery;
            $tipo->date_cancel        = $request->date_cancel;
            $tipo->fk_sucursal        = $request->fk_sucursal;
            $tipo->response_mercantil = $request->response_mercantil=='null' ? null : $request->response_mercantil;

            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');

//            $tipo->order_open   = $request->order_open;
            $tipo->order_open                              = true;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;;

            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');
//dd($request->all());
            if (is_null($request->idUser)){
                $rr = User::where('numerIdentificacion' , $request->data_user['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
                    $user = new User($request->data_user);

                    $user->profile_fk = 3; //como cliente
                    $user->name       = $request->data_user['fullName'];
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
                    } else {
//                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser)){
                    if (User::find($request->idUser)){
                        $tipo->idUser = $request->idUser;
                    } else {
//                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;
            //buscando si el cliente tiene un descuento automatico activo
            $cliente = User::find($tipo->idUser);
            if ($cliente){
                if ($cliente->aplica_descuento_automatico==1){
                    $descuento = Descuento::where('idUsuario' , $tipo->idUser)
                        ->active()
                        ->get();
                    if (count($descuento) > 0){
                        $tipo->fk_descuento = $descuento[0]->idDescuento;
                    }

                }
            }
            $tipo->save();
            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
//                'pass_rand' => $pass ,
//                'credenciales' => $credenciales->original ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];
//        $response['asa']="dsa";
//            return $response;
//            $response->append(['as'=>asdasd]);


            return OrderBodyController::store2($request , new Request($response));
//            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public
    function store3(Request $request)
    {
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                = new OrderHeader();
            $tipo->fk_payform    = $request->fk_payform;
            $tipo->recibo_pago   = $request->recibo_pago;
            $tipo->notes         = $request->notes;
            $tipo->total_diseno  = $request->total_diseno;
            $tipo->fk_moneda     = $request->fk_moneda;
            $tipo->fk_descuento  = $request->fk_descuento;
            $tipo->fk_zonaEnvio  = $request->fk_zonaEnvio;
            $tipo->precio_total  = $request->precio_total;
            $tipo->fk_status     = $request->fk_status;
            $tipo->idUserCreated = @$request->cerrar_orden['idUser'];
            $tipo->idUsuario     = @$request->cerrar_orden['idUser'];;

            $tipo->date_recived  = $request->date_recived;
            $tipo->date_delivery = now();
            $tipo->date_cancel   = $request->date_cancel;
            $tipo->fk_sucursal   = $request->fk_sucursal;

            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');

//            $tipo->order_open   = $request->order_open;
            $tipo->order_open                              = true;
            $tipo->aplica_pago_extra_por_tiempo_de_entrega = @$request->aplica_pago_extra_por_tiempo_de_entrega;;

            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');

            if (is_null($request->idUser)){
                $rr = User::where('numerIdentificacion' , $request->data_user['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
                    $user = new User($request->data_user);

                    $user->profile_fk = 3; //como cliente
                    $user->name       = $request->data_user['fullName'];
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
//                        if ($rr[0]->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser)){
                    $rr = User::find($request->idUser);
                    if ($rr){
                        $tipo->idUser = $request->idUser;
//                        if ($rr->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;
            //buscando si el cliente tiene un descuento automatico activo
            $cliente = User::find($tipo->idUser);
            if ($cliente){
                if ($cliente->aplica_descuento_automatico==1){
                    $descuento = Descuento::where('idUsuario' , $tipo->idUser)
                        ->active()
                        ->get();
                    if (count($descuento) > 0){
                        $tipo->fk_descuento = $descuento[0]->idDescuento;
                    }

                }
            }
            $tipo->save();

            //cerrando orden
            $idOrderHeader = $tipo->idOrderHeader;


            $idUser     = $tipo->idUser;
            $idSucursal = $request->cerrar_orden['idSucursal'];
            $order      = OrderHeader::find($idOrderHeader);

            //buscando la caja activa  de la sucursal
            $caja = Caja::where('idSucursal' , $idSucursal)->active()->get();
//            dd($caja);
            if (count($caja)==0){
                DB::rollback();
                return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
            }
            $caja = $caja[0];

            if ($order){
                //actualizo la orden
                $order->update(['order_open' => false , 'fk_status' => 3]);
                $tasa = TasaMoneda::active()->first();
                foreach ($request->cerrar_orden['formas_pago'] as $forma_pago) {
//              return response()->json($forma_pago);
                    $operacion = new Operacion();
                    if ($forma_pago['idTipoTransaccion']==2){
                        $operacion->descripcion = 'Cambio de la Orden Nº: ' . $order->n_order;
                    } else {
                        $operacion->descripcion = 'Cierre de Orden Nº: ' . $order->n_order;
                    }
                    $operacion->monto             = $forma_pago['monto'];
                    $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                    $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                    $operacion->idCaja            = $caja->idCaja;
                    $operacion->idOrderHeader     = $idOrderHeader;
                    $operacion->n_order           = $order->n_order;
                    $operacion->fk_moneda         = $forma_pago['fk_moneda'];
//                    if ($forma_pago['fk_moneda']==2){
                    $operacion->tasa_moneda = $tasa->tasa;
//                    }
                    $operacion->idUser = $request->cerrar_orden['idUser'];
                    $operacion->save();


                }

            }

            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::store3($request , new Request($response));
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public
    function crea_ordenes_punto_venta(Request $request)
    {
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                 = new OrderHeader();
            $tipo->punto_de_venta = true;
            $tipo->fk_payform     = $request->fk_payform;
            $tipo->recibo_pago    = $request->recibo_pago;
            $tipo->notes          = $request->notes;
            $tipo->total_diseno   = $request->total_diseno;
            $tipo->fk_moneda      = $request->fk_moneda;
            $tipo->fk_descuento   = $request->fk_descuento;
            $tipo->fk_zonaEnvio   = $request->fk_zonaEnvio;
            $tipo->precio_total   = $request->precio_total;
            $tipo->fk_status      = $request->fk_status;

            $tipo->date_recived  = $request->date_recived;
            $tipo->date_delivery = now();
            $tipo->date_cancel   = $request->date_cancel;
            $tipo->fk_sucursal   = $request->fk_sucursal;

            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');

//            $tipo->order_open   = $request->order_open;
            $tipo->order_open           = true;
            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');

            if (is_null($request->idUser)){
                $rr = User::where('numerIdentificacion' , $request->data_user['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
                    $user = new User($request->data_user);

                    $user->profile_fk = 3; //como cliente
                    $user->name       = $request->data_user['fullName'];
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
//                        if ($rr[0]->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser)){
                    $rr = User::find($request->idUser);
                    if ($rr){
                        $tipo->idUser = $request->idUser;
//                        if ($rr->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;
            //buscando si el cliente tiene un descuento automatico activo
            $cliente = User::find($tipo->idUser);
            if ($cliente){
                if ($cliente->aplica_descuento_automatico==1){
                    $descuento = Descuento::where('idUsuario' , $tipo->idUser)
                        ->active()
                        ->get();
                    if (count($descuento) > 0){
                        $tipo->fk_descuento = $descuento[0]->idDescuento;
                    }

                }
            }
            $tipo->save();

            //cerrando orden
            $idOrderHeader = $tipo->idOrderHeader;
            $idUser        = $tipo->idUser;
            $idSucursal    = $request->cerrar_orden['idSucursal'];
            $order         = OrderHeader::find($idOrderHeader);

            //buscando la caja activa  de la sucursal
            $caja = Caja::where('idSucursal' , $idSucursal)->active()->get();
//            dd($caja);
            if (count($caja)==0){
                DB::rollback();
                return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
            }
            $caja = $caja[0];

            if ($order){
                //actualizo la orden
                $order->update(['order_open' => false , 'fk_status' => 3]);
                $tasa = TasaMoneda::active()->first();
                foreach ($request->cerrar_orden['formas_pago'] as $forma_pago) {
//              return response()->json($forma_pago);
                    $operacion = new Operacion();
                    if ($forma_pago['idTipoTransaccion']==2){
                        $operacion->descripcion = 'Cambio de la Orden Nº: ' . $order->n_order;
                    } else {
                        $operacion->descripcion = 'Cierre de Orden Nº: ' . $order->n_order;
                    }
                    $operacion->monto             = $forma_pago['monto'];
                    $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                    $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                    $operacion->idCaja            = $caja->idCaja;
                    $operacion->idOrderHeader     = $idOrderHeader;
                    $operacion->n_order           = $order->n_order;
                    $operacion->fk_moneda         = $forma_pago['fk_moneda'];
//                    if ($forma_pago['fk_moneda']==2){
                    $operacion->tasa_moneda = $tasa->tasa;
//                    }
                    $operacion->idUser = $request->cerrar_orden['idUser'];
                    $operacion->save();


                }

            }
            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,

                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::puntoventa2($request , new Request($response));
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public
    function crea_ordenes_punto_venta_para_cotizar(Request $request)
    {
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                 = new OrderHeader();
            $tipo->punto_de_venta = true;
            $tipo->cotizacion     = true;
            $tipo->fk_payform     = $request->fk_payform;
            $tipo->recibo_pago    = $request->recibo_pago;
            $tipo->notes          = $request->notes;
            $tipo->total_diseno   = $request->total_diseno;
            $tipo->fk_moneda      = $request->fk_moneda;
            $tipo->fk_descuento   = $request->fk_descuento;
            $tipo->fk_zonaEnvio   = $request->fk_zonaEnvio;
            $tipo->precio_total   = $request->precio_total;
            $tipo->fk_status      = $request->fk_status;

            $tipo->date_recived  = $request->date_recived;
            $tipo->date_delivery = now();
            $tipo->date_cancel   = $request->date_cancel;
            $tipo->fk_sucursal   = $request->fk_sucursal;

            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');

//            $tipo->order_open   = $request->order_open;
            $tipo->order_open           = true;
            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');

            if (is_null($request->idUser)){
                $rr = User::where('numerIdentificacion' , $request->data_user['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
                    $user = new User($request->data_user);

                    $user->profile_fk = 3; //como cliente
                    $user->name       = $request->data_user['fullName'];
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
//                        if ($rr[0]->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser)){
                    $rr = User::find($request->idUser);
                    if ($rr){
                        $tipo->idUser = $request->idUser;
//                        if ($rr->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;
            //buscando si el cliente tiene un descuento automatico activo
            $cliente = User::find($tipo->idUser);
            if ($cliente){
                if ($cliente->aplica_descuento_automatico==1){
                    $descuento = Descuento::where('idUsuario' , $tipo->idUser)
                        ->active()
                        ->get();
                    if (count($descuento) > 0){
                        $tipo->fk_descuento = $descuento[0]->idDescuento;
                    }

                }
            }
            $tipo->save();

            //cerrando orden
            $idOrderHeader = $tipo->idOrderHeader;
            $idUser        = $tipo->idUser;
            $idSucursal    = $request->cerrar_orden['idSucursal'];
            $order         = OrderHeader::find($idOrderHeader);

            //buscando la caja activa  de la sucursal
            $caja = Caja::where('idSucursal' , $idSucursal)->active()->get();
//            dd($caja);
            if (count($caja)==0){
                DB::rollback();
                return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
            }
            $caja = $caja[0];

            if ($order){
                //actualizo la orden
                $order->update(['order_open' => false , 'fk_status' => 1]);
                $tasa = TasaMoneda::active()->first();
                foreach ($request->cerrar_orden['formas_pago'] as $forma_pago) {
//              return response()->json($forma_pago);
                    $operacion = new Operacion();
                    if ($forma_pago['idTipoTransaccion']==2){
                        $operacion->descripcion = 'Cambio de la Orden Nº: ' . $order->n_order;
                    } else {
                        $operacion->descripcion = 'Cierre de Orden Nº: ' . $order->n_order;
                    }
                    $operacion->monto             = $forma_pago['monto'];
                    $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                    $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                    $operacion->idCaja            = $caja->idCaja;
                    $operacion->idOrderHeader     = $idOrderHeader;
                    $operacion->n_order           = $order->n_order;
                    $operacion->fk_moneda         = $forma_pago['fk_moneda'];
//                    if ($forma_pago['fk_moneda']==2){
                    $operacion->tasa_moneda = $tasa->tasa;
//                    }
                    $operacion->idUser = $request->cerrar_orden['idUser'];
                    $operacion->save();


                }

            }

            DB::commit();

            $response = [
                'msj' => 'Creado exitosamente' ,

                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::puntoventa($request , new Request($response));
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    public function crea_ordenes_por_credito(Request $request)
    {
        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo                 = new OrderHeader();
            $tipo->punto_de_venta = true;

            if (!is_null($request->idUser)){
                $rr = User::find($request->idUser);
                if ($rr->permitir_credito==false){
                    $tipo->cuenta_por_cobrar = false;
                    $tipo->fk_status         = 1;
                } else {
                    $tipo->cuenta_por_cobrar = true;
                    $tipo->fk_status         = $request->fk_status;
                }
            }
            $tipo->cotizacion   = false;
            $tipo->fk_payform   = $request->fk_payform;
            $tipo->recibo_pago  = $request->recibo_pago;
            $tipo->notes        = $request->notes;
            $tipo->total_diseno = $request->total_diseno;
            $tipo->fk_moneda    = $request->fk_moneda;
            $tipo->fk_descuento = $request->fk_descuento;
            $tipo->fk_zonaEnvio = $request->fk_zonaEnvio;
            $tipo->precio_total = $request->precio_total;


            $tipo->date_recived  = $request->date_recived;
            $tipo->date_delivery = now();
            $tipo->date_cancel   = $request->date_cancel;
            $tipo->fk_sucursal   = $request->fk_sucursal;

            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');

//            $tipo->order_open   = $request->order_open;
            $tipo->order_open           = true;
            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');

            if (is_null($request->idUser)){
                $rr = User::where('numerIdentificacion' , $request->data_user['numerIdentificacion'])->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
                    $user = new User($request->data_user);

                    $user->profile_fk = 3; //como cliente
                    $user->name       = $request->data_user['fullName'];
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
//                        if ($rr[0]->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser)){
                    $rr = User::find($request->idUser);

                    if ($rr){
                        $tipo->idUser = $request->idUser;
//                        if ($rr->lista_negra==true){
//                            return response()->json([
//                                'msj' => 'Disculpe, no se pudo crear porque el cliente posee deudas.' ,
//                            ] , 404);
//                        }
                    } else {
                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;
            //buscando si el cliente tiene un descuento automatico activo
            $cliente = User::find($tipo->idUser);
            if ($cliente){
                if ($cliente->aplica_descuento_automatico==1){
                    $descuento = Descuento::where('idUsuario' , $tipo->idUser)
                        ->active()
                        ->get();
                    if (count($descuento) > 0){
                        $tipo->fk_descuento = $descuento[0]->idDescuento;
                    }

                }
            }
            $tipo->save();

            //cerrando orden
            $idOrderHeader = $tipo->idOrderHeader;
            $idUser        = $tipo->idUser;
            $idSucursal    = $request->cerrar_orden['idSucursal'];
            $order         = OrderHeader::find($idOrderHeader);

            //buscando la caja activa  de la sucursal
            $caja = Caja::where('idSucursal' , $idSucursal)->active()->get();
//            dd($caja);
            if (count($caja)==0){
                DB::rollback();
                return response()->json(['msj' => 'No existe caja activa para la sucursal solicitada'] , 404);
            }
            $caja = $caja[0];

            if ($order){
                //actualizo la orden

                $order->update(['order_open' => false , 'fk_status' => 2]);
                $rr = User::find($idUser);

                if ($rr->permitir_credito==false){
                    $order->update(['fk_status' => 1]);
                }
                $tasa = TasaMoneda::active()->first();

                if ($rr->permitir_credito==true){
                    foreach ($request->cerrar_orden['formas_pago'] as $forma_pago) {
                        $operacion                    = new Operacion();
                        $operacion->descripcion       = 'Abono de la Orden Nº: ' . $order->n_order;
                        $operacion->monto             = str_replace(',' , '.' , $forma_pago['monto']);
                        $operacion->idTipoTransaccion = $forma_pago['idTipoTransaccion'];
                        $operacion->idFormaPago       = $forma_pago['idFormaPago'];
                        $operacion->idCaja            = $caja->idCaja;
                        $operacion->idOrderHeader     = $idOrderHeader;
                        $operacion->n_order           = $order->n_order;
                        $operacion->fk_moneda         = $forma_pago['fk_moneda'];
//                    if ($forma_pago['fk_moneda']==2){
                        $operacion->tasa_moneda = $tasa->tasa;
//                    }
                        $operacion->idUser = $request->cerrar_orden['idUser'];
                        $operacion->save();
                    }
                }

            }

            DB::commit();

            $response = [
                'msj' => 'Creado exitosamente' ,
                'bool' => true ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];

            return OrderBodyController::puntoventacredito($request , new Request($response));
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public
    function show($id)
    {
        $h = Tipo::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public
    function edit($id)
    {

    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public
    function update(Request $request , $id)
    {
        $body                  = json_decode($request->body);
        $cantidad_de_productos = count($body->items);
        $cantidad              = 3;
        if ($cantidad_de_productos >= 5){ //si hay mas de  productos en la orden entonces agrego un dia mas a los 3 por defecto
            $cantidad = 4;
        }

        $fecha_de_entrega = $this->retornar_dia_disponible_para_fechaEntrega($cantidad);

        $pass           = null;
        $recredenciales = null;
        $user           = null;
        $pass           = null;
        DB::beginTransaction();
        try {
            $tipo               = new OrderHeader();
            $tipo->fk_payform   = $request->fk_payform;
            $tipo->recibo_pago  = $request->recibo_pago;
            $tipo->notes        = $request->notes;
            $tipo->total_diseno = $request->total_diseno;
            $tipo->fk_moneda    = $request->fk_moneda;
            $tipo->fk_descuento = $request->fk_descuento;
            $tipo->fk_zonaEnvio = $request->fk_zonaEnvio;
            $tipo->precio_total = $request->precio_total;
            $tipo->fk_status    = $request->fk_status;

            $tipo->date_recived       = $request->date_recived;
            $tipo->date_delivery      = $fecha_de_entrega;
            $tipo->date_cancel        = $request->date_cancel;
            $tipo->fk_sucursal        = $request->fk_sucursal;
            $tipo->response_mercantil = $request->response_mercantil=='null' ? null : $request->response_mercantil;


            $tipo->tocken_url          = $request->tocken_url;
            $tipo->resumen_pago        = @$request->input('resumen_pago');
            $tipo->comprobante_factura = @$request->input('comprobante_factura');
            $tipo->enviar_a_taller     = @$request->input('enviar_a_taller');
//            $tipo->order_open   = $request->order_open;
            $tipo->order_open           = true;
            $tipo->tasa_moneda          = $request->tasa_moneda;
            $tipo->date_created         = now();
            $tipo->orden_creada_cliente = is_null(@$request->input('orden_creada_cliente')) ? 0 : @$request->input('orden_creada_cliente');

            if (is_null($request->idUser) || $request->idUser=='null'){
                $datas = json_decode($request->data_user);

                $rr = User::where('numerIdentificacion' , $datas->numerIdentificacion)->get();
                if (count($rr)==0){
                    $pass = Str::random(10);
//                    return response()->json(json_decode($request->data_user),404);
//                    $request->validate([
//                        'numerIdentificacion' => 'required|unique|users,numerIdentificacion,NULL,id,deleted_at,NULL' ,
//                        'email' => 'required|unique:users,email,NULL,id,deleted_at,NULL' ,
//                    ] , [
//                        'numerIdentificacion.required' => 'El número de identificación es requerida' ,
//                        'numerIdentificacion.unique' => 'El número de identificación es debe ser único' ,
//                        'email.unique' => 'Este email ya se encuentra en uso' ,
//                    ]);
                    $user                      = new User();
                    $user->email               = $datas->email;
                    $user->fullName            = $datas->fullName;
                    $user->numerIdentificacion = $datas->numerIdentificacion;
                    $user->phone               = $datas->phone;
                    $user->typeIdentification  = $datas->typeIdentification;
                    $user->user                = $datas->user;
                    $user->direccion           = @$datas->direccion;
//                    return response()->json($user , 404);
                    $user->profile_fk = 3; //como cliente
                    $user->name       = $datas->fullName;
                    $user->password   = bcrypt($pass);
                    $user->save();
                    $tipo->idUser = $user->id;
                } else {
                    if (count($rr) > 0){
                        $tipo->idUser = $rr[0]->id;
                    } else {
//                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario1' ,
                        ] , 404);
                    }

                }
            } else {
                if (!is_null($request->idUser) || $request->idUser!="null"){
                    if (User::find($request->idUser)){
                        $tipo->idUser = $request->idUser;
                    } else {
//                        DB::rollback();
                        return response()->json([
                            'msj' => 'No se encuentra el usuario2' ,
                        ] , 404);
                    }
                }
            }
            if ($request->exists('img_pago')){
                if (!is_null($request->img_pago)){
                    /*para la foto*/
                    $originalImage = $request->img_pago;

                    $thumbnailImage = Image::make($originalImage);
//                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                        $constraint->aspectRatio();
//                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\comprobantepago\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img_pago = $nombre_interno;
                }
            }
            $Numero_Pedido_max = OrderHeader::without(
                'interacciones' ,
                'estatus' ,
                'descuento' ,
                'usuario_creador' ,
                'usuario_lector' ,
                'customer' ,
                'zonaEnvio' ,
                'moneda_orderheader' ,
                'estadoOrden' ,
                'sucursal_orden' ,
                'body' ,
                'historiaStatus' ,
                'transaccion'
            )->withTrashed()->select('idOrderHeader')->orderby('idOrderHeader' , 'desc')->first();
            $numero_pedido     = null;
            if (is_null($Numero_Pedido_max)){
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-1';
            } else {
                $numero_pedido = 'N# ISP' . Carbon::now()->format('md') . '-' . ($Numero_Pedido_max->idOrderHeader + 1);
            }
            $tipo->n_order = $numero_pedido;

            $tipo->save();

//            DB::commit();
            $response = [
                'msj' => 'Creado exitosamente' ,
//                'pass_rand' => $pass ,
//                'credenciales' => $credenciales->original ,
                'user' => $user ,
                'pass' => $pass ,
                'orden' => OrderHeader::find($tipo->idOrderHeader) ,
            ];
//        $response['asa']="dsa";
//            return $response;
//            exit();
//            $response->append(['as'=>asdasd]);


            return OrderBodyController::store($request , new Request($response));
//            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en OrderHeader: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = OrderHeader::where('fk_status' , '=' , '16')
            ->where('idOrderHeader' , $id)
            ->get();
//        return response()->json(['msj' => $h] , 201);
        if (count($h)==0){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }
        $h[0]->delete();
        $h[0]->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}


