<?php

namespace App\Http\Controllers;

use App\Mail\ClaveTemporalMail;
use App\Mail\EnviarMailVallas;
use App\Mail\MailIslapixel;
use App\Mail\OrdenClienteMail;
use App\Models\Elemento;
use App\Models\ElementosOrden;
use App\Models\Galeria;
use App\Models\ImagenesProductos;
use App\Models\LogDataVallasBodyMail;
use App\Models\LogDataVallasMail;
use App\Models\Marca;
use App\Models\Modalidad;
use App\Models\Moneda;
use App\Models\Noticias;
use App\Models\OrderHeader;
use App\Models\Producto;
use App\Models\ProductosMasBuscados;
use App\Models\Promociones;
use App\Models\Rol;
use App\Models\ServiciosProducto;
use App\Models\SubTipo;
use App\Models\TasaMoneda;
use App\Models\TiemposDeEntrega;
use App\Models\Tipo;
use App\Models\TipoPersonal;
use App\Models\User;
use App\Models\ZonaEnvio;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Image;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class ProductoController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getTiemposDeEntrega()
    {
        $t = TiemposDeEntrega::get();
        return response()->json($t);
    }

    public function delete_imagenes_productos($idImagenesProductos)
    {
        $galeria = ImagenesProductos::find($idImagenesProductos);
        if (isset($galeria)){
            $galeria->delete();
        } else {
            $response = [
                'msj' => 'Foto no existe' ,
            ];
            return response()->json($response , 404);
        }
        $response = [
            'msj' => 'Foto eliminada correctamente' ,
        ];

        return response()->json($response , 200);
    }

    public function add_imagenes_productos(Request $request)
    {

        if ($request->exists('imgs')){
            foreach ($request->imgs as $img) {
                if (!is_null($img)){
                    /*para la foto*/
                    $originalImage = $img;

                    $thumbnailImage = Image::make($originalImage);
//                    return response()->json($thumbnailImage,404);
                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
                        $constraint->aspectRatio();
                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\producto\\galeria\\' . $nombre_interno , (string)$thumbnailImage->encode());

                    $imagen = new ImagenesProductos([
                        'nombre_imagen' => $nombre_interno ,
                        'fk_idProducto' => $request->idProducto
                    ]);
                    $imagen->save();
                }
            }
        }
        return response()->json(['msj' => 'Imagenes creadas exitosamente']);

    }

    public function buscar_todo(Request $request)
    {
        $palabra = "%" . $request->palabra . "%";
        if (!is_null($request->palabra)){
            $productos = null;
            $productos = Producto::where('name' , 'like' , $palabra)
                ->orWhere('descripcion' , 'like' , $palabra)
                ->orderby('name' , 'asc')
                ->get();
        }

        return response()->json($productos);
    }

    public function buscador_condicionado(Request $request)
    {
        $request->validate([
            'idPerfilUser' => 'required' ,
            'palabra' => 'required' ,
        ] , [
            'idPerfilUser.required' => ':attribute es requerido' ,
            'palabra.required' => ':attribute es requerido' ,
        ]);

        $idPerfilUser = $request->idPerfilUser;
        $palabra      = "%" . $request->palabra . "%";

        if (!is_null($idPerfilUser)){
            $perfil    = Rol::where('id_rol' , $idPerfilUser)->get();
            $productos = null;
            if (count($perfil) > 0){
                if ($idPerfilUser==3){
                    $productos = Producto::activo()
                        ->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4 , 6])
                        ->where('name' , 'like' , $palabra)
                        ->orderby('name' , 'asc')
                        ->get();
                } else {
                    $productos = Producto::activo()
                        ->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4 , 6])
                        ->where('name' , 'like' , $palabra)
                        ->orderby('name' , 'asc')
                        ->get();
                }
            } else {
                $productos = Producto::activo()
                    ->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4])
                    ->where('name' , 'like' , $palabra)
                    ->orderby('name' , 'asc')
                    ->get();
            }
        }

        //se agrega el término del cliente a los mas buscados
        $add                = new ProductosMasBuscados();
        $add->frase_buscada = $request->palabra;
        $add->save();


        return response()->json($productos);
    }

    public function buscador_condicionado_punto_de_venta(Request $request)
    {
        $request->validate([
            'palabra' => 'required' ,
        ] , [
            'palabra.required' => ':attribute es requerido' ,
        ]);

        $palabra = "%" . $request->palabra . "%";

        $productos = null;

        $productos = Producto::puntoventa()
            ->where('name' , 'like' , $palabra)
            ->where('fk_tipoModalidad' , '!=' , 5) //no materias primas
            ->orderby('name' , 'asc')
            ->get();

        //se agrega el término del cliente a los mas buscados
        $add                = new ProductosMasBuscados();
        $add->frase_buscada = $request->palabra;
        $add->save();

        return response()->json($productos);
    }


    public function index(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        return response()->json(
            Producto::without('elementos' , 'servicios')->orderby('name' , 'asc')
                ->offset($request->offset)
                ->limit($request->limit)
                ->get()
        );
    }

    //distinto de materia prima
    public function productosSinMateriaPrima()
    {
        return response()->json(
            Producto::without('elementos' , 'servicios')
                ->orderby('name' , 'asc')
                ->where('fk_tipoModalidad' , '!=' , 5) //todos menos los configurables
                ->get()
        );
    }

    public function enviarMail(Request $request)
    {
        $nombre  = $request->nombre;
        $email   = $request->email;
        $mensaje = $request->mensaje;

        Mail::to('islapixel@gmail.com')->send(new MailIslapixel($nombre , $email , $mensaje));
    }

    public function enviarMailVallas(Request $request)
    {
        $nombre        = $request->nombre;
        $email         = $request->email;
        $mensaje       = $request->mensaje;
        $observacion   = $request->observacion;
        $numero        = @explode("-" , @explode("Contacto:" , $mensaje)[1])[0];
        $numero_vallas = @trim(@explode("Contacto:" , explode("Cant.: " , $mensaje)[1])[0]);
        $is_debug      = true;

        // envio de sms
        SmsController::sendSMS(new Request([
            "destination" => $is_debug ? '04120822226' : env('NUMERO_ENVIO_SMS_ISLA_ARNALDO') ,
            "message" => 'Me gustaría cotizar VallaPixel, Nombre: ' . strtoupper($nombre) . ', Contacto: ' . $numero . ' - ' . $email . ', ' . $numero_vallas . ' vallas' ,
        ]));

        $header = new LogDataVallasMail([
            "nombre" => $nombre ,
            "email" => $email ,
            "mensaje" => $mensaje ,
            "observacion" => $observacion ,
            "numero" => $numero ,
            "cantidad_vallas" => $numero_vallas ,
            "n_documento" => @$request->n_documento
        ]);

        $header->save();

        if (count($request->vallas) > 0){
            foreach ($request->vallas as $valla) {
                $body = new LogDataVallasBodyMail([
                    'fk_idVallita' => $valla['idVallita'] ,
                    'fk_idHeader' => $header->idLogDataMailVallas ,
                    'descripcion' => $valla['descripcion'] ,
                    'dosCaras' => $valla['dosCaras'] ,
                    'unaCara' => $valla['unaCara'] ,
                    'observacion' => $valla['observacion']
                ]);
                $body->save();
            }
        }

        // fin envio sms

//        Mail::to('arnaldomoros@gmail.com')
//            ->send(new EnviarMailVallas($nombre , $email , $mensaje));
    }

    public function enviarMailPrueba(Request $request)
    {
        $nombre     = $request->nombre;
        $email      = $request->email;
        $mensaje    = $request->mensaje;
        $string     = 'alecortez240192@gmail.com';
        $string     = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
        $resultado  = stristr($string , 'generico');
        $resultado2 = stristr($string , 'noposee');
        if ($resultado!==false && $resultado2!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
        } else {
            Mail::to('alecortez240192@gmail.com')->send(new MailIslapixel($nombre , $email , $mensaje));
        }

    }

    public function enviarMailOrden(Request $request)
    {
        $idOrdenHeader = $request->idOrdenHeader;
        $email         = $request->email;
        //busco la ordenK
        $orden = OrderHeader::where('idOrderHeader' , $idOrdenHeader)
            ->where('orden_creada_cliente' , true)
            ->get();
        if (count($orden) > 0){
            Mail::to('alecortez240192@gmail.com')->send(new OrdenClienteMail($orden[0]->n_order , $email));
        }

    }

    //recibe el id de perfil, si el perfil es cliente entonces queda igual, si no, retornar los productos por unidad que esten apagados para la web
    public function listar_producto_condicionda($idPerfilUser = null)
    {
        if (!is_null($idPerfilUser)){
            $perfil    = Rol::where('id_rol' , $idPerfilUser)->get();
            $productos = null;
            if (count($perfil) > 0){
                if ($idPerfilUser==3 && $idPerfilUser==0){
                    $productos = Producto::activo()->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4])->orderby('name' , 'asc')->get();
                } else {
                    $productos = Producto::activo()->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4])->orderby('name' , 'asc')->get();
                }
            }
        }

        return response()->json($productos);
    }

    public function listar_producto_condicionda_paginador(Request $request , $idPerfilUser = null)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        if (!is_null($idPerfilUser)){
            $perfil    = Rol::where('id_rol' , $idPerfilUser)->get();
            $productos = null;
            if (count($perfil) > 0){
                if ($idPerfilUser==3 && $idPerfilUser==0){
                    $productos = Producto::activo()
                        ->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4])
                        ->orderby('name' , 'asc')
                        ->offset($request->offset)
                        ->limit($request->limit)
                        ->get();
                } else {
                    $productos = Producto::activo()
                        ->wherein('fk_tipoModalidad' , [3 , 1 , 2 , 4])
                        ->orderby('name' , 'asc')
                        ->offset($request->offset)
                        ->limit($request->limit)
                        ->get();
                }
            }
        }

        return response()->json($productos);
    }

    public function home()
    {    //listado de galerias
//        $primero             = Producto::whereIn('idProducto' , [501 , 502])->get(); //colocar de primero en el listado
//        $productosDestacados = Producto::activo()->inRandomOrder()->where('destacar' , 1)->orderby('name' , 'asc')->get(); //luego estos en forma aleatoria
//        foreach ($productosDestacados as $item) {
//            $primero->push($item);
//        }
//        $productosDestacados = $primero->unique(); //quitando duplicados

        $productosDestacados = Producto::activo()->inRandomOrder()->where('destacar' , 1)->orderby('name' , 'asc')->limit(20)->get();


        $productosPorCotizar = Producto::activo()->inRandomOrder()->where('fk_tipoModalidad' , 6)->orderby('name' , 'asc')->limit(16)->get();
        $galerias            = Galeria::limit(16)->get();
        $marcas              = Marca::limit(16)->get();
        $noticias            = Noticias::limit(16)->get();
        $promociones         = Promociones::limit(16)->get();
        $tipos               = Tipo::where('activar_para_web' , true)->with('subtipo')->get();
        return response()->json([
            'productosDestacados' => $productosDestacados ,
            'galeria' => $galerias ,
            'marcas' => $marcas ,
            'noticias' => $noticias ,
            'promociones' => $promociones ,
            'productosPorCotizar' => $productosPorCotizar ,
            'tipos' => $tipos
        ]);
    }

    public function recuperarProducto($idProducto)
    {
        Producto::withTrashed()->where('idProducto' , $idProducto)->restore();

    }

    public function getTodosSeedProductos()
    {
        $monedas                = Moneda::active()->get();
        $tipos                  = Tipo::all();
        $subTipos               = SubTipo::all();
        $modalidades            = Modalidad::orderBy('descripcion' , 'asc')->get();
        $productosParaServicios = DB::table('productos')->where('fk_tipoModalidad' , 4)->get();
        $productosParaElementos = DB::table('productos')->wherein('fk_tipoModalidad' , [1 , 3 , 5])->get();
        $tipoPerosnalAgrupado   = TipoPersonal::with('personal')->get();
//        $zonas_envio            = ZonaEnvio::active()->get();

        $idsMoneda = Moneda::active()->select('idMoneda')->groupBy('idMoneda')->get()->toArray();
        foreach ($idsMoneda as $key => $item) {
            $idMoneda                     = $item['idMoneda'];
            $tasa_moneda                  = TasaMoneda::active()->where('fk_moneda' , $idMoneda)->orderBy('created_at' , 'desc')->first();
            $monedas[$key]['tasa_moneda'] = $tasa_moneda;
//                $todo[] = ['idMoneda' => $idMoneda , 'tasa_moneda' => $tasa_moneda];

        }

        return response()->json([
            'monedas' => $monedas ,
            'tipos' => $tipos ,
            'subTipos' => $subTipos ,
            'modalidades' => $modalidades ,
            'productosParaServicios' => $productosParaServicios ,
            'productosParaElementos' => $productosParaElementos ,
            'tipoPerosnalAgrupado' => $tipoPerosnalAgrupado ,
//            'zonas_envio' => $zonas_envio ,
        ] , 200);

    }

    public function indexSinAutenticar()
    {
        return response()->json(Producto::activo()->where('fk_tipoModalidad' , '!=' , 5)->where('fk_tipoModalidad' , '!=' , 6)->get());
    }

    public function editar_especificaciones_del_producto(Request $request)
    {
        $request->validate([
            'idProducto' => 'required' ,
            'especificaciones_del_producto' => 'required' ,
        ] , [
            'idProducto.required' => ':attribute es requerido' ,
            'especificaciones_del_producto.required' => ':attribute es requerido' ,
        ]);
        $producto = Producto::find($request->idProducto);
        if ($producto){
            $producto->update(
                ['especificaciones_del_producto' => $request->especificaciones_del_producto]
            );
        }
        return response()->json(['msj' => 'Especificacaiones del producto actualizada correctamente']);
    }

    public function porId($id)
    {
        $todo          = Producto::where('idProducto' , $id)->get();
        $zonas_envio   = ZonaEnvio::active()->get();
        $tipo_personal = TipoPersonal::with('personal')->get();
        foreach ($todo as $key => $item) {
//            return response()->json($item['fk_subType']);
            $relaciones                   = Producto::activo()->where('fk_subType' , $item['fk_subType'])
                ->where('idProducto' , '!=' , $item['idProducto']) //que no sea igual al que esta consultado
                ->where('fk_tipoModalidad' , '!=' , 5) //distinto de 5
                ->where('destacar' , 1) //que sea destacado
                ->limit(4) //solo 4 productos
                ->get();
            $todo[$key]->prodRelacionados = $relaciones;
        }
        return response()->json(['productos' => $todo[0] , 'zona_envio' => $zonas_envio , 'tipo_personal' => $tipo_personal]);
    }

    public function getProductosPorSubTipo(Request $request)
    {
        return response()->json(
            Producto::activo()
                ->puntoventa()
                ->where('fk_subType' , $request->fk_subType)
                ->where('fk_subType' , $request->fk_subType)
                ->get()
        );
    }

    public
    function getProductosParaServicios()
    {
        return response()->json(Producto::activo()->where('fk_tipoModalidad' , 4)->get());
    }

    public
    function getProductosParaElementos()
    {
        return response()->json(Producto::activo()->where('fk_tipoModalidad' , 5)->get());
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public
    function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public
    function store(Request $request)
    {
//       return response()->json($request->all(),404);


        /*Tener presente lo siguiente */
        /**
         *
         * (4) Los productos tipo servicio son los asociados a un producto,ni servicio ni elementos
         * (1) Los productos Estandar tendras los elementos y servicios
         * (2) Los productos configurables tendran los elementos y servicios (se activa el removible de elementos o no)
         * (3) Los productos por unidad, ni servicio ni elementos
         * (5) Los productos por materia prima (procesar) no tendran ni elementos ni servicios
         *
         */
        $request->validate([
            'descripcion' => 'required' ,
            'fk_subType' => 'required' ,
            'precio' => 'required' ,
            'fk_moneda' => 'required' ,
            'destacar' => 'required' ,
            'fk_tipoModalidad' => 'required' ,
            'name' => 'required|unique:productos,name,null,idProducto,deleted_at,NULL' ,
        ] , [
            'name.unique' => 'Este producto :input ya se encuentra registrado' ,
            'name.required' => ':attribute es requerido' ,
            'descripcion.required' => ':attribute es requerido' ,
            'fk_subType.required' => ':attribute es requerido' ,
            'precio.required' => ':attribute es requerido' ,
            'fk_moneda.required' => ':attribute es requerido' ,
            'destacar.required' => ':attribute es requerido' ,
            'fk_tipoModalidad.required' => ':attribute es requerido' ,
        ]);


        DB::beginTransaction();

        try {
            $response                   = new \stdClass();
            $response_errors_particular = new \stdClass();

            $producto = new Producto($request->all());
            if ($request->fk_idTipopersonal=='null'){
                $producto->fk_idTipopersonal = null;
            }
//            $producto->elaborated    = (boolean)$request->elaborated;
//            $producto->materia_prima = (boolean)$request->materia_prima;
            $producto->destacar  = (boolean)$request->destacar;
            $producto->fk_status = 1; //activo
            $response->msj       = 'Producto creado exitosamente';

            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);
                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
                        $constraint->aspectRatio();
                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\producto\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $producto->img = $nombre_interno;
                }
            }

            $producto->save();

//            if ($request->exists('imgs')){
//                foreach ($request->imgs as $img) {
//                    if (!is_null($img)){
//                        /*para la foto*/
//                        $originalImage = $img;
//
//                        $thumbnailImage = Image::make($originalImage);
//                        $thumbnailImage->fit(2048 , 2048 , function ($constraint){
//                            $constraint->aspectRatio();
//                        });
//                        $nombre_publico = $originalImage->getClientOriginalName();
//                        $extension      = 'png';
//                        //$extension = $originalImage->getClientOriginalExtension();
//                        $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
//                        $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
//                        Storage::disk('public')->put('\\producto\\galeria\\' . $nombre_interno , (string)$thumbnailImage->encode());
//
//                        $imagen = new ImagenesProductos([
//                            'nombre_imagen' => $nombre_interno ,
//                            'fk_idProducto' => $producto->idProducto
//                        ]);
//                        $imagen->save();
//                    }
//                }
//            }
            $array_preventivo  = [];
            $array_preventivo2 = [];
            $salida            = false;


            if ($request->fk_tipoModalidad==1 || $request->fk_tipoModalidad==2){ //si es un producto elaborado entonces obtengo cuales son los ingredientes
                if ($request->exists('elementos')){
                    if (count($request->elementos) > 0){
//                        return response()->json(json_decode($request->elementos[0])->cantidad);
                        foreach ($request->elementos as $elemento) {
                            $elemento = json_decode($elemento);
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $elemento->idProducto)->wherein('fk_tipoModalidad' , [1 , 3 , 5])->get();
                            if (count($con) > 0){
                                Elemento::create([
                                    "fk_producto" => $elemento->idProducto ,
                                    "cantidad" => $elemento->cantidad ,
                                    "precio" => $elemento->precio ,
                                    "removible" => (boolean)$elemento->removible ,
                                    "product_name" => $elemento->product_name ,
                                    "total" => $elemento->total ,
                                    "fk_idProducto" => $producto->idProducto ,
                                    "sum_ancho_bool" => $elemento->sum_ancho_bool ,
                                    "sum_alto_bool" => $elemento->sum_alto_bool ,
                                    "div_x_cm" => $elemento->div_x_cm ,
                                    "calcular_cm" => $elemento->calcular_cm ,
                                    "sobrante" => (boolean)$elemento->sobrante ,
                                    "instalacion" => (boolean)$elemento->instalacion ,
                                ]);
                            } else {
                                array_push($array_preventivo , $elemento->idProducto);
                            }
                        }
                    }
                }

                if ($request->exists('servicios')){
                    if (count($request->servicios) > 0){
                        foreach ($request->servicios as $servicio) {
                            $servicio = json_decode($servicio);
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $servicio->idProducto)->where('fk_tipoModalidad' , 4)->get();
                            if (count($con) > 0){
                                ServiciosProducto::create([
                                    "fk_producto" => $servicio->idProducto ,
                                    "product_name" => $servicio->product_name ,
                                    "precio" => $servicio->precio ,
                                    "cantidad" => $servicio->cantidad ,
                                    "total" => $servicio->total ,
                                    "removible" => (boolean)$servicio->removible ,
                                    "fk_idProducto" => $producto->idProducto
                                ]);
                            } else {
                                array_push($array_preventivo2 , $servicio->idProducto);
                            }
                        }
                    }
                }

                if (count($array_preventivo2) > 0){
                    $response_errors_particular->productos_no_existen_como_servicios = Producto::find($array_preventivo2);
                    $salida                                                          = true;
                }
            }
            if (count($array_preventivo) > 0){
                $response_errors_particular->productos_no_existen_como_elementos = Producto::find($array_preventivo);
                $salida                                                          = true;
            };
            if ($salida){
                DB::rollback();
                return response()->json(
                    [
                        "message" => "Los Datos son invalidos" ,
                        "errors" => [
                            $response_errors_particular
                        ]
                    ] , 422);
            }

            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en ProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,] , 500);
        }
    }

    public
    function store2(Request $request)
    {
//        return response()->json($request->all(),404);
//       return response()->json(json_decode($request->elementos[0])->cantidad);


        /*Tener presente lo siguiente */
        /**
         *
         * (4) Los productos tipo servicio son los asociados a un producto,ni servicio ni elementos
         * (1) Los productos Estandar tendras los elementos y servicios
         * (2) Los productos configurables tendran los elementos y servicios (se activa el removible de elementos o no)
         * (3) Los productos por unidad, ni servicio ni elementos
         * (5) Los productos por materia prima (procesar) no tendran ni elementos ni servicios
         *
         */
        $request->validate([
            'descripcion' => 'required' ,
            'fk_subType' => 'required' ,
            'precio' => 'required' ,
            'fk_moneda' => 'required' ,
            'destacar' => 'required' ,
            'fk_tipoModalidad' => 'required' ,
            'name' => 'required|unique:productos,name,null,idProducto,deleted_at,NULL' ,
        ] , [
            'name.unique' => 'Este producto :input ya se encuentra registrado' ,
            'name.required' => ':attribute es requerido' ,
            'descripcion.required' => ':attribute es requerido' ,
            'fk_subType.required' => ':attribute es requerido' ,
            'precio.required' => ':attribute es requerido' ,
            'fk_moneda.required' => ':attribute es requerido' ,
            'destacar.required' => ':attribute es requerido' ,
            'fk_tipoModalidad.required' => ':attribute es requerido' ,
        ]);


        DB::beginTransaction();

        try {
            $response                   = new \stdClass();
            $response_errors_particular = new \stdClass();

            $producto = new Producto($request->all());
//            $producto->elaborated    = (boolean)$request->elaborated;
//            $producto->materia_prima = (boolean)$request->materia_prima;
            $producto->destacar  = (boolean)$request->destacar;
            $producto->fk_status = 1; //activo
            $response->msj       = 'Producto creado exitosamente';

            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);
                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
                        $constraint->aspectRatio();
                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\producto\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $producto->img = $nombre_interno;
                }
            }
            $producto->save();
            $array_preventivo  = [];
            $array_preventivo2 = [];
            $salida            = false;


            if ($request->fk_tipoModalidad==1 || $request->fk_tipoModalidad==2){ //si es un producto elaborado entonces obtengo cuales son los ingredientes
                if ($request->exists('elementos')){
                    if (count($request->elementos) > 0){
//                        return response()->json(json_decode($request->elementos[0])->cantidad);
                        foreach ($request->elementos as $elemento) {
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $elemento['idProducto'])->wherein('fk_tipoModalidad' , [1 , 3 , 5])->get();
                            if (count($con) > 0){
                                Elemento::create([
                                    "fk_producto" => $elemento['idProducto'] ,
                                    "cantidad" => $elemento['cantidad'] ,
                                    "precio" => $elemento['precio'] ,
                                    "removible" => (boolean)$elemento['removible'] ,
                                    "product_name" => $elemento['product_name'] ,
                                    "total" => $elemento['total'] ,
                                    "fk_idProducto" => $producto['idProducto'] ,
                                    "sum_ancho_bool" => $elemento['sum_ancho_bool'] ,
                                    "sum_alto_bool" => $elemento['sum_alto_bool'] ,
                                    "div_x_cm" => $elemento['div_x_cm'] ,
                                    "calcular_cm" => $elemento['calcular_cm'] ,
                                    "sobrante" => (boolean)$elemento['sobrante'] ,
                                    "instalacion" => (boolean)$elemento['instalacion'] ,
                                ]);
                            } else {
                                array_push($array_preventivo , $elemento['idProducto']);
                            }
                        }
                    }
                }

                if ($request->exists('servicios')){
                    if (count($request->servicios) > 0){
                        foreach ($request->servicios as $servicio) {
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $servicio['idProducto'])->where('fk_tipoModalidad' , 4)->get();
                            if (count($con) > 0){
                                ServiciosProducto::create([
                                    "fk_producto" => $servicio['idProducto'] ,
                                    "product_name" => $servicio['product_name'] ,
                                    "precio" => $servicio['precio'] ,
                                    "cantidad" => $servicio['cantidad'] ,
                                    "total" => $servicio['total'] ,
                                    "removible" => (boolean)$servicio['removible'] ,
                                    "fk_idProducto" => $producto['idProducto']
                                ]);
                            } else {
                                array_push($array_preventivo2 , $servicio['idProducto']);
                            }
                        }
                    }
                }

                if (count($array_preventivo2) > 0){
                    $response_errors_particular->productos_no_existen_como_servicios = Producto::find($array_preventivo2);
                    $salida                                                          = true;
                }
            }
            if (count($array_preventivo) > 0){
                $response_errors_particular->productos_no_existen_como_elementos = Producto::find($array_preventivo);
                $salida                                                          = true;
            };
            if ($salida){
                DB::rollback();
                return response()->json(
                    [
                        "message" => "Los Datos son invalidos" ,
                        "errors" => [
                            $response_errors_particular
                        ]
                    ] , 422);
            }

            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en ProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param \App\Models\Producto $producto
     * @return \Illuminate\Http\Response
     */
    public
    function show(Producto $producto)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param \App\Models\Producto $producto
     * @return \Illuminate\Http\Response
     */
    public
    function edit(Producto $producto)
    {
        //
    }

    public
    function image_fix_orientation($file , $img_url)
    {
        $data = Image::make($file)->exif();
        //return $data;
        if (!empty($data['Orientation'])){
            $image = imagecreatefromjpeg($file);

            return $image;
            switch ($data['Orientation']) {
                case 3:
                    $image = imagerotate($image , 180 , 0);
                    break;

                case 6:
                    $image = imagerotate($image , -90 , 0);
                    break;

                case 8:
                    $image = imagerotate($image , 90 , 0);
                    break;
            }

            return '  we';
            //return $image;
            exit();

            imagejpeg($image , $img_url , 90);
        }

    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param \App\Models\Producto $producto
     * @return \Illuminate\Http\Response
     */
    public
    function update(Request $request , $idProducto)
    {
//        return response()->json($request->all());
        $producto = Producto::where('idProducto' , $idProducto)->get();
        if (count($producto) > 0){
            $producto = $producto[0];
        } else {
            return response()->json(
                [
                    "message" => "El producto no existe" ,
                ] , 404);
        }
        DB::beginTransaction();

        try {
            $response                   = new \stdClass();
            $response_errors_particular = new \stdClass();

            $producto->fill($request->all());
//            $producto->elaborated    = (boolean)$request->elaborated;
//            $producto->materia_prima = (boolean)$request->materia_prima;
            $producto->destacar  = (boolean)$request->destacar;
            $producto->fk_status = 1; //activo
            $response->msj       = 'Producto actualizado exitosamente';

            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);
                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
                        $constraint->aspectRatio();
                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\producto\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $producto->img = $nombre_interno;
                }
            }
            $producto->save();
            $array_preventivo  = [];
            $array_preventivo2 = [];
            $salida            = false;


            if ($request->fk_tipoModalidad==1 || $request->fk_tipoModalidad==2){ //si es un producto elaborado entonces obtengo cuales son los ingredientes
                if ($request->exists('elementos')){
                    $idElementos       = Elemento::select('idElemento')->where('fk_idProducto' , $idProducto)->without('producto')->pluck('idElemento')->toArray();
                    $array_idElementos = [];
                    if (count($request->elementos) > 0){
                        foreach ($request->elementos as $elemento) {
                            $elemento = json_decode($elemento);
//                            return response()->json($elemento,404);
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $elemento->fk_producto)->wherein('fk_tipoModalidad' , [1 , 3 , 5])->get();
                            if (count($con) > 0){
                                if (!is_null($elemento->idElemento)){
                                    $rr = Elemento::find($elemento->idElemento);
                                    if ($rr){
                                        $rr->update([
                                            "fk_producto" => $elemento->fk_producto ,
                                            "cantidad" => $elemento->cantidad ,
                                            "precio" => $elemento->precio ,
                                            "removible" => (boolean)$elemento->removible ,
                                            "product_name" => $elemento->product_name ,
                                            "total" => $elemento->total ,
//                                            "fk_idProducto" => $idProducto ,
                                            "sum_ancho_bool" => $elemento->sum_ancho_bool ,
                                            "sum_alto_bool" => $elemento->sum_alto_bool ,
                                            "div_x_cm" => $elemento->div_x_cm ,
                                            "calcular_cm" => $elemento->calcular_cm ,
                                            "sobrante" => $elemento->sobrante ,
                                            "instalacion" => $elemento->instalacion ,
                                        ]);
                                        array_push($array_idElementos , $elemento->idElemento);
                                    }
//                                    return response()->json([$idElementos,$array_idElementos] , 404);
                                } else {
                                    Elemento::create([
                                        "fk_producto" => $elemento->fk_producto ,
                                        "cantidad" => $elemento->cantidad ,
                                        "precio" => $elemento->precio ,
                                        "removible" => (boolean)$elemento->removible ,
                                        "product_name" => $elemento->product_name ,
                                        "total" => $elemento->total ,
                                        "fk_idProducto" => $idProducto ,
                                        "sum_ancho_bool" => $elemento->sum_ancho_bool ,
                                        "sum_alto_bool" => $elemento->sum_alto_bool ,
                                        "div_x_cm" => $elemento->div_x_cm ,
                                        "calcular_cm" => $elemento->calcular_cm ,
                                        "sobrante" => $elemento->sobrante ,
                                        "instalacion" => $elemento->instalacion ,
                                    ]);
                                }
                            } else {
//                                return response()->json($elemento,404);
                                array_push($array_preventivo , $elemento->fk_producto);
                            }
                        }
                        $diff = array_diff($idElementos , $array_idElementos);
//                                    return response()->json($diff , 404);
                        Elemento::destroy($diff);
                    }
                } else {
                    /*Borro los campos antes de reemplazarlos*/
                    $t                 = Elemento::select('idElemento')->where('fk_idProducto' , $idProducto)->without('producto')->get()->toArray();
                    $elementos_ordenes = ElementosOrden::wherein('fk_elemento' , $t)->get();
                    if (count($elementos_ordenes) > 0){
                        return response()->json("No se pueden eliminar los elementos" , 404);
                    } else {
                        Elemento::destroy($t);
                    }
                }

                if ($request->exists('servicios')){
                    $idServicios              = ServiciosProducto::select('idServicioProducto')->where('fk_idProducto' , $idProducto)->without('producto')->pluck('idServicioProducto')->toArray();
                    $array_idServicioProducto = [];
                    if (count($request->servicios) > 0){
                        foreach ($request->servicios as $servicio) {
                            $servicio = json_decode($servicio);
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $servicio->fk_producto)->where('fk_tipoModalidad' , 4)->get();
                            if (count($con) > 0){
                                $rr = ServiciosProducto::find($servicio->idServicioProducto);
                                if ($rr){
                                    $rr->update([
                                        "fk_producto" => $servicio->fk_producto ,
                                        "product_name" => $servicio->product_name ,
                                        "precio" => $servicio->precio ,
                                        "cantidad" => $servicio->cantidad ,
                                        "total" => $servicio->total ,
                                        "removible" => (boolean)$servicio->removible ,
                                        "fk_idProducto" => $idProducto ,
                                    ]);
                                    array_push($array_idServicioProducto , $servicio->idServicioProducto);
                                } else {
                                    ServiciosProducto::create([
                                        "fk_producto" => $servicio->fk_producto ,
                                        "product_name" => $servicio->product_name ,
                                        "precio" => $servicio->precio ,
                                        "cantidad" => $servicio->cantidad ,
                                        "total" => $servicio->total ,
                                        "removible" => (boolean)$servicio->removible ,
                                        "fk_idProducto" => $idProducto ,
                                    ]);
                                }
                            } else {
                                array_push($array_preventivo2 , $servicio->idProducto);
                            }
                        }
                        $diff = array_diff($idServicios , $array_idServicioProducto);
                        ServiciosProducto::destroy($diff);
                    }
                } else {
                    /*Borro los campos antes de reemplazarlos*/
                    $t = ServiciosProducto::select('idServicioProducto')->where('fk_idProducto' , $idProducto)->without('producto')->get()->toArray();
                    ServiciosProducto::destroy($t);
                }

                if (count($array_preventivo2) > 0){
                    $response_errors_particular->productos_no_existen_como_servicios = Producto::find($array_preventivo2);
                    $salida                                                          = true;
                }
            }
            if (count($array_preventivo) > 0){
                $response_errors_particular->productos_no_existen_como_elementos = Producto::find($array_preventivo);
                $salida                                                          = true;
            };
            if ($salida){
                DB::rollback();
                return response()->json(
                    [
                        "message" => "Los Datos son invalidos" ,
                        "errors" => [
                            $response_errors_particular
                        ]
                    ] , 422);
            }
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en ProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,
                'todo' => $e->getTrace()] , 500);
        }
    }

    public
    function update2(Request $request , $idProducto)
    {
        $producto = Producto::where('idProducto' , $idProducto)->get();
        if (count($producto) > 0){
            $producto = $producto[0];
        } else {
            return response()->json(
                [
                    "message" => "El producto no existe" ,
                ] , 404);
        }
        DB::beginTransaction();

        try {
            $response                   = new \stdClass();
            $response_errors_particular = new \stdClass();

            $producto->fill($request->all());
//            $producto->elaborated    = (boolean)$request->elaborated;
//            $producto->materia_prima = (boolean)$request->materia_prima;
            $producto->destacar  = (boolean)$request->destacar;
            $producto->fk_status = 1; //activo
            $response->msj       = 'Producto actualizado exitosamente';

            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);
                    $thumbnailImage->fit(2048 , 2048 , function ($constraint){
                        $constraint->aspectRatio();
                    });
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\producto\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $producto->img = $nombre_interno;
                }
            }
            $producto->save();
            $array_preventivo  = [];
            $array_preventivo2 = [];
            $salida            = false;


            if ($request->fk_tipoModalidad==1 || $request->fk_tipoModalidad==2){ //si es un producto elaborado entonces obtengo cuales son los ingredientes
                if ($request->exists('elementos')){
                    /*Borro los campos antes de reemplazarlos*/
                    $t = Elemento::select('idElemento')->where('fk_idProducto' , $idProducto)->without('producto')->get()->toArray();
                    Elemento::destroy($t);
                    if (count($request->elementos) > 0){
                        foreach ($request->elementos as $elemento) {
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $elemento['idProducto'])->wherein('fk_tipoModalidad' , [1 , 3 , 5])->get();
                            if (count($con) > 0){
                                Elemento::create([
                                    "fk_producto" => $elemento['idProducto'] ,
                                    "cantidad" => $elemento['cantidad'] ,
                                    "precio" => $elemento['precio'] ,
                                    "removible" => (boolean)$elemento['removible'] ,
                                    "product_name" => $elemento['product_name'] ,
                                    "total" => $elemento['total'] ,
                                    "fk_idProducto" => $idProducto ,
                                    "sum_ancho_bool" => $elemento['sum_ancho_bool'] ,
                                    "sum_alto_bool" => $elemento['sum_alto_bool'] ,
                                    "div_x_cm" => $elemento['div_x_cm'] ,
                                    "calcular_cm" => $elemento['calcular_cm'] ,
                                    "sobrante" => $elemento['sobrante'] ,
                                    "instalacion" => $elemento['instalacion'] ,
                                ]);
                            } else {
                                array_push($array_preventivo , $elemento['idProducto']);
                            }
                        }
                    }
                }

                if ($request->exists('servicios')){
                    /*Borro los campos antes de reemplazarlos*/
                    $t = ServiciosProducto::select('idServicioProducto')->where('fk_idProducto' , $idProducto)->without('producto')->get()->toArray();;
                    ServiciosProducto::destroy($t);
                    if (count($request->servicios) > 0){
                        foreach ($request->servicios as $servicio) {
                            //busco el producto si existe
                            $con = Producto::where('idProducto' , $servicio['idProducto'])->where('fk_tipoModalidad' , 4)->get();
                            if (count($con) > 0){
                                ServiciosProducto::create([
                                    "fk_producto" => $servicio['idProducto'] ,
                                    "product_name" => $servicio['product_name'] ,
                                    "precio" => $servicio['precio'] ,
                                    "cantidad" => $servicio['cantidad'] ,
                                    "total" => $servicio['total'] ,
                                    "removible" => (boolean)$servicio['removible'] ,
                                    "fk_idProducto" => $idProducto ,
                                ]);
                            } else {
                                array_push($array_preventivo2 , $servicio['idProducto']);
                            }
                        }
                    }
                }

                if (count($array_preventivo2) > 0){
                    $response_errors_particular->productos_no_existen_como_servicios = Producto::find($array_preventivo2);
                    $salida                                                          = true;
                }
            }
            if (count($array_preventivo) > 0){
                $response_errors_particular->productos_no_existen_como_elementos = Producto::find($array_preventivo);
                $salida                                                          = true;
            };
            if ($salida){
                DB::rollback();
                return response()->json(
                    [
                        "message" => "Los Datos son invalidos" ,
                        "errors" => [
                            $response_errors_particular
                        ]
                    ] , 422);
            }
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en ProductoController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json(['message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
                'linea' => $e->getLine() ,] , 500);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param \App\Models\Producto $producto
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $producto = Producto::find($id);
        if (isset($producto)){
            $producto->delete();
        } else {
            $response = [
                'msj' => 'Producto no existe' ,
            ];
            return response()->json($response , 404);
        }
        $response = [
            'msj' => 'Producto eliminado correctamente' ,
        ];

        return response()->json($response , 200);
    }
}
