<?php

namespace App\Http\Controllers;

use App\Jobs\GenerarPdfCotizacionJob;
use App\Jobs\GenerarPdfJob;
use App\Jobs\GenerarPdfOrdenJob;
use App\Mail\DestacadosMail;
use App\Mail\PdfOrdenNuevo;
use App\Mail\PromocionesMail;
use App\Models\Producto;
use App\Models\Promociones;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Image;

class PromocionesController extends Controller
{
    public function index()
    {
        return Promociones::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
//            'descripcion' => 'required' ,
//             'img' => 'required' ,
        ] , [
//            'descripcion.required' => ':attribute es requerido' ,
//            'img.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new Promociones($request->all());
            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);

                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\promociones\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $tipo->img = $nombre_interno;
                }
            }
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en PersonalController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Promociones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
//            'descripcion' => 'required' ,
//            'img' => 'required' ,
        ] , [
//            'descripcion.required' => ':attribute es requerido' ,
//            'img.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = Promociones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el tipo de personal solicitado'] , 401);
        }

        try {
            $h->fill($request->all());
            if ($request->exists('img')){
                if (!is_null($request->img)){
                    /*para la foto*/
                    $originalImage = $request->img;

                    $thumbnailImage = Image::make($originalImage);

                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    //$extension = $originalImage->getClientOriginalExtension();
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\promociones\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/

                    $h->img = $nombre_interno;
                }
            }
            $h->save();
            $response = [
                'msj' => 'Promocion actualizada exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TipoPersonalController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Promociones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }

    public static function enviarMailPromociones()
    {
        $prodPromo = Producto::where('is_offer' , true)
            ->get();
        $clientes  = DB::table('users')->select('email')->where('profile_fk' , 3)->get()->toArray();
//        return response()->json($prodPromo);
        //modo prueba
        Mail::to(['alecortez240192@gmail.com','banezcak27@gmail.com'])->send(new PromocionesMail($prodPromo));
    }
    public static function enviarMailDestacados()
    {
        $prodPromo = Producto::where('destacar' , true)
            ->get();
        $clientes  = DB::table('users')->select('email')->where('profile_fk' , 3)->get()->toArray();
//        return response()->json($prodPromo);
        //modo prueba
        Mail::to(['alecortez240192@gmail.com','banezcak27@gmail.com'])->send(new DestacadosMail($prodPromo));
    }
    public static function enviarMailOrdenNuevo($idOrden=19821)
    {
//        $idOrden=21595;
        GenerarPdfOrdenJob::dispatch($idOrden);
    }
    public static function enviarMailCotizacionNuevo($idOrden=19821)
    {
//        $idOrden=21595;
        GenerarPdfCotizacionJob::dispatch($idOrden);
    }
}
