<?php

namespace App\Http\Controllers;

use App\Exports\OperacionesExport;
use App\Exports\OperacionesSeniatExport;
use App\Exports\OrdenesPorAsesorExport;
use App\Exports\OrdenesPorEntregarExport;
use App\Exports\ReporteConciliacion;
use App\Exports\ReporteIngresosMensual;
use App\Exports\ReporteOrdenesClienteExport;
use App\Exports\ReporteOrdenesDiarioExport;
use App\Exports\ReporteOrdenesFacturadas;
use App\Exports\ReporteOrdenesPagadas;
use App\Exports\ReporteOrdenesPagadasNoFacturadas;
use App\Exports\ReporteOrdenesPagadasSinIgtf;
use App\Models\Operacion;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class ReportesController extends Controller
{
    public function reporte_ordenes_facturadas(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeaderNecesarioReporte')
            ->whereHas('orderHeaderNecesarioReporte' , function ($sql) use ($request , $año , $mes){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->where('fk_sucursal' , $request->idSucursal);
                    }
                }
            })
            ->get();

        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json($r);
        $r->sum_bs      = $sum_bs;
        $r->sum_dolares = $sum_dolares;

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_facturadas' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteOrdenesFacturadas($r)) , '\\excel\\reporte_ordenes_facturadas\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG' , null) . '\\storage\\app\\public\\excel\\reporte_ordenes_facturadas\\' . $nombre_archivo);


    }

    public function reporte_conciliacion_banco(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeaderNecesarioReporte')
            ->whereHas('orderHeaderNecesarioReporte' , function ($sql) use ($request , $año , $mes){

                if ($request->usar_fecha_facturacion==1){
                    $sql->whereYear('fecha_facturacion' , '=' , $año);
                    $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                }
                if ($request->exists('idSucursal')){
                    if ($request->usar_fecha_facturacion==1){
                        if ($request->idSucursal!=''){
                            $sql->where('sucursal_facturada' , $request->idSucursal);
                        }
                    }else{
                        if ($request->idSucursal!=''){
                            $sql->where('fk_sucursal' , $request->idSucursal);
                        }
                    }
                }


            })
            ->where(function ($sql2) use ($request , $año , $mes){
                if ($request->usar_fecha_facturacion==0){
                    $sql2->whereYear('created_at' , '=' , $año);
                    $sql2->whereMonth('created_at' , '=' , $mes);
                }
                $sql2->where(function ($sql3){
                    $sql3->whereNotNull('banco');
                    $sql3->orWhereNotNull('numero_transaccion');
                    $sql3->orWhereNotNull('descripcion_operacion');
                });
            })
            ->get();

        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json($r);
        $r->sum_bs      = $sum_bs;
        $r->sum_dolares = $sum_dolares;

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_de_conciliacion' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteConciliacion($r)) , '\\excel\\reporte_de_conciliacion\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG' , null) . '\\storage\\app\\public\\excel\\reporte_de_conciliacion\\' . $nombre_archivo);


    }

    public function reporte_ordenes_facturadas_a_credito(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeaderNecesarioReporte')
            ->whereHas('orderHeaderNecesarioReporte' , function ($sql) use ($request , $año , $mes){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->where('cuenta_por_cobrar' , 1); //a credito
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->where('sucursal_facturada' , $request->idSucursal);
                    }
                }
            })
            ->get();

        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json($r);
        $r->sum_bs      = $sum_bs;
        $r->sum_dolares = $sum_dolares;

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_facturadas_a_credito' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteOrdenesFacturadas($r)) , '\\excel\\reporte_ordenes_facturadas_a_credito\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG' , null) . '\\storage\\app\\public\\excel\\reporte_ordenes_facturadas_a_credito\\' . $nombre_archivo);


    }

    public function reporte_operaciones_facturadas_seniat()
    {

        $fecha_inicio = '2022-07-01 00:00:00';
        $fecha_fin    = '2022-12-31 23:59:59';

        $r_efectivo = Operacion::with('orderHeader')
            ->whereHas('orderHeader' , function ($sql) use ($fecha_fin , $fecha_inicio){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->wherenotnull('n_factura_seniat'); //que tenga factura de seniat
                $sql->wherehas('sucursal_orden' , function ($sql2){
                    $sql2->where('idSucursal' , 6); //sucursal los robles
                });
                $sql->whereBetween('date_delivery' , [$fecha_inicio , $fecha_fin]);


            })
            ->where('idFormaPago' , 1)
            ->where('fk_moneda' , 1)
            ->get();

        $r_paypal = Operacion::with('orderHeader')
            ->whereHas('orderHeader' , function ($sql) use ($fecha_fin , $fecha_inicio){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->wherenotnull('n_factura_seniat'); //que tenga factura de seniat
                $sql->wherehas('sucursal_orden' , function ($sql2){
                    $sql2->where('idSucursal' , 6); //sucursal los robles
                });
                $sql->whereBetween('date_delivery' , [$fecha_inicio , $fecha_fin]);


            })
            ->where('idFormaPago' , 4)
            ->where('fk_moneda' , 1)
            ->get();

        $r_zelle = Operacion::with('orderHeader')
            ->whereHas('orderHeader' , function ($sql) use ($fecha_fin , $fecha_inicio){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->wherenotnull('n_factura_seniat'); //que tenga factura de seniat
                $sql->wherehas('sucursal_orden' , function ($sql2){
                    $sql2->where('idSucursal' , 6); //sucursal los robles
                });
                $sql->whereBetween('date_delivery' , [$fecha_inicio , $fecha_fin]);


            })
            ->where('idFormaPago' , 7)
            ->where('fk_moneda' , 1)
            ->get();

        $r = Operacion::with('orderHeader')
            ->whereHas('orderHeader' , function ($sql) use ($fecha_fin , $fecha_inicio){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->wherenotnull('n_factura_seniat'); //que tenga factura de seniat
                $sql->wherehas('sucursal_orden' , function ($sql2){
                    $sql2->where('idSucursal' , 6); //sucursal los robles
                });
                $sql->whereBetween('date_delivery' , [$fecha_inicio , $fecha_fin]);


            })
            ->wherein('idFormaPago' , [1 , 4 , 7])
            ->where('fk_moneda' , 1)
            ->get();


        $nombre_archivo_efectivo = null;
        $nombre_archivo_paypal   = null;
        $nombre_archivo_zelle    = null;
        $todo                    = null;

        $nombre_archivo_efectivo = 'reporte_seniat_efectivo_' . time() . strval(rand(100 , 999)) . '.xlsx';
        Excel::store((new OperacionesSeniatExport($r_efectivo)) , '\\excel\\operaciones_seniat\\' . $nombre_archivo_efectivo , 'public');

        $nombre_archivo_paypal = 'reporte_seniat_paypal_' . time() . strval(rand(100 , 999)) . '.xlsx';
        Excel::store((new OperacionesSeniatExport($r_paypal)) , '\\excel\\operaciones_seniat\\' . $nombre_archivo_paypal , 'public');

        $nombre_archivo_zelle = 'reporte_seniat_zelle_' . time() . strval(rand(100 , 999)) . '.xlsx';
        Excel::store((new OperacionesSeniatExport($r_zelle)) , '\\excel\\operaciones_seniat\\' . $nombre_archivo_zelle , 'public');


        $todo = 'reporte_seniat_todos_' . time() . strval(rand(100 , 999)) . '.xlsx';
        Excel::store((new OperacionesSeniatExport($r)) , '\\excel\\operaciones_seniat\\' . $todo , 'public');

        return response()->json([
//            'operaciones_efectivo' => asset('storage\\app\\public\\excel\\operaciones_seniat\\' . $nombre_archivo_efectivo) ,
//            'operaciones_paypal' => asset('storage\\app\\public\\excel\\operaciones_seniat\\' . $nombre_archivo_paypal) ,
//            'operaciones_zelle' => asset('storage\\app\\public\\excel\\operaciones_seniat\\' . $nombre_archivo_zelle) ,
//            'operaciones' => asset('storage\\app\\public\\excel\\operaciones_seniat\\' . $todo) ,


            'operaciones_efectivo' => env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\operaciones_seniat\\' . $nombre_archivo_efectivo ,
            'operaciones_paypal' => env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\operaciones_seniat\\' . $nombre_archivo_paypal ,
            'operaciones_zelle' => env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\operaciones_seniat\\' . $nombre_archivo_zelle ,
            'operaciones' => env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\operaciones_seniat\\' . $todo ,
        ]);
    }

    public function buscar_todo(Request $request)
    {
        //falta condicionar date_delivery a fechas menores a la que se envie en fecha, ejemplo si se envia 18/11/2022 debo consultar las fechas
        $r              = OrderHeader::where('fk_status' , 2) //ordenes en produccion
        ->whereDate('date_delivery' , '<=' , substr($request->fecha , 0 , 10))
            ->get();
//        return response()->json($r[0]->customer->fullName);
        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_por_entregar_' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new OrdenesPorEntregarExport($r , $request->fecha)) , '\\excel\\reporte_ordenes_por_entregar\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_por_entregar\\' . $nombre_archivo);

    }

    public function reporte_por_asesor(Request $request)
    {
        $r              = OrderHeader::where('fk_status' , 2) //ordenes en produccion
        ->where('idUsuario' , $request->idUsuario)
            ->whereBetween('date_delivery' , [substr($request->desde , 0 , 10) . ' 00:00:00' , substr($request->hasta , 0 , 10) . ' 23:59:59'])
            ->get();
        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_por_asesor' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new OrdenesPorAsesorExport($r)) , '\\excel\\reporte_ordenes_por_asesor\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_por_asesor\\' . $nombre_archivo);

    }

    public function reporte_diario(Request $request)
    {
        $r              = OrderHeader::whereDate('fecha_facturacion' , $request->fecha)
            ->where(function ($sql) use ($request){
                if ($request->exists('fk_sucursal')){
                    if ($request->fk_sucursal!=''){
                        $sql->where('fk_sucursal' , $request->fk_sucursal);
                    }
                }
            })
            ->get();
        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_diario' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteOrdenesDiarioExport($r)) , '\\excel\\reporte_ordenes_diario\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_diario\\' . $nombre_archivo);

    }

    public function reporte_ordenes_cliente(Request $request)
    {
        $r    = OrderHeader::where('idUser' , $request->idUser)->get();
        $user = User::find($request->idUser);
        if ($user){
            $nombre_archivo = null;
            $nombre_archivo = str_replace(' ' , '_' , strtolower($user->fullName)) . '-' . time() . strval(rand(100 , 999)) . '.xlsx';
            Excel::store((new ReporteOrdenesClienteExport($r)) , '\\excel\\reporte_ordenes_cliente\\' . $nombre_archivo , 'public');
            return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_cliente\\' . $nombre_archivo);
        } else {
            return response('No se encontro el cliente' , 404);
        }


    }

    public function reporte_ordenes_cliente_solo_json(Request $request)
    {
        $r = OrderHeader::where('idUser' , $request->idUser)
            ->get();

        return response()->json($r);

    }

    public function reporte_diario_solo_json(Request $request)
    {
        $r = OrderHeader::whereDate('fecha_facturacion' , $request->fecha)
            ->where(function ($sql) use ($request){
                if ($request->exists('opcional')){
                    if ($request->opcional!=''){
                        $sql->whereHas('customer' , function ($sql) use ($request){
                            $sql->where('fullName' , 'like' , '%' . $request->opcional . '%');
                        });
                        $sql->orWhere('idOrderHeader' , $request->opcional);
                    }
                }
                if ($request->exists('fk_sucursal')){
                    if ($request->fk_sucursal!=''){
                        $sql->where('fk_sucursal' , $request->fk_sucursal);
                    }
                }
            })
            ->get();
        return response()->json($r);

    }

    public function buscar_todo2()
    {
        $r = OrderHeader::where('fk_status' , 3)
//            ->whereBetween('created_at' , [substr($request->desde , 0 , 10) . ' 00:00:00' , substr($request->hasta , 0 , 10) . ' 23:59:59'])
            ->limit(1)
            ->get();

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_por_entregar' . time() . '_' . strval(rand(100 , 999)) . '.xlsx';

        Excel::store((new OrdenesPorEntregarExport($r)) , '\\excel\\reporte_ordenes_por_entregar\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_por_entregar\\' . $nombre_archivo);

    }

    public function enviar_sms_ordenes_()
    {
        //busco las ordenes con estatus
        $ordenes = OrderHeader::withOnly('customer' , 'estatus')
            ->where('fk_status' , 3)
//            ->limit(2)
            ->get(); //Lista para retirar
        foreach ($ordenes as $orden) {
            // envio de sms
//            dd($orden->customer->phone);
//            exit();
            SmsController::sendSMS(new Request([
                "destination" => $orden->customer->phone ,
                "message" => "Islapixel le informa que su orden numero: " . $orden->n_order . ", se encuentra " . strtoupper($orden->estatus->nombre) . ", debe dirigirse a nuestras sucursales para solicitarla."
            ]));
//            if (env('SOLO_PRUEBAS_NOTIFICAR_SMS')){
//                exit();
//            }
        }
        return response()->json(['msj' => 'Se enviaron correctamente los sms']);
    }

    public function reporte_ingresos_mensual_cuenta_por_cobrar(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeader')
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                $sql->where('cuenta_por_cobrar' , true);
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }

            })
//            ->whereYear('fecha_facturacion' , '=' , $año)
//            ->whereMonth('fecha_facturacion' , '=' , $mes)
//            ->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"))
            ->get();

        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json([$sum_bs,$sum_dolares]);
        $r->sum_bs      = $sum_bs;
        $r->sum_dolares = $sum_dolares;
        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ingresos_mensual_cuenta_cobrar' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteIngresosMensual($r)) , '\\excel\\reporte_ingresos_mensual_cuenta_cobrar\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ingresos_mensual_cuenta_cobrar\\' . $nombre_archivo);

    }

    public function reporte_ingresos_mensual_cuenta_por_cobrar_solo_json(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeader_solo_customer')
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                $sql->where('cuenta_por_cobrar' , true);
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }

            })

//            ->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"))
            ->get();

//        foreach ($r as $key=>$item) {
//            $r[$key]->customer = $item->orderHeader_solo_customer->customer;
//       }


        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json([$sum_bs,$sum_dolares]);

        return response()->json([
            'data' => $r ,
            'sum_bs' => number_format($sum_bs , 2 , "," , ".") ,
            'sum_dolares' => number_format($sum_dolares , 2 , "," , ".")]);

    }

    public function reporte_ingresos_mensual(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeader')
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }

            })

//            ->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"))
            ->get();

        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json([$sum_bs,$sum_dolares]);
        $r->sum_bs      = $sum_bs;
        $r->sum_dolares = $sum_dolares;
        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ingresos_mensual' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteIngresosMensual($r)) , '\\excel\\reporte_ingresos_mensual\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ingresos_mensual\\' . $nombre_archivo);

    }

    public function reporte_ingresos_mensual_solo_json(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año = $date_separado[0];
        $mes = $date_separado[1];
        $r   = Operacion::with('orderHeader_solo_customer')
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }

            })

//            ->groupBy(DB::raw("DATE_FORMAT(created_at, '%Y-%m-%d')"))
            ->get();

//        foreach ($r as $key=>$item) {
//            $r[$key]->customer = $item->orderHeader_solo_customer->customer;
//       }


        $sum_bs      = $r->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $r->where('fk_moneda' , 1)->sum('monto');
//        return response()->json([$sum_bs,$sum_dolares]);

        return response()->json([
            'data' => $r ,
            'sum_bs' => number_format($sum_bs , 2 , "," , ".") ,
            'sum_dolares' => number_format($sum_dolares , 2 , "," , ".")]);

    }

    public function reporte_ordenes_pagadas_sin_igtf(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año         = $date_separado[0];
        $mes         = $date_separado[1];
        $operaciones = Operacion::with('orderHeader')
            ->where(function ($sql){
                $sql->whereNull('idOrderHTax');
                $sql->orWhere('idOrderHTax' , 0);
            })
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->where('precio_total' , "!=" , 0);

                $sql->whereYear('created_at' , '=' , $año);
                $sql->whereMonth('created_at' , '=' , $mes);

                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }
            })
            ->groupBy('idOrderHeader')
            ->get();

//        return response()->json($operaciones);
//        exit();
        $ordenes_pagas = array();

        foreach ($operaciones as $key => $operacion) {

            $AbonoTotalDolares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $abonoTotal = round($totalAbono , 2);
            $deudaTotal = ($operacion->orderHeader->precio_total < $totalAbono) ? 0 : round(($operacion->orderHeader->precio_total - $totalAbono) , 2);

            if ($deudaTotal==0){
                $result = Operacion::with('orderHeader')
                    ->where('idOrderHeader' , $operacion->idOrderHeader)
                    ->get();
                foreach ($result as $key => $item) {
                    if ($item->idOperacion==$operacion->idOperacion){
                        $item->total_pago_dolares_equi = $abonoTotal;
                    }
                    $ordenes_pagas[] = $item;
                }
            }

        }

        $ordenes_pagas = collect($ordenes_pagas);


        $sum_bs                     = $ordenes_pagas->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares                = $ordenes_pagas->where('fk_moneda' , 1)->sum('monto');
        $ordenes_pagas->sum_bs      = $sum_bs;
        $ordenes_pagas->sum_dolares = $sum_dolares;

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_pagadas_sin_igtf' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteOrdenesPagadasSinIgtf($ordenes_pagas)) , '\\excel\\reporte_ordenes_pagadas_sin_igtf\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_pagadas_sin_igtf\\' . $nombre_archivo);

    }

    public function reporte_ordenes_pagadas_sin_igtf_solo_json(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);

        $año         = $date_separado[0];
        $mes         = $date_separado[1];
        $operaciones = Operacion::with('orderHeader')
            ->where(function ($sql){
                $sql->whereNull('idOrderHTax');
                $sql->orWhere('idOrderHTax' , 0);
            })
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->where('precio_total' , "!=" , 0);

                $sql->whereYear('created_at' , '=' , $año);
                $sql->whereMonth('created_at' , '=' , $mes);

                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }
            })
            ->groupBy('idOrderHeader')
            ->get();

//        return response()->json($operaciones);
//        exit();
        $ordenes_pagas = array();

        foreach ($operaciones as $key => $operacion) {

            $AbonoTotalDolares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $abonoTotal = round($totalAbono , 2);
            $deudaTotal = ($operacion->orderHeader->precio_total < $totalAbono) ? 0 : round(($operacion->orderHeader->precio_total - $totalAbono) , 2);

            if ($deudaTotal==0){
                $result = Operacion::with('orderHeader')
                    ->where('idOrderHeader' , $operacion->idOrderHeader)
                    ->get();
                foreach ($result as $key => $item) {
                    if ($item->idOperacion==$operacion->idOperacion){
                        $item->total_pago_dolares_equi = $abonoTotal;
                    }
                    $ordenes_pagas[] = $item;
                }
            }

        }

        $ordenes_pagas = collect($ordenes_pagas);


        $sum_bs      = $ordenes_pagas->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares = $ordenes_pagas->where('fk_moneda' , 1)->sum('monto');

        return response()->json([
            'data' => $ordenes_pagas ,
            'sum_bs' => number_format($sum_bs , 2 , "," , ".") ,
            'sum_dolares' => number_format($sum_dolares , 2 , "," , ".")]);

    }

    public function reporte_ordenes_pagadas_no_facturadas(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);
        $año           = $date_separado[0];
        $mes           = $date_separado[1];

        $operaciones = Operacion::with('orderHeader')
            ->where('idOrderHeader' , "!=" , 0)
            ->where('idFormaPago' , $request->idFormaPago)
            ->whereYear('created_at' , '=' , $año)
            ->whereMonth('created_at' , '=' , $mes)
            ->whereHas('orderHeader' , function ($sql){
                $sql->where('precio_total' , "!=" , 0);
            })
            ->whereHas('orderHeader' , function ($sql) use ($request){
//                $sql->whereYear('created_at' , '=' , $año);
//                $sql->whereMonth('created_at' , '=' , $mes);

//                $sql->whereNull('fecha_facturacion');
//                $sql->whereNull('sucursal_facturada');
                $sql->where('cuenta_por_cobrar' , 0);

                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }
            })
            ->groupBy('idOrderHeader')
//            ->limit(100)
            ->get();
//        dd($operaciones);
        $ordenes_pagas = array();

        foreach ($operaciones as $key => $operacion) {

            $AbonoTotalDolares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $abonoTotal = round($totalAbono , 2);
            $deudaTotal = ($operacion->orderHeader->precio_total < $totalAbono) ? 0 : round(($operacion->orderHeader->precio_total - $totalAbono) , 2);

            if ($deudaTotal==0){
                $result = Operacion::with('orderHeader')
                    ->where('idOrderHeader' , $operacion->idOrderHeader)
                    ->get();
                foreach ($result as $key => $item) {
                    if ($item->idOperacion==$operacion->idOperacion){
                        $item->total_pago_dolares_equi = $abonoTotal;
                    }
                    $ordenes_pagas[] = $item;
                }
            }

        }

        $ordenes_pagas = collect($ordenes_pagas);


        $sum_bs                     = $ordenes_pagas->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares                = $ordenes_pagas->where('fk_moneda' , 1)->sum('monto');
        $ordenes_pagas->sum_bs      = $sum_bs;
        $ordenes_pagas->sum_dolares = $sum_dolares;

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_pagadas_no_facturadas' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteOrdenesPagadasNoFacturadas($ordenes_pagas)) , '\\excel\\reporte_ordenes_pagadas_no_facturadas\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_pagadas_no_facturadas\\' . $nombre_archivo);

    }

    public function reporte_ordenes_pagadas_no_facturadas_solo_json(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);
        $año           = $date_separado[0];
        $mes           = $date_separado[1];

        $operaciones = Operacion::with('orderHeader')
            ->where('idOrderHeader' , "!=" , 0)
            ->where('idFormaPago' , $request->idFormaPago)
            ->whereHas('orderHeader' , function ($sql){
                $sql->where('precio_total' , "!=" , 0);
            })
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->whereYear('created_at' , '=' , $año);
                $sql->whereMonth('created_at' , '=' , $mes);

//                $sql->whereNull('fecha_facturacion');
//                $sql->whereNull('sucursal_facturada');
                $sql->where('cuenta_por_cobrar' , 0);

                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }
            })
            ->groupBy('idOrderHeader')
//            ->limit(100)
            ->get();
//        dd($operaciones);
        $ordenes_pagas = array();

        foreach ($operaciones as $key => $operacion) {

            $AbonoTotalDolares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $abonoTotal = round($totalAbono , 2);
            $deudaTotal = ($operacion->orderHeader->precio_total < $totalAbono) ? 0 : round(($operacion->orderHeader->precio_total - $totalAbono) , 2);

            if ($deudaTotal==0){
                $result = Operacion::with('orderHeader')
                    ->where('idOrderHeader' , $operacion->idOrderHeader)
                    ->get();
                foreach ($result as $key => $item) {
                    if ($item->idOperacion==$operacion->idOperacion){
                        $item->total_pago_dolares_equi = $abonoTotal;
                    }
                    $ordenes_pagas[] = $item;
                }
            }

        }

        $ordenes_pagas = collect($ordenes_pagas);


        $sum_bs                     = $ordenes_pagas->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares                = $ordenes_pagas->where('fk_moneda' , 1)->sum('monto');
        $ordenes_pagas->sum_bs      = $sum_bs;
        $ordenes_pagas->sum_dolares = $sum_dolares;

        $nombre_archivo = null;

        return response()->json([
            'data' => $ordenes_pagas ,
            'sum_bs' => number_format($sum_bs , 2 , "," , ".") ,
            'sum_dolares' => number_format($sum_dolares , 2 , "," , ".")
        ]);

    }

    public function reporte_ordenes_pagadas(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);
        $año           = $date_separado[0];
        $mes           = $date_separado[1];

        $operaciones = Operacion::with('orderHeader')
            ->where('idOrderHeader' , "!=" , 0)
            ->where('idFormaPago' , $request->idFormaPago)
            ->whereHas('orderHeader' , function ($sql){
                $sql->where('precio_total' , "!=" , 0);
            })
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }
            })
            ->groupBy('idOrderHeader')
//            ->limit(100)
            ->get();
//        dd($operaciones);
        $ordenes_pagas = array();

        foreach ($operaciones as $key => $operacion) {

            $AbonoTotalDolares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $abonoTotal = round($totalAbono , 2);
            $deudaTotal = ($operacion->orderHeader->precio_total < $totalAbono) ? 0 : round(($operacion->orderHeader->precio_total - $totalAbono) , 2);

            if ($deudaTotal==0){
                $result = Operacion::with('orderHeader')
                    ->where('idOrderHeader' , $operacion->idOrderHeader)
                    ->get();
                foreach ($result as $key => $item) {
                    if ($item->idOperacion==$operacion->idOperacion){
                        $item->total_pago_dolares_equi = $abonoTotal;
                    }
                    $ordenes_pagas[] = $item;
                }
            }

        }

        $ordenes_pagas = collect($ordenes_pagas);


        $sum_bs                     = $ordenes_pagas->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares                = $ordenes_pagas->where('fk_moneda' , 1)->sum('monto');
        $ordenes_pagas->sum_bs      = $sum_bs;
        $ordenes_pagas->sum_dolares = $sum_dolares;

        $nombre_archivo = null;
        $nombre_archivo = 'reporte_ordenes_pagadas' . time() . strval(rand(100 , 999)) . '_' . $request->fecha . '.xlsx';
        Excel::store((new ReporteOrdenesPagadas($ordenes_pagas)) , '\\excel\\reporte_ordenes_pagadas\\' . $nombre_archivo , 'public');
        return response()->json(env('BASE_URL_IMG') . '\\storage\\app\\public\\excel\\reporte_ordenes_pagadas\\' . $nombre_archivo);

    }

    public function reporte_ordenes_pagadas_solo_json(Request $request)
    {
        $date_separado = explode("-" , $request->fecha);
        $año           = $date_separado[0];
        $mes           = $date_separado[1];

        $operaciones = Operacion::with('orderHeader')
            ->where('idOrderHeader' , "!=" , 0)
            ->where('idFormaPago' , $request->idFormaPago)
            ->whereHas('orderHeader' , function ($sql){
                $sql->where('precio_total' , "!=" , 0);
            })
            ->whereHas('orderHeader' , function ($sql) use ($request , $año , $mes){
                $sql->whereYear('fecha_facturacion' , '=' , $año);
                $sql->whereMonth('fecha_facturacion' , '=' , $mes);
                $sql->where('fk_status' , '!=' , 10); //que las ordenes no estan canceladas
                if ($request->exists('idSucursal')){
                    if ($request->idSucursal!=''){
                        $sql->whereHas('sucursal_orden_factuacion' , function ($sql2) use ($request){
                            $sql2->where('idSucursal' , $request->idSucursal);
                        });
                    }
                }
            })
            ->groupBy('idOrderHeader')
//            ->limit(100)
            ->get();
//        dd($operaciones);
        $ordenes_pagas = array();

        foreach ($operaciones as $key => $operacion) {

            $AbonoTotalDolares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 1)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum('monto');

            $AbonoTotalBolivares = Operacion::withOnly('formaPago')->where('idTipoTransaccion' , 1)
                ->where('fk_moneda' , 2)
                ->where('idOrderHeader' , $operacion->idOrderHeader)
                ->sum(DB::raw('monto/tasa_moneda'));

//                $totalAbono = ($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares;
            $totalAbono = $AbonoTotalBolivares + $AbonoTotalDolares;

            $abonoTotal = round($totalAbono , 2);
            $deudaTotal = ($operacion->orderHeader->precio_total < $totalAbono) ? 0 : round(($operacion->orderHeader->precio_total - $totalAbono) , 2);

            if ($deudaTotal==0){
                $result = Operacion::with('orderHeader')
                    ->where('idOrderHeader' , $operacion->idOrderHeader)
                    ->get();
                foreach ($result as $key => $item) {
                    if ($item->idOperacion==$operacion->idOperacion){
                        $item->total_pago_dolares_equi = $abonoTotal;
                    }
                    $ordenes_pagas[] = $item;
                }
            }

        }

        $ordenes_pagas = collect($ordenes_pagas);


        $sum_bs                     = $ordenes_pagas->where('fk_moneda' , 2)->sum('monto');
        $sum_dolares                = $ordenes_pagas->where('fk_moneda' , 1)->sum('monto');
        $ordenes_pagas->sum_bs      = $sum_bs;
        $ordenes_pagas->sum_dolares = $sum_dolares;

        $nombre_archivo = null;

        return response()->json([
            'data' => $ordenes_pagas ,
            'sum_bs' => number_format($sum_bs , 2 , "," , ".") ,
            'sum_dolares' => number_format($sum_dolares , 2 , "," , ".")
        ]);

    }
}
