<?php

namespace App\Http\Controllers;

use App\Models\Caja;
use App\Models\FormasDePagoSucursal;
use App\Models\Sucursal;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class SucursalController extends Controller
{
    public function index()
    {
        return Sucursal::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'nombre' => 'required|unique:sucursales,nombre,NULL,idSucursal,deleted_at,NULL' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
            'nombre.unique' => 'Sucursal ya existe' ,
        ]);
        DB::beginTransaction();
        try {
            $re = new Sucursal($request->all());
            $re->save();
            if ($request->exists('forma_de_pago')){
                if (count($request->forma_de_pago) > 0){
                    foreach ($request->forma_de_pago as $forma_de_pago) {
                        $formPago              = new FormasDePagoSucursal($forma_de_pago);
                        $formPago->fk_sucursal = $re->idSucursal;
                        $formPago->status      = 1;
                        $formPago->save();
                    }
                }
            }

            //creando la caja
            $caja = new CajaController();
            $caja->store(new Request([
                'nombre' => 'Caja de sucursal ' . $re->nombre ,
                'fecha_creacion' => now() ,
                'idSucursal' => $re->idSucursal ,
            ]));


            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => Sucursal::find($re->idSucursal) ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en SucursalController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Sucursal::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'nombre' => 'required|unique:sucursales,nombre,' . $id . ',idSucursal,deleted_at,NULL' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
            'nombre.unique' => 'Sucursal ya existe' ,
        ]);
        DB::beginTransaction();
        $h = Sucursal::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el tipo solicitado'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();

            if ($request->exists('forma_de_pago')){
//                if (count($request->forma_de_pago) > 0){
                /*Borro los campos antes de reemplazarlos*/
                $t = FormasDePagoSucursal::select('idFormaPagoSucursal')
                    ->where('fk_sucursal' , $id)
                    ->without('sucursal' , 'formaDePago' , 'status')
                    ->get()
                    ->toArray();
//                   return response()->json($t);
                FormasDePagoSucursal::destroy($t);
                foreach ($request->forma_de_pago as $forma_de_pago) {
                    $formPago              = new FormasDePagoSucursal($forma_de_pago);
                    $formPago->fk_sucursal = $id;
                    $formPago->status      = 1;
                    $formPago->save();
                }
//                }
            }
            $response = [
                'msj' => 'Tipo actualizado exitosamente' ,
                'tipo' => Sucursal::find($id) ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en SucursalController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Sucursal::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
