<?php

namespace App\Http\Controllers;

use App\Models\Suscripciones;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class SuscripcionesController extends Controller
{
    public function index()
    {
        return Suscripciones::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'email' => 'required|email|unique:suscripciones,email,NULL,id,deleted_at,NULL' ,
        ] , [
            'email.unique' => 'Este Email ya se encuentra en uso' ,
            'email.email' => 'El Email debe de tener un formato ejemplo@ejemplo.com' ,
            'email.required' => 'El Email es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new Suscripciones($request->all());
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'suscripcion' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en SuscripcionesController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Suscripciones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['suscripcion' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'email' => 'required|email|unique:suscripciones,email,NULL,id,deleted_at,NULL' ,
        ] , [
            'email.unique' => 'Este Email ya se encuentra en uso' ,
            'email.email' => 'El Email debe de tener un formato ejemplo@ejemplo.com' ,
            'email.required' => 'El Email es requerido' ,
        ]);
        DB::beginTransaction();
        $h = Suscripciones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la suscripción'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Suscripcion actualizada exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en SuscripcionesController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Suscripciones::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
