<?php

namespace App\Http\Controllers;

use App\Models\Moneda;
use App\Models\Producto;
use App\Models\TasaMoneda;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class TasaMonedaController extends Controller
{
    public function index()
    {
        $todo = TasaMoneda::active()->with('moneda')->orderBy('idTasaMoneda' , 'desc')->get();
        return response()->json($todo);
    }

    public function listarAgrupado()
    {
        $todo      = [];
        $idsMoneda = TasaMoneda::active()->select('fk_moneda')->groupBy('fk_moneda')->get()->toArray();
        foreach ($idsMoneda as $item) {
            $idMoneda    = $item['fk_moneda'];
            $tasa_moneda = TasaMoneda::active()->where('fk_moneda' , $idMoneda)->orderBy('created_at' , 'desc')->first();
            $moneda      = Moneda::find($idMoneda);
            if ($moneda['status']){ //valido si la moneda esta o no activa
                $todo[] = ['idMoneda' => $idMoneda , 'descripcion' => $moneda , 'tasa_moneda' => $tasa_moneda];
            }
        }
        return response()->json($todo);

    }

    static function listarAgrupado2(Producto $producto)
    {
        $todo     = [];
        $idMoneda = $producto->fk_moneda;
        //buscanlo las monedas distintas al del producto
        $idsMoneda = TasaMoneda::active()->select('fk_moneda')
            ->where('fk_moneda' , '!=' , $idMoneda)
            ->groupBy('fk_moneda')
            ->get()
            ->toArray();
        foreach ($idsMoneda as $item) {
            $idMoneda    = $item['fk_moneda'];
            $tasa_moneda = TasaMoneda::active()->where('fk_moneda' , $idMoneda)->orderBy('created_at' , 'desc')->first();
            $moneda      = Moneda::find($idMoneda);
            if ($moneda['status']){ //valido si la moneda esta o no activa
                $tasa_moneda = TasaMoneda::active()->where('fk_moneda' , $idMoneda)->orderBy('created_at' , 'desc')->first();
//                $tasa_moneda->equivalencia = (float)$tasa_moneda['tasa'] * (float)$producto['precio'];
                $tasa_moneda->moneda = $moneda;

                $todo[] = $tasa_moneda;
            }
        }

        return $todo;

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'tasa' => 'required' ,
            'fk_moneda' => 'required' ,
            'fecha_vencimiento' => 'required' ,
        ] , [
            'tasa.required' => ':attribute es requerido' ,
            'fk_moneda.required' => ':attribute es requerido' ,
            'fecha_vencimiento.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $re                    = new TasaMoneda($request->all());
            $re->fecha_vencimiento = $request->fecha_vencimiento;
            $re->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $re ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TasaMonedaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = TasaMoneda::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'tasa' => 'required' ,
//            'fk_moneda' => 'required' ,
            'fecha_vencimiento' => 'required' ,
        ] , [
            'tasa.required' => ':attribute es requerido' ,
//            'fk_moneda.required' => ':attribute es requerido' ,
            'fecha_vencimiento.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = TasaMoneda::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la tasa de moneda solicitada'] , 401);
        }

        try {
            $h->tasa              = $request->tasa;
            $h->fecha_vencimiento = $request->fecha_vencimiento;

            $h->save();
            $response = [
                'msj' => 'Tasa de moneda actualizado exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TasaMonedaController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
//        $h = TasaMoneda::find($id);
//
//        if (is_null($h)){
//            return response()->json(['msj' => 'no se encontro resultado'] , 401);
//        }
//
//        $h->delete();
//        $h->save();
//        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
