<?php

namespace App\Http\Controllers;

use App\Models\SubTipo;
use App\Models\Tipo;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class TiposController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return Tipo::get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'nombre' => 'required' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new Tipo($request->all());
            $tipo->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'tipo' => $tipo ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TiposController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Tipo::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['tipo' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'nombre' => 'required' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = Tipo::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir el tipo solicitado'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Tipo actualizado exitosamente' ,
                'tipo' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en TiposController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Tipo::find($id);
        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }
        $subtipos = SubTipo::where('fk_tipo' , $id)->get();
        if (count($subtipos) > 0){
            return response()->json(['msj' => 'no se puede eliminar el registro pues existen subtipos asociados' , 'data' => $subtipos] , 404);
        } else {
            $h->delete();
            $h->save();
            return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
        }

    }
}
