<?php

namespace App\Http\Controllers;

use App\Mail\NuevaClaveUser;
use App\Mail\RecuperarClave;
use App\Models\Modalidad;
use App\Models\Moneda;
use App\Models\Operacion;
use App\Models\Rol;
use App\Models\SubTipo;
use App\Models\Tipo;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;

ini_set('memory_limit' , '512M');
ini_set('max_execution_time' , '300');

class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function aplica_descuento_automatico(Request $request)
    {
        $this->validate($request , [
            'idUser' => 'required' ,
        ] , [
            'idUser.required' => 'El id del usuario es requerido' ,
        ]);

        $user = User::find($request->idUser);
        if ($user){
            if ($user->aplica_descuento_automatico!=$request->aplica_descuento_automatico){
                $user->aplica_descuento_automatico = $request->aplica_descuento_automatico;
                $user->save();
                $rr = '';

                if ($request->aplica_descuento_automatico){
                    $rr = 'agregado a';
                } else {
                    $rr = 'quitado de';
                }

                return response()->json(['msj' => ucfirst(strtolower($user->fullName)) . ' ' . $rr . ' la lista']);
            } else {
                return response()->json(['msj' => "Ya existe en la lista"]);
            }
        } else {
            return response()->json(['msj' => 'No se encontró al usuario solicitado' , 404]);
        }
    }

    public function get_users_lista_negra()
    {

        $user = User::where('lista_negra' , true)->get();
        if (count($user) > 0){
            return response()->json($user);
        } else {
            return response()->json(['msj' => 'No hay usuarios en lista negra' , 404]);
        }

    }

    public function set_user_a_credito(Request $request)
    {
        $this->validate($request , [
            'idUser' => 'required' ,
        ] , [
            'idUser.required' => 'El id del usuario es requerido' ,
        ]);

        $user = User::find($request->idUser);
        if ($user){
            if ($user->permitir_credito!=$request->permitir_credito){
                $user->permitir_credito = $request->permitir_credito;
                $user->save();
                $rr = '';

                if ($request->permitir_credito){
                    $rr = 'agregado a';
                } else {
                    $rr = 'quitado de';
                }

                return response()->json(['msj' => ucfirst(strtolower($user->fullName)) . ' ' . $rr . ' la lista']);
            } else {
                return response()->json(['msj' => "Ya existe en la lista"]);
            }
        } else {
            return response()->json(['msj' => 'No se encontró al usuario solicitado' , 404]);
        }

    }

    public function set_lista_negra(Request $request)
    {
        $this->validate($request , [
            'idUser' => 'required' ,
        ] , [
            'idUser.required' => 'El id del usuario es requerido' ,
        ]);

        $user = User::find($request->idUser);
        if ($user){
            if ($user->lista_negra!=$request->lista_negra){
                $user->lista_negra = $request->lista_negra;
                $user->save();
                $rr = '';
                if ($request->lista_negra){
                    $rr = 'agregado a';
                } else {
                    $rr = 'quitado de';
                }
                return response()->json(['msj' => ucfirst(strtolower($user->fullName)) . ' ' . $rr . ' la lista']);
            } else {
                return response()->json(['msj' => "Ya existe en la lista"]);
            }
        } else {
            return response()->json(['msj' => 'No se encontró al usuario solicitado' , 404]);
        }

    }

    public function buscador(Request $request)
    {
        $palabra = "%" . $request->descripcion . "%";
        return User::where('fullName' , 'like' , $palabra)
            ->orWhere('numerIdentificacion' , 'like' , $palabra)
            ->with('descuento')
//            ->where('profile_fk' , 3)
            ->get();
    }

    public function listado_clientes_con_deuda()
    {
        $depurar = false; //solo para pruebas
        $users   = User::with('ordenes')
//            ->where('email' , 'lagranjitamgta@gmail.com')
            ->whereHas('ordenes' , function ($query){
                $query->where('cuenta_por_cobrar' , true);
                $query->where('fk_status' , '!=' , 10);
            })->get();
        $arra    = [];
        foreach ($users as $key0 => $user) {

            $totalAbono           = 0;
            $precio_total_ordenes = 0;
            $deuda_por_orden      = 0;
//           if ($user->id==199){
//               return response()->json($user);
//               exit();
//           }
            foreach ($user->ordenes as $key => $orden) {
                $AbonoTotalDolares   = 0;
                $AbonoTotalBolivares = 0;
                $AbonoTotalDolares   = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 1)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum('monto');

                $AbonoTotalBolivares = Operacion::where('idTipoTransaccion' , 1)
                    ->where('fk_moneda' , 2)
                    ->where('idOrderHeader' , $orden->idOrderHeader)
                    ->sum(DB::raw('monto/tasa_moneda'));

                if ($depurar){
                    $arra[] = [
                        'idUser' => $user->id ,
                        'idOrderHeader' => $orden->idOrderHeader ,
                        'AbonoTotalDolares' => $AbonoTotalDolares ,
                        'AbonoTotalBolivares' => $AbonoTotalBolivares ,
                        'DeudaTotal' => $orden->precio_total - ($AbonoTotalBolivares + $AbonoTotalDolares) ,
                        'orden' => $orden
                    ];
                }

                $precio_total_ordenes += $orden->precio_total;
//                $totalAbono           += (($AbonoTotalBolivares / $orden->tasa_moneda) + $AbonoTotalDolares);
                $totalAbono      += ($AbonoTotalBolivares + $AbonoTotalDolares);
                $deuda_por_orden += $orden->precio_total - ($AbonoTotalBolivares + $AbonoTotalDolares);

            }

            $users[$key0]->AbonoTotal = round($totalAbono , 2);
            $users[$key0]->DeudaTotal = ($precio_total_ordenes < $totalAbono) ? 0 : round($deuda_por_orden , 2);
            $users[$key0]->deuda=$users[$key0]->deuda;
        }
        if ($depurar){
            return response()->json($arra);
        } else {
            return response()->json($users);
        }
    }

    public function index()
    {
        return response()->json('error');
    }

    public function index2(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        return response()->json(User::where('fk_status' , 1)
            ->orderby('fullName' , 'asc')
            ->offset($request->offset)
            ->limit($request->limit)
            ->get());
    }

    public function listar_clientes()
    {
        $clientes = User::where('profile_fk' , 3)
            ->withOnly('status')
            ->where('fk_status' , 1)->get();
        return response()->json($clientes);
    }

    public function getUser()
    {
        return response()->json(Auth::user());
    }

    public function getUserId($iduser)
    {
        return response()->json(User::find($iduser));
    }

    public function getPerfiles()
    {
        return response()->json(Rol::all());
    }

    public function getMonedas()
    {
        return response()->json(Moneda::get());
    }

    public function geTipos()
    {
        return response()->json(Tipo::all());
    }

    public function getSubTipos()
    {
        return response()->json(SubTipo::all());
    }

    public function getModalidades()
    {
        return response()->json(Modalidad::orderBy('descripcion' , 'asc')->get());
    }

    public function recuperarclave(Request $request)
    {
        $this->validate($request , [
            'email' => 'required' ,
        ] , [
            'email.required' => 'El Email es requerido' ,
        ]);

        $user = User::where('email' , $request->email)->first();

        if (is_null($user)){

            $response = [
                'msj' => 'Si eres parte de la plataforma revisa tu bandeja de entrada o spam, en donde te enviamos los pasos a seguir para restablecer la contraseña' ,
            ];

            return response()->json($response , 200);
        } else {
            $clave_nueva    = Str::random(8);
            $user->password = bcrypt($clave_nueva);
//            $user->statusUser = '1';
            $user->save();

            $string    = $user->email;
            $string    = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
            $resultado = stristr($string , 'generico');
            if ($resultado!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
            } else {
                Mail::to($user->email)->send(new RecuperarClave($user , $clave_nueva));
            }

            $response = [
                'msj' => 'Si eres parte de la plataforma revisa tu bandeja de entrada o spam, en donde te enviamos los pasos a seguir para restablecer la contraseña' ,
                'user' => $user ,
            ];

            return response()->json($response , 200);
        }

    }

    public function crear_user_clave_aleatoria(Request $request)
    {
        $this->validate($request , [
            'fullName' => 'required' ,
            'typeIdentification' => 'required' ,
            'numerIdentificacion' => 'required|unique:users,numerIdentificacion,NULL,id,deleted_at,NULL' ,
            'phone' => 'required' ,
            'email' => 'required|email' ,

        ] , [
            'fullName.required' => 'El :attribute es requerido' ,
            'typeIdentification.required' => 'El :attribute es requerido' ,
            'numerIdentificacion.required' => 'El :attribute es requerido' ,
            'numerIdentificacion.unique' => 'El número de identificación ya se encuentra registrado' ,
            'phone.required' => 'El :attribute es requerido' ,
            'email.email' => 'El email debe de tener un formato ejemplo@ejemplo.com' ,
            'email.required' => 'El email es requerido'
        ]);

        $user                      = new User();
        $user->fullName            = $request->fullName;
        $user->name                = $request->fullName;
        $user->typeIdentification  = $request->typeIdentification;
        $user->numerIdentificacion = $request->numerIdentificacion;
        $user->phone               = $request->phone;
        $user->email               = $request->email;
        $user->user                = $request->email;
        $user->direccion           = $request->direccion;
        $user->profile_fk          = 3; //cliente
        $user->fk_status           = 1; //activo

        $clave_nueva    = Str::random(8);
        $user->password = bcrypt($clave_nueva);
        $user->save();

        $string    = $user->email;
        $string    = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
        $resultado = stristr($string , 'generico');
        if ($resultado!==false){
//                        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
        } else {
            Mail::to($user->email)->send(new NuevaClaveUser($user , $clave_nueva));
        }


        $response = [
            'msj' => 'Usuario creado exitosamente' ,
            'user' => $user->refresh() ,
        ];

        return response()->json($response , 200);


    }

    public function listar(Request $request)
    {
        $listOnlyClient = $request->listOnlyClient;
        if ($listOnlyClient){
            return response()->json(User::where('profile_fk' , 3)
                ->orderby('fullName' , 'asc')
                ->get());
        } else {
            return response()->json(User::where('profile_fk' , '!=' , 3)
                ->orderby('fullName' , 'asc')
                ->get());
        }

    }

    public function user_buscar(Request $request)
    {
        $this->validate($request , [
            'offset' => 'integer|min:0' ,
            'limit' => 'integer|min:0' ,
        ] , [
            'offset.integer' => 'Debe ser numérico' ,
            'limit.integer' => 'Debe ser numérico' ,

            'offset.min' => 'Debe tener al menos un número' ,
            'limit.min' => 'Debe tener al menos un número' ,
        ]);

        $listOnlyClient = $request->listOnlyClient;
        if ($listOnlyClient){

            $result = User::with('ordenes')->with('descuento')->where('profile_fk' , 3)
                ->where(function ($query) use ($request){
                    if ($request->buscar!='null' && strtolower($request->buscar)!='deuda'){
                        $query->where('email' , 'like' , "%" . $request->buscar . "%");
                        $query->orwhere('fullName' , 'like' , "%" . $request->buscar . "%");
                        $query->orwhere('numerIdentificacion' , 'like' , "%" . $request->buscar . "%");
                        $query->orwhere('phone' , 'like' , "%" . $request->buscar . "%");
                    }
                    if ((strtolower($request->buscar))=='deuda'){
                        $query->whereHas('ordenes' , function ($query) use ($request){
                            $query->where('cuenta_por_cobrar' , true);
                        });
                    }
                })
                ->orderby('fullName' , 'asc')
                ->offset($request->offset)
                ->limit($request->limit)
                ->get();
            return response()->json($result);
        } else {
            return response()->json(User::where('profile_fk' , '!=' , 3)
                ->where(function ($query) use ($request){
                    if ($request->buscar!='null'){
                        $query->where('email' , 'like' , "%" . $request->buscar . "%");
                        $query->orwhere('fullName' , 'like' , "%" . $request->buscar . "%");
                        $query->orwhere('numerIdentificacion' , 'like' , "%" . $request->buscar . "%");
                        $query->orwhere('phone' , 'like' , "%" . $request->buscar . "%");
                    }
                })
                ->orderby('fullName' , 'asc')
                ->offset($request->offset)
                ->limit($request->limit)
                ->get());
        }

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $request->validate([
            'fullName' => 'required|min:2' ,
//            'fullName' => 'required|min:2|regex:/^[\pL\s\-]+$/u' ,
            'typeIdentification' => 'required' ,
            'numerIdentificacion' => 'required|unique:users,numerIdentificacion,NULL,id,deleted_at,NULL' ,
            'phone' => 'required' ,
            'profile_fk' => 'required' ,
            'email' => 'required|email|unique:users,email,NULL,id,deleted_at,NULL' ,
            'password' => 'min:8|confirmed' ,
            'password_confirmation' => 'required|min:8' ,
        ] , [
            'fullName.required' => 'El Nombre completo es requerido' ,
//            'fullName.regex' => 'El Nombre debe ser solo letras' ,
            'fullName.min' => 'El Nombre completo no puede tener menos de 2 caracteres' ,
            'typeIdentification.required' => 'La identificacion es requerido' ,
            'numerIdentificacion.required' => 'El número de identificación es requerida' ,
            'email.unique' => 'Este email ya se encuentra en uso' ,
            'numerIdentificacion.unique' => 'El número de identificación ya se encuentra registrado' ,
            'email.email' => 'El email debe de tener un formato ejemplo@ejemplo.com' ,
            'email.required' => 'El email es requerido' ,
            'password.min' => 'La contraseña debe de tener minimo 8 caracteres' ,
            'password_confirmation.required' => 'Este campo es requerido' ,
            'password.confirmed' => 'Las contraseña no coinciden vuelva a intentar' ,
        ]);


        DB::beginTransaction();

        try {
            $response = new \stdClass();

            $usuario            = new User($request->all());
            $usuario->fk_status = 1; //lo marco como activo
            $usuario->name      = $request->user;
            $response->msj      = 'Usuario Creado Exitosamente';

            $usuario->password = bcrypt($request->password);
            $usuario->save();
            $usuario->perfil;

            $response->user = $usuario;

            DB::commit();

            return response()->json($response , 201);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en UserController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage() ,
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $request->validate([
            'fullName' => 'min:2' ,
            'typeIdentification' => 'min:1' ,
            'numerIdentificacion' => 'min:1|unique:users,numerIdentificacion,' . $id . ',id,deleted_at,NULL' ,
            'phone' => 'min:5' ,
            'profile_fk' => 'min:1' ,
            'email' => 'email' ,
        ] , [
            'fullName.min' => 'El campo no puede estar vacio' ,
            'email.email' => 'El correo debe estar en formato correo' ,
            'numerIdentificacion.unique' => 'El Número de identificación ya esta registrado' ,
            'typeIdentification.min' => 'El campo no puede estar vacio' ,
            'numerIdentificacion.min' => 'El campo no puede estar vacio' ,
            'phone.min' => 'El campo no puede estar vacio' ,
            'profile_fk.min' => 'El campo no puede estar vacio' ,
        ]);
        DB::beginTransaction();
        try {
            $user = User::find($id);
            if ($user){
                $pass_last = $user->password;
                $user->fill([
                    "fullName" => is_null($request->fullName) ? $user->fullName : $request->fullName ,
                    "typeIdentification" => is_null($request->typeIdentification) ? $user->typeIdentification : $request->typeIdentification ,
                    "numerIdentificacion" => is_null($request->numerIdentificacion) ? $user->numerIdentificacion : $request->numerIdentificacion ,
                    "phone" => is_null($request->phone) ? $user->phone : $request->phone ,
                    "profile_fk" => is_null($request->profile_fk) ? $user->profile_fk : $request->profile_fk ,
                    "user" => is_null($request->user) ? $user->user : $request->user ,
                    "email" => is_null($request->email) ? $user->email : $request->email ,
                    "fk_status" => is_null($request->fk_status) ? $user->fk_status : $request->fk_status ,
                    "direccion" => @$request->input('direccion') ,
                ]);

                if ($request->password!=null && !empty($request->password)){
                    $user->password = bcrypt($request->password);
                    //Mail::to($request->email)->send(new NuevoPasswordMail($usuario));
                } else {
                    $user->password = $pass_last;
                }

//            if (!is_null($request->fotoPerfil)){
//                $originalImage = $request->fotoPerfil;
//
//                $thumbnailImage = Image::make($originalImage);
//                @$thumbnailImage->orientate();
//
//                $nombre_publico = $originalImage->getClientOriginalName();
//                $extension      = 'png';
//
//                $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
//                $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
//
//                Storage::disk('local')->put('\\fotoPerfil\\' . $nombre_interno , (string)$thumbnailImage->encode());
//                $user->fotoPerfil = $nombre_interno;
//            }

                $user->save();
//            if (!is_null($user->fotoPerfil)){
//                $user->fotoPerfilURL = asset('storage\\fotoPerfil\\' . $user->fotoPerfil);
//            }
                $response = [
                    'user' => $user ,
                ];
                DB::commit();
                return response()->json($response , 200);
            } else {
                return response()->json(['msj' => 'Usuario no existe'] , 404);
            }


        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en UserController@update: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de actualizar los datos.' ,
            ] , 500);
        }


    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        DB::beginTransaction();
        $status = 200;
        try {
            if (Auth::user()->id!=$id){
                $user = User::where('id' , $id)->get();
                if (count($user) > 0){
                    $user[0]->delete();

                    $response = [
                        'msj' => 'Usuario eliminado Correctamente'
                    ];

                    DB::commit();
                } else {
                    $response = [
                        'msj' => 'El usuario que intenta eliminar no existe'
                    ];
                    $status   = 404;
                }
            } else {
                $response = [
                    'msj' => 'No puede borrar su propio usuario'
                ];
                $status   = 404;
            }

            return response()->json($response , $status);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Ha ocurrido un error en UserController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de eliminar los datos.' ,
            ] , 500);
        }
    }
}
