<?php

namespace App\Http\Controllers;

use App\Models\LogDataVallasMail;
use App\Models\Vallas;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Image;
use function GuzzleHttp\Promise\all;

class VallasController extends Controller
{
    public function index()
    {
        return Vallas::get();
    }

    public function listar_las_vallas_activas()
    {
        return Vallas::where('disponible' , 1)->get();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'nombre' => 'required' ,
            'ubicacion' => 'required' ,
            'latitud' => 'required' ,
            'longitud' => 'required' ,
            'foto' => 'required' ,
            'observacion' => 'required' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
            'ubicacion.required' => ':attribute es requerido' ,
            'latitud.required' => ':attribute es requerido' ,
            'longitud.required' => ':attribute es requerido' ,
            'foto.required' => ':attribute es requerido' ,
            'observacion.required' => ':attribute es requerido' ,
        ]);

        try {
            $valla = new Vallas($request->all());
            if ($request->exists('foto')){
                if (!is_null($request->foto)){
                    /*para la foto*/
                    $originalImage  = $request->foto;
                    $thumbnailImage = Image::make($originalImage);
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\vallas\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/
                    $valla->foto = $nombre_interno;
                }
            }
            $valla->save();
            $response = [
                'msj' => 'Creado exitosamente' ,
                'valla' => $valla ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en VallasController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    public function lote_store(Request $request)
    {
        $this->validate($request , [
            'vallas.*.nombre' => 'required' ,
            'vallas.*.ubicacion' => 'required' ,
            'vallas.*.latitud' => 'required' ,
            'vallas.*.longitud' => 'required' ,
            'vallas.*.observacion' => 'required' ,
            'vallas.*.dosCaras' => 'required' ,
            'vallas.*.switch1Cara' => 'required' ,
            'vallas.*.switch2Caras' => 'required' ,
        ] , [
            'vallas.*.nombre.required' => ':attribute es requerido' ,
            'vallas.*.ubicacion.required' => ':attribute es requerido' ,
            'vallas.*.latitud.required' => ':attribute es requerido' ,
            'vallas.*.longitud.required' => ':attribute es requerido' ,
            'vallas.*.observacion.required' => ':attribute es requerido' ,
            'vallas.*.dosCaras.required' => ':attribute es requerido' ,
            'vallas.*.switch1Cara.required' => ':attribute es requerido' ,
            'vallas.*.switch2Caras.required' => ':attribute es requerido' ,
        ]);

        try {
            foreach ($request->vallas as $valla) {

                $registrosDb = Vallas::where('latitud' , $valla['latitud'])->where('longitud' , $valla['longitud'])->get();
//                return response()->json($registrosDb);
                if (count($registrosDb) > 0){
                    $registrosDb[0]->update($valla);

                } else {
                    $valla_db = new Vallas($valla);
                    $valla_db->save();
                }
            }
            $response = [
                'msj' => 'Creado exitosamente'
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en VallasController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    public function img_lote_store(Request $request)
    {
        $lista = json_decode($request->ids);
//        return response()->json($lista);
//        $this->validate($request , [
//            'vallas.*.nombre' => 'required' ,
//            'vallas.*.ubicacion' => 'required' ,
//            'vallas.*.latitud' => 'required' ,
//            'vallas.*.longitud' => 'required' ,
////            'vallas.*.foto' => 'required' ,
//            'vallas.*.observacion' => 'required' ,
//        ] , [
//            'vallas.*.nombre.required' => ':attribute es requerido' ,
//            'vallas.*.ubicacion.required' => ':attribute es requerido' ,
//            'vallas.*.latitud.required' => ':attribute es requerido' ,
//            'vallas.*.longitud.required' => ':attribute es requerido' ,
////            'vallas.*.foto.required' => ':attribute es requerido' ,
//            'vallas.*.observacion.required' => ':attribute es requerido' ,
//        ]);

        try {
            foreach ($lista as $id_valla) {
                $valla = Vallas::find($id_valla);
                if ($request->exists('imagen') && $valla){
                    if (!is_null($request->imagen)){
                        /*para la foto*/
                        $originalImage  = $request->imagen;
                        $thumbnailImage = Image::make($originalImage);
                        $nombre_publico = $originalImage->getClientOriginalName();
                        $extension      = 'png';
                        $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                        $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                        Storage::disk('public')->put('\\vallas\\' . $nombre_interno , (string)$thumbnailImage->encode());
                        /*para la foto*/
                        $valla->foto = $nombre_interno;
                    }
                    $valla->save();
                }


            }
            $response = [
                'msj' => 'Imágen asigana exitosamente'
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en VallasController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = Vallas::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['Vallas' => $h] , 201);
        }
    }

    public function log_vallas_por_id($id = null)
    {
        if (!is_null($id)){
            $h = LogDataVallasMail::find($id);

            if (is_null($h)){
                return response()->json(['msj' => 'no se encontro resultado'] , 401);
            } else {
                return response()->json(['log_de_vallas_mail' => $h] , 201);
            }
        } else return response()->json(['msj' => 'no se encontro resultado'] , 401);

    }

    public function log_vallas_listar_todas()
    {
        return LogDataVallasMail::get();
    }


    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {

        $this->validate($request , [
            'nombre' => 'required' ,
            'ubicacion' => 'required' ,
            'latitud' => 'required' ,
            'longitud' => 'required' ,
            'foto' => 'required' ,
            'observacion' => 'required' ,
        ] , [
            'nombre.required' => ':attribute es requerido' ,
            'ubicacion.required' => ':attribute es requerido' ,
            'latitud.required' => ':attribute es requerido' ,
            'longitud.required' => ':attribute es requerido' ,
            'foto.required' => ':attribute es requerido' ,
            'observacion.required' => ':attribute es requerido' ,
        ]);

        $h = Vallas::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la Valla solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            if ($request->exists('foto')){
                if (!is_null($request->foto)){
                    /*para la foto*/
                    $originalImage  = $request->foto;
                    $thumbnailImage = Image::make($originalImage);
                    $nombre_publico = $originalImage->getClientOriginalName();
                    $extension      = 'png';
                    $nombre_interno = str_replace('.' . $extension , '' , $nombre_publico);
                    $nombre_interno = str_slug($nombre_interno , '-') . '-' . time() . '-' . strval(rand(100 , 999)) . '.' . $extension;
                    Storage::disk('public')->put('\\vallas\\' . $nombre_interno , (string)$thumbnailImage->encode());
                    /*para la foto*/
                    $h->foto = $nombre_interno;
                }
            }
            $h->save();
            $response = [
                'msj' => 'Valla actualizada exitosamente' ,
                'valla' => $h ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en VallasController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = Vallas::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
