<?php

namespace App\Http\Controllers;

use App\Models\ZonaEnvio;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class ZonaEnvioController extends Controller
{
    public function index()
    {
        return ZonaEnvio::get();
    }
    public function listarPorId($idZonaEnvio)
    {
        return ZonaEnvio::find($idZonaEnvio);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request , [
            'descripcion' => 'required|unique:zona_envios,descripcion,NULL,idZonaEnvio,deleted_at,NULL' ,
            'precio' => 'required' ,
            'fk_moneda' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
            'descripcion.unique' => ':input ya esta registrado' ,
            'precio.required' => ':attribute es requerido' ,
            'fk_moneda.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        try {
            $tipo = new ZonaEnvio($request->all());
            $tipo->save();
            $tipo->moneda;
            $response = [
                'msj' => 'Creado exitosamente' ,
                'zona_envio' => $tipo->refresh() ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en ZonaEnvioController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
                'error' => $e->getMessage()
            ] , 500);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $h = ZonaEnvio::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        } else {
            return response()->json(['zona_envio' => $h] , 201);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request , $id)
    {
        $this->validate($request , [
            'descripcion' => 'required|unique:zona_envios,descripcion,' . $id . ',idZonaEnvio,deleted_at,NULL' ,
            'precio' => 'required' ,
            'fk_moneda' => 'required' ,
        ] , [
            'descripcion.required' => ':attribute es requerido' ,
            'precio.required' => ':attribute es requerido' ,
            'fk_moneda.required' => ':attribute es requerido' ,
        ]);
        DB::beginTransaction();
        $h = ZonaEnvio::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'No se logro conseguir la zona de envío solicitada'] , 401);
        }

        try {
            $h->fill($request->all());
            $h->save();
            $response = [
                'msj' => 'Zona de envío actualizado exitosamente' ,
                'zona_envio' => $h->refresh() ,
            ];
            DB::commit();
            return response()->json($response , 201);
        } catch (\Exception $e) {

            DB::rollback();
            Log::error('Ha ocurrido un error en ZonaEnvioController: ' . $e->getMessage() . ', Linea: ' . $e->getLine());

            return response()->json([
                'message' => 'Ha ocurrido un error al tratar de guardar los datos.' ,
            ] , 500);
        }

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $h = ZonaEnvio::find($id);

        if (is_null($h)){
            return response()->json(['msj' => 'no se encontro resultado'] , 401);
        }

        $h->delete();
        $h->save();
        return response()->json(['msj' => 'Se elimino exitosamente'] , 201);
    }
}
