<?php

namespace App\Jobs;

use App\Http\Controllers\OrderHeaderController;
use App\Mail\QRmail;
use App\Mail\ResumenCompraMail;
use App\Mail\ResumenCotizacionMail;
use App\Models\OrderHeader;
use App\Models\TiemposDeEntrega;
use Barryvdh\DomPDF\Facade as PDF;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use QrCode;

class GenerarPdfCotizacionJob /*implements ShouldQueue*/
{
    use Dispatchable , InteractsWithQueue , Queueable , SerializesModels;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    protected $idOrden;

    public function __construct($idOrden)
    {
        $this->idOrden = $idOrden;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $orden = OrderHeader::find($this->idOrden);
        $data           = OrderHeaderController::resumen_orden_reusable($this->idOrden)->getData();
        $orden->resumen = $data;
        $pdf = PDF::loadView('qr.index5_cotizacion' , ["orden" => $orden]);
        //Renderiza el archivo primero
        $pdf->render();
        $nombre_interno = $this->idOrden . '_cotizacion_' . time() . ".pdf";
        //Guardalo en una variable
        $output = $pdf->output();
        try {
            $email = $orden->customer->email;
//            $email = 'alecortez240192@gmail.com';
            Mail::to($email)->send(new ResumenCotizacionMail($output , $orden->n_order , $orden));
        } catch (\Exception $e) {
            Log::error('No se pudo enviar el mail con id ' . $this->idOrden . ', se muestra el error: ' . $e->getMessage() . ', Linea: ' . $e->getLine());
        }
    }

    public static function get_qr($palabra)
    {
        $qr_64 = base64_encode(QrCode::format('png')->size(1000)->generate($palabra));
        return $qr_64;
    }
}
