<?php

namespace App\Jobs;

use App\Models\LogSMS;
use App\Models\User;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Http\Request;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class SendSMSJob implements ShouldQueue
{
    use Dispatchable , InteractsWithQueue , Queueable , SerializesModels;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    protected $destino , $mensaje;

    public function __construct($destino , $mensaje)
    {
        $this->destino = $destino;
        $this->mensaje  = $mensaje;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        //configuracion Gateway SMS
        $url         = "https://www.interconectados.net/api2/?phonenumber=";
        $phonenumber = $this->destino;
        $text        = $this->mensaje;
        $user        = "islapixel";
        $password    = env('TOCKEN_ENVIO_SMS','null');
        $texto       = urlencode($text); //para enviar cadena de texto codificada
        //Asignamos variables
        try {
            // Se crea un manejador CURL para realizar la petición
            $ch = curl_init();
            // Se establece la URL y algunas opciones
            curl_setopt($ch , CURLOPT_URL , "" . $url . "" . $phonenumber . "&text=" . $texto . "&user=" . $user . "&password=" . $password . "");
            curl_setopt($ch , CURLOPT_RETURNTRANSFER , 1);
            // Se obtiene la URL del SMS Gateway
            $res = curl_exec($ch);
            // Obtener la respuesta del mensaje recibido por el gateway
//            Log::info(['msm'=>$res] );
            throw new \Exception($res);
        } catch (\Exception $e) {
            //Manejos de Excepciones
//            return response()->json($e->getMessage());

            $r = null;
            if ($e->getMessage()==401){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Autenticacion Invalida"]));
            }
            if ($e->getMessage()==400){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "El mensaje no puede ser NULL"]));
            }
            if ($e->getMessage()==405){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "El númeo de telefono no puede ser NULL"]));
            }
            if ($e->getMessage()==402){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Creditos Insuficientes"]));
            }
            if ($e->getMessage()==403){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Cuenta Inactiva"]));
            }
            if ($e->getMessage()==501){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Tipo de Cuenta Invalida"]));
            }
            if ($e->getMessage()==502){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Peticion Sobrecargada"]));
            }
            if ($e->getMessage()=="200 / 1"){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Mensaje Enviado"]));
            }
            if ($e->getMessage()=="200 / 2"){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Mensaje muy largo"]));
            }
            if ($e->getMessage()=="200 / 0"){
                self::loga(new Request(['text' => $text , 'phonenumber' => $phonenumber , 'e' => $e , 'r' => "Número de teléfono erroneo"]));
            }
        }
// Se cierra el recurso CURL y se liberan los recursos del sistema
        curl_close($ch);
    }
    static function loga(Request $request)
    {
        $u   = null;
        $log = new LogSMS([
            'mensaje' => $request->text ,
            'destinatario' => $request->phonenumber ,
            'idUser' => User::where('phone' , $request->phonenumber)->first() ? User::where('phone' , $request->phonenumber)->first()['id'] : 0 ,
            'respuestaapi' => $request->e->getMessage() ,
            'respuestaapiinterpretado' => $request->r ,
        ]);
        $log->save();
        return $log;
    }

}
