<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Caja extends Model
{
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d H:i:s');
    }

    use SoftDeletes;
    use HasFactory;
    protected $table = 'cajas';
    protected $primaryKey = 'idCaja';
    protected $with = ['userClose' , 'sucursal',  'estado', 'operaciones'];
//    protected $appends = ['total_operaciones_ingresos','total_operaciones_egresos'];
//    protected $casts = ['status' => 'boolean' ];
    protected $fillable = [
        'nombre' ,
        'idUserClose' ,
        'fecha_creacion' ,
        'fecha_cierre' ,
        'status' ,
        'idSucursal'
    ];
    protected $hidden = [
        'created_at' ,
        'updated_at' ,
        'deleted_at' ,
    ];
    protected $dates = ['fecha_creacion' , 'fecha_cierre'];

    public function scopeActive($query)
    {
        $query->where('status' , 1);
    }
    public function scopeCerrada($query)
    {
        $query->where('status' , 2);
    }

    public function userClose()
    {
        return $this->belongsTo(User::class , 'idUserClose');
    }

    public function estado()
    {
        return $this->belongsTo(EstadoCaja::class , 'status');
    }

    public function sucursal()
    {
        return $this->belongsTo(Sucursal::class , 'idSucursal');
    }

    public function operaciones()
    {
        return $this->hasMany(Operacion::class , 'idCaja' , 'idCaja')->without('orderHeader');
    }

//    public function getTotalOperacionesIngresosAttribute()
//    {
//        return round($this->operaciones()->where('idTipoTransaccion',1)->sum('monto'),2);
//    }
//    public function getTotalOperacionesEgresosAttribute()
//    {
//        return round($this->operaciones()->where('idTipoTransaccion',2)->sum('monto'),2);
//    }


}
