<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Operacion extends Model
{
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d H:i:s');
    }

    use SoftDeletes;
    use HasFactory;
    protected $table = 'operaciones';
    protected $primaryKey = 'idOperacion';
    protected $with = ['formaPago' , 'tipoOperacion' , 'moneda' , 'usuario_creador'];
    protected $casts = [
        'monto' => 'decimal:2' ,
        'created_at' => 'date:Y-m-d' ,
        'impuesto' => 'boolean' ,
    ];
    protected $fillable = [
        'descripcion' ,
        'monto' ,
        'idTipoTransaccion' ,
        'idFormaPago' ,
        'idCaja' ,
        'idOrderHeader' ,
        'n_order' ,
        'fk_moneda' ,
        'idUser' ,
        'tasa_moneda' ,

        'impuesto' ,
        'idOrderHTax' ,
        'historia_borrado',

        'banco',
        'numero_transaccion',
        'descripcion_operacion',
    ];
    protected $hidden = [
        'updated_at' ,
        'deleted_at' ,
    ];

//    protected $dates = ['created_at'];

    public function usuario_creador()
    {
        return $this->belongsTo(User::class , 'idUser');
    }

    public function formaPago()
    {
        return $this->belongsTo(FormaDePago::class , 'idFormaPago');
    }

    public function moneda()
    {
        return $this->belongsTo(Moneda::class , 'fk_moneda');
    }

    public function tipoOperacion()
    {
        return $this->belongsTo(TipoOperacion::class , 'idTipoTransaccion');
    }

    public function caja()
    {
        return $this->belongsTo(Caja::class , 'idCaja');
    }

    public function orderHeader()
    {
        return $this->belongsTo(OrderHeader::class , 'idOrderHeader')->withOnly('estatus');
    }

    public function orderHeaderNecesarioReporte()
    {
        return $this->belongsTo(OrderHeader::class , 'idOrderHeader')
            ->withOnly(['body','formaPago' , 'moneda_orderheader'])
            ->with('body' , function ($sql){
                $sql->withOnly('estatus');
            });

    }

    public function orderHeader_solo_customer()
    {
        return $this->belongsTo(OrderHeader::class , 'idOrderHeader')->withOnly('customer');
    }
}
