<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class OrderBody extends Model
{
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d');
    }

    use HasFactory;
    use SoftDeletes;
//    protected $softCascade = ['servicios' , 'elementos'];
    protected $table = 'order_bodys';
    protected $with = ['valla','tiempo_entrega','zona_envio' , 'estatus' , 'producto' , 'servicios' , 'elementos' , 'historiaStatus' , 'personal'];
    protected $primaryKey = 'idOrderBody';
    protected $casts = ['is_express_delivery' => 'boolean'];
//    protected $appends = ['UrlImagen'];
    protected $fillable = [
        'fk_orderHeader' ,
        'idProduct' ,
        'cantidad' ,
        'precio' ,
        'fk_moneda' ,
        'fk_status' ,
        'ancho' ,
        'alto' ,
        'idPersonal' ,
        'archivo' ,
        'especificaciones_del_producto' ,
        'fk_idZonaEnvio' ,
        'color' ,
        'tiempo_de_entrega' ,
        'monto_extra_de_tiempo_de_entrega' ,
        'fk_idTiempoEntrega' ,
        'is_express_delivery',

        'fk_idValla' //solo para tema de vallas
    ];

    public function tiempo_entrega()
    {
        return $this->belongsTo(TiemposDeEntrega::class , 'fk_idTiempoEntrega');
    }

    public function valla()
    {
        return $this->belongsTo(Vallas::class , 'fk_idValla');
    }

    public function personal()
    {
        return $this->belongsTo(Personal::class , 'idPersonal');
    }

    public function moneda()
    {
        return $this->belongsTo(Moneda::class , 'fk_moneda');
    }

    public function estatus()
    {
        return $this->belongsTo(EstatusOrden::class , 'fk_status');
    }

    public function producto()
    {
        return $this->belongsTo(Producto::class , 'idProduct')->withTrashed();
    }

    public function orderHeader()
    {
        return $this->belongsTo(OrderHeader::class , 'fk_orderHeader')->without('body');
    }

    public function servicios()
    {
        return $this->hasMany(ServiciosOrden::class , 'fk_orden_body' , 'idOrderBody')->withTrashed();
    }

    public function elementos()
    {
        return $this->hasMany(ElementosOrden::class , 'fk_orden_body' , 'idOrderBody')->withTrashed();
    }

    public function historiaStatus()
    {
        return $this->hasMany(HistoriaStatusOB::class , 'idOrderBody' , 'idOrderBody');
    }

    public function zona_envio()
    {
        return $this->belongsTo(ZonaEnvio::class , 'fk_idZonaEnvio');
    }
//    public function getUrlImagenAttribute()
//    {
//        return is_null($this->archivo) ? null : asset('storage\\app\\public\\archivobodycotizacion\\' . $this->archivo);
//    }
}
