<?php

namespace App\Models;

use Askedio\SoftCascade\Traits\SoftCascadeTrait;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use phpDocumentor\Reflection\DocBlock\Tags\Uses;
use phpDocumentor\Reflection\Utils;

class OrderHeader extends Model
{

    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d');
    }

    use HasFactory;
    use SoftDeletes;
//    use SoftCascadeTrait;
    protected $softCascade = ['body'];
    protected $table = 'order_headers';
    protected $with = ['fueImpresa','tiempo_entrega' , 'lista_impuestos' , 'formaPago' , 'interacciones' , 'estatus' , 'descuento' , 'usuario_creador' , 'usuario_lector' , 'customer' , 'zonaEnvio' , 'moneda_orderheader' , 'estadoOrden' , 'sucursal_orden' , 'body' , 'historiaStatus' , 'transaccion'];
    protected $primaryKey = 'idOrderHeader';
    protected $appends = ['UrlImagen'];
    protected $casts = ['pedido_enviado' => 'boolean' , 'is_express_delivery' => 'boolean' , 'isPrint' => 'boolean' , 'punto_de_venta' => 'boolean' , 'orden_creada_cliente' => 'boolean' , 'cuenta_por_cobrar' => 'boolean' , 'cotizacion' => 'boolean' , 'order_open' => 'boolean' , 'esta_leido' => 'boolean'];
    protected $fillable = [
        'idUser' ,
        'fk_payform' ,
        'recibo_pago' ,
        'notes' ,
        'total_diseno' ,
        'fk_moneda' ,
        'fk_descuento' ,
        'fk_zonaEnvio' ,
        'precio_total' ,
        'fk_status' ,
        'date_created' ,
        'date_recived' ,
        'date_delivery' ,
        'date_cancel' ,
        'fk_sucursal' ,
        'img_pago' ,
        'n_order' ,
        'tocken_url' ,
        'order_open' ,
        'tasa_moneda' ,

        'idUsuario' , //para notificacion
        'esta_leido' , //para notificacion
        'fecha_leido' , //para notificacion

        'resumen_pago' ,
        'comprobante_factura' ,
        'enviar_a_taller' ,
        'idUserCreated' ,
        'cotizacion' ,
        'n_factura_seniat' ,
        'response_mercantil' ,
        'cuenta_por_cobrar' ,
        'a_quien_entregan_orden' ,
        'nota_de_credito' ,
        'created_at' ,
        'orden_creada_cliente' ,
        'punto_de_venta' ,
        'isPrint' ,
        'fecha_facturacion' ,
        'aplica_pago_extra_por_tiempo_de_entrega' ,
        'fk_idTiempoEntrega' , //para cuando la orden sea urgente, normal o relax
        'is_express_delivery' , //para identificar la orden como urgente
        'sucursal_facturada' ,
        'pedido_enviado' ,  //por defecto false, si es true la tabla ordenes_enviadas_impresas tendra el idHeader de relacion para buscar la relacion

        'fecha_vencimiento', //solo para vallas
        'estatus_contrato',  //solo para vallas  1 y 2, activo y vencido
    ];

    protected $dates = [
        'date_created' ,
        'date_delivery' ,
        'fecha_facturacion'
    ];

    public function scopeLeido($query)
    {
        $query->where('esta_leido' , true);
    }

    public function scopeNoLeido($query)
    {
        $query->where('esta_leido' , false)->orWhereNull('esta_leido');
    }

    public function formaPago()
    {
        return $this->belongsTo(FormaDePago::class , 'fk_payform');
    }
    public function fueImpresa()
    {
        return $this->hasMany(OrdenesEnviadasImpresas::class , 'fk_idOrdenHeader','idOrderHeader');
    }

    public function customer()
    {
        return $this->belongsTo(User::class , 'idUser')->withTrashed();
    }

    public function tiempo_entrega()
    {
        return $this->belongsTo(TiemposDeEntrega::class , 'fk_idTiempoEntrega');
    }

    public function descuento()
    {
        return $this->belongsTo(Descuento::class , 'fk_descuento');
    }

    public function usuario_lector()
    {
        return $this->belongsTo(User::class , 'idUsuario')->withTrashed();
    }

    public function usuario_creador()
    {
        return $this->belongsTo(User::class , 'idUserCreated');
    }

    public function estadoOrden()
    {
        return $this->belongsTo(EstatusOrden::class , 'fk_status');
    }

    public function estatus()
    {
        return $this->belongsTo(EstatusOrden::class , 'fk_status');
    }

    public function moneda_orderheader()
    {
        return $this->belongsTo(Moneda::class , 'fk_moneda');
    }

    public function zonaEnvio()
    {
        return $this->belongsTo(ZonaEnvio::class , 'fk_zonaEnvio');
    }

    public function sucursal_orden()
    {
        return $this->belongsTo(Sucursal::class , 'fk_sucursal');
    }

    public function sucursal_orden_factuacion()
    {
        return $this->belongsTo(Sucursal::class , 'sucursal_facturada');
    }

    public function body()
    {
        return $this->hasMany(OrderBody::class , 'fk_orderHeader' , 'idOrderHeader');
    }

    public function getUrlImagenAttribute()
    {
        return is_null($this->img_pago) ? null : env('BASE_URL_IMG') . '\\storage\\app\\public\\comprobantepago\\' . $this->img_pago;
    }

    public function historiaStatus()
    {
        return $this->hasMany(HistoriaStatusOH::class , 'idOrderHeader' , 'idOrderHeader');
    }

    public function transaccion()
    {
        return $this->hasMany(Operacion::class , 'idOrderHeader' , 'idOrderHeader')->without('orderHeader');
    }

    public function lista_impuestos()
    {
        return $this->hasMany(Operacion::class , 'idOrderHTax' , 'idOrderHeader')->without('orderHeader');
    }

    public function interacciones()
    {
        return $this->hasMany(Interacciones::class , 'fk_idHeader' , 'idOrderHeader');
    }

    public function getAbonoTotalAttribute() //abono
    {
        return (float)$this->transaccion()->where('idTipoTransaccion' , 1)->sum('monto');
    }

    public function getDeudaTotalAttribute() //deuda
    {
        return ((float)$this->precio_total - (float)$this->transaccion()->where('idTipoTransaccion' , 1)->sum('monto'));
    }

}
