<?php

namespace App\Models;

use App\Http\Controllers\TasaMonedaController;
use Askedio\SoftCascade\Traits\SoftCascadeTrait;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Producto extends Model
{
    use SoftCascadeTrait;

    protected $with = ['galeria_imagenes','status' , 'subTipo' , 'moneda' , 'elementos' , 'servicios' , 'modalidad' , 'tipoPersonal'];
    protected $softCascade = ['elementos' , 'servicios'];
    use SoftDeletes;
    use HasFactory;
    protected $table = 'productos';
    protected $primaryKey = 'idProducto';
    protected $casts = ['mostrar_en_punto_de_venta' => 'boolean' ,'permitir_un_elemento' => 'boolean' ,'is_offer' => 'boolean' ,'aplica_redondeo' => 'boolean' , 'destacar' => 'boolean' , 'used_size' => 'boolean' , 'active' => 'boolean' , 'permite_solo_cantidades' => 'boolean' , 'mostrar_detalle' => 'boolean'];
    protected $appends = [ 'UrlImagen' , 'monedas_convertir'];

    protected $fillable = [
        'name' ,
        'descripcion' ,
        'fk_subType' ,
        'precio' ,
        'fk_status' ,
        'fk_moneda' ,
        'destacar' ,
        'fk_tipoModalidad' ,
        'img' ,
        'used_size' ,
        'dias_entrega' ,
        'comision_porcentaje' ,
        'comision' ,
        'fk_idTipopersonal' ,
        'idPersonal' ,
        'alto_estandard' ,
        'ancho_estandard' ,
        'active' ,  //activar para mostrar en la web
        'permite_solo_cantidades' ,
        'mostrar_detalle' ,
        'infomacion_adicional' ,
        'especificaciones_del_producto' ,
        'aplica_redondeo' ,
        'color' ,
        'is_offer' ,
        'permitir_un_elemento' ,
        'mostrar_en_punto_de_venta' ,
    ];

    protected $hidden = [

    ];

    public function scopeActivo($query)
    {
        $query->where('active' , true);
    }
    public function scopePuntoventa($query)
    {
        $query->where('mostrar_en_punto_de_venta' , true);
    }

    public function getIdValueAttribute()
    {
        return $this->idProducto;
    }

    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d H:i:s');
    }

    public function status()
    {
        return $this->belongsTo(Estado::class , 'fk_status');
    }

    public function galeria_imagenes()
    {
        return $this->hasMany(ImagenesProductos::class , 'fk_idProducto','idProducto');
    }

    public function prodRelacionados()
    {
        return $this->hasMany(Producto::class , 'fk_subType' , 'fk_subType')
            ->where('idProducto' , '!=' , $this->id_value) //que no sea igual al que esta consultado
            ->where('fk_tipoModalidad' , '!=' , 5) //distinto de 5
            ->where('destacar' , 1) //que sea destacado
            ->limit(4) //solo 4 productos
            ->without('prodRelacionados');
    }

    public function tipoPersonal()
    {
        return $this->belongsTo(TipoPersonal::class , 'fk_idTipopersonal');
    }

    public function subTipo()
    {
        return $this->belongsTo(SubTipo::class , 'fk_subType');
    }

    public function moneda()
    {
        return $this->belongsTo(Moneda::class , 'fk_moneda');
    }

    public function modalidad()
    {
        return $this->belongsTo(Modalidad::class , 'fk_tipoModalidad');
    }

    public function getMonedasConvertirAttribute()
    {
        return TasaMonedaController::listarAgrupado2($this);
    }

    public function elementos()
    {
        return $this->hasMany(Elemento::class , 'fk_idProducto' , 'idProducto');
    }

    public function servicios()
    {
        return $this->hasMany(ServiciosProducto::class , 'fk_idProducto' , 'idProducto');
    }

    public function getUrlImagenAttribute()
    {
        return is_null($this->img) ? null : env('BASE_URL_IMG').'\\storage\\app\\public\\producto\\' . $this->img;
    }
    public function inventario_header($idSucursal)
    {
        return $this->hasMany(InventarioDeProducto::class , 'fk_idProducto' , 'idProducto')
            ->where('fk_idSucursal' , $idSucursal)->without('producto');
    }
    public function inventario()
    {
        return $this->hasMany(InventarioDeProducto::class , 'fk_idProducto' , 'idProducto')->without('producto');
    }

//    public function inventario()
//    {
//        return $this->hasMany(InventarioDeProducto::class , 'fk_idProducto' , 'idProducto')
//            ->with(['inventario_header'=>function ($sql){
//                $sql->without('body');
//            }]);
//    }

//    public function getStockAttribute()
//    {
//        return $this->inventario()->where('incremento' , 1)->sum('cantidad') - $this->inventario()->where('incremento' , 0)->sum('cantidad');
//    }
//
//    public function getInventarioentradaAttribute()
//    {
//        return (int) $this->inventario()->whereHas('inventario_header',function ($sql){
//            $sql->where('fk_tipo_inventario' , 1);
//        })->sum('cantidad');
//    }
//
//    public function getInventariosalidaAttribute()
//    {
//        return (int) $this->inventario()->whereHas('inventario_header',function ($sql){
//            $sql->where('fk_tipo_inventario' , 2);
//        })->sum('cantidad');
//    }

//    public function getInventarioautoconsumoAttribute()
//    {
//        return (int) $this->inventario()->whereHas('inventario_header',function ($sql){
//            $sql->where('fk_tipo_inventario' , 3);
//        })->sum('cantidad');
//    }

}
