<?php

namespace App\Models;

use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Laravel\Passport\HasApiTokens;


//use Laravel\Sanctum\HasApiTokens;

class User extends Authenticatable
{
    protected function serializeDate(\DateTimeInterface $date)
    {
        return $date->format('Y-m-d H:i:s');
    }

//    protected $dateFormat = 'U';

    use SoftDeletes;
    use HasApiTokens , HasFactory , Notifiable;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $with = ['historial_notificaciones_cobro' , 'perfil' , 'status' , 'notificaciones_cobros'];
//    protected $appends = ['deuda'];
    protected $fillable = [
        'email' ,
        'password' ,
        'fullName' ,
        'typeIdentification' ,
        'numerIdentificacion' ,
        'phone' ,
        'profile_fk' ,
        'user' ,
        'fk_status' ,
        'direccion' ,
        'lista_negra' ,
        'permitir_credito' ,
        'aplica_descuento_automatico' ,
    ];
    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'name' ,
        'password' ,
        'remember_token' ,
        'email_verified_at' ,
//        'deleted_at' ,
        'profile_fk'
    ];
    protected $dates = [
        'created_at' ,
        'deleted_at' ,
        'updated_at'
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime' ,
        'lista_negra' => 'boolean' ,
        'permitir_credito' => 'boolean' ,
        'aplica_descuento_automatico' => 'boolean'
    ];

    public function perfil()
    {
        return $this->belongsTo(Rol::class , 'profile_fk');
    }

    public function status()
    {
        return $this->belongsTo(Estado::class , 'fk_status');
    }

    public function orden_cuenta_por_cobrar()
    {
        return $this->hasMany(OrderHeader::class , 'idUser' , 'id')
            ->with('estatus')
            ->withOnly('estatus')
            ->where('cuenta_por_cobrar' , true);
    }

    public function descuento()
    {
        return $this->hasMany(Descuento::class , 'idUsuario' , 'id')
            ->where('fecha_vencimiento' , '>=' , now()->format('Y-m-d H:i:s'));
    }

    public function ordenes() //con cuenta por cobrar
    {
        return $this->hasMany(OrderHeader::class , 'idUser' , 'id')
            ->without(
            'interacciones' ,
            'estatus' ,
            'descuento' ,
            'usuario_creador' ,
            'customer' ,
            'zonaEnvio' ,
            'moneda_orderheader' ,
            'estadoOrden' ,
            'sucursal_orden' ,
            'body' ,
            'historiaStatus'
        )->where('cuenta_por_cobrar' , true)
            ->where('fk_status' , '!=' , 10);
    }

    public function getDeudaAttribute()
    {
        return $this->orden_cuenta_por_cobrar()->count() > 0 ? true : false;
    }

    public function notificaciones_cobros()
    {
        return $this->hasMany(NotificacionCobro::class , 'fk_idCliente' , 'id')
            ->with('asesor');
    }

    public function historial_notificaciones_cobro()
    {
        return $this->hasMany(HistorialNotificacionCobroSms::class , 'idUserNotificado' , 'id');
    }


}
