<?php

use App\Http\Controllers\AuthController;
use App\Http\Controllers\CajaController;
use App\Http\Controllers\ControladorTest;
use App\Http\Controllers\DatosAdicionalesOrdenesController;
use App\Http\Controllers\DescuentoController;
use App\Http\Controllers\EstatusOrdenesController;
use App\Http\Controllers\FechaDeNoEntregaController;
use App\Http\Controllers\FormaDePagoController;
use App\Http\Controllers\GaleriaController;
use App\Http\Controllers\InteraccionesController;
use App\Http\Controllers\InventarioDeProductoController;
use App\Http\Controllers\MarcaController;
use App\Http\Controllers\MetricasController;
use App\Http\Controllers\MonedaController;
use App\Http\Controllers\NoticiasController;
use App\Http\Controllers\NotificacionCobroController;
use App\Http\Controllers\OperacionController;
use App\Http\Controllers\OrderBodyController;
use App\Http\Controllers\OrderHeaderController;
use App\Http\Controllers\PersonalController;
use App\Http\Controllers\ProductoController;
use App\Http\Controllers\PromocionesController;
use App\Http\Controllers\ProveedoresController;
use App\Http\Controllers\ReportesController;
use App\Http\Controllers\SmsController;
use App\Http\Controllers\SubTipoController;
use App\Http\Controllers\SucursalController;
use App\Http\Controllers\SuscripcionesController;
use App\Http\Controllers\TasaMonedaController;
use App\Http\Controllers\TipoPersonalController;
use App\Http\Controllers\TiposController;
use App\Http\Controllers\UserController;
use App\Http\Controllers\VallasController;
use App\Http\Controllers\ZonaEnvioController;
use App\Models\Elemento;
use App\Models\ElementosOrden;
use App\Models\EstatusOrden;
use App\Models\FechaDeNoEntrega;
use App\Models\OrderBody;
use App\Models\OrderHeader;
use App\Models\Sucursal;
use App\Models\TasaMoneda;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use function Nette\Utils\data;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/


Route::prefix('v1')->middleware('auth:api')->group(function (){
    Route::post('filtro_ordenes_endeuda_por_asesor' , [OrderHeaderController::class , 'filtro_ordenes_endeuda_por_asesor']);
    Route::get('solo_clientes' , [UserController::class , 'listar_clientes']);
    Route::get('productosSinMateriaPrima' , [ProductoController::class , 'productosSinMateriaPrima']);
    Route::get('listado_clientes_con_deuda' , [UserController::class , 'listado_clientes_con_deuda']);
    Route::get('validar_autenticacion' , [AuthController::class , 'validar']);
    Route::resource('user' , UserController::class)->except('store');
    Route::post('user_listar_paginador' , [UserController::class , 'index2']);
    Route::post('user_buscar' , [UserController::class , 'user_buscar']);
    Route::post('buscador_user' , [UserController::class , 'buscador']);
    Route::get('logout' , [AuthController::class , 'logout']); //cerrar sesion
    Route::get('getUser' , [AuthController::class , 'user']);  //Obtener usuarios autenticados
    Route::get('getUserAuth' , [UserController::class , 'getUser']);
    Route::get('getUser/{idUser}' , [UserController::class , 'getUserId']);
    Route::get('user_listar' , [UserController::class , 'listar']);
    Route::resource('producto' , ProductoController::class)->except('update' , 'index');

    //nueva ruta para obtener todos los productos con paginacion
    Route::post('obtener_todos_productos' , [ProductoController::class , 'index']);

    Route::get('listar_producto_condicionda/{idPerfilUser?}' , [ProductoController::class , 'listar_producto_condicionda']);
    Route::post('listar_producto_condicionda_paginador/{idPerfilUser?}' , [ProductoController::class , 'listar_producto_condicionda_paginador']);

    Route::post('producto2' , [ProductoController::class , 'store2']); //solo para uso de postman
    Route::get('producto/recuperarProducto/{idProducto}' , [ProductoController::class , 'recuperarProducto']);
    Route::post('producto/edit/{idProducto}' , [ProductoController::class , 'update']);
    Route::post('producto/edit2/{idProducto}' , [ProductoController::class , 'update2']); //solo para uso de postman

    Route::get('getProductosParaServicios' , [ProductoController::class , 'getProductosParaServicios']);
    Route::get('getProductosParaElementos' , [ProductoController::class , 'getProductosParaElementos']);
    Route::get('moneda/changeStatus/{idMoneda}' , [MonedaController::class , 'cambiarStatus']);
    Route::post('moneda' , [MonedaController::class , 'store']);

    Route::post('moneda/establecerMonedaPorDefecto' , [MonedaController::class , 'establecerMonedaPorDefecto']);

    Route::resource('tipo' , TiposController::class);
    Route::resource('fecha_no_entrega' , FechaDeNoEntregaController::class);

    Route::resource('sub_tipo' , SubTipoController::class);
    Route::resource('sucursal' , SucursalController::class);
    Route::resource('tasamoneda' , TasaMonedaController::class);
    Route::resource('zonaenvio' , ZonaEnvioController::class)->except('index');
    Route::get('zonaenvio/{idZonaEnvio}' , [ZonaEnvioController::class , 'listarPorId']);
    Route::delete('ordenHeader/{idOrdenHeader}' , [OrderHeaderController::class , 'destroy']);

    Route::post('cambiarStatusHeader' , [OrderHeaderController::class , 'cambiarStatusHeader']);
    Route::post('cambiarStatusBody' , [OrderBodyController::class , 'cambiarStatusBody']);
    Route::post('leer_orden' , [OrderHeaderController::class , 'leer']);
    Route::resource('caja' , CajaController::class);
    Route::get('caja/filter/{idFormaPago}' , [CajaController::class , 'filter']);
    Route::resource('operacion' , OperacionController::class);
    Route::post('operacion_sin_caja' , [OperacionController::class , 'storesincaja']);
    Route::post('operaciones_impuestos' , [OperacionController::class , 'operaciones_impuestos']);

    Route::post('operacion/edit/{id?}' , [OperacionController::class , 'editar_operacion']);

    Route::post('cerrar_orden_cuenta_por_cobrar' , [OperacionController::class , 'cerrar_orden_cuenta_por_cobrar']);

    Route::post('cerrarCaja' , [CajaController::class , 'cerrarCaja']);
    Route::post('cajascerradas' , [CajaController::class , 'cajascerradas']);
    Route::post('cerrarOrder' , [CajaController::class , 'cerrarOrder']);
    Route::post('cerrarOrderCuentaPorCobrar' , [CajaController::class , 'cerrarOrderCuentaPorCobrar']);
    Route::get('totalesPorSucursal/{idSucursal}' , [OperacionController::class , 'totalesPorSucursal']);
    Route::post('orden_cambio_a_taller' , [OrderHeaderController::class , 'orden_cambio_a_taller']);
    Route::post('orden/actualizarfacturaseniat' , [OrderHeaderController::class , 'actualizarfacturaseniat']);
    Route::post('enviarsms' , [SmsController::class , 'sendSMS']);
    Route::post('orden/cambiar/cuenta_por_cobrar' , [OrderHeaderController::class , 'cambiar_cuenta_por_cobrar']);
    Route::resource('inventario_producto' , InventarioDeProductoController::class)->except('index');
    Route::post('inventario_producto/listar_paginador' , [InventarioDeProductoController::class , 'index']);

    Route::post('inventario_producto_agrupado/{idProducto}' , [InventarioDeProductoController::class , 'inventario_producto_agrupado']);
    Route::delete('inventario_borrar_body/{idBody}' , [InventarioDeProductoController::class , 'destroyBody']);
    Route::post('listar_productos_con_stock' , [InventarioDeProductoController::class , 'listar_productos_con_stock']);
    Route::post('listar_productos_con_stock_por_fecha' , [InventarioDeProductoController::class , 'listar_productos_con_stock_por_fecha']);
    Route::post('listar_productos_con_todos_los_pametros' , [InventarioDeProductoController::class , 'listar_productos_con_todos_los_pametros']);
    Route::post('crear_inventario_desde_orden' , [InventarioDeProductoController::class , 'crear_inventario_desde_orden']);
    Route::post('inventario_cambiar_status' , [InventarioDeProductoController::class , 'inventario_cambiar_status']);
    Route::get('listar_movimiento_por_producto/{idProducto}' , [InventarioDeProductoController::class , 'listar_movimiento_por_producto']);
    Route::post('cambiar_cuenta_por_cobrar_false' , [OrderHeaderController::class , 'cambiar_cuenta_por_cobrar_false']);
    Route::post('actualizar_nota_de_credito' , [OrderHeaderController::class , 'actualizar_nota_de_credito']);
    Route::post('actualizar_comprobante_factura' , [OrderHeaderController::class , 'actualizar_comprobante_factura']);
    Route::post('buscador_pedidos' , [OrderHeaderController::class , 'buscador_pedidos']);
    Route::get('periodos_de_cajas_cerradas' , [CajaController::class , 'periodos_de_cajas_cerradas']);
    Route::post('reportecajas' , [CajaController::class , 'export']);
    Route::post('reportecomisionporventas' , [CajaController::class , 'reportecomisionporventas']);
    Route::post('cambiar_cotizacion_a_orden' , [OrderHeaderController::class , 'cambiar_cotizacion_a_orden']);
    Route::post('ordenHeader_listar_con_buscador' , [OrderHeaderController::class , 'ordenHeader_listar_con_buscador']); //buscador general
    Route::post('ordenHeader_listar_con_buscador2' , [OrderHeaderController::class , 'ordenHeader_listar_con_buscador2']);
    Route::post('orderList_drafts' , [OrderHeaderController::class , 'orders_drafts_lists']);
    Route::post('buscador_de_ordenes_general' , [OrderHeaderController::class , 'buscador_general']); //buscador general
    Route::post('buscador_de_ordenes_general_2' , [OrderHeaderController::class , 'buscador_general_2']); //buscador general2
//    Route::get('listar_sucursales_y_usuarios' , [OrderHeaderController::class , 'listar_sucursales_y_usuarios']);
    Route::post('ordenes_listar_parametros' , [OrderHeaderController::class , 'indexListarParametros2']);
    //idorderheader  precio, y actualiza precio total y cotizacion=false
    Route::post('actualizar_precio_body' , [OrderBodyController::class , 'actualizar_precio_body']);
    Route::resource('datos_adicionales_ordenes' , DatosAdicionalesOrdenesController::class);
    Route::get('datos_adicionales_ordenes_por_idOrden/{idOrdenHeader?}' , [DatosAdicionalesOrdenesController::class , 'datos_adicionales_ordenes_por_idOrden']);
    Route::post('totalesPorCajaCerrada' , [OperacionController::class , 'totalesPorCajaCerrada']);
    Route::post('operaciones_por_mes' , [OperacionController::class , 'operaciones_por_mes']);
    Route::post('operaciones_por_rango' , [OperacionController::class , 'operaciones_por_rango3']);
    Route::post('quitar_n_seniat_orden' , [OrderHeaderController::class , 'quitar_n_seniat_orden']);
    Route::post('add_img_orden' , [OrderHeaderController::class , 'add_img_orden']);
    Route::put('edit_orden_body/{idOrdenBody?}' , [OrderBodyController::class , 'edit']);
    Route::put('edit_body_WHD/{idOrdenBody?}' , [OrderHeaderController::class , 'updateWHD']);
    Route::put('edit_orden_body_all' , [OrderBodyController::class , 'edit_all']); //actualizar el body comppleto de la orden
    Route::post('add_imagenes_productos' , [ProductoController::class , 'add_imagenes_productos']);
    Route::delete('delete_imagenes_productos/{idImagenesProductos}' , [ProductoController::class , 'delete_imagenes_productos']);
    Route::post('cambiar_zona_envio_orden' , [OrderHeaderController::class , 'cambiar_zona_envio_orden']);
    Route::post('editar_especificaciones_del_producto' , [ProductoController::class , 'editar_especificaciones_del_producto']);
    Route::post('actualizar_descuento_en_orden' , [OrderHeaderController::class , 'actualizar_descuento_en_orden']);
    Route::post('cambiar_status_body_masivamente' , [OrderBodyController::class , 'cambiar_status_body_masivamente']);
    Route::post('enviar_notificacion_socket' , [OrderHeaderController::class , 'enviar_notificacion_socket']);
    Route::post('listado_de_ordenes_por_fecha' , [OrderHeaderController::class , 'ordenes_listar']);
    Route::post('buscador_productos_descripcion' , [ProductoController::class , 'buscar_todo']);
    Route::post('reporte_ordenes_por_entrega' , [ReportesController::class , 'buscar_todo']);
    Route::get('enviar_sms_ordenes_' , [ReportesController::class , 'enviar_sms_ordenes_']);
    Route::post('reporte_por_asesor' , [ReportesController::class , 'reporte_por_asesor']);
    Route::post('reporte_diario' , [ReportesController::class , 'reporte_diario']);
    Route::post('reporte_diario_solo_json' , [ReportesController::class , 'reporte_diario_solo_json']);

    Route::post('reporte_ingresos' , [ReportesController::class , 'reporte_ingresos_mensual']);
    Route::post('reporte_ingresos_solo_json' , [ReportesController::class , 'reporte_ingresos_mensual_solo_json']);

    Route::post('reporte_ordenes_pagadas_sin_igtf' , [ReportesController::class , 'reporte_ordenes_pagadas_sin_igtf']);
    Route::post('reporte_ordenes_pagadas_sin_igtf_solo_json' , [ReportesController::class , 'reporte_ordenes_pagadas_sin_igtf_solo_json']);


    Route::post('reporte_ingresos_cuenta_por_cobrar' , [ReportesController::class , 'reporte_ingresos_mensual_cuenta_por_cobrar']);
    Route::post('reporte_ingresos_cuenta_por_cobrar_solo_json' , [ReportesController::class , 'reporte_ingresos_mensual_cuenta_por_cobrar_solo_json']);

    Route::post('reporte_ordenes_cliente' , [ReportesController::class , 'reporte_ordenes_cliente']);
    Route::post('reporte_ordenes_cliente_solo_json' , [ReportesController::class , 'reporte_ordenes_cliente_solo_json']);

    Route::post('inventario/filtro' , [InventarioDeProductoController::class , 'filtro']);
    Route::resource('notificacion_cobro' , NotificacionCobroController::class)->except('update');
    Route::post('ordenes_canceladas' , [OrderHeaderController::class , 'ordenes_canceladas']);
    Route::post('set_lista_negra' , [UserController::class , 'set_lista_negra']); //colocar a usuarios a lista negra
    Route::post('set_user_a_credito' , [UserController::class , 'set_user_a_credito']); //colocar usuarios a creditos
    Route::get('get_users_lista_negra' , [UserController::class , 'get_users_lista_negra']); //colocar a usuarios a lista negra
    Route::post('enviar_notificacion_cobro' , [OrderHeaderController::class , 'enviar_notificacion_cobro']);
    Route::post('enviar_notificacion_cobro_por_cliente' , [OrderHeaderController::class , 'enviar_notificacion_cobro_por_cliente']);
    Route::get('reporte_operaciones_facturadas_seniat' , [ReportesController::class , 'reporte_operaciones_facturadas_seniat']);
//listado de ordenes en deuda por asesor
    Route::post('listado_ordenes_en_deuda_por_asesor' , [OrderHeaderController::class , 'listado_ordenes_en_deuda_por_asesor']);
    Route::post('borrar_body' , [OrderBodyController::class , 'destroy2']);
    Route::post('cambiar_aplica_descuento_automatico' , [UserController::class , 'aplica_descuento_automatico']); //colocar usuario con descuento automatico
    Route::post('operacion/borrar/{idOperacion}' , [OperacionController::class , 'destroy']);


    //Nuevas 10/04/2023
    Route::post('orden/agregar_body/{idOrderHeader?}' , [OrderHeaderController::class , 'add_body_extra']);
    Route::post('orden/cambiar_tiempo_entrega/{id}' , [OrderHeaderController::class , 'cambiar_tiempo_entrega']);
    Route::post('orden/cambiar_cliente' , [OrderHeaderController::class , 'cambiar_cliente']);
    Route::post('orden/asignar_sucursal' , [OrderHeaderController::class , 'asignar_sucursal']);
    Route::post('orden/cambiar_orden_a_cotizacion' , [OrderHeaderController::class , 'cambiar_orden_a_cotizacion']);
    Route::post('orden/cambiar_orden_a_credito' , [OrderHeaderController::class , 'cambiar_orden_a_credito']);
    Route::post('orden/resumen_orden' , [OrderHeaderController::class , 'resumen_orden']);
    Route::post('orden/add_img_subtipo' , [OrderHeaderController::class , 'add_img_subtipo']);
    Route::post('orden/close_order_point_sale' , [OrderHeaderController::class , 'close_order_point_sale']);

    Route::post('reporte_ordenes_facturadas' , [ReportesController::class , 'reporte_ordenes_facturadas']);
    Route::post('reporte_conciliacion_banco' , [ReportesController::class , 'reporte_conciliacion_banco']);
    Route::post('reporte_ordenes_facturadas_a_credito' , [ReportesController::class , 'reporte_ordenes_facturadas_a_credito']);

    Route::delete('eliminar_order_header/{idOrderHeader}' , [OrderHeaderController::class , 'destroy']);
    Route::resource('vallas' , VallasController::class);
    Route::post('lote_vallas' , [VallasController::class , 'lote_store']);
    Route::post('imgenes_lote_vallas' , [VallasController::class , 'img_lote_store']);

    Route::post('reporte_ordenes_pagadas' , [ReportesController::class , 'reporte_ordenes_pagadas']);
    Route::post('reporte_ordenes_pagadas_solo_json' , [ReportesController::class , 'reporte_ordenes_pagadas_solo_json']);

    Route::post('reporte_ordenes_pagadas_no_facturadas' , [ReportesController::class , 'reporte_ordenes_pagadas_no_facturadas']);
    Route::post('reporte_ordenes_pagadas_no_facturadas_solo_json' , [ReportesController::class , 'reporte_ordenes_pagadas_no_facturadas_solo_json']);

    Route::post('orden/chek_enviado' , [OrderHeaderController::class , 'chek_enviado']);
    Route::post('orden/listar_imprimir' , [OrderHeaderController::class , 'listar_imprimir']);
    Route::get('log_vallas/listar/{id?}' , [VallasController::class , 'log_vallas_por_id']);
    Route::get('log_vallas/listar_todo' , [VallasController::class , 'log_vallas_listar_todas']);

    Route::post('crer_orden_desde_valla' , [OrderHeaderController::class , 'crer_orden_desde_valla']);
});

Route::get('vallas' , [VallasController::class , 'listar_las_vallas_activas']);
Route::get('vallas/{id}' , [VallasController::class , 'show']);

Route::get('sucursal' , [SucursalController::class , 'index']);
Route::get('getTiemposDeEntrega' , [ProductoController::class , 'getTiemposDeEntrega']);

//Route::post('totalesPorCajaCerrada' , [OperacionController::class , 'totalesPorCajaCerrada']);
//Route::resource('fecha_no_entrega' , FechaDeNoEntregaController::class);
//Route::post('operaciones_por_mes' , [OperacionController::class , 'operaciones_por_mes']);
Route::get('enviarMailPromociones' , [PromocionesController::class , 'enviarMailPromociones']);
Route::get('enviarMailDestacados' , [PromocionesController::class , 'enviarMailDestacados']);
Route::get('enviarMailOrdenNuevo' , [PromocionesController::class , 'enviarMailOrdenNuevo']);
//crea un usuario y le manda clave via mail
Route::get('order/noprintbyorder/{idSucursal}/{idEnvironment}/{idOrder}' , [OrderHeaderController::class , 'noprintbyorder']);
Route::get('order/isprint/{idOrder}' , [OrderHeaderController::class , 'isprint']);
Route::post('crear_user_clave_aleatoria' , [UserController::class , 'crear_user_clave_aleatoria']);
Route::get('listado_clientes_con_deuda' , [UserController::class , 'listado_clientes_con_deuda']);
Route::get('actualizar_dias_de_entrega' , [OrderHeaderController::class , 'refrescar_fechas_de_entrega']); //solo para actualizar los dias de entrega
Route::get('reportecomisionporventas2' , [CajaController::class , 'reportecomisionporventas2']); //solo para pruebas
Route::get('inventarioProductos' , [InventarioDeProductoController::class , 'export']);

Route::get('get_proveedores_productos' , [ProveedoresController::class , 'get_proveedores_productos']);
Route::resource('suscripciones' , SuscripcionesController::class);
Route::resource('proveedores' , ProveedoresController::class);
Route::resource('clientes_con_cuentas_por_cobrar' , ProveedoresController::class);
Route::post('listado_ordenes_cuenta_por_cobrar' , [OrderHeaderController::class , 'listado_ordenes_cuenta_por_cobrar']);


//Route::post('enviarMail' , [ProductoController::class , 'enviarMail']);
Route::post('enviarMail' , [ProductoController::class , 'enviarMailVallas']);
Route::post('enviarMailVallas' , [ProductoController::class , 'enviarMailVallas']);
Route::post('enviarMailPrueba' , [ProductoController::class , 'enviarMailPrueba']);
Route::post('enviarMailOrden' , [ProductoController::class , 'enviarMailOrden']);
Route::post('fecha_no_entrega/parametro' , [FechaDeNoEntregaController::class , 'parametro']);
Route::post('buscador_condicionado' , [ProductoController::class , 'buscador_condicionado']);
Route::post('buscador_condicionado_punto_de_venta' , [ProductoController::class , 'buscador_condicionado_punto_de_venta']);
Route::post('orden_actualizar_personal' , [OrderBodyController::class , 'actualizarPersonal']);
Route::get('initORder' , [OrderHeaderController::class , 'OrdenesNoleidas']);
Route::resource('descuento' , DescuentoController::class);
Route::post('descuento/buscar' , [DescuentoController::class , 'buscador']);
Route::post('getOrdenPorIdBody' , [OrderBodyController::class , 'getOrdenPorIdBody']);
Route::resource('galeria' , GaleriaController::class)->except('update');
Route::resource('estatusorden' , EstatusOrdenesController::class);
Route::post('galeria/editar/{idGaleria}' , [GaleriaController::class , 'update']);
Route::resource('formapago' , FormaDePagoController::class)->except('update');
Route::post('formapago/editar/{idFormaPago}' , [FormaDePagoController::class , 'update']);
//Route::get('galeria/{idGaleria}' , [GaleriaController::class,'porId']);
Route::get('init/carrio' , [OrderHeaderController::class , 'initcarrito']);
Route::get('tasamonedalistaragrupado' , [TasaMonedaController::class , 'listarAgrupado']);
Route::get('zonaenvio' , [ZonaEnvioController::class , 'index'])->name('zonaenvio.index');
Route::post('getProductosPorSubTipo' , [ProductoController::class , 'getProductosPorSubTipo']);
Route::post('user' , [UserController::class , 'store']); //para crear el usuario
Route::post('user/recuperarclave' , [UserController::class , 'recuperarclave']); //para crear el usuario
Route::post('login' , [AuthController::class , 'login']); //logear
Route::get('producto' , [ProductoController::class , 'indexSinAutenticar']);
Route::get('producto/porid/{id}' , [ProductoController::class , 'porId']);
Route::get('home' , [ProductoController::class , 'home']);

Route::post('prueba23' , [OrderBodyController::class , 'prueba']);

Route::resource('ordenHeader' , OrderHeaderController::class)->except('destroy');

Route::post('ordenHeader/cotizacion' , [OrderHeaderController::class , 'cotizacion']);
Route::post('ordenHeader/cotizacion/get' , [OrderHeaderController::class , 'cotizacion_listar']);

Route::post('ordenHeader/puntoventa/get' , [OrderHeaderController::class , 'cotizacion_listar_punto_venta']);
Route::post('ordenHeader/cotizacionpostman' , [OrderHeaderController::class , 'cotizacionpostman']);


Route::post('ordenHeader2' , [OrderHeaderController::class , 'store2']); //solo para uso de postman
Route::post('ordenHeader3' , [OrderHeaderController::class , 'store3']); //esa crear la orden y la cierra

Route::post('crea_ordenes_punto_venta' , [OrderHeaderController::class , 'crea_ordenes_punto_venta']);
Route::post('crea_ordenes_punto_venta_para_cotizar' , [OrderHeaderController::class , 'crea_ordenes_punto_venta_para_cotizar']);
Route::post('crea_ordenes_por_credito' , [OrderHeaderController::class , 'crea_ordenes_por_credito']);

Route::get('ordenHeader/porid/{idOrderHeader}' , [OrderHeaderController::class , 'porid']);

Route::post('topPedidos' , [OrderHeaderController::class , 'topPedidos']);
Route::resource('tipoPersonal' , TipoPersonalController::class);
Route::resource('personal' , PersonalController::class);
Route::get('personalportipo' , [PersonalController::class , 'agrupado']);
Route::resource('promociones' , PromocionesController::class)->except('update');
Route::post('promociones/editar/{idPromocion}' , [PromocionesController::class , 'update']);
Route::resource('noticias' , NoticiasController::class)->except('update');
Route::post('noticias/editar/{idPromocion}' , [NoticiasController::class , 'update']);
Route::resource('marcas' , MarcaController::class)->except('update');
Route::post('marcas/editar/{idPromocion}' , [MarcaController::class , 'update']);
//Route::get('/', [UserController::class, 'listar']);
Route::post('buscarOrdenPorParametros' , [OrderHeaderController::class , 'findParametros']); //recibe variable parametros

Route::get('getPerfiles' , [UserController::class , 'getPerfiles']);
Route::get('getMonedas' , [UserController::class , 'getMonedas']);
Route::get('geTipos' , [UserController::class , 'geTipos']);
Route::get('getSubTipos' , [UserController::class , 'getSubTipos']);
Route::get('getModalidades' , [UserController::class , 'getModalidades']);
Route::get('producto/getTodosSeedProductos' , [ProductoController::class , 'getTodosSeedProductos']);
Route::get('error' , function (){
    return response()->json(['msj' => 'No se encuentra autenticado'] , 401);
})->name('error');


Route::get('limpiar_cache' , function (){
    Artisan::call('config:clear');
    Artisan::call('cache:clear');
    Artisan::call('view:clear');
    Artisan::call('route:clear');

    return response()->json('cache borrada y configurada');
});

Route::get('mostrarqr/{idOrden}' , [OrderHeaderController::class , 'gereararPdf']); //enviar mail con pdf de qr
Route::get('generarqr/{idOrden}' , [OrderHeaderController::class , 'gereararPdf2']); //regresar solo qr
Route::get('generarqrbody/{idOrdenBody}' , [OrderHeaderController::class , 'gereararPdf3']); //regresar solo qr

Route::post('generarqruniversal' , [OrderHeaderController::class , 'gereararQRUniversal']); //regresar solo qr
Route::get('quitar' , function (){
    return preg_replace('/[0-9\@\.\;\" "]+/' , " " , "2824000000 AMZ.AC. INV. INICIAL RESERVAS PROBADAS ADQUIS. 3º");
}); //regresar solo qr

Route::resource('interacciones' , InteraccionesController::class)->except('index');
Route::post('interacciones_listar' , [InteraccionesController::class , 'index']);
Route::post('interacciones/actualizarLeido' , [InteraccionesController::class , 'actualizarLeido']);

Route::post('convertir_cotizacion_en_orden' , [OrderHeaderController::class , 'convertir_cotizacion_en_orden']);
Route::post('actualizar_campo_entrega_orden' , [OrderHeaderController::class , 'actualizar_campo_entrega_orden']);
Route::get('actualizar_sucursal_en_body_de_inventario' , [InventarioDeProductoController::class , 'p100']);
Route::get('obtenercomprobantepago/{idOrderHeader}' , [OrderHeaderController::class , 'obtenercomprobantepago']);

Route::post('get' , [MarcaController::class , 'prueba']);

//funcion para actualizar la tasa de las operaciones en dolares que aun estaban en null, se actualiza con tasas validas de los dias de creacion
Route::get('actualizar_tasa_operaciones' , [OperacionController::class , 'actualizar_tasa']);
Route::get('get-users' , function (){
    $user = User::find(1);
    return response()->json($user , 401);
});

Route::get('p23' , function (){
    $user = Sucursal::find(1);
    return response()->json(substr($user->nombre , 10) , 401);
});
Route::get('p101' , function (){
    $string    = 'genericosmargarita@gmail.com';
    $string    = iconv("UTF-8" , "ASCII//TRANSLIT" , $string);
    $resultado = stristr($string , 'generico');
    if ($resultado!==false){
        return response()->json("El término 'generico' se encuentra en la palabra '$string' y el resultado es '$resultado'");
    } else {
        return response()->json("El término 'generico' no se encuentra en la palabra '$string'");
    }
});

Route::get('mail_reporte_semanal' , [MetricasController::class , 'mail_reporte_semanal']);


Route::get('p470' , function (){
    $json     = '{"productos": [{"nombre": "Producto 1", "cantidad": 10}, {"nombre": "Producto 1", "cantidad": 50}, {"nombre": "Producto 3", "cantidad": 15}, {"nombre": "Producto 4", "cantidad": 20}]}';
    $array    = json_decode($json , true);
    $recuento = array();
    foreach ($array['productos'] as $producto) {
        if (isset($recuento[$producto['nombre']])){
            $recuento[$producto['nombre']] += $producto['cantidad'];
        } else {
            $recuento[$producto['nombre']] = $producto['cantidad'];
        }
    }
    arsort($recuento);
    $top_10_productos = array_slice($recuento , 0 , 10);
    echo 'Los últimos 10 productos más vendidos son: ' . json_encode($top_10_productos);
});
Route::get('p471' , function (){
    $now      = Carbon::now();
    $lastWeek = Carbon::now()->subWeek();

    echo "Lunes de la semana actual: " . $now->startOfWeek()->format('Y-m-d') . "\n";
    echo "Domingo de la semana actual: " . $now->endOfWeek()->format('Y-m-d') . "\n";
    echo "Lunes de la semana pasada: " . $lastWeek->startOfWeek()->format('Y-m-d') . "\n";
    echo "Domingo de la semana pasada: " . $lastWeek->endOfWeek()->format('Y-m-d') . "\n";
});
Route::get('actualizar_sucursal_facturada_en_orden' , [OrderHeaderController::class , 'updated_sucursal_facturada']);
Route::get('actualizar_tiempo_entrega' , [OrderHeaderController::class , 'actualizar_tiempo_entrega']);
Route::get('mostrar_en_punto_de_venta ' , [OrderHeaderController::class , 'mostrar_en_punto_de_venta']);

Route::post('z' , [OrderHeaderController::class , 'qw']);
Route::post('z_total_sin_sucursal' , [OrderHeaderController::class , 'qw2']);
Route::get('p22/{cantidad}' , [OrderHeaderController::class , 'retornar_dia_disponible_para_fechaEntrega']);


Route::get('p472' , function (){
    $string   = "Cant.: 10 Contacto:04120822226-Hola";
    $separado = @explode("Contacto:" , @explode("Cant.: " , $string)[1])[0];
    return response()->json($separado);
});
